#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: ssl_forms.cgi                                                     #
#                                                                             #
# Description: Contains CGI scripts needed to collect SSL information.        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageTime;
use LanguageResource;
use DisplayMain;
use CookieData;
use SSL;
use CGI;

###############################################################################
# GetCookieCrumbs
###############################################################################

sub GetCookieCrumbs
{
	my ($cgi, $CookieFile, $DisplayDataRef) = @_;
	my $Cookie				= new CookieData();

	# Attach to the cookie file...
	if (0 == $Cookie->Attach($CookieFile))
	{
		my $PreferredLanguage = (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		
		# Run through at most five back reference links...
		for (my $i = 0; $i < 5; ++$i)
		{
			# If all of the 'back' information is available...
			if ((defined($Cookie->GetValueKeyed("b-uri$i"))) &&
				(defined($Cookie->GetValueKeyed("b-class$i"))) &&
				(defined($Cookie->GetValueKeyed("b-label$i"))))
			{
				my $ResourcePath	= (defined($Cookie->GetValueKeyed("b-rpath$i"))) ? $Cookie->GetValueKeyed("b-rpath$i") : '/usr/ins/intel/gui/locale';
				my $Class			= $Cookie->GetValueKeyed("b-class$i") . '.lrc';
				my $TitleResource	= new LanguageResource($PreferredLanguage, $ResourcePath, [ $Class ]);
				my $Name			= $TitleResource->_S($Cookie->GetValueKeyed("b-label$i"));
				my $Location		= $Cookie->GetValueKeyed("b-uri$i");

				# Add on the next back reference.
				push(@{ $DisplayDataRef->{'cookiecrumbs'} }, { 'name' => $Name, 'location' => $Location });
			}
			else
			{
				last;
			}
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}
}

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef, $FormType)	= @_;
	my $Return = 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If there is data...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}

	# Validate based on the form type...
	if ('default' eq $FormType)
	{
		# If SSL is active...
		if ($FormDataRef->{'ssl_active'})
		{
			# Based on the certificate type...
			if ('self' eq $FormDataRef->{'certificate_type'})
			{
				# Validate the data...
				if ($FormDataRef->{'country'} !~ /^[a-zA-Z]{2}$/)
				{
					# Send back an error.
					$Return = 140001;
				}
				elsif (($FormDataRef->{'province'} !~ /^\S+$/) ||
					   ($FormDataRef->{'province'} =~ /,/))
				{
					# Send back an error.
					$Return = 140002;
				}
				elsif (($FormDataRef->{'locality'} !~ /^\S+(\s*\S+)*$/) ||
					   ($FormDataRef->{'locality'} =~ /,/))
				{
					# Send back an error.
					$Return = 140003;
				}
				elsif (($FormDataRef->{'organization'} !~ /^\S+(\s*\S+)*$/) ||
					   ($FormDataRef->{'organization'} =~ /,/))
				{
					# Send back an error.
					$Return = 140004;
				}
				elsif (($FormDataRef->{'organizational_unit'} !~ /^\S+(\s*\S+)*$/) ||
					   ($FormDataRef->{'organizational_unit'} =~ /,/))
				{
					# Send back an error.
					$Return = 140005;
				}
				elsif ($FormDataRef->{'common_name'} !~ /^(\w|-|\.|_)+$/)
				{
					# Send back an error.
					$Return = 140006;
				}
				elsif ($FormDataRef->{'e_mail'} !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
				{
					# Send back an error.
					$Return = 140007;
				}
			}
			elsif ('ca' eq $FormDataRef->{'certificate_type'})
			{
				my $Random			= int(rand() * 10000);
				my $SSLData			= new SSL('/usr/ins/intel/apache/cert', "test-$Random");
				
				# Validate the data...
				if ($FormDataRef->{'private_key'} !~ /^\s*-[\s\w\+\/=-]+-\s*$/)
				{
					# Send back an error.
					$Return = 140008;
				}
				elsif (($FormDataRef->{'certificate'} !~ /^\s*-[\s\w\+\/=-]+-\s*$/) ||
					   ($SSLData->SetCertificate($FormDataRef->{'certificate'})))
				{
					# Send back an error.
					$Return = 140009;
				}
				
				# Clean out the object.
				$SSLData->SetCertificate();
			}
			else
			{
				# Send back an error.
				$Return = 140000;
			}
		}
	}
	elsif ('generate_csr' eq $FormType)
	{
		# Validate the data...
		if ($FormDataRef->{'int_country'} !~ /^[a-zA-Z]{2}$/)
		{
			# Send back an error.
			$Return = 140001;
		}
		elsif (($FormDataRef->{'int_province'} !~ /^\S+$/) ||
			   ($FormDataRef->{'int_province'} =~ /,/))
		{
			# Send back an error.
			$Return = 140002;
		}
		elsif (($FormDataRef->{'int_locality'} !~ /^\S+(\s*\S+)*$/) ||
			   ($FormDataRef->{'int_locality'} =~ /,/))
		{
			# Send back an error.
			$Return = 140003;
		}
		elsif (($FormDataRef->{'int_organization'} !~ /^\S+(\s*\S+)*$/) ||
			   ($FormDataRef->{'int_organization'} =~ /,/))
		{
			# Send back an error.
			$Return = 140004;
		}
		elsif (($FormDataRef->{'int_organizational_unit'} !~ /^\S+(\s*\S+)*$/) ||
			   ($FormDataRef->{'int_organizational_unit'} =~ /,/))
		{
			# Send back an error.
			$Return = 140005;
		}
		elsif ($FormDataRef->{'int_common_name'} !~ /^(\w|-|\.|_)+$/)
		{
			# Send back an error.
			$Return = 140006;
		}
		elsif ($FormDataRef->{'int_e_mail'} !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
		{
			# Send back an error.
			$Return = 140007;
		}
	}

	return $Return;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef)	= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'ssl_forms.lrc', 'common.lrc' ]);
	my $Cookie					= new CookieData(('firstboot' eq $cgi->param('cookie-file')) ? 0 : undef);
	my %_R						= ( 'IDT:1400'				=> '',
									'ENABLE_SSL'			=> '',
									'INSTRUCTIONS'			=> '',
									'SIGNED_SELF'			=> '',
									'COUNTRY'				=> '',
									'COUNTRY_HINT'			=> '',
									'PROVINCE'				=> '',
									'LOCALITY'				=> '',
									'ORGANIZATION'			=> '',
									'ORGANIZATIONAL_UNIT'	=> '',
									'COMMON_NAME'			=> '',
									'COMMON_NAME_HINT'		=> '',
									'E_MAIL'				=> '',
									'SIGNED_CA'				=> '',
									'PRIVATE_KEY'			=> '',
									'GENERATE_KEY'			=> '',
									'CERTIFICATE'			=> '',
									'CANCEL'				=> '',
									'APPLY'					=> '' );
	my $CookieFileID			= $cgi->param('cookie-file');
	my $CookieFile;
	my @Keys					= ( 'post_uri',
									'ssl_active',
									'certificate_type',
									'country',
									'province',
									'locality',
									'organization',
									'organizational_unit',
									'common_name',
									'e_mail',
									'private_key',
									'certificate' );
	my $GenerateCSR;
	my $CancelButton;
	my $PageWidth;
	my $TextAreaWidth;
	my $Key;
	my %FormData;
	my $CertificateSelf;
	my $CertificateCA;
	my $Result;
	my $Feedback;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe default(s).
	$CookieFile	= "/usr/ins/intel/gui/cookies/" . $CookieFileID;
	
	# Run through all of the keys...
	foreach $Key (@Keys)
	{
		# Load in the item.
		$FormData{$Key} = $cgi->param($Key);
	}
	
	# Attach to the cookie.
	$Cookie->Attach($CookieFile);
	
	# See if this is posted information...
	unless ((defined($cgi->param('apply'))) ||
		    (defined($cgi->param('generate_csr'))))
	{
		# Overriding defaults...
		foreach $Key (@Keys)
		{
			# Load in the item.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key) unless(defined($FormData{$Key}));
		}
	}
	
	# If the post URI is not available...
	unless (defined($FormData{'post_uri'}))
	{
		# Dredge up the URI.
		$FormData{'post_uri'} = $Cookie->GetValueKeyed('post_uri');
	}
	
	# Safe default(s)...
	foreach $Key (@Keys)
	{											   
		# Set a dafe default.
		$FormData{$Key} = '' unless(defined($FormData{$Key}));
	}

	# Safe default(s)...
	$FormData{'certificate_type'} = '' unless(defined($FormData{'certificate_type'}));
	
	# If cancel was pressed...
	if (defined($cgi->param('cancel')))
	{
		# Redirect back to the parent.
		print $cgi->redirect( -url => $FormData{'post_uri'} );
		exit(0);
	}
	
	# Run through all of the keys...
	foreach $Key (@Keys)
	{
		# Save the data.
		$Cookie->SetValueKeyed($Key, $FormData{$Key});
	}
	
	# Set the display modifiers.
	$DisplayDataRef->{'root-link'}		= $Cookie->GetValueKeyed('root-link') || undef;
	$DisplayDataRef->{'title-html'}		= $Cookie->GetValueKeyed('ssl-signature-title-html') || undef;
	$DisplayDataRef->{'margin-left'}	= $Cookie->GetValueKeyed('ssl-signature-margin-left') || undef;
	$DisplayDataRef->{'helpIndicator'}	= ($Cookie->GetValueKeyed('ssl-signature-no-help')) ? 'none' : 'default';

	# If there are no cookie crumbs...
	if ($Cookie->GetValueKeyed('ssl-signature-no-cookie-crumbs'))
	{
		# No CSR - need cookie crumbs.
		$GenerateCSR = '';
		
		# Set the cancel button.
		$CancelButton = "\t\t\t\t\t<input type=submit name=cancel value='$_R{'CANCEL'}' class=clsButton>\n";
	}
	else
	{
		# Get the cookie crumbs...
		GetCookieCrumbs($cgi, $CookieFile, $DisplayDataRef) unless($Cookie->GetValueKeyed('ssl-signature-no-cookie-crumbs'));

		# Get the CSR option...
		$GenerateCSR	= ($Cookie->GetValueKeyed('ssl-signature-no-generate-csr')) ? '' : "<input type=submit name=generate_csr value='$_R{'GENERATE_KEY'}' class=clsButton>";

		# No cancel button.
		$CancelButton = '';
	}

	# Get the page and text area widths...
	$PageWidth		= ($Cookie->GetValueKeyed('ssl-signature-page-width')) ? $Cookie->GetValueKeyed('ssl-signature-page-width') : 375;
	$TextAreaWidth	= int($PageWidth / 10) + 1;

	# Detach from the cookie.
	$Cookie->Detach();
	
    # If a button has been pressed...
	if ((defined($cgi->param('apply'))) ||
		(defined($cgi->param('generate_csr'))))
	{
		# If there is data being applied...
		if (defined($cgi->param('apply')))
		{
			# Validate the posted data.
			if ($Result = ValidateFormData(\%FormData, 'default'))
			{
				# Get the error code.
				$Feedback = $StringResource->_S($Result);
			}
			else
			{
				# Attach to the cookie.
				$Cookie->Attach($CookieFile);
				
				# Run through all of the keys...
				foreach $Key (@Keys)
				{
					# Save the data.
					$Cookie->SetValueKeyed($Key, $FormData{$Key});
				}

				# Detach from the cookie.
				$Cookie->Detach();
				
				# Redirect back to the parent.
				print $cgi->redirect( -url => ($FormData{'post_uri'} . (($FormData{'post_uri'} =~ /\?/) ? '&' : '?') . "cookie-file=$CookieFileID") );
				exit(0);
			}
		}
		elsif (defined($cgi->param('generate_csr')))
		{
			# Get rid of the cookie crumbs.
			undef(@{ $DisplayDataRef->{'cookiecrumbs'} });

			# Goto the generation page.
			return 'generate_csr';
		}
	}

	# Reform the data.
	$FormData{'ssl_active'} = ($FormData{'ssl_active'}) ? 'checked' : '';
	
	# Which type of certificate is selected.
	if ('ca' eq $FormData{'certificate_type'})
	{
		# Certificate authority.
		$CertificateSelf	= '';
		$CertificateCA		= 'checked';
	}
	else
	{
		# Self-signed.
		$CertificateSelf	= 'checked';
		$CertificateCA		= '';
	}

	$html = <<HERE;
<form method=post action=/cgi-bin/common/ssl_forms.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=$PageWidth bgcolor=#eeeeee>
	<tr>
		<td bgcolor=#eeeeee>
			<input type=hidden name=type value=default>
			<input type=hidden name=cookie-file value="$CookieFileID">
			<table border=0 width=100%>
			<tr>
				<td colspan=2><b><input type=checkbox name=ssl_active value=checked $FormData{'ssl_active'}>$_R{'ENABLE_SSL'}</b></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;$_R{'INSTRUCTIONS'}</td>
			</tr> 
			<tr>
				<td colspan=2>&nbsp;<b><input type=radio name=certificate_type value=self $CertificateSelf>$_R{'SIGNED_SELF'}</b></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'COUNTRY'}</b><br><nobr>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$_R{'COUNTRY_HINT'}</nobr></td>
				<td><input type=text name=country size=3 maxlength=2 value="$FormData{'country'}"></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'PROVINCE'}</b></td>
				<td><input type=text name=province size=15 maxlength=25 value="$FormData{'province'}"></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'LOCALITY'}</b></td>
				<td><input type=text name=locality size=20 maxlength=25 value="$FormData{'locality'}"></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'ORGANIZATION'}</b></td>
				<td><input type=text name=organization size=20 maxlength=25 value="$FormData{'organization'}"></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'ORGANIZATIONAL_UNIT'}</b></td>
				<td><input type=text name=organizational_unit size=20 maxlength=25 value="$FormData{'organizational_unit'}"></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'COMMON_NAME'}</b><br><nobr>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$_R{'COMMON_NAME_HINT'}</nobr></td>
				<td><input type=text name=common_name size=20 maxlength=40 value="$FormData{'common_name'}"></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'E_MAIL'}</b></td>
				<td><input type=text name=e_mail size=20 maxlength=40 value="$FormData{'e_mail'}"></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;<b><input type=radio name=certificate_type value=ca $CertificateCA>$_R{'SIGNED_CA'}</b></td>
			</tr>
			<tr>
				<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'PRIVATE_KEY'}</b></td>
				<td align=right>$GenerateCSR&nbsp;&nbsp;&nbsp;&nbsp;</td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<textarea name=private_key rows=4 cols=$TextAreaWidth wrap=physical>$FormData{'private_key'}</textarea></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'CERTIFICATE'}</b></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<textarea name=certificate rows=4 cols=$TextAreaWidth wrap=physical>$FormData{'certificate'}</textarea></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td align=right width=100% colspan=3>
$CancelButton					<input type=submit name=apply value='$_R{'APPLY'}' class=clsButton>
	    		</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	
	# Set up the display parameters.
	$DisplayDataRef->{'q'}				= $cgi;
	$DisplayDataRef->{'html'}			= $html;
	$DisplayDataRef->{'class'}			= 'ssl_forms';
	$DisplayDataRef->{'id'}				= 1400;
	$DisplayDataRef->{'title'}			= $_R{'IDT:1400'};
	$DisplayDataRef->{'result'}			= $Result;
	$DisplayDataRef->{'feedback'}		= $Feedback;
	
	return 0;
}

###############################################################################
# RenderGenerateCSR
###############################################################################

sub RenderGenerateCSR
{
	my ($cgi, $DisplayDataRef)	= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'ssl_forms.lrc', 'common.lrc' ]);
	my $Cookie					= new CookieData();
	my %_R						= ( 'IDT:1400'				=> '',
									'IDT:1500'				=> '',
									'COUNTRY'				=> '',
									'COUNTRY_HINT'			=> '',
									'PROVINCE'				=> '',
									'LOCALITY'				=> '',
									'ORGANIZATION'			=> '',
									'ORGANIZATIONAL_UNIT'	=> '',
									'COMMON_NAME'			=> '',
									'COMMON_NAME_HINT'		=> '',
									'E_MAIL'				=> '',
									'CSR_STATUS'			=> '',
									'VIEW'					=> '',
									'SAVE'					=> '',
									'GENERATE'				=> '' );
	my $CookieFileID			= $cgi->param('cookie-file');
	my $CookieFile;
	my @Keys					= ( 'int_country',
									'int_province',
									'int_locality',
									'int_organization',
									'int_organizational_unit',
									'int_common_name',
									'int_e_mail' );
	my $PageWidth;
	my $Key;
	my %FormData;
	my $IntStatus;
	my $Result;
	my $Feedback;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe default(s).
	$CookieFile	= "/usr/ins/intel/gui/cookies/" . $CookieFileID;
	
	# Get the private key and CSR.
	$Cookie->Attach($CookieFile);
	$FormData{'int_private_key'}	= $Cookie->GetValueKeyed('int_private_key');
	$FormData{'int_csr'}			= $Cookie->GetValueKeyed('int_csr');
	
	# Set the display modifiers.
	$DisplayDataRef->{'root-link'}		= $Cookie->GetValueKeyed('root-link') || undef;
	$DisplayDataRef->{'margin-left'}	= $Cookie->GetValueKeyed('ssl-signature-margin-left') || undef;
	$DisplayDataRef->{'helpIndicator'}	= ($Cookie->GetValueKeyed('ssl-signature-no-help')) ? 'none' : 'generate_csr';

	# Get the page width.
	$PageWidth	= ($Cookie->GetValueKeyed('ssl-signature-page-width')) ? $Cookie->GetValueKeyed('ssl-signature-page-width') : 375;

	# Detach from the cookie.
	$Cookie->Detach();

	# Get the cookie crumbs.
	GetCookieCrumbs($cgi, $CookieFile, $DisplayDataRef);
	
	# Only use safe defaults if applying data...
	if ((defined($cgi->param('view'))) ||
		(defined($cgi->param('save'))) ||
		(defined($cgi->param('generate'))))
	{
		# Get the posted data...
		foreach $Key (@Keys)
		{
			# Load in the item.
			$FormData{$Key} = $cgi->param($Key);
		}
	}
	else
	{
		my $Preset = 0;

		# Get the cookie data.
		$Cookie->Attach($CookieFile);
		
		# Get the posted data...
		foreach $Key (@Keys)
		{
			# Load in the item.
			$Preset = 1 if($FormData{$Key} = $Cookie->GetValueKeyed($Key));
		}
		
		# Detach from the cookie.
		$Cookie->Detach();
		
		# Check for pre-set data...
		unless ($Preset)
		{
			# Re-used defaults.
			$Cookie->Attach($CookieFile);
			$FormData{'int_country'}				= $Cookie->GetValueKeyed('country');
			$FormData{'int_province'}				= $Cookie->GetValueKeyed('province');
			$FormData{'int_locality'}				= $Cookie->GetValueKeyed('locality');
			$FormData{'int_organization'}			= $Cookie->GetValueKeyed('organization');
			$FormData{'int_organizational_unit'}	= $Cookie->GetValueKeyed('organizational_unit');
			$FormData{'int_common_name'}			= $Cookie->GetValueKeyed('common_name');
			$FormData{'int_e_mail'}					= $Cookie->GetValueKeyed('e_mail');
			$Cookie->Detach();
		}
	}

	# Safe default(s)...
	foreach $Key (@Keys)
	{
		# Set a dafe default.
		$FormData{$Key} = '' unless(defined($FormData{$Key}));
	}
	
	# Add on the next back reference.
	push(@{ $DisplayDataRef->{'cookiecrumbs'} }, { 'name' => $_R{'IDT:1400'}, 'location' => "ssl_forms.cgi?type=default&cookie-file=$CookieFileID" });
	
	# If a button has been pressed...
	if ((defined($cgi->param('view'))) ||
		(defined($cgi->param('save'))) ||
		(defined($cgi->param('generate'))))
	{
		my $SaveCookies = 0;
		my $Return;

		# Which button was pressed...
		if (defined($cgi->param('generate')))
		{
			# Validate the posted data.
			if ($Result = ValidateFormData(\%FormData, 'generate_csr'))
			{
				# Get the error code.
				$Feedback = $StringResource->_S($Result);
			}
			else
			{
				my $Random	= int(rand() * 10000);
				my $SSLData = new SSL('/usr/ins/intel/apache/cert', "csr-$Random");
				my %Subject = ( 'country'				=> $FormData{'int_country'},
								'province'				=> $FormData{'int_province'},
								'locality'				=> $FormData{'int_locality'},
								'organization'			=> $FormData{'int_organization'},
								'organizational_unit'	=> $FormData{'int_organizational_unit'},
								'common_name'			=> $FormData{'int_common_name'},
								'e_mail'				=> $FormData{'int_e_mail'} );
	
				# Try to set the subject and get a private key...
				if ((0 == $SSLData->SetSubject(\%Subject)) &&
					($FormData{'int_private_key'} = $SSLData->GetPrivateKey(1)))
				{
					# Try to get the CSR.
					$Result = ($FormData{'int_csr'} = $SSLData->GetCertificateSigningRequest(1)) ? 0 : 150001;
				}
				else
				{
					$Result = 150000;
				}
	
				# If there was a problem...
				if ($Result)
				{
					# Get the error code.
					$Feedback = $StringResource->_S($Result);
				}
				else
				{
					# Save the cookies.
					$SaveCookies = 1;
				}
	
				# Remove the cached data.
				$SSLData->SetPrivateKey();
				$SSLData->SetCertificateSigningRequest();
			}
		}
		elsif (defined($cgi->param('view')))
		{
			# Save the cookies.
			$SaveCookies = 1;
			
			# Get rid of the cookie crumbs.
			undef(@{ $DisplayDataRef->{'cookiecrumbs'} });
	
			# Goto the review page.
			$Return = 'review_csr';
		}
		elsif (defined($cgi->param('save')))
		{
			# Get a suggested file name.
			my $TimeResource	= new LanguageTime($PreferredLanguage, '/usr/ins/intel/gui/locale');
			my $FileName		= $TimeResource->FormatTime($StringResource->_S('BACKUP_CSR_TIME_FORMAT'), localtime());
			my $Buffer			= $FormData{'int_csr'} . $FormData{'int_private_key'};
	
			# If the data is going to a Windows box...
			if ($ENV{'HTTP_USER_AGENT'} =~ /MSIE/)
			{
				# Add the carrage return...
				$Buffer =~ s/\n/\r\n/g;
			}
			
			# Puke up the requested information.
			print $cgi->header( -type => 'application/octet-stream', 'Content-Disposition' => "attachment; filename=$FileName");
			syswrite(STDOUT, $Buffer);
			exit(0);
		}

		# If the cookies need to be saved...
		if ($SaveCookies)
		{
			# Add on the cookies...
			$Cookie->Attach($CookieFile);
			
			# Run through all of the keys...
			foreach $Key (@Keys)
			{
				# Save the data.
				$Cookie->SetValueKeyed($Key, $FormData{$Key});
			}
			
			# Save the CSR.
			$Cookie->SetValueKeyed('int_csr', $FormData{'int_csr'});
			$Cookie->SetValueKeyed('int_private_key', $FormData{'int_private_key'});
			
			# Detach from the cookie.
			$Cookie->Detach();
		}

		# If the return value is set...
		if ($Return)
		{
			return $Return;
		}
	}

	# If there is a CSR and private key...
	if (($FormData{'int_csr'}) &&
		($FormData{'int_private_key'}))
	{
		# Set the status.
		$IntStatus = <<HERE;
		<table border=0 width=$PageWidth bgcolor=#eeeeee>
		<tr>
			<td bgcolor=#eeeeee>
				<table border=0 width=100%>
				<tr>
					<td><b>$_R{'CSR_STATUS'}</b></td>
					<td align=right>
						<input type=submit name=view value='$_R{'VIEW'}' class=clsButton>
						<input type=submit name=save value='$_R{'SAVE'}' class=clsButton>
					</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
		<br>
HERE
	}
	else
	{
		# Safe default.
		$IntStatus = '';
	}
	
	$html = <<HERE;
<form method=post action=/cgi-bin/common/ssl_forms.cgi enctype="application/x-www-form-urlencoded">
	$IntStatus
	<table border=0 width=$PageWidth bgcolor=#eeeeee>
	<tr>
		<td bgcolor=#eeeeee>
            <input type=hidden name=type value=generate_csr>
			<input type=hidden name=cookie-file value="$CookieFileID">
			<table border=0 width=100%>
            <tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td><b>$_R{'COUNTRY'}</b><br><nobr>&nbsp;$_R{'COUNTRY_HINT'}</nobr></td>
				<td><input type=text name=int_country size=3 maxlength=2 value="$FormData{'int_country'}"></td>
			</tr>
			<tr>
				<td><b>$_R{'PROVINCE'}</b></td>
				<td><input type=text name=int_province size=15 maxlength=25 value="$FormData{'int_province'}"></td>
			</tr>
			<tr>
				<td><b>$_R{'LOCALITY'}</b></td>
				<td><input type=text name=int_locality size=20 maxlength=25 value="$FormData{'int_locality'}"></td>
			</tr>
			<tr>
				<td><b>$_R{'ORGANIZATION'}</b></td>
				<td><input type=text name=int_organization size=20 maxlength=25 value="$FormData{'int_organization'}"></td>
			</tr>
			<tr>
				<td><b>$_R{'ORGANIZATIONAL_UNIT'}</b></td>
				<td><input type=text name=int_organizational_unit size=20 maxlength=25 value="$FormData{'int_organizational_unit'}"></td>
			</tr>
			<tr>
				<td><b>$_R{'COMMON_NAME'}</b><br><nobr>&nbsp;$_R{'COMMON_NAME_HINT'}</nobr></td>
				<td><input type=text name=int_common_name size=20 maxlength=40 value="$FormData{'int_common_name'}"></td>
			</tr>
			<tr>
				<td><b>$_R{'E_MAIL'}</b></td>
				<td><input type=text name=int_e_mail size=20 maxlength=40 value="$FormData{'int_e_mail'}"></td>
			</tr>
            <tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td align=right width=100% colspan=3>
					<input type=submit name=generate value='$_R{'GENERATE'}' class=clsButton>
	    		</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	
	# Set up the display parameters.
	$DisplayDataRef->{'q'}				= $cgi;
	$DisplayDataRef->{'html'}			= $html;
	$DisplayDataRef->{'class'}			= 'ssl_forms';
	$DisplayDataRef->{'id'}				= 1500;
	$DisplayDataRef->{'title'}			= $_R{'IDT:1500'};
	$DisplayDataRef->{'result'}			= $Result;
	$DisplayDataRef->{'feedback'}		= $Feedback;

	return 0;
}

###############################################################################
# RenderReview
###############################################################################

sub RenderReview
{
	my ($cgi, $DisplayDataRef)	= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'ssl_forms.lrc', 'common.lrc' ]);
	my $Cookie					= new CookieData();
	my %_R						= ( 'IDT:1400'						=> '',
									'IDT:1500'						=> '',
									'IDT:1600'						=> '',
									'CERTIFICATE_SIGNING_REQUEST'	=> '',
									'PRIVATE_KEY'					=> '' );
	my $PageWidth;
	my $CookieFileID			= $cgi->param('cookie-file');
	my $CookieFile;
	my $IntPrivateKey;
	my $IntCSR;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe default(s).
	$CookieFile	= "/usr/ins/intel/gui/cookies/" . $CookieFileID;
	
	# Get the private key and CSR.
	$Cookie->Attach($CookieFile);
	$IntPrivateKey	= $Cookie->GetValueKeyed('int_private_key');
	$IntCSR			= $Cookie->GetValueKeyed('int_csr');
	
	# Set the display modifiers.
	$DisplayDataRef->{'root-link'}		= $Cookie->GetValueKeyed('root-link') || undef;
	$DisplayDataRef->{'margin-left'}	= $Cookie->GetValueKeyed('ssl-signature-margin-left') || undef;

	# Get the page width.
	$PageWidth	= ($Cookie->GetValueKeyed('ssl-signature-page-width')) ? $Cookie->GetValueKeyed('ssl-signature-page-width') : 375;

	# Detach from the cookie.
	$Cookie->Detach();
	
	# Get the cookie crumbs.
	GetCookieCrumbs($cgi, $CookieFile, $DisplayDataRef);
	
	# Safe default(s).
	$IntPrivateKey	= '' unless(defined($IntPrivateKey));
	$IntCSR			= '' unless(defined($IntCSR));
	
	# Add the carrage return...
	$IntPrivateKey	=~ s/\n/<br>\r\n/g;
	$IntCSR			=~ s/\n/<br>\r\n/g;

	# Add on the next back references.
	push(@{ $DisplayDataRef->{'cookiecrumbs'} }, { 'name' => $_R{'IDT:1400'}, 'location' => "ssl_forms.cgi?type=default&cookie-file=$CookieFileID" });
	push(@{ $DisplayDataRef->{'cookiecrumbs'} }, { 'name' => $_R{'IDT:1500'}, 'location' => "ssl_forms.cgi?type=generate_csr&cookie-file=$CookieFileID" });

	$html = <<HERE;
<form method=post action=/cgi-bin/common/ssl_forms.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=$PageWidth bgcolor=white>
	<tr>
		<td bgcolor=white>
			<table border=0 width=100%>
			<tr>
				<td><b>$_R{'CERTIFICATE_SIGNING_REQUEST'}</b></td>
			</tr>
            <tr>
				<td><tt><font size=1>
$IntCSR</font></tt></td>
			</tr>
			<tr>
				<td><b>$_R{'PRIVATE_KEY'}</b></td>
			</tr>
            <tr>
				<td><tt><font size=1>
$IntPrivateKey</font></tt></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	
	# Set up the display parameters.
	$DisplayDataRef->{'q'}		= $cgi;
	$DisplayDataRef->{'html'}	= $html;
	$DisplayDataRef->{'class'}	= 'ssl_forms';
	$DisplayDataRef->{'id'}		= 1600;
	$DisplayDataRef->{'title'}	= $_R{'IDT:1600'};

	return 0;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'      eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);		last SWITCH; }
		if ('generate_csr' eq $FormType) { $FormType = RenderGenerateCSR($cgi, \%DisplayData);	last SWITCH; }
		if ('review_csr'   eq $FormType) { $FormType = RenderReview($cgi, \%DisplayData);		last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		goto SWITCH;
	}
	else
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
