#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_alerts.cgi                                                 #
#                                                                             #
# Description: CGI needed to display and modify alert configurations.         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use MonSettings;
use Authorize;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;
	
	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my $MailList		= $cgi->param('mailto');
		my @MailListList	= '';
		my $BadAddressCount	= 0;

		# Clean up the list.
		$MailList =~ s/\n/ /sg;
		$MailList =~ s/\,/ /g;
		$MailList =~ s/\;/ /g;
		$MailList =~ s/\cM//g;

		# Get the individual addresses.
		@MailListList = split(/\s+/, $MailList);

		# Run though the mail list...
		for (@MailListList)
		{
			# Look for bad addresses...
			if ($_ =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
			{
				next;
			}
			else
			{
				# Increment the bad list count.
				++$BadAddressCount;
			}
		}

		# If there are any bad addresses...
		if ($BadAddressCount)
		{
			# Set the return code.
			$Return = (1 < $BadAddressCount) ? 640001 : 640000;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
	my ($Domain, $Level)	= Authorize::getTargetDomain();
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_alerts.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:6400'					=> '',
								'SELECT_ALERT_ACTION'		=> '',
								'E-MAIL_ALERT'				=> '',
								'SERVICES'					=> '',
								'WEB'						=> '',
								'MAIL'						=> '',
								'FTP'						=> '',
								'E-MAIL_ALERT_ADDRESSES'	=> '',
								'APPLY'						=> '' );
	my $ServiceCheckState	= { };
	my $MailList			= $cgi->param('mailto');
	my $Cache;
	my $Key;
	my $Feedback;
	my $Result;
	my $html;

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	#Safe default(s).
	$MailList = '' unless(defined($MailList));

	# Get the data from the form.
	$ServiceCheckState->{'http'}	= 'checked' if(defined($cgi->param('http')));
	$ServiceCheckState->{'smtp'}	= 'checked' if(defined($cgi->param('mail')));
	$ServiceCheckState->{'ftp'}		= 'checked' if(defined($cgi->param('ftp')));

	# If applying a change...
	if (defined($cgi->param('apply')))
	{
		# Validate the data.
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			my $Monitor	= new MonSettings($Domain);

			# Open the configuration.
			if (0 == $Monitor->open())
			{
				my $Recipient;
				
				# Set the alert state.
				$Monitor->setAlertState('http', (($ServiceCheckState->{'http'}) ? 1 : 0));
				$Monitor->setAlertState('smtp', (($ServiceCheckState->{'smtp'}) ? 1 : 0));
				$Monitor->setAlertState('ftp',  (($ServiceCheckState->{'ftp' }) ? 1 : 0));

				# Clean out the e-mail addresses.
				$Monitor->deleteAllEmailAddresses();

				# Clean up the list.
				$MailList =~ s/\n/ /sg;
				$MailList =~ s/\,/ /g;
				$MailList =~ s/\;/ /g;
				$MailList =~ s/\cM//g;
				
				# Run through the recipients
				foreach $Recipient (split(/ /, $MailList))
				{
					# Add the e-mail address.
					$Monitor->addEmailAddress($Recipient) if('' ne $Recipient);
				}
				
				# Close the configuration.
				$Monitor->close();
				
				# Success.
				$Result = 0;
			}
			else
			{
				$Result = 640003;
			}
		}
		
		# If there is a problem...
		if ($Result)
		{
			# Get the feedback message.
			$Feedback = $StringResource->_S($Result);
		}
	}
	else
	{
		my $Monitor	= new MonSettings($Domain);

		# Open the configuration.
		if (0 == $Monitor->open())
		{
			my $Enumerator;

			# Get the service check state.
			$ServiceCheckState->{'http'}	= ($Monitor->getAlertState('http')) ? 'checked' : '';
			$ServiceCheckState->{'smtp'}	= ($Monitor->getAlertState('smtp')) ? 'checked' : '';
			$ServiceCheckState->{'ftp'}		= ($Monitor->getAlertState('ftp' )) ? 'checked' : '';
			
			# If there are e-mail addresses...
			if ($Enumerator = $Monitor->enumeratorEmailAddresses())
			{
				# While there are more elements...
				while ($Enumerator->hasMoreElements())
				{
					# Grow the mail list.
					$MailList .= $Enumerator->nextElement() . ' ';
				}
			}
		}
	}
	
	# Clean up the list.
	$MailList =~ s/\n/ /sg;
	$MailList =~ s/\,/ /g;
	$MailList =~ s/\;/ /g;
	$MailList =~ s/\cM//g;
	$MailList =~ s/^\s*(.*?)\s*$/$1/;
	$MailList =~ s/\s+/;\n/g;
	
	$html = <<HERE;
<form method=post action=domain_alerts.cgi enctype=application/x-www-form-urlencoded>
	<table border=0 bgcolor=#eeeeee width=375>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td>$_R{'SELECT_ALERT_ACTION'}</td>
			</tr>
			<tr>
				<td align=left valign=top>
					<table bordercolor=#eeeeee cellspacing=0 border=1 cellpadding=2 bgcolor=#ffffff width=100%>
					<tr bgcolor=gray>
						<td align=left width=140>&nbsp;</td>
						<td align=left width=220><b><font color=white>$_R{'E-MAIL_ALERT'}</font></b></td>
					</tr>
					<tr>
						<td colspan=2 bgcolor=white height=25><b>$_R{'SERVICES'}</b></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=domain_http.cgi>$_R{'WEB'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=http value=on $ServiceCheckState->{'http'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=domain_mail.cgi>$_R{'MAIL'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=mail value=on $ServiceCheckState->{'smtp'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=domain_ftp.cgi>$_R{'FTP'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=ftp value=on $ServiceCheckState->{'ftp'}></td>
					</tr>
					</table>
				</td>
			</tr>
			<tr>
				<td>&nbsp</td>
			</tr>
			<tr>
				<td>&nbsp;<b>$_R{'E-MAIL_ALERT_ADDRESSES'}</b></td>
			</tr>
			<tr>
				<td align=left>&nbsp;&nbsp; <textarea name=mailto rows=3 cols=35>$MailList</textarea></td>
			</tr>
			<tr>
				<td>&nbsp</td>
			</tr>
			<tr>
				<td align=right width=100%><input type=submit name=apply value='$_R{'APPLY'}' class=clsbutton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'domain_alerts', 'id' => 6400, 'title' => $_R{'IDT:6400'}, 
								 'root-link' => 'domain_alerts', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
