#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_mail.cgi                                                   #
#                                                                             #
# Description: CGI needed to display and modify mail settings.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use MonSettings;
use Authorize;
use Daemon;
use Mail;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_mail.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:6100'						=> '',
								'SERVICE'						=> '',
								'STATUS'						=> '',
								'CONTROLS'						=> '',
								'MAIL'							=> '',
								'ENABLE'   						=> '',
								'DISABLE'						=> '',
								'RUNNING'						=> '',
								'DISABLED'						=> '' );
	my $MailDaemon			= new Daemon('sendmail', ': accepting connections');
	my $Domain;
	my $Level;
	my $Site;
	my $HelpIndicator;
	my $ServiceStatus;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }

	# Get the associated site...
	$Site = Mail::get_site_from_domain($Domain);
	
	# If toggling the configuration state...
	if ((defined($cgi->param('control_enable'))) ||
		(defined($cgi->param('control_disable'))))
	{
		my $Monitor = new MonSettings($Domain);
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('smtp', (($cgi->param('control_enable')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}
	
	SWITCH:
	{
		# Control the services.
		if (defined($cgi->param('control_enable')))		{ Mail::enable_mail($Site, $Level); sleep(1); last SWITCH; }
		if (defined($cgi->param('control_disable')))	{ Mail::disable_mail($Site, $Level); sleep(1); last SWITCH; }
	}
	
	# Get the status information...
	$ServiceStatus	= ($MailDaemon->GetStatus() && ('checked' ne Mail::mail_check($Site))) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'DISABLED'}<font>";
	
	$html = <<HERE;
<form method=post action=domain_mail.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
			<tr> 
				<td width=100%> 
					<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
					<tr>
						<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
						<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
						<td width=40% align=left bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'MAIL'}</b></td>
						<td width=15%>$ServiceStatus</td>
						<td width=40% align=left nowrap>
							<input type=submit class=clsButton value='$_R{'ENABLE'}' name=control_enable>
							<input type=submit class=clsButton value='$_R{'DISABLE'}' name=control_disable>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			</table>
			<br>
		</td>
	</tr>
	</table>
</form>
HERE

	# Get the appropriate help indicator.
	$HelpIndicator = (10000 == (getpwnam($cgi->remote_user()))[3]) ? 'server' : 'domain';
	
	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'domain_mail', 'id' => 6100, 'title' => $_R{'IDT:6100'},
								 'root-link' => 'domain_mail', 'helpIndicator' => $HelpIndicator });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
