#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_alerts.cgi                                                     #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              alert configuration of element nodes.                          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;
	
	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my $MailList		= $cgi->param('mailto');
		my @MailListList	= '';
		my $BadAddressCount	= 0;

		# Clean up the list.
		$MailList =~ s/\n/ /sg;
		$MailList =~ s/\,/ /g;
		$MailList =~ s/\;/ /g;
		$MailList =~ s/\cM//g;

		# Get the individual addresses.
		@MailListList = split(/\s+/, $MailList);

		# Run though the mail list...
		for (@MailListList)
		{
			# Look for bad addresses...
			if ($_ =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
			{
				next;
			}
			else
			{
				# Increment the bad e-mail address counter.
				++$BadAddressCount;
			}
		}

		# If there are any bad addresses...
		if ($BadAddressCount)
		{
			# Set the return code.
			$Return = (1 < $BadAddressCount) ? 1801 : 1800;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_alerts.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_alerts', 18, '/cgi-bin/emc/cp_alerts.cgi', 5555);
	my %_R							= ( 'IDT:18'					=> '',
										'SELECT_ALERT_ACTION'		=> '',
										'E-MAIL_ALERT'				=> '',
										'SERVICES'					=> '',
										'WEB'						=> '',
										'MAIL'						=> '',
										'FTP'						=> '',
										'HARDWARE'					=> '',
										'CPU'						=> '',
										'MEMORY'					=> '',
										'DRIVES'					=> '',
										'WARNING'					=> '',
										'CRITICAL'					=> '',
										'TEMPERATURE'				=> '',
										'FANS'						=> '',
										'VOLTAGES'					=> '',
										'E-MAIL_ALERT_ADDRESSES'	=> '',
										'APPLY'						=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('apply')) ? 1 : 0;
	my $CheckHTTP					= $cgi->param('http') || '';
	my $CheckMail					= $cgi->param('mail') || '';
	my $CheckFTP					= $cgi->param('ftp') || '';
	my $CheckCPU					= $cgi->param('cpu') || '';
	my $CheckMemory					= $cgi->param('memory') || '';
	my $CheckDriveWarning			= $cgi->param('drive_warning') || '';
	my $CheckDriveCritical			= $cgi->param('drive_critical') || '';
	my $CheckTemperature			= $cgi->param('temperatures') || '';
	my $CheckFans					= $cgi->param('fans') || '';
	my $CheckVoltages				= $cgi->param('voltages') || '';
	my $MailList					= $cgi->param('mailto');
    
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe defaults.
	$MailList = '' unless(defined($MailList));

	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

	$html = <<HERE;
<form method="POST" action="go_alerts.cgi" ENCTYPE="application/x-www-form-urlencoded">
  <table border="0" bgcolor="#eeeeee" width="375">
  <tr>
    <td>
	<table border="0" width="100%">
	<tr>
	  <td>$_R{'SELECT_ALERT_ACTION'}</td>
	</tr>
    <tr>
	  <td align="left" valign="top">
	    <table bordercolor="#eeeeee" cellspacing="0" border="1" cellpadding="2" bgcolor="#ffffff" width="100%">
	    <tr bgcolor="gray">
		  <td align="left" width="140">&nbsp;
		  </td>
          <td align="left" width="220"><b><font color="white">$_R{'E-MAIL_ALERT'}</font></b>
		  </td>
		</tr>
        <tr>
		  <td colspan="2" bgcolor="white" height="25"><b>$_R{'SERVICES'}</b>
		  </td>
		</tr>
		<tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'WEB'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=http DISABLED value=checked $CheckHTTP>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=http value=checked $CheckHTTP>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'MAIL'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=mail DISABLED value=checked $CheckMail>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=mail value=checked $CheckMail>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'FTP'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=ftp DISABLED value=checked $CheckFTP>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=ftp value=checked $CheckFTP>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td colspan="2" bgcolor="white" height="25"><b>$_R{'HARDWARE'}</b>
		  </td>
		</tr>
		<tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'CPU'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=cpu DISABLED value=checked $CheckCPU>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=cpu value=checked $CheckCPU>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'MEMORY'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=memory DISABLED value=checked $CheckMemory>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=memory value=checked $CheckMemory>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td align="left" bgcolor="white"><b>&nbsp;$_R{'DRIVES'}</b><br>&nbsp; $_R{'WARNING'}<br>&nbsp; $_R{'CRITICAL'}
		  </td> 
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<br> <input type=checkbox name=drive_warning DISABLED value=checked $CheckDriveWarning>");
			document.write("<br> <input type=checkbox name=drive_critical DISABLED value=checked $CheckDriveCritical>");
		  }
		  else
		  {	   
		    document.write("<br> <input type=checkbox name=drive_warning value=checked $CheckDriveWarning>");
			document.write("<br> <input type=checkbox name=drive_critical value=checked $CheckDriveCritical>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'TEMPERATURE'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=temperatures DISABLED value=checked $CheckTemperature>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=temperatures value=checked $CheckTemperature>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'FANS'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=fans DISABLED value=checked $CheckFans>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=fans value=checked $CheckFans>");
		  }
          </script>
          </td>
		</tr>
        <tr>
		  <td bgcolor="white"><b>&nbsp;$_R{'VOLTAGES'}</b>
		  </td>
		  <td align="left" bgcolor="white">
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=checkbox name=voltages DISABLED value=checked $CheckVoltages>");
		  }
		  else
		  {	   
		    document.write("<input type=checkbox name=voltages value=checked $CheckVoltages>");
		  }
          </script>
		  </td>
		</tr>
	    </table>
        <tr>
		  <td>&nbsp;
		  </td>
		</tr>
        <tr>
		  <td>&nbsp;<b>$_R{'E-MAIL_ALERT_ADDRESSES'}</b>
		  </td>
		</tr>
        <tr>
	      <td align="left">&nbsp;&nbsp;
		  <script>
		  if(mysel == 0)
		  {
	        document.write("<textarea name=mailto DISABLED rows=3 cols=35>");
		  }
		  else
		  {	   
		    document.write("<textarea name=mailto rows=3 cols=35>");
		  }
          </script>$MailList</textarea>
          </td>
	    </tr>
        <tr>
	      <td>&nbsp;
		  </td>
	    </tr>
        <tr>
	      <td align="right" width="367">
	      <script>
		  if(mysel == 0)
		  {
	        document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=apply>");
		  }
		  else
		  {	   
		    document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>");
		  }
          </script>
	      </td>
	    </tr>
	  </td>
	  </tr>
	  </table>
    </td>
  </tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:18'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_alerts' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
