#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_mail.cgi                                                       #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for mail #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my %FormData;
		my @HostAliases;
		my @RelayDomains;
		my @RejectList;
		my $Item;

		# Cache the form data.
		$FormData{'post_master_address'}		= $cgi->param('post_master_address');
		$FormData{'smart_relay_host'}			= $cgi->param('smart_relay_host');
		$FormData{'host_aliases'}				= $cgi->param('host_aliases');
		$FormData{'relay_domains'}				= $cgi->param('relay_domains');
		$FormData{'reject_list'}				= $cgi->param('reject_list');
		$FormData{'max_incoming_message_size'}	= $cgi->param('max_incoming_message_size');
		$FormData{'max_outgoing_message_size'}	= $cgi->param('max_outgoing_message_size');
		
		# Get rid of control characters.
		$FormData{'host_aliases'}	=~ s/\n/;/sg;
		$FormData{'host_aliases'}	=~ s/\,/;/g;
		$FormData{'host_aliases'}	=~ s/\cM//g;
		$FormData{'relay_domains'}	=~ s/\n/;/sg;
		$FormData{'relay_domains'}	=~ s/\,/;/g;
		$FormData{'relay_domains'}	=~ s/\cM//g;
		$FormData{'reject_list'}	=~ s/\n/;/sg;
		$FormData{'reject_list'}	=~ s/\,/;/g;
		$FormData{'reject_list'}	=~ s/\cM//g;

		# Break out the names.
		@HostAliases	= split(/;/, $FormData{'host_aliases'});
		@RelayDomains	= split(/;/, $FormData{'relay_domains'});
		@RejectList		= split(/;/, $FormData{'reject_list'});
        
		if (($FormData{'post_master_address'}) &&
			(!(($FormData{'post_master_address'} =~ /^(\w|-|_)+$/) ||
			   ($FormData{'post_master_address'} =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/))))
		{
			# Set the return code.
			$Return = 1600;
		}
		elsif (($FormData{'max_incoming_message_size'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 1601;
		}
		elsif (($FormData{'max_outgoing_message_size'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 1602;
		}
		elsif ($FormData{'smart_relay_host'} !~ /^(\w|-|\.|_)*$/)
		{
			# Set the return code.
			$Return = 1603;
		}

		# If things are going well..
		if (0 == $Return)
		{
			# Run through all of the host aliases...
			foreach $Item (@HostAliases)
			{
				# Trim the white space.
				$Item =~ s/^\s*(.*?)\s*$/$1/;

				# If the item is not a host/domain.
				if ($Item !~ /^(\w|-|\.|_)*$/)
				{
					# Set the error code.
					$Return = 1604;
					last;
				}
			}
		}

		# If things are going well..
		if (0 == $Return)
		{
			# Run through all of the host aliases...
			foreach $Item (@RelayDomains)
			{
				# Trim the white space.
				$Item =~ s/^\s*(.*?)\s*$/$1/;
				
				# If the item is not a host/domain.
				if ($Item !~ /^(\w|-|\.|_)*$/)
				{
					# Set the error code.
					$Return = 1605;
					last;
				}
			}
		}
		
		# If things are going well..
		if (0 == $Return)
		{
			# Run through all of the host aliases...
			foreach $Item (@RejectList)
			{
				# Trim the white space.
				$Item =~ s/^\s*(.*?)\s*$/$1/;
				
				# If the item is not a host/domain.
				if (($Item) &&
					(!(($Item =~ /^(\w|-|\.|_)*$/) ||               
					   ($Item =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/))))
				{
					# Set the error code.
					$Return = 1606;
					last;
				}
			}
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_mail.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_mail', 16, '/cgi-bin/emc/cp_mail.cgi', 5555);
	my %_R							= ( 'IDT:16'						=> '',
										'SERVICE'						=> '',
										'CONTROLS'						=> '',
										'MAIL'							=> '',
										'START'							=> '',
										'STOP'							=> '',
										'RESTART'						=> '',
										'POSTMASTER_E-MAIL_ADDRESS'		=> '',
										'SMART_RELAY_HOSTNAME'			=> '',
										'REJECT_UNRESOLVED_HOST_E-MAIL'	=> '',
										'ENABLE_RBL'					=> '',
										'E-MAIL_ALIASES'				=> '',
										'E-MAIL_RELAY'					=> '',
										'E-MAIL_REJECT'					=> '',
										'MAX_MESSAGE_SIZE_INCOMING'		=> '',
										'MAX_MESSAGE_SIZE_OUTGOING'		=> '',
										'MB'							=> '',
										'APPLY'							=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('apply') || $cgi->param('control_start'  ) ||
															   $cgi->param('control_stop'   ) ||
															   $cgi->param('control_restart')) ? 1 : 0;
	my $PostMasterAddress			= $cgi->param('post_master_address');
	my $SmartRelayHost				= $cgi->param('smart_relay_host');
	my $UnresolvedHosts				= $cgi->param('unresolvable_domains') || '';
	my $BlackholeList				= $cgi->param('rbl') || '';
	my $HostAliases					= $cgi->param('host_aliases');
	my $RelayDomains				= $cgi->param('relay_domains');
	my $RejectList					= $cgi->param('reject_list');
	my $IncMessageSizeMax			= $cgi->param('max_incoming_message_size');
	my $OutMessageSizeMax			= $cgi->param('max_outgoing_message_size');

	# Safe defaults.
	$PostMasterAddress	= '' unless(defined($PostMasterAddress));
	$SmartRelayHost		= '' unless(defined($SmartRelayHost));
	$HostAliases		= '' unless(defined($HostAliases));
	$RelayDomains		= '' unless(defined($RelayDomains));
	$RejectList			= '' unless(defined($RejectList));
	$IncMessageSizeMax	= 50 unless(defined($IncMessageSizeMax));
	$OutMessageSizeMax	= 50 unless(defined($OutMessageSizeMax));

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

    $html = <<HERE;
<form method="POST" action="go_mail.cgi" ENCTYPE="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        &nbsp;
        <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
        <tr> 
          <td width="100%"> 
          <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE"> 
          <tr> 
			<td width="58%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td>
			<td width="42%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'CONTROLS'}</font></b></td>
          </tr> 
          <tr> 
		    <td width="58%"><b>$_R{'MAIL'}</b></td>
		    <td width="42%" align="left" nowrap>
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'STOP'}' name=control_stop> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'RESTART'}' name=control_restart>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop> ");
				document.write("<input type=submit class=clsButton value='$_R{'RESTART'}' name=control_restart>");
			  }
			</script>
          </tr> 
        </table> 
      </td> 
    </tr> 
  </table> 
  <table border="0" width="100%">
    <tr>
      <td width="100%" colspan="4">&nbsp;</td>
    </tr>
    <tr>
      <td width="100%" colspan="4">
	    <font color="black"><b>$_R{'POSTMASTER_E-MAIL_ADDRESS'}</b></font>&nbsp;
	  	<script>
		if(mysel == 0)
		{
	      document.write("<input type=text DISABLED name=post_master_address size=21 maxlength=35 value='$PostMasterAddress'>");
		}
		else
		{	   
		  document.write("<input type=text name=post_master_address size=21 maxlength=35 value='$PostMasterAddress'>");
		}
        </script>				  
	  </td>
    </tr>
	<tr>
      <td width="100%" colspan="4">
	    <font color="black"><b>$_R{'SMART_RELAY_HOSTNAME'}</b></font>&nbsp;
	    <script>
		if(mysel == 0)
		{
	      document.write("<input type=text DISABLED name=smart_relay_host size=26 maxlength=40 value='$SmartRelayHost'>");
		}
		else
		{	   
		  document.write("<input type=text name=smart_relay_host size=26 maxlength=40 value='$SmartRelayHost'>");
		}
        </script>				  
	  </td>
    </tr>
    <tr>
      <td width="100%" valign="top" colspan="4">&nbsp;</td>
    </tr>
    <tr>
      <td width="100%" valign="top" colspan="4">
	  	<script>
		if(mysel == 0)
		{
	      document.write("<input type=checkbox DISABLED name=unresolvable_domains value=checked $UnresolvedHosts>");
		}
		else
		{	   
		  document.write("<input type=checkbox name=unresolvable_domains value=checked $UnresolvedHosts>");
		}
        </script><font color="black"><b>$_R{'REJECT_UNRESOLVED_HOST_E-MAIL'}</b></font>
	  </td>
    </tr>
    <tr>
      <td width="100%" valign="top" colspan="4">
	    <script>
		if(mysel == 0)
		{
	      document.write("<input type=checkbox DISABLED name=rbl value=checked $BlackholeList>");
		}
		else
		{	   
		  document.write("<input type=checkbox name=rbl value=checked $BlackholeList>");
		}
        </script><font color="black"><b>$_R{'ENABLE_RBL'}</b></font>
	  </td>
    </tr>
    <tr>
      <td width="100%" colspan="4">&nbsp;</td>
    </tr>
    <tr>
      <td width="100%" colspan="4"><font color="black"><b>$_R{'E-MAIL_ALIASES'}</b></font></td>
    </tr>
    <tr>
      <td width="100%" colspan="4">
	  <script>
	  if(mysel == 0)
	  {
	    document.write("<textarea name=host_aliases DISABLED rows=2 cols=40 wrap=physical>");
	  }
	  else
	  {	   
	    document.write("<textarea name=host_aliases rows=2 cols=40 wrap=physical>");
	  }
      </script>$HostAliases</textarea>
	  </td>
    </tr>
    <tr>
      <td width="100%" colspan="4"><font color="black"><b>$_R{'E-MAIL_RELAY'}</b></font></td>
    </tr>
    <tr>
      <td width="100%" colspan="4">
	    <script>
	    if(mysel == 0)
	    {
		  document.write("<textarea name=relay_domains DISABLED rows=2 cols=40 wrap=physical>");
	    }
	    else
	    {	   
	      document.write("<textarea name=relay_domains rows=2 cols=40 wrap=physical>");
	    }
        </script>$RelayDomains</textarea>
	  </td>
    </tr>
    <tr>
      <td width="100%" colspan="4"><font color="black"><b>$_R{'E-MAIL_REJECT'}</b></font></td>
    </tr>
    <tr>
      <td width="100%" colspan="4">
	    <script>
	    if(mysel == 0)
	    {
	      document.write("<textarea name=reject_list DISABLED rows=2 cols=40 wrap=physical>");
	    }
	    else
	    {	   
	      document.write("<textarea name=reject_list rows=2 cols=40 wrap=physical>");
	    }
        </script>$RejectList</textarea>
      </td>
    </tr>
	<tr>
	  <td width="100%" colspan="2">&nbsp;</td>
	</tr>
    <tr>
      <td width="74%"><b><font color="black">$_R{'MAX_MESSAGE_SIZE_INCOMING'}</font></b></td>
      <td width="19%" align="right" nowrap colspan="2">
	  	<script>
	    if(mysel == 0)
	    {
	      document.write("<input type=text name=max_incoming_message_size DISABLED size=3 maxlength=3 value='$IncMessageSizeMax'>");
	    }
	    else
	    {	   
	      document.write("<input type=text name=max_incoming_message_size size=3 maxlength=3 value='$IncMessageSizeMax'>");
	    }
        </script>$_R{'MB'}</td>
      <td width="7%"></td>
    </tr>
    <tr>
      <td width="74%"><b><font color="black">$_R{'MAX_MESSAGE_SIZE_OUTGOING'}</font></b></td>
      <td width="19%" align="right" nowrap colspan="2">
	  	<script>
	    if(mysel == 0)
	    {
	      document.write("<input type=text name=max_outgoing_message_size DISABLED size=3 maxlength=3 value='$OutMessageSizeMax'>");
	    }
	    else
	    {	   
	      document.write("<input type=text name=max_outgoing_message_size size=3 maxlength=3 value='$OutMessageSizeMax'>");
	    }
        </script>$_R{'MB'}</td>
      <td width="7%"></td>
    </tr>
    <tr>
      <td width="100%" colspan="4">&nbsp;</td>
    </tr>
    <tr>
      <td width="100%" colspan="4" align="right">			    
	    <script>
		if(mysel == 0)
		{
	      document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=apply>");
		}
		else
		{	   
		  document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>");
		}
        </script>				  
	  </td>
    </tr>
  </table>
  </td>
</tr>
</table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:16'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_mail' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
