#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_network.cgi                                                    #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              network configuration of element nodes.                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		# Get the IP addresses and their octets.
		my $AddressDNSPrimary		= $cgi->param('dnsa_p');
		my $AddressDNSSecondary		= $cgi->param('dnsa_s');
		my $AddressDNSTertiary		= $cgi->param('dnsa_t');
		my $AddressDefaultGateway	= $cgi->param('dga');
		my @OctetsPrimary			= split(/\./, $AddressDNSPrimary);
		my @OctetsSecondary			= split(/\./, $AddressDNSSecondary);
		my @OctetsTertiary			= split(/\./, $AddressDNSTertiary);
		my @OctetsDefaultGateway	= split(/\./, $AddressDefaultGateway);
		my $Match					= '255.255.255.255';

		# Default values.
		$AddressDNSPrimary		= '' unless(defined($AddressDNSPrimary));
		$AddressDNSSecondary	= '' unless(defined($AddressDNSSecondary));
		$AddressDNSTertiary		= '' unless(defined($AddressDNSTertiary));
		$AddressDefaultGateway	= '' unless(defined($AddressDefaultGateway));
		
		# Validate the data.
		if (($AddressDNSPrimary ne '') &&
			(($AddressDNSPrimary =~ /.*$Match.*/) ||
			 ($AddressDNSPrimary !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			 (255 < $OctetsPrimary[0]) ||
			 (255 < $OctetsPrimary[1]) ||
			 (255 < $OctetsPrimary[2]) ||
			 (255 < $OctetsPrimary[3])))
		{
			# Set the return code.
			$Return = 1400;
		}
		elsif (($AddressDNSSecondary ne '') &&
			   (($AddressDNSSecondary =~ /.*$Match.*/) ||
			    ($AddressDNSSecondary !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			    (255 < $OctetsSecondary[0]) ||
			    (255 < $OctetsSecondary[1]) ||
				(255 < $OctetsSecondary[2]) ||
				(255 < $OctetsSecondary[3])))
		{
			# Set the return code.
			$Return = 1401;
		}
		elsif (($AddressDNSTertiary ne '') &&
			   (($AddressDNSTertiary =~ /.*$Match.*/) ||
			    ($AddressDNSTertiary !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			    (255 < $OctetsTertiary[0]) ||
			    (255 < $OctetsTertiary[1]) ||
				(255 < $OctetsTertiary[2]) ||
				(255 < $OctetsTertiary[3])))
		{
			# Set the return code.
			$Return = 1402;
		}
		elsif (($AddressDefaultGateway ne '') &&
			   (($AddressDefaultGateway =~ /.*$Match.*/) ||
			    ($AddressDefaultGateway !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			    (255 < $OctetsDefaultGateway[0]) ||
			    (255 < $OctetsDefaultGateway[1]) ||
				(255 < $OctetsDefaultGateway[2]) ||
				(255 < $OctetsDefaultGateway[3])))
		{
			# Set the return code.
			$Return = 1403;
		}
		elsif (((length($AddressDNSPrimary)) && 
				(length($AddressDNSSecondary)) && 
				($AddressDNSPrimary eq $AddressDNSSecondary)) ||
			   ((length($AddressDNSPrimary)) && 
				(length($AddressDNSTertiary)) && 
				($AddressDNSPrimary eq $AddressDNSTertiary)) ||
			   ((length($AddressDNSSecondary)) && 
				(length($AddressDNSTertiary)) && 
				($AddressDNSSecondary eq $AddressDNSTertiary)))
		{
			# Set the return code.
			$Return = 1404;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_network.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_network', 14, '/cgi-bin/emc/cp_network.cgi', 5555);
	my %_R							= ( 'IDT:14'					=> '',
										'SERVER_ADDRESS_PRIMARY'	=> '',
										'SERVER_ADDRESS_SECONDARY'	=> '',
										'SERVER_ADDRESS_TERTIARY'	=> '',
										'DEFAULT_GATEWAY'			=> '',
										'APPLY'						=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('apply')) ? 1 : 0;
	my $AddressDNSPrimary			= $cgi->param('dnsa_p');
	my $AddressDNSSecondary			= $cgi->param('dnsa_s');
	my $AddressDNSTertiary			= $cgi->param('dnsa_t');
	my $AddressDefaultGateway		= $cgi->param('dga');
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe defaults.
	$AddressDNSPrimary		= '' unless(defined($AddressDNSPrimary));
	$AddressDNSSecondary	= '' unless(defined($AddressDNSSecondary));
	$AddressDNSTertiary		= '' unless(defined($AddressDNSTertiary));
	$AddressDefaultGateway	= '' unless(defined($AddressDefaultGateway));

	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

	$html = <<HERE;
<form method="post" action="go_network.cgi" enctype="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td width="100%">
  <table border="0" width="100%">
    <tr>
      <td width="359" colspan="2">&nbsp;</td>
    </tr>
    <tr>
      <td width="70%" align="left" nowrap><b>$_R{'SERVER_ADDRESS_PRIMARY'}</b></td>
      <td width="30%" align="left">
	  <script>
	    if(mysel == 0)
	    {
		  document.write("<input type=text DISABLED name=dnsa_p size=15 maxlength=15 value='$AddressDNSPrimary'>");
		}
		else
		{
	      document.write("<input type=text name=dnsa_p size=15 maxlength=15 value='$AddressDNSPrimary'>");
		}
      </script>
    </tr>
    <tr>
      <td width="70%" align="left" nowrap><b>$_R{'SERVER_ADDRESS_SECONDARY'}</b></td>
      <td width="30%" align="left">
	  <script>
	    if(mysel == 0)
	    {
		  document.write("<input type=text DISABLED name=dnsa_s size=15 maxlength=15 value='$AddressDNSSecondary'>");
		}
		else
		{
	      document.write("<input type=text name=dnsa_s size=15 maxlength=15 value='$AddressDNSSecondary'>");
		}
      </script>
	  </td>
    </tr>
    <tr>
      <td width="70%" align="left" nowrap><b>$_R{'SERVER_ADDRESS_TERTIARY'}</b></td>
      <td width="30%" align="left">
	  <script>
	    if(mysel == 0)
	    {
		  document.write("<input type=text DISABLED name=dnsa_t size=15 maxlength=15 value='$AddressDNSTertiary'>");
		}
		else
		{
	      document.write("<input type=text name=dnsa_t size=15 maxlength=15 value='$AddressDNSTertiary'>");
		}
      </script>
	  </td>
    </tr>
    <tr>
      <td colspan="2" width="359">&nbsp;</td>
    </tr>
    <tr>
      <td width="70%" align="left" nowrap><b>$_R{'DEFAULT_GATEWAY'}</b></td>
      <td width="30%" align="left">
	  <script>
	    if(mysel == 0)
	    {
		  document.write("<input type=text DISABLED name=dga size=15 maxlength=15 value='$AddressDefaultGateway'>");
		}
		else
		{
	      document.write("<input type=text name=dga size=15 maxlength=15 value='$AddressDefaultGateway'>");
		}
      </script>
	  </td>
    </tr>
	<tr>
      <td colspan="2" width="359">&nbsp;</td>
    </tr>
    <tr>
      <td width="100%" align="right" nowrap colspan="2">
	  <script>
	    if(mysel == 0)
	    {
		  document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=apply>");
		}
		else
		{
	      document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>");
		}
      </script>
      </td>
    </tr>
  </table>
      </td>
    </tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:14'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_network' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
