#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: go_result.cgi                                                     #
#                                                                             #
# Description: Contains CGI script needed to display group operation results. #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;
use LanguageResource;
use DirectiveManager;
use GroupOperations;
use CGI;
use IO::File;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $ID							= $cgi->param('ID');
	my $RootLink					= $cgi->param('RL');
	my $html1						= '';
	my $html2						= '';
	my $html3						= '';
	my $html4						= '';
	my $html5						= '';
	my $Back;
	my $Title;
	my $Class;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'GroupOperations.lrc' ]);
	my $DirectiveManager			= new DirectiveManager('/usr/ins/intel/bin/wcdd-source', '/usr/ins/intel/bin/wcdd-results');
	my $GroupOperations				= new GroupOperations($PreferredLanguage);
	my %_R							= ( 'SELECT_SERVERS'	=> '',
										'JOB_RESULT'		=> '',
										'JOB_RESULTS'		=> '',
										'SERVER'			=> '',
										'PROCESSING'		=> '',
										'SUCCESSFUL'		=> '',
										'FAILED'			=> '' );
	my $ResultTarget				= '';
	my $ResultStatus				= '';
	my $ResultContent				= '';
	my $File;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
    
	# Try to get a result target...
	if ($ResultTarget = $DirectiveManager->GetResultTarget($ID))
	{
		my $ConfigurationData = new ConfigurationData;

		# Get the result status.
		$ResultStatus = $DirectiveManager->GetResultTargetStatus($ResultTarget);
				 
		# Try to get the associated class...
		if (0 == $ConfigurationData->Attach($ResultTarget))
		{
			my $TitleResource;

			# Get the associated class information.
			$Class	= $ConfigurationData->GetValueKeyed('CID_CLASS');
			$Back	= "$Class.cgi";
			
			# Create the title resource object...
			if (defined($TitleResource = new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ "$Class.lrc" ])))
			{
				# Get the title for the results.
				$Title = $TitleResource->_S("IDT:$ID");
			}

			# Detach from the configuration data.
			$ConfigurationData->Detach(1);
		}
	}

	# If the class is available...
	if (defined($Class))
	{
		# Set up the object.
		$GroupOperations->SetID($ID);
		
		# Get the result content...
		if (defined($ResultContent = $GroupOperations->GetResultContent($ResultTarget, $ResultStatus, $RootLink)))
		{
			# Tack on a line break.
			$ResultContent .= '<br>';
		}
	}
	
	# Safe defaults.
	$Back	= 'servers.cgi' unless(defined($Back));
	$Title	= $_R{'SELECT_SERVERS'} unless(defined($Title));
	$Class	= 'servers_list' unless(defined($Class));

	# Set up the cookie crumb...
	$Back	= "$Back?RL=$RootLink" if(defined($RootLink));
	
	# Build the root link if needed...
	if (defined($RootLink))
	{
		$RootLink = <<HERE;
	<script>
		selected_item = "$RootLink";
		parent.menu_frame.selectitem("$RootLink");
	</script>
HERE
	}
	else
	{
		# Safe default.
		$RootLink = '';
	}

    $html1 = <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<link rel=stylesheet href=/style/style.css type=text/css>
		<meta name=copyright content="Copyright (c) 2000 Intel Corporation, et al.">
	</head>
	<body bgcolor=#ffffff topmargin=5 leftmargin=15>
$RootLink
	<table cellspacing=1 width=100%> 
	<tr>
		<td align=left colspan=3>
			<b><font face=Arial size=1><u><a href=$Back>&lt; $Title</a></u>&nbsp;</font></b>
			<p class=clsTitlePage>$Title - $_R{'JOB_RESULTS'}</p>
		</td>
	</tr> 
	<tr>
		<td valign=top width=350>
HERE
	
	# Try to open the result file.
	$File = new IO::File("< $ResultTarget");

	# If there is a result file...
	if ($File)
	{
		my @TargetList;
		my $Target;
		my $TargetSecurity;
		my $TargetURL;
		my $TargetIP;
		my $Status;
		my $bgcolor		= '#EEEEEE';
		my $i			= 1;

		$html2 = <<HERE;
			$ResultContent
			<table border=0 width=375>
			<tr class=clsHeader>
			  <td nowrap width=60% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVER'}</font></b><font color=#ffffff>&nbsp;&nbsp;</font></td>
			  <td nowrap width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'JOB_RESULT'}</font></b><font color=#ffffff>&nbsp;&nbsp;</font></td>
			</tr>
HERE
		# Read the file into a target list.
		@TargetList = <$File>;

		# Loop through the target list...
		foreach $Target (@TargetList)
		{
			# If the header is cleared...
			if ((3 < $i) &&
				($Target =~ /=/s))
			{
				# Get the name/address and the results.
				($Target, $Status)			= split(/=/, $Target, 2);
				($Target, $TargetIP)		= split(/:/, $Target, 2);
				($TargetSecurity, $Status)	= split(/:/, $Status, 2);

				# Build the target URL.
				if (1 == $TargetSecurity)
				{
					# Use a SSL link.
					$TargetURL = "<a href='https://$TargetIP:5555' target='_blank'>" . $Target . "</a>";
				}
				else
				{
					# Use a non-SSL link.
					$TargetURL = "<a href='http://$TargetIP:5555' target='_blank'>" . $Target . "</a>";
				}

				SWITCH :
				{
					if (-1 == $Status) { $Status = "<font color=#000080>$_R{'PROCESSING'}</font>"; last SWITCH; }
					if ( 0 == $Status) { $Status = "<font color=green>$_R{'SUCCESSFUL'}</font>"; last SWITCH; }

					$Status = "<font color=red>$_R{'FAILED'}</font>";
				};

		$html3 .= <<HERE;
			<tr bgcolor=$bgcolor>
			  <td width=60%>$TargetURL</td>
			  <td width=40%>$Status</td>
			</tr>
HERE
				# Toggle the background color.
				$bgcolor = ($bgcolor eq '#EEEEEE') ? 'white' : '#EEEEEE';
			}
			else
			{
				# Move the the next header item.
				++$i;
			}
		}
	
		$html4 = <<HERE;
			</table>
HERE
	}
	else
	{
	}

	$html5 = <<HERE;
		</td> 
	</table>
	</body>
</html>
HERE

	print($cgi->header());
	print($html1 . $html2 . $html3 . $html4 . $html5);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
