#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_serverlistwait.cgi                                             #
#                                                                             #
# Description: Contains CGI script needed to render the group operations      #
#              server list wait frame.                                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use QuickHelp;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
    my $html;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_serverlist.lrc' ]);
	my $ImageResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/images');
	my $QuickHelp			= new QuickHelp($PreferredLanguage, 'go_serverlist', 1);
	my %_R					= ( 'IDT:1'				=> '',
								'ADD'				=> '',
								'DELETE'			=> '',
								'DISCOVER'			=> '',
								'REFRESH'			=> '',
								'SERVER'			=> '',
								'IP_ADDRESS'		=> '',
								'WEB'           	=> '',
								'MAIL'				=> '',
								'HARDWARE'			=> '' );
	my $ImgPathWait			= $ImageResource->ResolveResourceFile('please_wait.gif');
	my $QuickHelpTitle;
	my $QuickHelpContent;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe defaults.
	$ImgPathWait	= '' unless(defined($ImgPathWait));

	# Get just the hidden part of the file name.
	$ImgPathWait	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;

	# Get the quick help information...
	if ((defined($QuickHelpTitle	= $QuickHelp->GetTitle())) &&
		(defined($QuickHelpContent	= $QuickHelp->GetContent())) &&
		(length($QuickHelpTitle)) && (length($QuickHelpContent)))
	{
		$QuickHelpContent = <<HERE;
		<td valign=top width=100%></td> 
		<td valign=top width=177>
			<table align=left border=1 width=60% cellspacing=0 cellpadding=3 bordercolor=#808080>
			<tr bgcolor=gray>
				<td bgcolor=#808080><b><font color=#ffffff>$QuickHelpTitle</font></b></td>
			</tr>
			<tr bgcolor=#ffffcc>
				<td width=160 valign=top align=left>
					$QuickHelpContent
					<p><img width=183 src='/images/blank.gif' height=1></p>
				</td>
			</tr>
			</table>
		</td> 
HERE
	}
	else
	{
		# Safe default.
		$QuickHelpContent = '';
	}
	
	$html = <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<meta name=copyright content="Copyright (c) 2000 Intel Corporation, et al.">
		<link rel=stylesheet href=/style/style.css type=text/css>
	</head>
	<script>
	function dorefresh()
	{
		document.location='/cgi-bin/ema/go_serverlist.cgi';
	}
	</script>
	<body bgcolor=#ffffff topmargin=0 leftmargin=15>
    <table align=left cellspacing=1 border=0 cellpadding=0 bgcolor=white width=100%>
	<tr align=left valign=top>
		<td align=left colspan=3>
			<p class=clsTitlePage>$_R{'IDT:1'}</p>
			<tr align=left valign=top>
				<td valign=top>
					<table border=0 width=450 bgcolor=#eeeeee>
					<tr>
						<td>
							<table border=0 cellpadding=2 cellspacing=0 width=100% bordercolor=#eeeeee>
							<form>
							<tr>
								<td width=50% align=left>
									<input type=button class=clsButton DISABLED value='$_R{'ADD'}'>
									<input type=button class=clsButton DISABLED value='$_R{'DELETE'}'>
									<input type=button class=clsButton DISABLED value='$_R{'DISCOVER'}'>
								</td>
								<td width=50% align=right>
									<input type=button class=clsButton DISABLED value='$_R{'REFRESH'}'>
								</td>
							</tr>
							</form>
							</table>
							<br>
							<table border=0 cellpadding=0 cellspacing=0 width=100%>
							<tr>
								<td>
									<table border=1 cellspacing=0 cellpadding=2 width=100% bordercolor=#eeeeee bgcolor=#ffffff>
									<tr>
										<td width=19 align=center bgcolor=#808080>&nbsp; </td>
										<td width=135 bgcolor=#808080>
											<b><font color=#ffffff>$_R{'SERVER'}</font></b>
										</td>
										<td width=100 bgcolor=#808080>
											<b><font color=#ffffff>$_R{'IP_ADDRESS'}</font></b>
										</td>
										<td width=70 bgcolor=#808080>
											<b><font color=#ffffff>$_R{'HARDWARE'}</font></b>
										</td>
										<td width=50 bgcolor=#808080>
											<b><font color=#ffffff>$_R{'WEB'}</font></b>
										</td>
										<td width=50 bgcolor=#808080>
											<b><font color=#ffffff>$_R{'MAIL'}</font></b>
										</td>
									</tr>
									<tr>
										<td width=100% align=center bgcolor=white colspan=6>
											<img border=0 src=$ImgPathWait width=200 height=150>
										</td>
									</tr>
									</table>
								</td>
							</tr>
							</table>
						</td>
					</tr>
					</table>
				</td> 
$QuickHelpContent
			</tr>
		</td>
	</tr>
	</table>
	</body>
</html>
HERE

	print($html);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);

