#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_swupdate.cgi                                                   #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              software updates on element nodes.                             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ProductInformation;
use LanguageResource;
use GroupOperations;
use SoftwareSupport;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my ($html0, $html1, $html2);
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_swupdate.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_swupdate', 20, '/cgi-bin/emc/cp_swupdate.cgi', 5555);
	my $ProductInfo	        		= new ProductInformation();
	my %_R							= ( 'IDT:20'						=> '',
										'WARNING'						=> '',
										'INSTALL_SOFTWARE_HEADER'		=> '',
										'INSTALL_SOFTWARE_PACKAGES'		=> '',
										'SOFTWARE_PACKAGES'				=> '',
										'VERSION_DOWNLOADED'			=> '',
										'NO_PACKAGES_AVAILABLE'			=> '',
										'DOWNLOAD_PACKAGES'				=> '',
										'UNSUPPORTED_PACKAGE_WARNING'	=> '',
										'INSTALL'						=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('install')) ? 1 : 0;
	my @Manifest;
	my @PackagesSupported;
	my @PackagesUnsupported;
	my $SoftwareSupport;
	my $ManifestName;
	my $i;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
    
	# Get the product model and manifest.
	$ManifestName		= $ProductInfo->getManufacturerID() . "_" . $ProductInfo->getFamily() . "_" . $ProductInfo->getSeries();
	$ManifestName		= lc($ManifestName);
	
	# Get available update manifest from the manufacturer support site.
	$SoftwareSupport	= new SoftwareSupport($ManifestName, $ProductInfo->getVersion(), '/home/updates/');

	# Make sure the hash arrays are empty.
	undef(@Manifest);
	undef(@PackagesSupported);
	undef(@PackagesUnsupported);
	
	# Get the manifest for the product.        
	$SoftwareSupport->GetOfficialProductManifest(\@Manifest);
	
	# Cache whatever manifest was available and reload it off disk.
	$SoftwareSupport->CacheProductManifest(1);

	# Get the package lists.
	$SoftwareSupport->GetCachedPackageListSupported(\@PackagesSupported);
	$SoftwareSupport->GetCachedPackageListUnsupported(\@PackagesUnsupported);
	
	# If applying changes...
	if ($PostActivity)
	{
		# Generate the distribution directive...
		unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
		{
			# Get the error string.
			$Status = $GroupOperations->GetErrorString($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();
	
	$html0 = <<HERE;
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
	  <table border="0" width="367">
          <tr>
            <td width="359" colspan="2"><b>$_R{'INSTALL_SOFTWARE_HEADER'}</b></td>
          </tr>
          <tr>
            <td width="359" colspan="2">$_R{'INSTALL_SOFTWARE_PACKAGES'}</td>
          </tr>
          <tr>
            <td width="359" colspan="2">
              <table border="0" cellpadding="0" cellspacing="0" width="355" bgcolor="#EEEEEE">
                <tr>
                  <td width="100%">
                    <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                      <tr>
                        <td width="40%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SOFTWARE_PACKAGES'}</font></b></td>
                        <td width="20%" align="left" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'VERSION_DOWNLOADED'}</font></b></td>
                        <td width="20%" align="left" bgcolor="#808080">&nbsp;</td>
                      </tr>
HERE
	
	# Check to see if the downloaded packages are available...
	if ((defined(@PackagesSupported)) || 
		(defined(@PackagesUnsupported)))
	{
        # If there are any supported packages...
		if (defined(@PackagesSupported))
		{
			# Loop through the packages...
			for $i (0 .. $#PackagesSupported)
			{
				# Re-join the model numbers.
				my $Model = join(',', @{ $PackagesSupported[$i]{'model'} });
				
				$html1 .= <<HERE;
					  <tr>
					  <form method="POST" action="go_swupdate.cgi" ENCTYPE="application/x-www-form-urlencoded">
						<td width="40%"><b>$PackagesSupported[$i]{'display-name'}</b></td>
						<td width="20%" align="left" nowrap>$PackagesSupported[$i]{'version'}</td>
						<td width="20%" align="center">
		                  <script>
		                  if(mysel == 0)
		                  {
  	                        document.write("<input type=submit DISABLED class=clsButton value='$_R{'INSTALL'}' name=install>");
		                  }
		                  else
		                  {	   
		                    document.write("<input type=submit class=clsButton value='$_R{'INSTALL'}' name=install>");
		                  }
                          </script>                   
						  <input type="hidden" name="location" value="http://$ENV{'SERVER_ADDR'}/updates/">
						  <input type="hidden" name="filename" value="$PackagesSupported[$i]{'file-name'}">
						  <input type="hidden" name="model" value="$Model">
						  $TargetListContentHidden
						</td>
					  </form>
					  </tr>
HERE
			}
		}
		
		# If there are any unsupported packages...
		if (defined(@PackagesUnsupported))
		{
			# Loop through the packages...
			for $i (0 .. $#PackagesUnsupported)
			{
				# Re-join the model numbers.
				my $Model = join(',', @{ $PackagesUnsupported[$i]{'model'} });
				
				$html1 .= <<HERE;
					  <tr>
					  <form method="POST" action="go_swupdate.cgi" ENCTYPE="application/x-www-form-urlencoded">
						<td width="40%"><b><font color=\"#FF0000\">*</font>$PackagesUnsupported[$i]{'display-name'}</b></td>
						<td width="20%" align="left" nowrap>$PackagesUnsupported[$i]{'version'}</td>
						<td width="20%" align="center">
		                  <script>
		                  if(mysel == 0)
		                  {
  	                        document.write("<input type=submit DISABLED class=clsButton value='$_R{'INSTALL'}' name=install>");
		                  }
		                  else
		                  {	   
		                    document.write("<input type=submit class=clsButton value='$_R{'INSTALL'}' name=install>");
		                  }
                          </script>                   
						  <input type="hidden" name="location" value="http://$ENV{'SERVER_ADDR'}/updates/">
						  <input type="hidden" name="filename" value="$PackagesUnsupported[$i]{'file-name'}">
						  <input type="hidden" name="model" value="$Model">
						  $TargetListContentHidden
						</td>
					  </form>
					  </tr>
HERE
			}
		}
	}
	else
	{
		$html1 = "<tr><td width=40%><b><a href=go_swdownload.cgi?RL=go_swupdate>$_R{'NO_PACKAGES_AVAILABLE'}</a></b></td><td width=20% align=left nowrap>&nbsp;</td><td width=20% align=center>&nbsp;</td></tr>";
	}

	$html1 .= "<tr><td width=396 bgcolor=#EEEEEE colspan=3 align=left><a href=go_swdownload.cgi?RL=go_swupdate>$_R{'DOWNLOAD_PACKAGES'}</a></td></tr></table></td></tr></table></td></tr><tr><td width=2 align=left></td>";

	# Check to see if the warning needs to be displayed...
	if (defined(@PackagesUnsupported))
	{
		# Add the warning.
		$html1 .= "<td width=396 align=left><b><font color=red>$_R{'WARNING'}</font></b>&nbsp;$_R{'UNSUPPORTED_PACKAGE_WARNING'}</td></tr>";
	}
          
	$html2 = <<HERE;
		  <tr>
            <td width="404" align="right" colspan="2">
            &nbsp;
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  <br>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:20'}, 'html' => ($html0 . $html1 . $html2), 'result' => $Result, 'status' => $Status, 'root-link' => 'go_swupdate' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);