#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: cp_ftp.cgi                                                        #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for FTP  #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use VirtualServ;
use MonSettings;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
	my $Status;
	my $ConfigurationFile	= '/etc/inetd.conf';
	my $INetDaemon			= '/etc/rc.d/init.d/inet';
	my @Line;
	my $Item;
	
	# If toggling the daemon state...
	if ((defined($cgi->param('control_start'))) ||
		(defined($cgi->param('control_stop'))))
	{
		my $Monitor = new MonSettings();
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('ftp', (($cgi->param('control_start')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}
	
    # Enact the command.
	if ($cgi->param('control_stop')) 
	{
		# Read in the file.
		open(FHH, $ConfigurationFile);
		while(<FHH>) 
		{
			push(@Line, $_);
		}
		close(FHH);
	
		# Write out the new configuration.
		open(FHH, ">$ConfigurationFile");
		flock(FHH, 2);
		foreach $Item (@Line) 
		{
			if ($Item =~ /^ftp/)
			{
				print FHH "#$Item";
			}
			else 
			{ 
				print FHH $Item; 
			}
		}
		close(FHH);

		# Update the daemon.
		Safe::system($INetDaemon, 'restart');

		$Status = 0;
	}
	elsif ($cgi->param('control_start')) 
	{
		# Read in the file.
		open(FHH, $ConfigurationFile);
		while(<FHH>) 
		{
			push(@Line, $_);
		}
		close(FHH);
	
		# Write out the new configuration.
		open(FHH, ">$ConfigurationFile");
		flock(FHH, 2);
		foreach $Item (@Line) 
		{
			if ($Item =~ /(\#+)(ftp.*)$/)
			{
				print FHH "$2\n";
			}
			else 
			{ 
				print FHH "$Item"; 
			}
		}
		close(FHH);
		
		# Update the daemon.
		Safe::system($INetDaemon, 'restart');

		$Status = 0;
	}
	elsif ($cgi->param('apply'))
	{
		$cgi->import_names('G');

		# Set the FTP configuration information.
		VirtualServ::setftp($G::max_login_failure_attempts, $G::max_file_transfer_idle, $G::max_file_transfer_size, $G::max_login_users, $G::banner_anonymous_users, $G::banner_system_users, $G::enable_anonymous_ftp);
			
		$Status = 0;
	}
	else
	{
		$Status = 1;
	}
	
	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
