#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: cp_snmp.cgi                                                       #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for SNMP #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Daemon;
use Safe;
use CGI;

###############################################################################
# SetConfigData
###############################################################################

sub SetConfigData
{
	my ($Community) = @_;
	
	# Apply the new community settings.
	Safe::backtick("/usr/ins/intel/bin/asm/bakAsm set_snmp $Community");

	return 1;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)		= @_;
	my $Status		= 1;
	my $SNMPDaemon	= new Daemon('snmpd');
	my $Community;

	SWITCH:
	{
		# Control the service.
		if (defined($cgi->param('control_start')))		{ $SNMPDaemon->Control(1); $Status = 0; last SWITCH; }
		if (defined($cgi->param('control_stop')))		{ $SNMPDaemon->Control(0); $Status = 0; last SWITCH; }
		if (defined($cgi->param('control_restart')))	{ $SNMPDaemon->Control(2); $Status = 0; last SWITCH; }
	}
	
	# If applying a change...
	if ($cgi->param('apply'))
	{
		my $Success;
		my $CommunityLength;
	
		# Cache the input data.
		$Community 			= $cgi->param('community');
		$CommunityLength	= length($Community);
	
		# Make sure that the data is valid.
		unless ($Community !~ /\w{$CommunityLength}/)
		{
			# Try to set the data.
			$Status = (SetConfigData($Community)) ? 0 : 1;
		}
	}
	
	# Set the page and display it.
	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi	= new CGI;

Main($cgi);

