#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: cp_telnet.cgi                                                     #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              Telnet service control on element nodes.                       #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Daemon;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
	my $Status;
	my $SSHDaemon			= new Daemon('sshd');
	my $ConfigurationFile	= '/etc/inetd.conf';
	my $INetDaemon			= '/etc/rc.d/init.d/inet';
	my @Line;
	my $Item;
	
	# Enact the command.
	if ($cgi->param('control_stop_telnet')) 
	{
		# Read in the file.
		open(FHH, $ConfigurationFile);
		while(<FHH>) 
		{
			push(@Line, $_);
		}
		close(FHH);
	
		# Write out the new configuration.
		open(FHH, ">$ConfigurationFile");
		flock(FHH, 2);
		foreach $Item (@Line) 
		{
			if ($Item =~ /^telnet/)
			{
				print FHH "#$Item";
			}
			else 
			{ 
				print FHH $Item; 
			}
		}
		close(FHH);

		# Update the daemon.
		Safe::system($INetDaemon, 'restart');

		$Status = 0;
	}
	elsif ($cgi->param('control_start_telnet')) 
	{
		# Read in the file.
		open(FHH, $ConfigurationFile);
		while(<FHH>) 
		{
			push(@Line, $_);
		}
		close(FHH);
	
		# Write out the new configuration.
		open(FHH, ">$ConfigurationFile");
		flock(FHH, 2);
		foreach $Item (@Line) 
		{
			if ($Item =~ /(\#+)(telnet.*)$/)
			{
				print FHH "$2\n";
			}
			else 
			{ 
				print FHH "$Item"; 
			}
		}
		close(FHH);
		
		# Update the daemon.
		Safe::system($INetDaemon, 'restart');

		$Status = 0;
	}
	elsif ($cgi->param('control_start_openssh')) 
	{
		# Start the process.
		$SSHDaemon->Control(1);
		$Status = 0;
	}
	elsif ($cgi->param('control_stop_openssh')) 
	{
		# Stop the process.
		$SSHDaemon->Control(0);
		$Status = 0;
	}
	else
	{
		$Status = 1;
	}
	
	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);