#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot3.cgi                                                    #
#                                                                             #
# Description: CGI needed to display the first boot security page.            #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use CookieData;
use CGI;

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $UserID;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
		else
		{
			# Set an empty value.
			$FormDataRef->{$Key} = '';
		}
	}
	
	# Validate the data...
	if (('root' eq $FormDataRef->{'name_user'}) ||
		($FormDataRef->{'name_user'} !~ /^(\w|-|_)+$/))
	{
		# Set the return code.
		$Return = 1040000;
	}
	elsif ($FormDataRef->{'user_password'} !~ /^\S+$/)
	{
		# Set the return code.
		$Return = 1040001;
	}
	elsif ($FormDataRef->{'user_password'} ne $FormDataRef->{'user_password_confirm'})
	{
		# Set the return code.
		$Return = 1040002;
	}
	elsif ($UserID = getpwnam($FormDataRef->{'name_user'}))
	{
		# Set the return code.
		$Return = 1040003;
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot3.lrc', 'server_domains.lrc', 'common.lrc' ]);
	my $Cookie				= new CookieData(0);
	my %_R					= ( 'IDT:10400'						=> '',
								'INTRODUCTION'					=> '',
								'STEP_2_OF_3'					=> '',
								'ADMIN_LOGIN_NAME'				=> '',
								'PASSWORD'						=> '',
								'PASSWORD_CONFIRM'				=> '',
								'USE_SSL'						=> '',
								'CONFIGURE_SSL'					=> '',
								'BACK'							=> '',
								'NEXT'							=> '',
								'REQUIRED_FIELD_NOTE'			=> '' );
	my %FormData			= ( 'name_user'						=> '',
								'user_password'					=> '',
								'user_password_confirm'			=> '',
								'ssl_active'					=> '',
								'certificate_type'				=> '' );
	my %Arrow;
	my $CookieFile			= '/usr/ins/intel/gui/cookies/firstboot';
	my $SSLDetails;
	my $Item;
	my $Key;
	my $Result;
	my $Feedback;
	my $htmlTitle;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Based on the user agent...
	if (-1 < (index($cgi->user_agent(), 'MSIE')))
	{
		# Set the Microsoft arrows.
		$Arrow{'left'}	=  '&larr;';
		$Arrow{'right'}	=  '&rarr;';
	}
	else
	{
		# Set the other arrows.
		$Arrow{'left'}	=  '<-';
		$Arrow{'right'}	=  '->';
	}
	
	# If a button is pressed...
	if (defined($cgi->param('back')))
	{
		# Redirect to the previous form.
		print $cgi->redirect( -url => 'firstboot2.cgi' );
		return;
	}
	elsif ((defined($cgi->param('next'))) ||
		   (defined($cgi->param('configure_ssl'))))
	{
		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
		}

		# Validate the form data...
		if (0 == ($Result = ValidateFormData(\%FormData)))
		{
			# Attach to the cookie file.
			$Cookie->Attach($CookieFile);

			# Run through all of the keys...
			foreach $Key (keys(%FormData))
			{
				# Save the data.
				$Cookie->SetValueKeyed($Key, $FormData{$Key});
			}

			# Detach from the cookie.
			$Cookie->Detach();

			# If moving on...
			if (defined($cgi->param('next')))
			{
				# Redirect to the next form.
				print $cgi->redirect( -url => 'firstboot4.cgi' );
				return;
			}
			else
			{
				my $URL	= '/cgi-bin/common/ssl_forms.cgi?cookie-file=firstboot';

				$htmlTitle = <<HERE;
				<table width=548 border=0 cellspacing=0 cellpadding=0>
				<tr>
					<td align=left><p class=clstitlepage><b><img align=absmiddle border=0 width=24 src=/images/2.gif height=22>&nbsp;$_R{'IDT:10400'}&nbsp;-&nbsp;$_R{'CONFIGURE_SSL'}</b></p></td>
					<td align=right><b>$_R{'STEP_2_OF_3'}</b></td>
				</tr>
				</table>
HERE

				# Attach to the cookie file.
				$Cookie->Attach($CookieFile);

				# Set the POST URI.
				$Cookie->SetValueKeyed('post_uri', '/cgi-bin/firstboot/firstboot3.cgi');

				# Set up the cookie crumbs.
				$Cookie->SetValueKeyed('b-class0', 'firstboot3');
				$Cookie->SetValueKeyed('b-label0', 'IDT:10400');
				$Cookie->SetValueKeyed('b-rpath0', '/usr/ins/intel/gui/locale');
				$Cookie->SetValueKeyed('b-uri0',   '/cgi-bin/firstboot/firstboot3.cgi');

				# Set the object parameters.
				$Cookie->SetValueKeyed('ssl-signature-margin-left', 3);
				$Cookie->SetValueKeyed('ssl-signature-no-cookie-crumbs', 1);
				$Cookie->SetValueKeyed('ssl-signature-no-generate-csr', 1);
				$Cookie->SetValueKeyed('ssl-signature-page-width', 550);
				$Cookie->SetValueKeyed('ssl-signature-no-help', 1);
				$Cookie->SetValueKeyed('ssl-signature-title-html', $htmlTitle);

				# Detach from the cookie.
				$Cookie->Detach();

				# Redirect to the SSL forms.
				print $cgi->redirect( -url => $URL );
				return;
			}
		}
		else
		{
			# Get the error code.
			$Feedback = $StringResource->_S($Result);
		}
	}
	elsif ( -e $CookieFile )
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Run through all of the keys...
		foreach $Key (keys(%FormData))
		{
			# Fetch the data.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key);
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}

	# Set the SSL status text...
	if ($FormData{'ssl_active'})
	{
		# Using SSL...
		$SSLDetails = '<font color=green>' . (('self' eq $FormData{'certificate_type'}) ? $StringResource->_S('SSL_CONFIGURED_SELF') : $StringResource->_S('SSL_CONFIGURED_CA')) . '<font>';
	}
	else
	{
		# Not using SSL.
		$SSLDetails = '<font color=red>' . $StringResource->_S('SSL_NOT_CONFIGURED') . '<font>';
	}
	
	$htmlTitle = <<HERE;
	<table width=548 border=0 cellspacing=0 cellpadding=0>
	<tr>
		<td align=left><p class=clstitlepage><b><img align=absmiddle border=0 width=24 src=/images/2.gif height=22>&nbsp;$_R{'IDT:10400'}</b></p></td>
		<td align=right><b>$_R{'STEP_2_OF_3'}</b></td>
	</tr>
	</table>
HERE

	$html = <<HERE;
<table border=0 width=550 bgcolor=white>
<tr>
	<td>$_R{'INTRODUCTION'}</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td bgcolor=#eeeeee>
		<table border=0 width=100% bgcolor=#eeeeee>
		<tr>
			<td>&nbsp;
				<table cellspacing=1 border=0 width=100% bgcolor=#eeeeee>
				<form method=post action=firstboot3.cgi enctype=application/x-www-form-urlencoded>
                <input type=hidden name=ssl_active value='$FormData{'ssl_active'}'>
				<input type=hidden name=certificate_type value='$FormData{'certificate_type'}'>
				<tr>
					<td width=1% valign=top><font size=3 color=red><sup>*</sup></font></td>
					<td width=40% nowrap><b>$_R{'ADMIN_LOGIN_NAME'}</b></td>
					<td width=60%><input type=text name=name_user size=20 maxlength=25 value='$FormData{'name_user'}'></td>
				</tr>
				<tr>
					<td width=1% valign=top><font size=3 color=red><sup>*</sup></font></td>
					<td width=40% nowrap><b>$_R{'PASSWORD'}</b></td>
					<td width=60%><input type=password name=user_password size=20 maxlength=25 value='$FormData{'user_password'}'></td>
				</tr>
				<tr>
					<td width=1% valign=top><font size=3 color=red><sup>*</sup></font></td>
					<td width=40% nowrap><b>$_R{'PASSWORD_CONFIRM'}</b></td>
					<td width=60%><input type=password name=user_password_confirm size=20 maxlength=25 value='$FormData{'user_password_confirm'}'></td>
				</tr>
				<tr>
					<td colspan=3>&nbsp;</td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td nowrap><b>$_R{'USE_SSL'}</b></td>
					<td><input type=submit name=configure_ssl value='$_R{'CONFIGURE_SSL'}' class=clsButton></td>
				</tr>
				<tr>
					<td width=1%>&nbsp;</td>
					<td colspan=2>&nbsp;$SSLDetails</td>
				</tr>
    			<tr>
					<td colspan=3>&nbsp;</td>
				</tr>
				<tr>
					<td nowrap colspan=3 align=right width=532>
						<input type=submit class=clsButton value='$Arrow{'left'}&nbsp;$_R{'BACK'}' name=back>
						<input type=submit class=clsButton value='$_R{'NEXT'}&nbsp;$Arrow{'right'}' name=next>
					</td>
				</tr>
				</form>
				</table>
			</td>
		</tr>
		</table>
	</td>
</tr>
<tr>
	<td>$_R{'REQUIRED_FIELD_NOTE'}</td>
</tr>
</table>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'title-html' => $htmlTitle, 'margin-left' => 3, 
								 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
