#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot4.cgi                                                    #
#                                                                             #
# Description: CGI needed to display the first boot installation page.        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use CookieData;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot4.lrc', 'common.lrc' ]);
	my $Cookie				= new CookieData(0);
	my %_R					= ( 'IDT:10500'					=> '',
								'INTRODUCTION'				=> '',
								'STEP_3_OF_3'				=> '',
								'GO_POINT_1'				=> '',
								'GO_POINT_2'				=> '',
								'NOTE'						=> '',
								'INSTALLATION_NOTE'			=> '',
								'INSTALL_GROUP_OPERATIONS'	=> '',
								'BACK'						=> '',
								'APPLY_SETTINGS'			=> '' );
	my %FormData			= ( 'group_operations'			=> '' );
	my %Arrow;
	my $CookieFile			= '/usr/ins/intel/gui/cookies/firstboot';
	my $Key;
	my $Result;
	my $Feedback;
	my $htmlTitle;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Based on the user agent...
	if (-1 < (index($cgi->user_agent(), 'MSIE')))
	{
		# Set the Microsoft arrows.
		$Arrow{'left'}	=  '&larr;';
	}
	else
	{
		# Set the other arrows.
		$Arrow{'left'}	=  '<-';
	}
	
	# If a button is pressed...
	if (defined($cgi->param('back')))
	{
		# Redirect to the previous form.
		print $cgi->redirect( -url => 'firstboot3.cgi' );
		return;
	}
	elsif (defined($cgi->param('finish')))
	{
		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
		}

		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Run through all of the keys...
		foreach $Key (keys(%FormData))
		{
			# Save the data.
			$Cookie->SetValueKeyed($Key, $FormData{$Key});
		}

		# Detach from the cookie.
		$Cookie->Detach();
				
		# Redirect to the next form.
		print $cgi->redirect( -url => 'firstboot_finalize.cgi' );
		return;
	}
	elsif ( -e $CookieFile )
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Run through all of the keys...
		foreach $Key (keys(%FormData))
		{
			# Fetch the data.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key);
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}

	$htmlTitle = <<HERE;
	<table width=548 border=0 cellspacing=0 cellpadding=0>
	<tr>
		<td align=left><p class=clstitlepage><b><img align=absmiddle border=0 width=24 src=/images/3.gif height=22>&nbsp;$_R{'IDT:10500'}</b></p></td>
		<td align=right><b>$_R{'STEP_3_OF_3'}</b></td>
	</tr>
	</table>
HERE

	$html = <<HERE;
<table border=0 width=550 bgcolor=white>
<tr>
	<td>
		$_R{'INTRODUCTION'}
		<ul>
			<li>$_R{'GO_POINT_1'}</li>
			<li>$_R{'GO_POINT_2'}</li>
		</ul>
		<b>$_R{'NOTE'}</b>&nbsp;$_R{'INSTALLATION_NOTE'}
	</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td bgcolor=#eeeeee>
		<table border=0 width=100% bgcolor=#eeeeee>
		<tr>
			<td>&nbsp;
				<table cellspacing=1 border=0 width=100% bgcolor=#eeeeee>
				<form method=post action=firstboot4.cgi enctype=application/x-www-form-urlencoded>
				<tr>
					<td width=1%>&nbsp;</td>
					<td width=100% colspan=2 align=left>
						<input type=checkbox name=group_operations value=checked $FormData{'group_operations'}>&nbsp;<font color=black><b>$_R{'INSTALL_GROUP_OPERATIONS'}</b></font>
					</td>
				</tr>
				<tr>
					<td colspan=3>&nbsp;</td>
				</tr>
				<tr>
					<td nowrap colspan=3 align=right width=532>
						<input type=submit class=clsButton value='$Arrow{'left'}&nbsp;$_R{'BACK'}' name=back>
						<input type=submit class=clsButton value='$_R{'APPLY_SETTINGS'}' name=finish>
					</td>
				</tr>
				</form>
				</table>
			</td>
		</tr>
		</table>
	</td>
</tr>
</table>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'title-html' => $htmlTitle, 'margin-left' => 3, 
								 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
