#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_alerts.cgi                                                 #
#                                                                             #
# Description: CGI needed to display and modify alert configurations.         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;
use LanguageResource;
use LanguageLogCGI;
use DisplayMain;
use MonSettings;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;
	
	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my $MailList		= $cgi->param('mailto');
		my @MailListList	= '';
		my $BadAddressCount	= 0;

		# Clean up the list.
		$MailList =~ s/\n/ /sg;
		$MailList =~ s/\,/ /g;
		$MailList =~ s/\;/ /g;
		$MailList =~ s/\cM//g;

		# Get the individual addresses.
		@MailListList = split(/\s+/, $MailList);

		# Run though the mail list...
		for (@MailListList)
		{
			# Look for bad addresses...
			if ($_ =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
			{
				next;
			}
			else
			{
				# Increment the bad list count.
				++$BadAddressCount;
			}
		}

		# If there are any bad addresses...
		if ($BadAddressCount)
		{
			# Set the return code.
			$Return = (1 < $BadAddressCount) ? 210001 : 210000;
		}
	}

	return $Return;
}

###############################################################################
# SetSubConfiguration
###############################################################################

sub SetSubConfiguration
{
	my ($Action, $Compound, $Key, $Marker) = @_;

	# Safe default.
	$Marker = ',' unless(defined($Marker));

	# Based on the action...
	if (0 == $Action)
	{
		# Look for the key in the compound...
		if ($Compound =~ /$Key/)
		{
			my @Values = split(/$Marker/, $Compound);
			my $Value;

			# Clean out the value.
			$Compound = '';

			# Run through the values...
			foreach $Value (@Values)
			{
				# If the value is not the key...
				if ($Value ne $Key)
				{
					# Add the value back in.
					$Compound .= ('' ne $Compound) ? "$Marker$Value" : $Value;
				}
			}
		}
	}
	elsif (1 == $Action)
	{
		# Look for the key in the compound...
		if ($Compound !~ /$Key/)
		{
			# Add the item to the compound.
			$Compound .= ('' ne $Compound) ? "$Marker$Key" : $Key;
		}
	}

	return $Compound; 
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_alerts.lrc', 'common.lrc' ]);
	my $Configuration		= new ConfigurationData();
	my %_R					= ( 'IDT:2100'					=> '',
								'SELECT_ALERT_ACTION'		=> '',
								'E-MAIL_ALERT'				=> '',
								'SERVICES'					=> '',
								'WEB'						=> '',
								'MAIL'						=> '',
								'FTP'						=> '',
								'HARDWARE'					=> '',
								'CPU'						=> '',
								'MEMORY'					=> '',
								'DRIVES'					=> '',
								'WARNING'					=> '',
								'CRITICAL'					=> '',
								'TEMPERATURE'				=> '',
								'FANS'						=> '',
								'VOLTAGES'					=> '',
								'E-MAIL_ALERT_ADDRESSES'	=> '',
								'APPLY'						=> '' );
	my %CheckState			= (	'cpu'		=> '',
								'mem'		=> '',
								'drvwarn'	=> '',
								'drvcrit'	=> '',
								'temp'		=> '',
								'fan'		=> '',
								'volt'		=> '' );
	my $ServiceCheckState	= { };
	my $MailList			= $cgi->param('mailto');
	my $Cache;
	my $Key;
	my $Feedback;
	my $Result;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	#Safe default(s).
	$MailList = '' unless(defined($MailList));

	# Run through the check box states...
	foreach $Key (keys(%CheckState))
	{
		# Read in the data.
		$CheckState{$Key} = 'checked' if(defined($cgi->param($Key)));
	}
	
	# Get the data from the form.
	$ServiceCheckState->{'http'}	= 'checked' if(defined($cgi->param('http')));
	$ServiceCheckState->{'smtp'}	= 'checked' if(defined($cgi->param('mail')));
	$ServiceCheckState->{'ftp'}		= 'checked' if(defined($cgi->param('ftp')));

	# If applying a change...
	if (defined($cgi->param('apply')))
	{
		# Validate the data.
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			my $Monitor	= new MonSettings();

			# Open the configuration.
			if (0 == $Monitor->open())
			{
				my $Recipient;
				my $i = 0;
				
				# Set the alert state.
				$Monitor->setAlertState('http', (($ServiceCheckState->{'http'}) ? 1 : 0));
				$Monitor->setAlertState('smtp', (($ServiceCheckState->{'smtp'}) ? 1 : 0));
				$Monitor->setAlertState('ftp',  (($ServiceCheckState->{'ftp' }) ? 1 : 0));

				# Clean out the e-mail addresses.
				$Monitor->deleteAllEmailAddresses();

				# Clean up the list.
				$MailList =~ s/\n/ /sg;
				$MailList =~ s/\,/ /g;
				$MailList =~ s/\;/ /g;
				$MailList =~ s/\cM//g;
				
				# Run through the recipients
				foreach $Recipient (split(/ /, $MailList))
				{
					# Add the e-mail address.
					$Monitor->addEmailAddress($Recipient) if('' ne $Recipient);
				}
				
				# Close the configuration.
				$Monitor->close();
			
				# Try to open the configuration file for writing...
				if (0 == $Configuration->Attach('/etc/asm/asm.conf', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0600 }))
				{
                    # Set the hardware alerts.
					$Cache = ($Configuration->GetValueKeyed('PF_CPU_UTILIZATION_ACTIONS')) || '';
					$Configuration->SetValueKeyed('PF_CPU_UTILIZATION_ACTIONS', SetSubConfiguration((('' ne $CheckState{'cpu'}) ? 1 : 0), $Cache, 'Email'));

					$Cache = ($Configuration->GetValueKeyed('PF_MEMORY_UTILIZATION_ACTIONS')) || '';
					$Configuration->SetValueKeyed('PF_MEMORY_UTILIZATION_ACTIONS', SetSubConfiguration((('' ne $CheckState{'mem'}) ? 1 : 0), $Cache, 'Email'));

    				$Cache = ($Configuration->GetValueKeyed('DISK_LOWER_ACTIONS')) || '';
					$Configuration->SetValueKeyed('DISK_LOWER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'drvwarn'}) ? 1 : 0), $Cache, 'Email'));
					
    				$Cache = ($Configuration->GetValueKeyed('DISK_LOWERCRITICAL_ACTIONS')) || '';
					$Configuration->SetValueKeyed('DISK_LOWERCRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'drvcrit'}) ? 1 : 0), $Cache, 'Email'));
					
    				$Cache = ($Configuration->GetValueKeyed('TEMP_UPPER_CRITICAL_ACTIONS')) || '';
					$Configuration->SetValueKeyed('TEMP_UPPER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('TEMP_UPPER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('TEMP_LOWER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('TEMP_LOWER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_TEMP_UPPER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_TEMP_UPPER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_TEMP_LOWER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_TEMP_LOWER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'temp'}) ? 1 : 0), $Cache, 'Email'));
					
                    $Cache = ($Configuration->GetValueKeyed('FAN_LOWER_ACTIONS')) || '';
					$Configuration->SetValueKeyed('FAN_LOWER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'fan'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('FAN_LOWER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'fan'}) ? 1 : 0), $Cache, 'Email'));
					
    				$Cache = ($Configuration->GetValueKeyed('VOLT_UPPER_CRITICAL_ACTIONS')) || '';
					$Configuration->SetValueKeyed('VOLT_UPPER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('VOLT_UPPER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('VOLT_LOWER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('VOLT_LOWER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_VOLT_UPPER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_VOLT_UPPER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_VOLT_LOWER_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					$Configuration->SetValueKeyed('PF_VOLT_LOWER_CRITICAL_ACTIONS', SetSubConfiguration((('' ne $CheckState{'volt'}) ? 1 : 0), $Cache, 'Email'));
					
					# Run through the recipients
					foreach $Recipient (split(/ /, $MailList))
					{
						# If there is a recipient...
						if ('' ne $Recipient)
						{
							# Add the e-mail address.
							$Configuration->SetValueKeyed('EMAIL_ALERTS_ADDRESS'.($i + 1), $Recipient);
							last if(4 < ++$i);
						}
					}
					
					# Clean out the other slots...
					for ($i; 3 > $i; ++$i)
					{
						# Empty the slot.
						$Configuration->SetValueKeyed('EMAIL_ALERTS_ADDRESS'.($i + 1), '');
					}
					
					# Success.
					$Result = 0;

					# Detach from the file.
					$Configuration->Detach();
				}
				else
				{
					my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_alerts.lrc' ]);

					# Log the error and set the result code.
					$Log->Append('error', 'critical', $Result = 210002);
				}
			}
			else
			{
				$Result = 210003;
			}
		}
		
		# If there is a problem...
		if ($Result)
		{
			# Get the feedback message.
			$Feedback = $StringResource->_S($Result);
		}
	}
	else
	{
		my $Monitor	= new MonSettings();

		# Open the configuration.
		if (0 == $Monitor->open())
		{
			my $Enumerator;

			# Get the service check state.
			$ServiceCheckState->{'http'}	= ($Monitor->getAlertState('http')) ? 'checked' : '';
			$ServiceCheckState->{'smtp'}	= ($Monitor->getAlertState('smtp')) ? 'checked' : '';
			$ServiceCheckState->{'ftp'}		= ($Monitor->getAlertState('ftp' )) ? 'checked' : '';
			
			# If there are e-mail addresses...
			if ($Enumerator = $Monitor->enumeratorEmailAddresses())
			{
				# While there are more elements...
				while ($Enumerator->hasMoreElements())
				{
					# Grow the mail list.
					$MailList .= $Enumerator->nextElement() . ' ';
				}
			}
			
			# Try to open the ASM configuration file for reading...
			if (0 == $Configuration->Attach('/etc/asm/asm.conf'))
			{
				# Get the checkbox states
				$Cache = $Configuration->GetValueKeyed('PF_CPU_UTILIZATION_ACTIONS');
				$CheckState{'cpu'}		= ($Cache =~ /Email/) ? 'checked' : '';
				$Cache = $Configuration->GetValueKeyed('PF_MEMORY_UTILIZATION_ACTIONS');
				$CheckState{'mem'}		= ($Cache =~ /Email/) ? 'checked' : '';
				$Cache = $Configuration->GetValueKeyed('DISK_LOWER_ACTIONS');
				$CheckState{'drvwarn'}	= ($Cache =~ /Email/) ? 'checked' : '';
				$Cache = $Configuration->GetValueKeyed('DISK_LOWERCRITICAL_ACTIONS');
				$CheckState{'drvcrit'}	= ($Cache =~ /Email/) ? 'checked' : '';
				$Cache = $Configuration->GetValueKeyed('TEMP_UPPER_ACTIONS');
				$CheckState{'temp'}		= ($Cache =~ /Email/) ? 'checked' : '';
				$Cache = $Configuration->GetValueKeyed('FAN_LOWER_ACTIONS');
				$CheckState{'fan'}		= ($Cache =~ /Email/) ? 'checked' : '';
				$Cache = $Configuration->GetValueKeyed('VOLT_UPPER_ACTIONS');
				$CheckState{'volt'}		= ($Cache =~ /Email/) ? 'checked' : '';
	
				# Detach from the file.
				$Configuration->Detach();
			}
		}
	}
	
	# Clean up the list.
	$MailList =~ s/\n/ /sg;
	$MailList =~ s/\,/ /g;
	$MailList =~ s/\;/ /g;
	$MailList =~ s/\cM//g;
	$MailList =~ s/^\s*(.*?)\s*$/$1/;
	$MailList =~ s/\s+/;\n/g;
	
	$html = <<HERE;
<form method=post action=server_alerts.cgi enctype=application/x-www-form-urlencoded>
	<table border=0 bgcolor=#eeeeee width=375>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td>$_R{'SELECT_ALERT_ACTION'}</td>
			</tr>
			<tr>
				<td align=left valign=top>
					<table bordercolor=#eeeeee cellspacing=0 border=1 cellpadding=2 bgcolor=#ffffff width=100%>
					<tr bgcolor=gray>
						<td align=left width=140>&nbsp;</td>
						<td align=left width=220><b><font color=white>$_R{'E-MAIL_ALERT'}</font></b></td>
					</tr>
					<tr>
						<td colspan=2 bgcolor=white height=25><b>$_R{'SERVICES'}</b></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_http.cgi>$_R{'WEB'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=http value=on $ServiceCheckState->{'http'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_mail.cgi>$_R{'MAIL'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=mail value=on $ServiceCheckState->{'smtp'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_ftp.cgi>$_R{'FTP'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=ftp value=on $ServiceCheckState->{'ftp'}></td>
					</tr>
					<tr>
						<td colspan=2 bgcolor=white height=25><b>$_R{'HARDWARE'}</b></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_cpu.cgi>$_R{'CPU'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=cpu value=on $CheckState{'cpu'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_memory.cgi>$_R{'MEMORY'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=mem value=on $CheckState{'mem'}></td>
					</tr>
					<tr>
						<td align=left bgcolor=white>&nbsp<a href=server_disk.cgi><b>$_R{'DRIVES'}</b></a><br>&nbsp; $_R{'WARNING'}<br>&nbsp; $_R{'CRITICAL'}</td>
						<td align=left bgcolor=white><br>
							<input type=checkbox name=drvwarn value=on $CheckState{'drvwarn'}><br>
							<input type=checkbox name=drvcrit value=on $CheckState{'drvcrit'}>
						</td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_temperature.cgi>$_R{'TEMPERATURE'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=temp value=on $CheckState{'temp'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_fan.cgi>$_R{'FANS'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=fan value=on $CheckState{'fan'}></td>
					</tr>
					<tr>
						<td bgcolor=white><b>&nbsp<a href=server_voltage.cgi>$_R{'VOLTAGES'}</a></b></td>
						<td align=left bgcolor=white><input type=checkbox name=volt value=on $CheckState{'volt'}></td>
					</tr>
					</table>
				</td>
			</tr>
			<tr>
				<td>&nbsp</td>
			</tr>
			<tr>
				<td>&nbsp;<b>$_R{'E-MAIL_ALERT_ADDRESSES'}</b></td>
			</tr>
			<tr>
				<td align=left>&nbsp;&nbsp; <textarea name=mailto rows=3 cols=35>$MailList</textarea></td>
			</tr>
			<tr>
				<td>&nbsp</td>
			</tr>
			<tr>
				<td align=right width=100%><input type=submit name=apply value='$_R{'APPLY'}' class=clsbutton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_alerts', 'id' => 2100, 'title' => $_R{'IDT:2100'}, 
								 'root-link' => 'server_alerts', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
