#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: server_date.cgi                                                   #
#                                                                             #
# Description: Contains CGI script needed to view and configure date/time     #
#              settings.                                                      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Chronos;
use Zone;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('applya'))
	{
		my $Time = $cgi->param('time');
		my ($Hour, $Minute, $Second);

		# Validate the time...
		if ($Time =~ /^\d{1,2}:\d{1,2}:\d{1,2}$/)
		{
			my ($Hour, $Minute, $Second) = split(/:/, $Time);

			# Validate the fields.
			if ((24 > $Hour) &&
				(60 > $Minute) &&
				(60 > $Second))
			{
				my %MonthDayHash	= ( '01' => 31,
									    '02' => 29, 
										'03' => 31,
										'04' => 30,
										'05' => 31,
										'06' => 30,
										'07' => 31,
										'08' => 31,
										'09' => 30,
										'10' => 31,
										'11' => 30,
										'12' => 31 );
				my $Month			= $cgi->param('month');
				my $Date			= $cgi->param('date');
				my $Year			= $cgi->param('year');
				
				# Check for a valid date other than February.
				if ($Date <= $MonthDayHash{$Month})
                {
					# Validate for February.
					if ((2 == $Month) &&
						($Date > (((0 == ($Year % 4)) && ((!(0 == ($Year % 100))) || ($Year % 400 == 0))) ? 29 : 28)))
					{
						# Set the return.
						$Return = 200001;
					}
				}
				else
				{
					# Set the return.
					$Return = 200001;
				}
			}
			else
			{
				# Set the return.
				$Return = 200000;
			}
		}
		else
		{
			# Set the return.
			$Return = 200000;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $TimeData					= new Chronos($PreferredLanguage, '/usr/ins/intel/gui/locale');
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_date.lrc', 'common.lrc' ]);
	my %_R							= ( 'IDT:2000'				=> '',
										'TIME'					=> '',
										'DATE'					=> '',
										'TIME_ZONE'				=> '',
										'NETWORK_TIME_PROTOCOL'	=> '',
										'NTP_SERVER_ADDRESS'	=> '',
										'JAN'					=> '',
										'FEB'					=> '',
										'MAR'					=> '',
										'APR'					=> '',
										'MAY'					=> '',
										'JUN'					=> '',
										'JUL'					=> '',
										'AUG'					=> '',
										'SEP'					=> '',
										'OCT'					=> '',
										'NOV'					=> '',
										'DEC'					=> '',
										'SERVICE'				=> '',
										'STATUS'				=> '',
										'CONTROLS'				=> '',
										'STOP'					=> '',
										'CONFIRM_STOP'			=> '',
										'START'					=> '',
										'RESTART'				=> '',
										'APPLY'					=> '' );
	my @NTPServers					= ( '' );
	my $PostActivity				= ($cgi->param('applya') || 
									   $cgi->param('applyb')) ? 1 : 0;
	my %NumberMonthHash;
	my %CurrentTime;
	my $Time						= $cgi->param('time');
	my $Month						= $cgi->param('month');
	my $Date						= $cgi->param('date');
	my $Year						= $cgi->param('year');
	my $TimeZone					= $cgi->param('zone');
	my $NTPServer					= $cgi->param('ntpserver');
	my $NTPTimeZone					= $cgi->param('ntptz');
	my $SelectionMonth				= '';
	my $SelectionDate				= '';
	my $SelectionYear				= '';
	my $SelectionTimeZone			= '';
	my $SelectionNTPTimeZone		= '';
	my $Selection;
	my @TimeZones					= Zone::tzone;
	my $XNTPStatus;
	my $Item;
	my $Result;
	my $Feedback;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	SWITCH:
	{
		# Control the service.
		if (defined($cgi->param('control_start')))		{ $TimeData->ControlNTP(1); last SWITCH; }
		if (defined($cgi->param('control_stop')))		{ $TimeData->ControlNTP(0); last SWITCH; }
		if (defined($cgi->param('control_restart')))	{ $TimeData->ControlNTP(2); last SWITCH; }
	}
	
	# Fill in the hash.
	%NumberMonthHash				= ( '01' => $_R{'JAN'}, 
									    '02' => $_R{'FEB'},
									    '03' => $_R{'MAR'},
									    '04' => $_R{'APR'},
									    '05' => $_R{'MAY'},
									    '06' => $_R{'JUN'},
									    '07' => $_R{'JUL'},
									    '08' => $_R{'AUG'},
									    '09' => $_R{'SEP'},
									    '10' => $_R{'OCT'},
									    '11' => $_R{'NOV'},
									    '12' => $_R{'DEC'} );

	# Get the current time.
	$TimeData->GetTime(\%CurrentTime);
	
	# Get the current NTP server list.
	$TimeData->GetNTPServers(\@NTPServers);

    # Default values.
	$Time				= sprintf("%02d:%02d:%02d", $CurrentTime{'hour'}, $CurrentTime{'minute'}, $CurrentTime{'second'}) unless(defined($Time));
	$Month				= sprintf("%02d", $CurrentTime{'month'}) unless(defined($Month));
	$Date				= $CurrentTime{'date'} unless(defined($Date));
	$Year				= $CurrentTime{'year'} unless(defined($Year));
	$TimeZone			= $TimeData->GetTimeZone() unless(defined($TimeZone));
	$NTPServer			= ($NTPServers[0] || '') unless(defined($NTPServer));
	$NTPTimeZone		= $TimeData->GetTimeZone() unless(defined($NTPTimeZone));

	# Run through all of the months...
	foreach $Item (sort(keys(%NumberMonthHash)))
	{
		$Selection		 = ($Item eq $Month) ? 'selected' : '';
		$SelectionMonth	.= "<option value=$Item $Selection>$NumberMonthHash{$Item}\n";
	} 

	# Run through all of the dates...
	foreach $Item (1..31) 
	{
		$Selection		 = ($Item eq $Date) ? 'selected' : '';
		$SelectionDate	.= "<option value=$Item $Selection>$Item\n"; 
	}

	# Run through all of the years...
	foreach $Item (1970 .. 2037) 
	{
		$Selection		 = ($Item eq $Year) ? 'selected' : '';
		$SelectionYear	.= "<option value=$Item $Selection>$Item\n"; 
	}

	# Run through all of the timezones...	
    foreach $Item (@TimeZones) 
	{
		$Selection			 = ($Item eq $TimeZone) ? 'selected' : '';
		$SelectionTimeZone	.= "<option value=$Item $Selection>$Item\n";
	}

	# Run through all of the timezones...	
    foreach $Item (@TimeZones) 
	{
		$Selection			 	= ($Item eq $NTPTimeZone) ? 'selected' : '';
		$SelectionNTPTimeZone	.= "<option value=$Item $Selection>$Item\n";
	}
	
	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Check to see if anything needs to be applied...
			if ($cgi->param('applya')) 
			{
				# Prime the time hash.
				$CurrentTime{'year'}	= $cgi->param('year');
				$CurrentTime{'month'}	= $cgi->param('month');
				$CurrentTime{'date'}	= $cgi->param('date');
				($CurrentTime{'hour'}, $CurrentTime{'minute'}, $CurrentTime{'second'}) = split(/:/, $cgi->param('time'));

				# Try to set the timezone...
				if (0 == ($Result = $TimeData->SetTimeZone($cgi->param('zone'))))
				{
   					# Set the time...
					$Result = $TimeData->SetTime(\%CurrentTime);
				}
			}
			elsif ($cgi->param('applyb'))
			{
				# Try to set the time...
				if ((0 == ($Result = $TimeData->SetTimeZone($cgi->param('ntptz')))) &&
				  	(0 == ($Result = $TimeData->SetNTPServers([ $cgi->param('ntpserver') ]))))
				{
					# If the service is running...
					if (1 == $TimeData->GetStatusNTP())
					{
						# Restart the service.
						$TimeData->ControlNTP(2);
					}
				}
			}

			# If there is an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $TimeData->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Feedback = $StringResource->_S($Result);
		}
	}
	
	# Get the XNTP process status...
	if (1 == ($XNTPStatus = $TimeData->GetStatusNTP()))
	{
		# Indicate running.
		$XNTPStatus = '<font color=green>' . $StringResource->_S('RUNNING') . '</font>';
	}
	else
	{
		# Indicate stopped.
		$XNTPStatus = '<font color=red>' . $StringResource->_S('STOPPED') . '</font>';
	}
	
    $html = <<HERE;
<form method=post action=server_date.cgi enctype=application/x-www-form-urlencoded>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 width=100%>
			<tr>
				<td width=35%><b>$_R{'TIME'}</b></td>
				<td nowrap width=65%><input type=text name=time size=11 maxlength=8 value=$Time></td>
			</tr>
			<tr>
				<td width=35%><b>$_R{'DATE'}</b></td>
				<td nowrap width=65%>
					<select size=1 name=month>$SelectionMonth</select>
					<select size=1 name=date>$SelectionDate</select>
					<select size=1 name=year>$SelectionYear</select>
				</td>
			</tr>
			<tr>
				<td width=35%><b>$_R{'TIME_ZONE'}</b></td>
				<td nowrap width=65%><select size=1 name=zone>$SelectionTimeZone</select></td>
			</tr>
			<tr>
				<td nowrap colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td nowrap colspan=2 align=right><input type=submit class=clsButton value='$_R{'APPLY'}' name=applya></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
<form method=post action=server_date.cgi enctype=application/x-www-form-urlencoded>
<table border=0 width=375 bgcolor=#eeeeee>
<tr>
	<td colspan=2 width=100%>
		&nbsp;
		<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
		<tr>
			<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
			<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
			<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
		</tr>
		<tr>
			<td width=40%><b>$_R{'NETWORK_TIME_PROTOCOL'}</b></td>
			<td width=15%>$XNTPStatus</td>
			<td width=40% align=left nowrap>
				<input type=submit class=clsButton value="$_R{'START'}" name=control_start>
				<input type=submit class=clsButton value="$_R{'STOP'}" name=control_stop onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
				<input type=submit class=clsButton value="$_R{'RESTART'}" name=control_restart> 
			</td>
		</tr>
		</table>
		<table border=0 width=100%>
		<tr>
			<td colspan=2 align=left>&nbsp;</td>
		</tr>
		<tr>
			<td width=35% nowrap colspan=2>&nbsp;<b>$_R{'NTP_SERVER_ADDRESS'}</b>&nbsp;&nbsp;
				<input type=text name=ntpserver size=19 maxlength=32 value=$NTPServer>
			</td>
		</tr>
		<tr>
			<td colspan=2>
				<table width=100%>
				<tr>
					<td width=35%><b>$_R{'TIME_ZONE'}</b></td>
					<td nowrap width=65%><select size=1 name=ntptz>$SelectionNTPTimeZone</select></td>
				</tr>
				</table>
			</td>
		</tr>
		<tr>
			<td width=418 valign=top align=left colspan=2>&nbsp;</td>
		</tr>
		<tr>
			<td colspan=2 align=right><input type=submit class=clsButton value='$_R{'APPLY'}' name=applyb></td>
		</tr>
		</table>
	</td>
</tr>
</table>
</form>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
								 'class' => 'server_date', 'id' => 2000, 'title' => $_R{'IDT:2000'},
								 'root-link' => 'server_date', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
