#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_diag.cgi                                                   #
#                                                                             #
# Description: CGI needed to diagnose the network settings.                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use CGI;

###############################################################################
# Ping
###############################################################################

sub Ping
{
	my ($StringResource, $Hostname) = @_;

	# Autoflush output.
	$| =1;
	
	# If there is a hostname...
	if ('' ne $Hostname)
	{
		my $Command = "/bin/ping -c 5 -s 56 $Hostname";

		print '<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>';
		print '<b>' . $StringResource->_S('NETWORK_PING_STATUS') . '</b>';
		print "<hr size=2 noshade align=left>\n";
		print "<pre>\n$Command\n";
		open (PNG, "$Command 2>&1 |");
		while(<PNG>) { print $_ ; }
		close (PNG);	
		print "</pre>\n<hr size=2 noshade align=left>\n\n";
	}
}	

###############################################################################
# DNSLookup
###############################################################################

sub DNSLookup
{ 
	my ($StringResource, $Hostname) = @_;

	# Autoflush output.
	$| =1;
	
	# If there is a hostname...
	if ('' ne $Hostname)
	{
		my $Command = "/usr/bin/nslookup -query=any $Hostname";

		print '<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>';
		print '<b>' . $StringResource->_S('DNS_LOOKUP_STATUS') . '</b>';
		print "<hr size=2 noshade align=centre>\n";
		print "<pre>\n$Command\n";
		open (PNG, "$Command 2>&1 |");
		while(<PNG>) { print $_ ; }
		close (PNG);	
		print "</pre>\n<hr size=2 noshade align=center>\n\n";
	}
}

###############################################################################
# TraceRoute
###############################################################################

sub TraceRoute
{	
	my ($StringResource, $Hostname) = @_;

	# Autoflush output.
	$| =1;
	
	# If there is a hostname...
	if ('' ne $Hostname)
	{
		my $Command = "/usr/sbin/traceroute -m 30 $Hostname 40 2>&1";
		
		print '<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>';
		print '<b>' . $StringResource->_S('TRACE_ROUTE_STATUS') . '</b>';
		print "<HR SIZE=2 NOSHADE ALIGN=centre>\n";
		print "<PRE>\n$Command\n";
		open (PNG, "$Command 2>&1 |");
		while(<PNG>) { print $_ ; }
		close (PNG);	
		print "</pre>\n<hr size=2 noshade align=center>\n\n";
	}
}	

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_network.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:4500'						=> '',
								'IDT:4800'						=> '',
								'HOSTNAME_IPADDRESS_LABEL'		=> '',
								'PING'							=> '',
								'DNS_LOOKUP'					=> '',
								'TRACE_ROUTE'					=> '');
	my $Hostname			= $cgi->param('hostname');
	my $Feedback;
	my $Result;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe default.
	$Hostname = '' unless(defined($Hostname));

	# If there is data posted...
	if ((($cgi->param('ping')) ||
		 ($cgi->param('nslookup')) ||
		 ($cgi->param('trace_route'))) &&
		(($Hostname) && !($Hostname =~ /^[A-z0-9\.\-]+$/)))
	{
		# Set the status.
		$Result		= 480000;
		$Feedback	= $StringResource->_S($Result);
	}

	$html = <<HERE;
<form method=post action=server_diag.cgi enctype=application/x-www-form-urlencoded>
<table border=0 width=375 bgcolor=#eeeeee>
<tr>
	<td>
		<table cellspacing=0 border=0 cellpadding=0 bgcolor=#eeeeee width=100%>
		<tr>
			<td width=100% colspan=2>&nbsp</td>
		</tr>
		<tr>
			<td align=center>&nbsp;<b>$_R{'HOSTNAME_IPADDRESS_LABEL'}</b></td>
			<td><input type=text name=hostname size=25 maxlength=25 value='$Hostname'></td>
		</tr>
		<tr>
			<td width=100% colspan=2>&nbsp;</td>
		</tr>
		<tr>
			<td align=center colspan=2>&nbsp;
				<input type=submit name=ping value='$_R{'PING'}' class=clsbutton>
				<input type=submit name=nslookup value='$_R{'DNS_LOOKUP'}' class=clsbutton>
				<input type=submit name=trace_route value='$_R{'TRACE_ROUTE'}' class=clsbutton>
			</td>
		</tr>
		<tr>
			<td width=100% colspan=2>&nbsp</td>
		</tr>
		</table>
	</td>
</tr>
</table>
</form>
HERE

	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:4500'}, 'location' => 'server_network.cgi' } ],
								 'class' => 'server_network', 'id' => 4800, 'title' => $_R{'IDT:4800'},
								 'root-link' => 'server_network', 'result' => $Result, 'feedback' => $Feedback });

	# If everything is going well...
	unless (defined($Result))
	{
		# Perform the activity...
		if ($cgi->param('ping'))
		{
			Ping($StringResource, $Hostname);
		}
		elsif ($cgi->param('nslookup'))
		{
			DNSLookup($StringResource, $Hostname);
		}
		elsif ($cgi->param('trace_route'))
		{
			TraceRoute($StringResource, $Hostname);
		}
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
