#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_addrecord_A.cgi                                        #
#                                                                             #
# Description: Creates A records (for host address records) for a zone on     #
#              this appliance.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use DNSZone;
use DNSConfig;

########################################
my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:9100'			=> '',
		'IDQH:9100'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',

		'USEDEFAULTDOMAINNAME'		=> '',
		'CREATENEWHOST'			=> '',
		'IPADDR'			=> '',
		'ADD'				=> '',

		'DEFAULT_MESSAGE'		=> '');


$StringResource->GetStrings(\%_R);

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'namechoice'		=> 'nc_alt',
			'domainname'		=> '',
			'ipaddr'		=> '',
		);

########################################
			
if ( $cgi->param('addrecord') )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}

# Set up the radio state.
$FormData{'namechoice_ext'}		= ('nc_ext' eq $FormData{'namechoice'}) ? 'checked' : '';
$FormData{'namechoice_alt'}		= ('nc_alt' eq $FormData{'namechoice'}) ? 'checked' : '';

########################################


my $PAGE = <<HERE;
<FORM ACTION="server_dns_addrecord_A.cgi" name=addrec METHOD="POST" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">
<INPUT TYPE="hidden" NAME="addrecord" VALUE="1">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <tr>
    <TD WIDTH="100%" colspan="2">&nbsp;
    </TD>
  </tr>
  <tr>
    <TD WIDTH="100%" colspan="2"><INPUT TYPE="radio" NAME="namechoice" VALUE="nc_ext" $FormData{'namechoice_ext'}> <B>$_R{'USEDEFAULTDOMAINNAME'} (DOMAINNAME)</B>
    </TD>
  </TR>
  <tr>
    <TD WIDTH="50%" NOWRAP><INPUT TYPE="radio" NAME="namechoice" VALUE="nc_alt" $FormData{'namechoice_alt'}> <b>$_R{'CREATENEWHOST'}:</b>
    </TD>
    <TD WIDTH="50%" NOWRAP><INPUT TYPE="text" NAME="domainname" SIZE=10 MAXLENGTH=48 VALUE="$FormData{'domainname'}"> <B>.DOMAINNAME</B>
    </TD>
  </tr>
  <tr>
    <TD WIDTH="50%" NOWRAP><b>&nbsp;$_R{'IPADDR'}:</b>
    </TD>
    <TD WIDTH="50%" NOWRAP><INPUT TYPE="text" NAME="ipaddr" SIZE=20 MAXLENGTH=15 VALUE="$FormData{'ipaddr'}">
    </TD>
  </tr>
  <TR>
    <TD colspan="2" width="100%">&nbsp;
    </TD>
  </TR>
  <TR>
    <TD colspan="2" align="right" width="100%"><input type="submit" class="clsButton" value=" $_R{'ADD'} " name="Add">
    </TD>
  </TR>
</TABLE>

</FORM>
HERE


###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html, $domainname) = @_;
	
	# Set the community information.
	$html =~ s/DOMAINNAME/$domainname/g;

	return $html;
}


###############################################################################
# InvalidIP
###############################################################################

sub InvalidIP
{
	my ($ipaddress,$useless) = @_;
	
	my ($ipA,$ipB,$ipC,$ipD);
	my ($t1,$t2);
	my $tempIP = $ipaddress;

	my $chkval = ( $tempIP =~ /(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})/ );

	if (!($chkval) )
	{
		# If the string does not match the approved format
		return 1;  # false
	}
	elsif ( ($` ne '') || ($' ne '') || ($& eq '') )
	{
		return 1;
		# Matched, but there's extra data before or after the string.
	}
	else
	{
		($ipA,$ipB,$ipC,$ipD) = ( $tempIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
		
		if ( ($ipA < 0) || ($ipA > 255) )
		{
			return 1;
		}
		elsif ( ($ipB < 0) || ($ipB > 255) )
		{
			return 1;
		}
		elsif ( ($ipC < 0) || ($ipC > 255) )
		{
			return 1;
		}
		elsif ( ($ipD < 0) || ($ipD > 255) )
		{
			return 1;
		}
		else
		{
			return 0;
		}

	}

}

###############################################################################
# InvalidFQDN
###############################################################################

sub InvalidFQDN
{
	my ($thisfqdn,$useless) = @_;
	
	# Look for valid fqdn host names.
	
	if ($thisfqdn =~ /_/)
	{
		return 1;
	}
	
	if ($thisfqdn =~ /^(\w|-)+(\.((\w|-)+))*$/ )
	{
		return 0;  
	}
	
	else
	{
		return 1;

	}

}

###############################################################################
# InvalidHost
###############################################################################

sub InvalidHost
{
	my ($thishost,$useless) = @_;
	
	# Look for valid host names only.
	
	if ($thishost =~ /_/)
	{
		return 1;
	}
	
	if ($thishost =~ /^(\w|-)+$/ )
	{
		return 0;  
	}
	
	else
	{
		return 1;

	}

}

###############################################################################
# The main body of the script.
###############################################################################

my $html;
my $ia;
my $ib;
my $i;
my $NumZones;

# If applying a change...
if ($cgi->param('Add'))
{
	my $zonename = $cgi->param('zname');        

	my @zone_dat_list;

	$ia = DNSConfig->zone_list(\@zone_dat_list);
	if ($ia ne 0)
	{
		$Result = 910002;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		goto DISPLAYMAINFORM;
	}

	$NumZones = scalar(@zone_dat_list);

	# Iterate through data structure and display meaningful information.
	for ($i = 0; $i < $NumZones; $i++)
	{
		# for each zone print out the records
		if($zone_dat_list[$i][0] eq $zonename)
		{
			my $realdomname = lc($cgi->param('domainname'));
			my $znamechoice = lc($cgi->param('namechoice'));
			my $ipaddr = lc($cgi->param('ipaddr'));

			if ( ($ipaddr eq '') || (InvalidIP($ipaddr)) )
			{
				# Missing or invalid IP address!
				$Result = 910009;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addrec';
				$FocusField = 'ipaddr';
				goto DISPLAYMAINFORM;
			}
			
			if ($znamechoice eq 'nc_ext')
			{
				$realdomname = $zonename . '.';
			}

			if ($realdomname eq '')
			{
				$Result = 910001;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				if ($znamechoice eq 'nc_alt')
				{
					$UseFocus = 1;
					$FocusForm = 'addrec';
					$FocusField = 'domainname';
				}
				
				goto DISPLAYMAINFORM;
			}
			
			if ( ($znamechoice eq 'nc_alt') && (InvalidFQDN($realdomname)) )
			{
				$Result = 910010;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';

				$UseFocus = 1;
				$FocusForm = 'addrec';
				$FocusField = 'domainname';
				goto DISPLAYMAINFORM;
			}
			
			$ib  = DNSZone->zone_addrecord($zonename,$zone_dat_list[$i][1],"A",$realdomname,$ipaddr); 

			if( $ib eq 0 )
			{
				DNSZone->zone_increment($zonename,$zone_dat_list[$i][1]);
				print $cgi->redirect("server_dns_edit.cgi?zname=$zonename&newrecordmessage=1");  
				exit(0);
			}
			else 
			{
				$Result = (($ib eq 215) ? 910004 : 910003);
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				goto DISPLAYMAINFORM;
			}
		}
	}
}


DISPLAYMAINFORM:

# Set the page and display it.
$html = SetPage($PAGE, $cgi->param('zname'));
my $domainname = $cgi->param('zname');


my %var;
$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 9100;
$var{'title'}			= $_R{'IDT:9100'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'} =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
				{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$domainname"},
			];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}

DisplayMain->printDocument(\%var);

exit(0);
