#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_addreverse.cgi                                         #
#                                                                             #
# Description: Can create a reverse-lookup DNS (RDNS) zone on this appliance. #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSZone;
use DNSConfig;
my $DNSPATH = '/home/named/var/named/';

########################################
my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8200'			=> '',
		'IDQH:8200'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',
		'IPADDRESSSUBNET'		=> '',
		'DNSZONENAME'			=> '',
		'IPADDRESS'			=> '',
		'ADMINEMAIL'			=> '',
		'REFRESHAFTER'			=> '',
		'RETRYAFTER'			=> '',
		'EXPIREAFTER'			=> '',
		'MINIMUMTTL'			=> '',
		'MASTER'			=> '',
		'SLAVE'				=> '',
		'MASTERZONE'			=> '',
		'SLAVEZONE'			=> '',
		'ADD'				=> '',
		'MASTERSERVERNAME'		=> '',

		'TIME_SECS'			=> '',
		'TIME_MINS'			=> '',
		'TIME_HOUR'			=> '',
		'TIME_DAYS'			=> '',
		'TIME_WEEK'			=> '',

		'DEFAULT_MESSAGE'		=> '');



$StringResource->GetStrings(\%_R);

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'ipaddr_A'		=> '',
			'ipaddr_B'		=> '',
			'ipaddr_C'		=> '',
			'adminemail'		=> '',

			'refresh'		=> '8',
			'retry'			=> '2',
			'expire'		=> '1',
			'minttl'		=> '1',
			'D1'			=> 'H',
			'D2'			=> 'H',
			'D3'			=> 'W',
			'D4'			=> 'D',
			
			'R1'			=> '1',
			'slavenetname'		=> '',
		);

########################################
			
if ( $cgi->param('addzone') )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}

# Set up the radio state.
$FormData{'R1_master'}		= ('1' eq $FormData{'R1'}) ? 'checked' : '';
$FormData{'R1_slave'}		= ('2' eq $FormData{'R1'}) ? 'checked' : '';

# Set up the option selectors for D1.
$FormData{'D1_S'}		= ('S' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_M'}		= ('M' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_H'}		= ('H' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_D'}		= ('D' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_W'}		= ('W' eq $FormData{'D1'}) ? 'selected' : '';

# Set up the option selectors for D2.
$FormData{'D2_S'}		= ('S' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_M'}		= ('M' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_H'}		= ('H' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_D'}		= ('D' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_W'}		= ('W' eq $FormData{'D2'}) ? 'selected' : '';

# Set up the option selectors for D3.
$FormData{'D3_S'}		= ('S' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_M'}		= ('M' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_H'}		= ('H' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_D'}		= ('D' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_W'}		= ('W' eq $FormData{'D3'}) ? 'selected' : '';

# Set up the option selectors for D4.
$FormData{'D4_S'}		= ('S' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_M'}		= ('M' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_H'}		= ('H' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_D'}		= ('D' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_W'}		= ('W' eq $FormData{'D4'}) ? 'selected' : '';

########################################



my $PAGE = <<HERE;
<SCRIPT>
function disabledforslave()
{
	document.addreverse.adminemail.disabled=true;	
	document.addreverse.refresh.disabled=true;	
	document.addreverse.retry.disabled=true;	
	document.addreverse.expire.disabled=true;	
	document.addreverse.minttl.disabled=true;	
	document.addreverse.D1.disabled=true;	
	document.addreverse.D2.disabled=true;	
	document.addreverse.D3.disabled=true;	
	document.addreverse.D4.disabled=true;	
	
}
function enableformaster()
{
	document.addreverse.adminemail.disabled=false;	
	document.addreverse.refresh.disabled=false;	
	document.addreverse.retry.disabled=false;	
	document.addreverse.expire.disabled=false;	
	document.addreverse.minttl.disabled=false;	
	document.addreverse.D1.disabled=false;	
	document.addreverse.D2.disabled=false;	
	document.addreverse.D3.disabled=false;	
	document.addreverse.D4.disabled=false;	
	
}
</SCRIPT>
<FORM METHOD="POST" NAME=addreverse ACTION="server_dns_addreverse.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="addzone" value="on">
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
	  </TD>
        </TR>
        <tr>
          <TD WIDTH="53%" colspan="2"><b>$_R{'IPADDRESSSUBNET'}:<br></b>
          </TD>
          <TD WIDTH="47%" NOWRAP>
            <INPUT TYPE="text" NAME="ipaddr_A" SIZE=2 MAXLENGTH=3 value="$FormData{'ipaddr_A'}"> .
            <INPUT TYPE="text" NAME="ipaddr_B" SIZE=2 MAXLENGTH=3 value="$FormData{'ipaddr_B'}"> .
            <INPUT TYPE="text" NAME="ipaddr_C" SIZE=2 MAXLENGTH=3 value="$FormData{'ipaddr_C'}">
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="1" $FormData{'R1_master'} onclick='enableformaster();'>
          </td>
          <td width="94%" colspan="2" align="left"><b>$_R{'MASTERZONE'}</b>
          </td>
        </tr>
	
	
	<tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'ADMINEMAIL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="adminemail" value="$FormData{'adminemail'}" SIZE=23 MAXLENGTH=45>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'REFRESHAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="refresh" SIZE=5 MAXLENGTH=15 value="$FormData{'refresh'}">
            <select size="1" name="D1">
              <option $FormData{'D1_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D1_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D1_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D1_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D1_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'RETRYAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="retry"  SIZE=5 MAXLENGTH=15 value="$FormData{'retry'}">
            <select size="1" name="D2">
              <option $FormData{'D2_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D2_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D2_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D2_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D2_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </TR>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'EXPIREAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="expire"  SIZE=5 MAXLENGTH=15 value="$FormData{'expire'}">
            <select size="1" name="D3">
              <option $FormData{'D3_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D3_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D3_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D3_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D3_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'MINIMUMTTL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="minttl"  SIZE=5 MAXLENGTH=15 value="$FormData{'minttl'}">
            <select size="1" name="D4">
              <option $FormData{'D4_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D4_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D4_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D4_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D4_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>


        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="2" $FormData{'R1_slave'} onclick='disabledforslave();'>
	  </td>
	  <td width="94%" colspan="2" align="left"><b>$_R{'SLAVEZONE'}</b>
          </td>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'MASTERSERVERNAME'}:</b>
	  </TD>
	  <TD WIDTH="47%"><input type="text" name="slavenetname" size="23" value="$FormData{'slavenetname'}">
          </td>
        </tr>        
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <TR>
          <TD WIDTH="100%" colspan="3" align="right">&nbsp;<input type="submit" class="clsButton" value=" $_R{'ADD'} " name="submit">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>
HERE

###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html) = @_;
	
	# Set the server status.
	#$html =~ s/SERVERSTATUS/$ServiceStatus/s;

    return $html;
}


###############################################################################
# InvalidOctet
###############################################################################

sub InvalidOctet
{
	my ($ipaddress,$useless) = @_;
	my ($t1,$t2);

	# If it's blank, dump it!

	if ($ipaddress eq '')
	{
		return 1;
	}
	
	# Check the octet for non-digit characters.

	$t1 = $ipaddress;
	if ($t1 =~ /\D/)
	{
		# Non-digit character found!
		return 1;
	}

	else
	{
		# Check for out of bounds values.
		if ( ($t1 < 0) || ($t1 > 255) )
		{
			return 1;
		}

		else
		{
			return 0;
		}
	}
}

###############################################################################
# InvalidTime
###############################################################################

sub InvalidTime
{
	my ($tval1,$useless) = @_;

	# Begin checking for the time values.
	# Valid include:
	# 1, 1S1, 1M1, 1H1, 1D1, 1W1, 1S1M1H1D1W1

	if ($tval1 =~ /^(((\d+)[SsMmHhDdWw])*)(\d+)$/ )
	{
		# Match value?
		return 0;
	}
	else
	{
		return 1;
	}

}

###############################################################################
# The main body of the script.
###############################################################################

my $html				= $PAGE;
my $fn_out1;
my $mymaster = $cgi->param('R1');
my $mymasterlen = length($cgi->param('slavenetname'));
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);
my ($ia,$ib,$ic,$ic1,$ic2);
my $filename;
my $teststr;
my $teststr1;

if ($cgi->param('submit') && $cgi->param('addzone'))
{
	my ($ipaddr_A, $ipaddr_B, $ipaddr_C);
	my $revsubzoneip = '';

	$ipaddr_A = $cgi->param('ipaddr_A');
	$ipaddr_B = $cgi->param('ipaddr_B');
	$ipaddr_C = $cgi->param('ipaddr_C');

	my $slavenetname = lc($cgi->param('slavenetname'));
	
	if ($ipaddr_A eq '')
	{
		$Result = 820006;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';

		$UseFocus = 1;
		$FocusForm = 'addreverse';
		$FocusField = 'ipaddr_A';
		goto DISPLAYMAINFORM;
	}
	
	if ( ($ipaddr_A ne '') && ($ipaddr_B eq '') && ($ipaddr_C ne '') )
	{
		$ipaddr_B = $ipaddr_C;
		$ipaddr_C = '';
	}
	
	if ( ($ipaddr_A ne '') && ($ipaddr_B ne '') && ($ipaddr_C ne '') )
	{
		# Use all three octet fields.

		if (InvalidOctet($ipaddr_A)) 
		{
			# Bad octet here!
			$Result = 820009;
			$FocusField = 'ipaddr_A';
		}
		elsif (InvalidOctet($ipaddr_B))
		{
			# Bad octet there!
			$Result = 820009;
			$FocusField = 'ipaddr_B';
		}
		elsif (InvalidOctet($ipaddr_C))
		{
			# Bad octet everywhere!
			$Result = 820009;
			$FocusField = 'ipaddr_C';
		}
		else
		{
			$revsubzoneip = $ipaddr_C . '.' . $ipaddr_B . '.' . $ipaddr_A ;
		}
		
	}
	elsif ( ($ipaddr_A ne '') && ($ipaddr_B ne '') )
	{
		if (InvalidOctet($ipaddr_A)) 
		{
			# Bad octet here!
			$Result = 820009;
			$FocusField = 'ipaddr_A';
		}
		elsif (InvalidOctet($ipaddr_B))
		{
			# Bad octet there!
			$Result = 820009;
			$FocusField = 'ipaddr_B';
		}
		else
		{
			$revsubzoneip = $ipaddr_B . '.' . $ipaddr_A ;
		}
		
	}
	else	# if $ipaddr_A ne ''
	{
		if (InvalidOctet($ipaddr_A)) 
		{
			# Bad octet here!
			$Result = 820009;
			$FocusField = 'ipaddr_A';
		}
		else
		{
			$revsubzoneip = $ipaddr_A ;
		}
		
	}

	if ($Result eq 820009)
	{
		# Aww, did we make a boo-boo?
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';

		$UseFocus = 1;
		$FocusForm = 'addreverse';
		goto DISPLAYMAINFORM;
	}

	# Otherwise, keep going.
		
	if ( ( ($mymaster eq '2') && ($mymasterlen) ) || ($mymaster eq '1') )
	{
		if($mymaster eq '2')
		{
			$ia = DNSConfig->zone_add($revsubzoneip.'.in-addr.arpa',\$fn_out1, $mymaster,$slavenetname );
		}
		else
		{
			if ($cgi->param('adminemail') eq '')
			{
				$Result = 820005;
				$FocusField = 'adminemail';
			}
			
			elsif ($cgi->param('refresh') eq '')
			{
				$Result = 820005;
				$FocusField = 'refresh';
			}
			elsif (InvalidTime($cgi->param('refresh')))
			{
				$Result = 820019;
				$FocusField = 'refresh';
			}
			
			elsif ($cgi->param('retry') eq '')
			{
				$Result = 820005;
				$FocusField = 'retry';
			}
			elsif (InvalidTime($cgi->param('retry')))
			{
				$Result = 820019;
				$FocusField = 'retry';
			}
			
			elsif ($cgi->param('expire') eq '')
			{
				$Result = 820005;
				$FocusField = 'expire';
			}
			elsif (InvalidTime($cgi->param('expire')))
			{
				$Result = 820019;
				$FocusField = 'expire';
			}
			
			elsif ($cgi->param('minttl') eq '')
			{
				$Result = 820005;
				$FocusField = 'minttl';
			}
			elsif (InvalidTime($cgi->param('minttl')))
			{
				$Result = 820019;
				$FocusField = 'minttl';
			}
			
			if ( ($Result eq 820005) || ($Result eq 820019) )
			{
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				$UseFocus = 1;
				$FocusForm = 'addreverse';
				goto DISPLAYMAINFORM;
			}
			
			if ($cgi->param('adminemail') !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
			{
				# Invalid e-mail address!

				$Result = 820010;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addreverse';
				$FocusField = 'adminemail';
				goto DISPLAYMAINFORM;
			}
			
			$ia = DNSConfig->zone_add($revsubzoneip.'.in-addr.arpa',\$fn_out1, $mymaster);

			if ($ia eq 0)
			{
				$ib = DNSZone->zone_construct_rev($revsubzoneip.'.in-addr.arpa.',$fn_out1,'admin@localhost.');

				if($ib eq 0)
				{
					$ic1 = DNSZone->zone_getinfo($revsubzoneip.'.in-addr.arpa.',$fn_out1,\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);
					$pz_soa[2] = $cgi->param('adminemail');
					$pz_soa[4] = $cgi->param('refresh') . $cgi->param('D1');
					$pz_soa[5] = $cgi->param('retry') . $cgi->param('D2');
					$pz_soa[6] = $cgi->param('expire') . $cgi->param('D3');
					$pz_soa[7] = $cgi->param('minttl') . $cgi->param('D4');

					$ic2 = DNSZone->zone_changeheader($revsubzoneip.'.in-addr.arpa.',$fn_out1,\@pz_soa);
				}
			}
		}
		
		if ($ia != 0 || $ib != 0)
		{
			# Unable to add or construct zone.
			$Result = 820002;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		elsif ($ic1 != 0)
		{
			# Unable to get info for newly created zone file.
			$Result = 820003;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		elsif ($ic2 != 0)
		{
			# Unable to set header info for newly created zone file.
			$Result = 820004;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		else
		{
			$cgi->redirect("server_dns.cgi?newzonemessage=1&zname=" . $revsubzoneip.'.in-addr.arpa');
			exit(0);
		}	
	}
	else
	{
		$Result = 820001;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';	
		
		$UseFocus = 1;
		$FocusForm = 'addreverse';
		$FocusField = 'slavenetname';
	}	
}

DISPLAYMAINFORM:

my %var;
$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8200;
$var{'title'}			= $_R{'IDT:8200'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'}		= [{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}

DisplayMain->printDocument(\%var);

exit(0);

