#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_edit_SOA.cgi                                           #
#                                                                             #
# Description: Can edit the SOA record of a zone on this appliance.           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSZone;
use DNSConfig;

########################################
my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8400'			=> '',
		'IDQH:8400'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',

		'DNSZONENAME'			=> '',
		'IPADDRESS'			=> '',
		'ADMINEMAIL'			=> '',
		'REFRESHAFTER'			=> '',
		'RETRYAFTER'			=> '',
		'EXPIREAFTER'			=> '',
		'MINIMUMTTL'			=> '',
		'MASTERZONE'			=> '',
		'SLAVEZONE'			=> '',
		'MASTER'			=> '',
		'SLAVE'				=> '',
		'APPLY'				=> '',
		'MASTERSERVERNAME'		=> '',
		
		'TIME_SECS'			=> '',
		'TIME_MINS'			=> '',
		'TIME_HOUR'			=> '',
		'TIME_DAYS'			=> '',
		'TIME_WEEK'			=> '',

		'DEFAULT_MESSAGE'		=> '');



$StringResource->GetStrings(\%_R);

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'adminemail'		=> '',

			'refresh'		=> '',
			'retry'			=> '',
			'expire'		=> '',
			'minttl'		=> '',
			'D1'			=> '',
			'D2'			=> '',
			'D3'			=> '',
			'D4'			=> '',
			
			'R1'			=> '',
			'slavenetname'		=> '',
		);

########################################
if ( $cgi->param('edit_SOA') )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}

# Set up the radio state.
$FormData{'R1_master'}		= ('1' eq $FormData{'R1'}) ? 'checked' : '';
$FormData{'R1_slave'}		= ('2' eq $FormData{'R1'}) ? 'checked' : '';

# Set up the option selectors for D1.
$FormData{'D1_S'}		= ('S' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_M'}		= ('M' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_H'}		= ('H' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_D'}		= ('D' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_W'}		= ('W' eq $FormData{'D1'}) ? 'selected' : '';

# Set up the option selectors for D2.
$FormData{'D2_S'}		= ('S' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_M'}		= ('M' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_H'}		= ('H' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_D'}		= ('D' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_W'}		= ('W' eq $FormData{'D2'}) ? 'selected' : '';

# Set up the option selectors for D3.
$FormData{'D3_S'}		= ('S' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_M'}		= ('M' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_H'}		= ('H' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_D'}		= ('D' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_W'}		= ('W' eq $FormData{'D3'}) ? 'selected' : '';

# Set up the option selectors for D4.
$FormData{'D4_S'}		= ('S' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_M'}		= ('M' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_H'}		= ('H' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_D'}		= ('D' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_W'}		= ('W' eq $FormData{'D4'}) ? 'selected' : '';

########################################


my $PAGE = <<HERE;
<SCRIPT>
function disabledforslave()
{
	document.editsoa.adminemail.disabled=true;	
	document.editsoa.refresh.disabled=true;	
	document.editsoa.retry.disabled=true;	
	document.editsoa.expire.disabled=true;	
	document.editsoa.minttl.disabled=true;	
	document.editsoa.D1.disabled=true;	
	document.editsoa.D2.disabled=true;	
	document.editsoa.D3.disabled=true;	
	document.editsoa.D4.disabled=true;	
	
}
function enableformaster()
{
	document.editsoa.adminemail.disabled=false;	
	document.editsoa.refresh.disabled=false;	
	document.editsoa.retry.disabled=false;	
	document.editsoa.expire.disabled=false;	
	document.editsoa.minttl.disabled=false;	
	document.editsoa.D1.disabled=false;	
	document.editsoa.D2.disabled=false;	
	document.editsoa.D3.disabled=false;	
	document.editsoa.D4.disabled=false;	
	
}
</SCRIPT>
<FORM METHOD="POST" NAME=editsoa ACTION="server_dns_edit_SOA.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">
<INPUT TYPE="hidden" NAME="masterslave" VALUE="MASTERSLAVE">
<INPUT TYPE="hidden" NAME="edit_SOA" VALUE="1">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <tr>
          <TD WIDTH="53%" colspan="2"><b>$_R{'DNSZONENAME'}:</b>
          </TD>
          <TD WIDTH="47%">DOMAINNAME
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="1" MASTERCHECK >
          </td>
          <td width="94%" colspan="2" align="left"><b>$_R{'MASTERZONE'}</b>
          </td>
        </tr>
	
	<tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'ADMINEMAIL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="adminemail" SIZE=23 MAXLENGTH=45 value="ADMINEMAIL">
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'REFRESHAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="refresh" SIZE=5 MAXLENGTH=15 value="REFRESHAFTER">
            <select size="1" name="D1">
              <option REFRESHAFTER_T_S value=S>$_R{'TIME_SECS'}</option>
              <option REFRESHAFTER_T_M value=M>$_R{'TIME_MINS'}</option>
              <option REFRESHAFTER_T_H value=H>$_R{'TIME_HOUR'}</option>
              <option REFRESHAFTER_T_D value=D>$_R{'TIME_DAYS'}</option>
              <option REFRESHAFTER_T_W value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'RETRYAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="retry"  SIZE=5 MAXLENGTH=15 value="RETRYAFTER">
            <select size="1" name="D2">
              <option RETRYAFTER_T_S value=S>$_R{'TIME_SECS'}</option>
              <option RETRYAFTER_T_M value=M>$_R{'TIME_MINS'}</option>
              <option RETRYAFTER_T_H value=H>$_R{'TIME_HOUR'}</option>
              <option RETRYAFTER_T_D value=D>$_R{'TIME_DAYS'}</option>
              <option RETRYAFTER_T_W value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </TR>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'EXPIREAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="expire"  SIZE=5 MAXLENGTH=15 value="EXPIREAFTER">
            <select size="1" name="D3">
              <option EXPIREAFTER_T_S value=S>$_R{'TIME_SECS'}</option>
              <option EXPIREAFTER_T_M value=M>$_R{'TIME_MINS'}</option>
              <option EXPIREAFTER_T_H value=H>$_R{'TIME_HOUR'}</option>
              <option EXPIREAFTER_T_D value=D>$_R{'TIME_DAYS'}</option>
              <option EXPIREAFTER_T_W value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'MINIMUMTTL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="minttl"  SIZE=5 MAXLENGTH=15 value="MINTTL">
            <select size="1" name="D4">
              <option MINTTL_T_S value=S>$_R{'TIME_SECS'}</option>
              <option MINTTL_T_M value=M>$_R{'TIME_MINS'}</option>
              <option MINTTL_T_H value=H>$_R{'TIME_HOUR'}</option>
              <option MINTTL_T_D value=D>$_R{'TIME_DAYS'}</option>
              <option MINTTL_T_W value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="2" SLAVECHECK >
	  </TD>
	  <td width="94%" colspan="2" align="left"><b>$_R{'SLAVE'}</b>
          </td>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'MASTERSERVERNAME'}:</b>
	  </TD>
	  <TD WIDTH="47%"><input type="text" name="slavenetname" size="20" value="SLAVECHECKNAME">
	  </td>
        </tr>
	<TR>
	  <TD WIDTH="100%" colspan="3">&nbsp;&nbsp;
	  </TD>
	</TR>
        <TR>
          <TD WIDTH="100%" colspan="3" align="right">&nbsp;<input type="submit" class="clsButton" value="$_R{'APPLY'}" name="Apply">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>

HERE




###############################################################################
# ConvertNum
###############################################################################
sub ConvertNum
{
	my ($strin, $num, $time_d) = @_;
	
	my $secs=0;
	my $mins=0;
	my $hours=0;
	my $days=0;
	my $weeks=0;
	my $tmp1;

	if($strin =~ /[SsMmHhDdWw]/)
	{ 
		($secs, $tmp1) = ($strin  =~ m/(\d+)[Ss]/); 
		($mins, $tmp1) = ($strin  =~ m/(\d+)[Mm]/); 
		($hours, $tmp1) = ($strin =~ m/(\d+)[Hh]/); 
		($days, $tmp1) = ($strin  =~ m/(\d+)[Dd]/); 
		($weeks, $tmp1) = ($strin =~ m/(\d+)[Ww]/);
	}
	else
	{
		$$num = $strin;
		$$time_d = "S";
		return 0;	
	}

	if($secs)
	{
		$secs += (($mins * 60) + ($hours * 3600) + ($days * 86400) + ($weeks * 604800)); #convert higher to secs
		$$num=$secs;
		$$time_d="S";
	}
	elsif($mins)
	{ 
		$mins += (($hours * 60) + ($days * 1440)+ ($weeks * 10080));  #convert higher to mins
		$$num=$mins;
		$$time_d="M";
	}
	elsif($hours)
	{ 
		$hours += (($days * 24) + ($weeks * 168)) ; #convert higher to hours
		$$num=$hours;
		$$time_d="H";
	}
	elsif($days)
	{ 
		$days += ($weeks * 7) ;
		$$num=$days;
		$$time_d="D";
	}
	elsif($weeks)
	{ 
		$$num = $weeks;
		$$time_d="W";
	}
	
	return 0;
}

###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my (	$html, $Domainname, $Adminemail, $Refresh, $Retry, $Expire, $MinTTL,
		$Masterslave, $Slavecheckname, $RefT, $RetT, $ExpT, $MinT) = @_;

	$html =~ s/DOMAINNAME/$Domainname/g;
	$html =~ s/ADMINEMAIL/$Adminemail/s;
	$html =~ s/REFRESHAFTER/$Refresh/s;
	$html =~ s/RETRYAFTER/$Retry/s;
	$html =~ s/EXPIREAFTER/$Expire/s;
	$html =~ s/MINTTL/$MinTTL/s;
	$html =~ s/MASTERSLAVE/$Masterslave/g;

	if ($Masterslave eq "1")
	{
		$html =~ s/MASTERCHECK/checked/s;
		$html =~ s/SLAVECHECK//s; 
		$html =~ s/SLAVECHECKNAME/$Slavecheckname/s;
	}
	else
	{
		$html =~ s/MASTERCHECK//s;
		$html =~ s/SLAVECHECK/checked/s; 
		$html =~ s/SLAVECHECKNAME/$Slavecheckname/s;
	}


	if ($RefT =~ /[Ss]/)
	{
		$html =~ s/REFRESHAFTER_T_S/selected/s;
		$html =~ s/REFRESHAFTER_T_M//s;
		$html =~ s/REFRESHAFTER_T_H//s;
		$html =~ s/REFRESHAFTER_T_D//s;
		$html =~ s/REFRESHAFTER_T_W//s;
	}
	elsif ($RefT =~ /[Mm]/)
	{
		$html =~ s/REFRESHAFTER_T_S//s;
		$html =~ s/REFRESHAFTER_T_M/selected/s;
		$html =~ s/REFRESHAFTER_T_H//s;
		$html =~ s/REFRESHAFTER_T_D//s;
		$html =~ s/REFRESHAFTER_T_W//s;
	}
	elsif ($RefT =~ /[Hh]/)
	{
		$html =~ s/REFRESHAFTER_T_S//s;
		$html =~ s/REFRESHAFTER_T_M//s;
		$html =~ s/REFRESHAFTER_T_H/selected/s;
		$html =~ s/REFRESHAFTER_T_D//s;
		$html =~ s/REFRESHAFTER_T_W//s;
	}
	elsif ($RefT =~ /[Dd]/)
	{
		$html =~ s/REFRESHAFTER_T_S//s;
		$html =~ s/REFRESHAFTER_T_M//s;
		$html =~ s/REFRESHAFTER_T_H//s;
		$html =~ s/REFRESHAFTER_T_D/selected/s;
		$html =~ s/REFRESHAFTER_T_W//s;
	}
	elsif ($RefT =~ /[Ww]/)
	{
		$html =~ s/REFRESHAFTER_T_S//s;
		$html =~ s/REFRESHAFTER_T_M//s;
		$html =~ s/REFRESHAFTER_T_H//s;
		$html =~ s/REFRESHAFTER_T_D//s;
		$html =~ s/REFRESHAFTER_T_W/selected/s;
	}


	if ($RetT =~ /[Ss]/)
	{
		$html =~ s/RETRYAFTER_T_S/selected/s;
		$html =~ s/RETRYAFTER_T_M//s;
		$html =~ s/RETRYAFTER_T_H//s;
		$html =~ s/RETRYAFTER_T_D//s;
		$html =~ s/RETRYAFTER_T_W//s;
	}
	elsif ($RetT =~ /[Mm]/)
	{
		$html =~ s/RETRYAFTER_T_S//s;
		$html =~ s/RETRYAFTER_T_M/selected/s;
		$html =~ s/RETRYAFTER_T_H//s;
		$html =~ s/RETRYAFTER_T_D//s;
		$html =~ s/RETRYAFTER_T_W//s;
	}
	elsif ($RetT =~ /[Hh]/)
	{
		$html =~ s/RETRYAFTER_T_S//s;
		$html =~ s/RETRYAFTER_T_M//s;
		$html =~ s/RETRYAFTER_T_H/selected/s;
		$html =~ s/RETRYAFTER_T_D//s;
		$html =~ s/RETRYAFTER_T_W//s;
	}
	elsif ($RetT =~ /[Dd]/)
	{
		$html =~ s/RETRYAFTER_T_S//s;
		$html =~ s/RETRYAFTER_T_M//s;
		$html =~ s/RETRYAFTER_T_H//s;
		$html =~ s/RETRYAFTER_T_D/selected/s;
		$html =~ s/RETRYAFTER_T_W//s;
	}
	elsif ($RetT =~ /[Ww]/)
	{
		$html =~ s/RETRYAFTER_T_S//s;
		$html =~ s/RETRYAFTER_T_M//s;
		$html =~ s/RETRYAFTER_T_H//s;
		$html =~ s/RETRYAFTER_T_D//s;
		$html =~ s/RETRYAFTER_T_W/selected/s;
	}


	if ($ExpT =~ /[Ss]/)
	{
		$html =~ s/EXPIREAFTER_T_S/selected/s;
		$html =~ s/EXPIREAFTER_T_M//s;
		$html =~ s/EXPIREAFTER_T_H//s;
		$html =~ s/EXPIREAFTER_T_D//s;
		$html =~ s/EXPIREAFTER_T_W//s;
	}
	elsif ($ExpT =~ /[Mm]/)
	{
		$html =~ s/EXPIREAFTER_T_S//s;
		$html =~ s/EXPIREAFTER_T_M/selected/s;
		$html =~ s/EXPIREAFTER_T_H//s;
		$html =~ s/EXPIREAFTER_T_D//s;
		$html =~ s/EXPIREAFTER_T_W//s;
	}
	elsif ($ExpT =~ /[Hh]/)
	{
		$html =~ s/EXPIREAFTER_T_S//s;
		$html =~ s/EXPIREAFTER_T_M//s;
		$html =~ s/EXPIREAFTER_T_H/selected/s;
		$html =~ s/EXPIREAFTER_T_D//s;
		$html =~ s/EXPIREAFTER_T_W//s;
	}
	elsif ($ExpT =~ /[Dd]/)
	{
		$html =~ s/EXPIREAFTER_T_S//s;
		$html =~ s/EXPIREAFTER_T_M//s;
		$html =~ s/EXPIREAFTER_T_H//s;
		$html =~ s/EXPIREAFTER_T_D/selected/s;
		$html =~ s/EXPIREAFTER_T_W//s;
	}
	elsif ($ExpT =~ /[Ww]/)
	{
		$html =~ s/EXPIREAFTER_T_S//s;
		$html =~ s/EXPIREAFTER_T_M//s;
		$html =~ s/EXPIREAFTER_T_H//s;
		$html =~ s/EXPIREAFTER_T_D//s;
		$html =~ s/EXPIREAFTER_T_W/selected/s;
	}


	if ($MinT =~ /[Ss]/)
	{
		$html =~ s/MINTTL_T_S/selected/s;
		$html =~ s/MINTTL_T_M//s;
		$html =~ s/MINTTL_T_H//s;
		$html =~ s/MINTTL_T_D//s;
		$html =~ s/MINTTL_T_W//s;
	}
	elsif ($MinT =~ /[Mm]/)
	{
		$html =~ s/MINTTL_T_S//s;
		$html =~ s/MINTTL_T_M/selected/s;
		$html =~ s/MINTTL_T_H//s;
		$html =~ s/MINTTL_T_D//s;
		$html =~ s/MINTTL_T_W//s;
	}
	elsif ($MinT =~ /[Hh]/)
	{
		$html =~ s/MINTTL_T_S//s;
		$html =~ s/MINTTL_T_M//s;
		$html =~ s/MINTTL_T_H/selected/s;
		$html =~ s/MINTTL_T_D//s;
		$html =~ s/MINTTL_T_W//s;
	}
	elsif ($MinT =~ /[Dd]/)
	{
		$html =~ s/MINTTL_T_S//s;
		$html =~ s/MINTTL_T_M//s;
		$html =~ s/MINTTL_T_H//s;
		$html =~ s/MINTTL_T_D/selected/s;
		$html =~ s/MINTTL_T_W//s;
	}
	elsif ($MinT =~ /[Ww]/)
	{
		$html =~ s/MINTTL_T_S//s;
		$html =~ s/MINTTL_T_M//s;
		$html =~ s/MINTTL_T_H//s;
		$html =~ s/MINTTL_T_D//s;
		$html =~ s/MINTTL_T_W/selected/s;
	}

	return $html;
}

###############################################################################
# InvalidTime
###############################################################################

sub InvalidTime
{
	my ($tval1,$useless) = @_;

	# Begin checking for the time values.
	# Valid include:
	# 1, 1S1, 1M1, 1H1, 1D1, 1W1, 1S1M1H1D1W1

	if ($tval1 =~ /^(((\d+)[SsMmHhDdWw])*)(\d+)$/ )
	{
		# Match value?
		return 0;
	}
	else
	{
		return 1;
	}

}

###############################################################################
# The main body of the script.
###############################################################################

my $html;
my $Records;
my $Domainname;
my $NumZones;
my ($ia,$ib,$ic,$id,$ie,$ij1,$ij2);
my ($i,$j);
my $count;
my $total;
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);

my $ErrorFlag = 0;

if ($cgi->param('Apply'))
{
	my $zonename = $cgi->param('zname');        
	my @zone_dat_list;

	my $slavenetname = lc($cgi->param('slavenetname'));

	$ia = DNSConfig->zone_list(\@zone_dat_list);
	$NumZones = scalar(@zone_dat_list);

	# Iterate through data structure and display meaningful information.
	for ($i = 0; $i < $NumZones; $i++)
	{
		# for each zone print out the records
		if ($zone_dat_list[$i][0] eq $zonename)
		{
			$ia = DNSZone->zone_getinfo($zonename,$zone_dat_list[$i][1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);
			if ($ia ne 0)
			{
				$Result = 840004;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				$ErrorFlag = 1;
				goto DISPLAYMAINFORM;
			}

			# Rememver that calls to zone_changeheader ignore any null values in the array below;
			# Null values are treated as no-op, no change.
			# However, we're going to be a lot more picky than that.
			
			if ($cgi->param('R1') eq '1')
			{	# If master...
			
				if ($cgi->param('adminemail') eq '')
				{
					$Result = 840007;
					$FocusField = 'adminemail';
				}

				elsif ($cgi->param('refresh') eq '')
				{
					$Result = 840007;
					$FocusField = 'refresh';
				}
				elsif (InvalidTime($cgi->param('refresh')))
				{
					$Result = 840019;
					$FocusField = 'refresh';
				}

				elsif ($cgi->param('retry') eq '')
				{
					$Result = 840007;
					$FocusField = 'retry';
				}
				elsif (InvalidTime($cgi->param('retry')))
				{
					$Result = 840019;
					$FocusField = 'retry';
				}

				elsif ($cgi->param('expire') eq '')
				{
					$Result = 840007;
					$FocusField = 'expire';
				}
				elsif (InvalidTime($cgi->param('expire')))
				{
					$Result = 840019;
					$FocusField = 'expire';
				}

				elsif ($cgi->param('minttl') eq '')
				{
					$Result = 840007;
					$FocusField = 'minttl';
				}
				elsif (InvalidTime($cgi->param('minttl')))
				{
					$Result = 840019;
					$FocusField = 'minttl';
				}

				if ( ($Result eq 840007) || ($Result eq 840019) )
				{
					$ErrorFlag = 1;
					$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
					
					$UseFocus = 1;
					$FocusForm = 'editsoa';
					goto DISPLAYMAINFORM;
				}
				
				if ($cgi->param('adminemail') !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
				{
					# Invalid e-mail address!

					$ErrorFlag = 1;
					$Result = 840010;
					$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';

					$UseFocus = 1;
					$FocusForm = 'editsoa';
					$FocusField = 'adminemail';
					goto DISPLAYMAINFORM;
				}
			
			}			

			$pz_soa[2] = $cgi->param('adminemail');
			$pz_soa[4] = $cgi->param('refresh') . $cgi->param('D1');
			$pz_soa[5] = $cgi->param('retry') . $cgi->param('D2');
			$pz_soa[6] = $cgi->param('expire') . $cgi->param('D3');
			$pz_soa[7] = $cgi->param('minttl') . $cgi->param('D4');

			$ib = DNSZone->zone_changeheader($zonename,$zone_dat_list[$i][1],\@pz_soa);
			if ($ib ne 0)
			{
				$Result = 840001;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				$ErrorFlag = 1;
				goto DISPLAYMAINFORM;
			}

			my $newmaster = $slavenetname;

			if ($cgi->param('R1') eq $zone_dat_list[$i][2])
			{
				# If no change in status (i.e., was master and still is, or was slave and still is)

				if ($cgi->param('R1') eq '2')
				{
					# If slave, check to see if the master server address has changed.

					# First check to make sure there is a new master server address.

					if ($newmaster eq '')
					{
						$ErrorFlag = 1;
						$Result = 840003;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						
						$UseFocus = 1;
						$FocusForm = 'editsoa';
						$FocusField = 'slavenetname';
						goto DISPLAYMAINFORM;
					}

					my $oldmaster = $zone_dat_list[$i][3];
					
					if ( $oldmaster ne $newmaster) 
					{
						# Time to change the master.  First toggle zone back into
						# Master status, then toggle back into Slave with new master.
						
						$ic = DNSConfig->zone_toggle($cgi->param('zname'),1);
						$id = DNSConfig->zone_toggle($cgi->param('zname'),2,$newmaster);
						
						if ( ($ic ne 0) || ($id ne 0) )
						{
							# Error.
							$Result = 840002;
							$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
							$ErrorFlag = 1;
							goto DISPLAYMAINFORM;
						}
					}
				}
			
				# If it was master and still is master, then do no such configuration changes here,
				# other than what has been done above.
			}
			
			else 
			{  
				# We are changing from master to slave, or from slave to master
				
				if ($cgi->param('R1') eq '2')
				{
					# If going to slave, validate that the new master server address is not null.
	
					if ($newmaster eq '')
					{
						$Result = 840003;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						$ErrorFlag = 1;

						$UseFocus = 1;
						$FocusForm = 'editsoa';
						$FocusField = 'slavenetname';
						goto DISPLAYMAINFORM;
					}

					$ic = DNSConfig->zone_toggle($cgi->param('zname'),'2',$newmaster);
				}
				else
				{
					# If master, just go ahead and change it.
					$ic = DNSConfig->zone_toggle($cgi->param('zname'),'1');

					my @zone_data;
					$ij1 = DNSConfig->zone_getinfo(\@zone_data,$cgi->param('zname'));
					$ij2 = DNSZone->zone_promotemaster($zone_data[0],$zone_data[1]);
					
					if ( ($ij1 ne 0) || ($ij2 ne 0) )
					{
						$Result = 840008;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						goto DISPLAYMAINFORM;

						# Do not set the error flag, as we want to display the freshly set zone values,
						# since most of the work has been done already.
					}
				}

				if ($ic ne 0)
				{
					# Error when changing states.
					$Result = ($cgi->param('R1') eq '2') ? 840005 : 840006;
					$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
					$ErrorFlag = 1;
					goto DISPLAYMAINFORM;
				}
			}


			# By this point, no errors have been encountered, and we can assume
			# that the zone information update was successful.

			DNSZone->zone_increment($zonename,$zone_dat_list[$i][1]);

			$cgi->redirect("server_dns_edit.cgi?zname=$zonename&update=1&newSOAvalue=1");
			exit(0);

		}
	}
}

DISPLAYMAINFORM:

#Get all the data for this domain's SOA record
my $zonename = $cgi->param('zname');        

my @zone_dat_list;

$ia = DNSConfig->zone_list(\@zone_dat_list);
$NumZones = scalar(@zone_dat_list);


if ($ErrorFlag eq 0)
{	# If no error condition caught,

	# Iterate through data structure and display values as they reside in the zone.
	for ($i = 0; $i < $NumZones; $i++)
	{
		# for each zone print out the records
		if($zone_dat_list[$i][0] eq $zonename)
		{
			$ib = DNSZone->zone_getinfo($zonename,$zone_dat_list[$i][1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);
			my $crefreshtime;
			my $crefreshtimechar;
			ConvertNum($pz_soa[4], \$crefreshtime, \$crefreshtimechar);
			my $cretrytime;
			my $cretrytimechar;
			ConvertNum($pz_soa[5], \$cretrytime, \$cretrytimechar);
			my $cexpiretime;
			my $cexpiretimechar;
			ConvertNum($pz_soa[6], \$cexpiretime, \$cexpiretimechar);
			my $cminttltime;
			my $cminttltimechar;
			ConvertNum($pz_soa[7], \$cminttltime, \$cminttltimechar);

			my $FormatAdminEmail = '';
			my $RawAdminEmail = $pz_soa[2];

			my (@a1,$t1,$t2);
		
			@a1 = split('\.',$RawAdminEmail);
			while ( (scalar(@a1)) && ($a1[0] eq '') )
			{
				# Strip off leading dots in the e-mail address.  They annoy us.
				shift(@a1);
			}
		
			if (scalar(@a1) > 1)
			{
				$t1 = shift(@a1);
			
				$FormatAdminEmail = $t1 . '@' . join('.',@a1);
			}
			else
			{
				$FormatAdminEmail = join('.',@a1);
			}

			# Set up all the values to replace
			$html = SetPage(	$PAGE,$cgi->param('zname'),$FormatAdminEmail,
						$crefreshtime,$cretrytime,$cexpiretime,$cminttltime,
						$zone_dat_list[$i][2],$zone_dat_list[$i][3],
						$crefreshtimechar,$cretrytimechar,$cexpiretimechar,$cminttltimechar); 
			goto OUTPAGE;
		}
	}
}

else
{	# If error condition caught...

	# Retrieve all the values from the form data, based on last submission
	$html = SetPage(	$PAGE,$cgi->param('zname'),$FormData{'adminemail'},
				$FormData{'refresh'},$FormData{'retry'},$FormData{'expire'},$FormData{'minttl'},
				$FormData{'R1'},$FormData{'slavenetname'},
				$FormData{'D1'},$FormData{'D2'},$FormData{'D3'},$FormData{'D4'});



}

OUTPAGE:

# Set the page and display it.
my $domainname = $cgi->param('zname');

my %var;
$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8400;
$var{'title'}			= $_R{'IDT:8400'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'} =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
				{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$domainname"},
			];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}


DisplayMain->printDocument(\%var);

exit(0);

