#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_edit_slaves.cgi                                        #
#                                                                             #
# Description: Manages NS records (for DNS slave servers) for a zone on this  #
#              appliance.                                                     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSConfig;
use DNSZone;


########################################

my $Result;
my $Feedback;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8600'			=> '',
		'IDQH:8600'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',
		
		'ADD'				=> '',
		'DELETE'			=> '',

		'DNSZONENAME'			=> '',
		'SLAVESERVERRECS'		=> '',
		'SLAVESERVERADDR'		=> '',
		'DEFAULTPAREN'			=> '',

		'ALLSELECTEDRECORDSMSG'		=> '',
		'NOSELECTEDRECORDSMSG'		=> '',
		'DEFAULT_MESSAGE'		=> '');


$StringResource->GetStrings(\%_R);

########################################

my $PAGE = <<HERE;
<script>
function checkall(count,type)
{
	if(eval(\"document.DNS.delall_\" + type + \".checked\") == true)
	{
		for(var i=0;i<count;i++)
			eval(\"document.DNS.\" + type + "" + i + \".checked='checked';\");
	}
	else
	{
		for(var i=0;i<count;i++)
			eval(\"document.DNS.\" + type + "" + i + \".checked='';\");
	}
}

function preparetodeleterecords(count,type)
{
	var boxcount=0;
	for(var i=0;i<count;i++)	
	{
		if (eval(\"document.DNS.\" + type + "" + i + \".checked\"))
		{
			boxcount++;
		}
	}
	
	if (0 < boxcount)
	{
		return confirm("$_R{'ALLSELECTEDRECORDSMSG'}");
	}
	else
	{
		alert("$_R{'NOSELECTEDRECORDSMSG'}");
		return false;
	}
}

</script>
<FORM METHOD="POST" NAME=DNS ACTION="server_dns_edit_slaves.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
        <tr>
          <TD WIDTH="50%" NOWRAP><b>$_R{'DNSZONENAME'}:</b>
          </TD>
          <TD WIDTH="50%" NOWRAP>DOMAINNAME
          </TD>
        </tr>
        <tr>
          <td width="100%" align="left" valign="bottom" colspan="2"><b>$_R{'SLAVESERVERRECS'}</b>
          </td>
        </tr>
        <tr>
          <td width="100%" align="left" valign="bottom" colspan="2"><input type="button" class="clsButton" value="$_R{'ADD'}" name="B16" onclick="javascript:location='server_dns_addrecord_NS.cgi?zname=DOMAINNAME'">
            <input type="submit" class="clsButton" value="$_R{'DELETE'}" name="DeleteRec" onclick=" return preparetodeleterecords(RECNUM,'rec');">
          </td>
        </tr>
        <tr>
          <td width="100%" align="left" valign="bottom" colspan="2">&nbsp;
          </td>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="2">
            <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
              <tr>
                <td width="100%">
                  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                    <tr>
                      <td width="3%" bgcolor="#808080"><input type="checkbox" name="delall_rec" value="delall_rec" onclick="javascript:checkall(RECNUM,'rec');">&nbsp;
                      </td>
                      <td width="97%" nowrap bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SLAVESERVERADDR'}</font></b>
                      </td>
                    </tr>
RECORDS
                  </table>
                </td>
              </tr>
            </table>
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          

</FORM>
HERE


###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html, $Records, $Domainname, $recordcount) = @_;
	
	# Set the DNS Records.
	$html =~ s/RECORDS/$Records/s;

	# Set the DNS Records.
	$html =~ s/DOMAINNAME/$Domainname/g;

	# Set the DNS Records.
	$html =~ s/RECNUM/$recordcount/g;

	return $html;
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi					= new CGI;
my $html;
my $Records;
my $SubzoneRecords;
my $Domainname;
my $NumZones;
my ($ia,$ib,$ic,$id);
my $i;
my $j;
my $count;
my $total;
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);


if ($cgi->param('DeleteRec'))
{
	my $zonename = $cgi->param('zname');        
	my $fname = $cgi->param('zonefilename');

	my $realdomname = $zonename.'.';

	# Iterate through data structure and display meaningful information.
	for ($i = 0; $i < $cgi->param('recordcount'); $i++)
	{
		# if the item is checked
		if($cgi->param("rec$i"))
		{	
			my $retype = $cgi->param("rectype$i");
			my $refrom = $cgi->param("recfrom$i");
			my $reto = $cgi->param("recto$i");
			my $repri = $cgi->param("recpri$i");

			$ib  += DNSZone->zone_delrecord($zonename,$fname,$retype,$refrom,$reto,$repri); 
		}
	}	

	if($ib eq 0)
	{
		$Result = 0;
		$Feedback = '<b><font color="green">' . $StringResource->_S('860002') . '</font></b>';
	}
	else
	{
		$Result = 860001;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
	}

	DNSZone->zone_increment($zonename,$fname);

}


my $zonename = $cgi->param('zname');        

if ($cgi->param('newrecordmessage'))
{
	$Result .= 0;
	$Feedback = '<b><font color="green">' . $StringResource->_S('860003') . '</font></b>';
	DNSConfig->named_update('named');
}

my @zone_dat_list;

$ia = DNSConfig->zone_list(\@zone_dat_list);
$NumZones = scalar(@zone_dat_list);

# Iterate through data structure and display meaningful information.
for ($i = 0; $i < $NumZones; $i++)
{
	# for each zone print out the records
	if($zone_dat_list[$i][0] eq $zonename)
	{
		$ib = DNSZone->zone_getinfo($zonename,$zone_dat_list[$i][1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);
	
		# Iterate through the data structure getting the NS records
		$count = scalar(@pz_ns);
				
		for ($j = 0; $j < $count; $j++)
		{
			my $num = $total + $j;
			my $comments;
			
			if ( ($pz_ns[$j][1] eq 'dns.' . $zonename . '.') || ($pz_ns[$j][1] eq 'dns') )
			{
				$comments = $_R{'DEFAULTPAREN'};
			}
			else
			{
				$comments = '';
			}

			my $perc = '%';
			
			$Records .= "<tr><td width=3$perc NOWRAP><input type=checkbox name=rec$num value=ON><input type=hidden name=rectype$num value=NS><input type=hidden name=recfrom$num value=$pz_ns[$j][0]><input type=hidden name=recto$num value=$pz_ns[$j][1]></td>";
			$Records .= "<td width=97$perc nowrap>$pz_ns[$j][1] " . $comments . " </td></tr>\n";
		}

		$total += $count;

		$Records .= "<input type=hidden name=zonefilename value=$zone_dat_list[$i][1]>";

		goto DISPLAYREALFORM;  # Break out of the loop since we found the domain
	}
}

DISPLAYREALFORM:

$Records .= "<input type=hidden name=recordcount value=$total>\n";

# Set the page and display it.
$html = SetPage($PAGE, $Records, $zonename, $total);

my %var;

$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8600;
$var{'title'}			= $_R{'IDT:8600'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'} =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
				{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$zonename"},
			];

DisplayMain->printDocument(\%var);

exit(0);

