#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_subzone.cgi                                            #
#                                                                             #
# Description: Can create subzones of a zone on this appliance.               #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSZone;
use DNSConfig;

my $DNSPATH = '/home/named/var/named/';

########################################

my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8700'			=> '',
		'IDQH:8700'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',
		
		'DNSSUBNAME'			=> '',
		'IPADDRESS'			=> '',
		'ADMINEMAIL'			=> '',
		'REFRESHAFTER'			=> '',
		'RETRYAFTER'			=> '',
		'EXPIREAFTER'			=> '',
		'MINIMUMTTL'			=> '',
		'MASTER'			=> '',
		'SLAVE'				=> '',
		'MASTERZONE'			=> '',
		'SLAVEZONE'			=> '',
		'ADD'				=> '',
		'MASTERSERVERNAME'		=> '',
		'SLAVESUBZONEDISP'		=> '',
		'MASTERLC'			=> '',


		'TIME_SECS'			=> '',
		'TIME_MINS'			=> '',
		'TIME_HOUR'			=> '',
		'TIME_DAYS'			=> '',
		'TIME_WEEK'			=> '',

		'DEFAULT_MESSAGE'		=> '');


$StringResource->GetStrings(\%_R);

###############################################################################
# ConvertNum
###############################################################################
sub ConvertNum
{
	my ($strin, $num, $time_d) = @_;
	
	my $secs=0;
	my $mins=0;
	my $hours=0;
	my $days=0;
	my $weeks=0;
	my $tmp1;

	if($strin =~ /[SsMmHhDdWw]/)
	{ 
		($secs, $tmp1) = ($strin  =~ m/(\d+)[Ss]/); 
		($mins, $tmp1) = ($strin  =~ m/(\d+)[Mm]/); 
		($hours, $tmp1) = ($strin =~ m/(\d+)[Hh]/); 
		($days, $tmp1) = ($strin  =~ m/(\d+)[Dd]/); 	
		($weeks, $tmp1) = ($strin =~ m/(\d+)[Ww]/); 		
	}
	else
	{
		$$num = $strin;
		$$time_d = "S";
		return 0;	
	}

	if($secs){
		$secs += (($mins * 60) + ($hours * 3600) + ($days * 86400) + ($weeks * 604800)); #convert higher to secs
		$$num=$secs;
		$$time_d="S";
	}
	elsif($mins){ 
		$mins += (($hours * 60) + ($days * 1440)+ ($weeks * 10080));  #convert higher to mins
		$$num=$mins;
		$$time_d="M";
	}
	elsif($hours){ 
		$hours += (($days * 24) + ($weeks * 168)) ; #convert higher to hours
		$$num=$hours;
		$$time_d="H";
	}
	elsif($days){ 
		$days += ($weeks * 7) ;
		$$num=$days;
		$$time_d="D";
	}
	elsif($weeks){ 
		$$num = $weeks;
		$$time_d="W";
	}
	
	return 0;
}

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'subzonename'		=> '',
			'subzoneipaddr'		=> '',
			'adminemail'		=> '',

			'refresh'		=> '8',
			'retry'			=> '2',
			'expire'		=> '1',
			'minttl'		=> '1',
			'D1'			=> 'H',
			'D2'			=> 'H',
			'D3'			=> 'W',
			'D4'			=> 'D',
			
			'R1'			=> '1',
			'slavenetname'		=> '',
		);

########################################

if ( ($cgi->param('addsubzone')) && ($cgi->param('submit')) )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}
else
{
	# If we started fresh and didn't submit anything,
	# Grab the adminemail and zone time values from the zone's parent if possible.
	
	my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);
	my @z_info;
	my $parentzone = $cgi->param('zname');
	my ($ia,$ib,$ic,$ic1,$ic2,$ic3,$ic4);

	$ia = DNSConfig->zone_getinfo(\@z_info,$parentzone);

	if ( ($z_info[0] ne '') && ($z_info[1] ne '') )
	{
		# Retrieve information about the parent zone.  Only really use the pz_soa array.
		
		$ib = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

		my $FormatAdminEmail = '';
		my $RawAdminEmail = $pz_soa[2];

		if ($RawAdminEmail ne '')
		{
			my (@a1,$t1,$t2);

			@a1 = split('\.',$RawAdminEmail);
			while ( (scalar(@a1)) && ($a1[0] eq '') )
			{
				# Strip off leading dots in the e-mail address.  They annoy us.
				shift(@a1);
			}

			if (scalar(@a1) > 1)
			{
				$t1 = shift(@a1);

				$FormatAdminEmail = $t1 . '@' . join('.',@a1);
			}
			else
			{
				$FormatAdminEmail = join('.',@a1);
			}
		}

		# Next, assign the re-formatted admin e-mail to the FormData hash.

		$FormData{'adminemail'} = $FormatAdminEmail;

		# Then convert the time values (number + digit modifier) and push them into the FormData hash.

		ConvertNum($pz_soa[4], \$FormData{'refresh'}, \$FormData{'D1'});
		ConvertNum($pz_soa[5], \$FormData{'retry'},   \$FormData{'D2'});
		ConvertNum($pz_soa[6], \$FormData{'expire'},  \$FormData{'D3'});
		ConvertNum($pz_soa[7], \$FormData{'minttl'},  \$FormData{'D4'});

		# Our work here is done.
	}
}


# Set up the radio state.
$FormData{'R1_master'}		= ('1' eq $FormData{'R1'}) ? 'checked' : '';
$FormData{'R1_slave'}		= ('2' eq $FormData{'R1'}) ? 'checked' : '';

# Set up the option selectors for D1.
$FormData{'D1_S'}		= ('S' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_M'}		= ('M' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_H'}		= ('H' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_D'}		= ('D' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_W'}		= ('W' eq $FormData{'D1'}) ? 'selected' : '';

# Set up the option selectors for D2.
$FormData{'D2_S'}		= ('S' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_M'}		= ('M' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_H'}		= ('H' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_D'}		= ('D' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_W'}		= ('W' eq $FormData{'D2'}) ? 'selected' : '';

# Set up the option selectors for D3.
$FormData{'D3_S'}		= ('S' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_M'}		= ('M' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_H'}		= ('H' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_D'}		= ('D' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_W'}		= ('W' eq $FormData{'D3'}) ? 'selected' : '';

# Set up the option selectors for D4.
$FormData{'D4_S'}		= ('S' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_M'}		= ('M' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_H'}		= ('H' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_D'}		= ('D' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_W'}		= ('W' eq $FormData{'D4'}) ? 'selected' : '';

########################################

my $PAGEFWD = <<HEREFWD;
<SCRIPT>
function disabledforslave()
{
	document.addsub.subzoneipaddr.disabled=true;
	document.addsub.adminemail.disabled=true;	
	document.addsub.refresh.disabled=true;	
	document.addsub.retry.disabled=true;	
	document.addsub.expire.disabled=true;	
	document.addsub.minttl.disabled=true;	
	document.addsub.D1.disabled=true;	
	document.addsub.D2.disabled=true;	
	document.addsub.D3.disabled=true;	
	document.addsub.D4.disabled=true;	
	
}
function enableformaster()
{
	document.addsub.subzoneipaddr.disabled=false;
	document.addsub.adminemail.disabled=false;	
	document.addsub.refresh.disabled=false;	
	document.addsub.retry.disabled=false;	
	document.addsub.expire.disabled=false;	
	document.addsub.minttl.disabled=false;	
	document.addsub.D1.disabled=false;	
	document.addsub.D2.disabled=false;	
	document.addsub.D3.disabled=false;	
	document.addsub.D4.disabled=false;	
	
}
</SCRIPT>

<FORM METHOD="POST" NAME="addsub" ACTION="server_dns_subzone.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="addsubzone" value="on">
<INPUT TYPE="hidden" NAME="zname" value="SUB_THISPARENTZONE">
<INPUT TYPE="hidden" NAME="direction" value="forward">
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <tr>
          <TD WIDTH="53%" NOWRAP colspan="2"><b>$_R{'DNSSUBNAME'}:</b>
          </TD>
          <TD NOWRAP WIDTH="47%"><INPUT TYPE="text" NAME="subzonename" value="$FormData{'subzonename'}" SIZE=10 MAXLENGTH=45> <B>.SUB_THISPARENTDISP </B>
          </TD>
        </tr>

        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="1" $FormData{'R1_master'} onclick='enableformaster();'>
          </td>
          <td width="94%" colspan="2" align="left"><b>$_R{'MASTERZONE'}</b>
          </td>
        </tr>
	        
	<tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'IPADDRESS'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="subzoneipaddr" value="$FormData{'subzoneipaddr'}"  SIZE=18 MAXLENGTH=15>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'ADMINEMAIL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="adminemail" value="$FormData{'adminemail'}" SIZE=23 MAXLENGTH=45>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'REFRESHAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="refresh" SIZE=5 MAXLENGTH=15 value="$FormData{'refresh'}">
            <select size="1" name="D1">
              <option $FormData{'D1_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D1_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D1_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D1_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D1_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'RETRYAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="retry"  SIZE=5 MAXLENGTH=15 value="$FormData{'retry'}">
            <select size="1" name="D2">
              <option $FormData{'D2_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D2_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D2_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D2_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D2_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </TR>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'EXPIREAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="expire"  SIZE=5 MAXLENGTH=15 value="$FormData{'expire'}">
            <select size="1" name="D3">
              <option $FormData{'D3_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D3_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D3_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D3_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D3_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'MINIMUMTTL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="minttl"  SIZE=5 MAXLENGTH=15 value="$FormData{'minttl'}">
            <select size="1" name="D4">
              <option $FormData{'D4_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D4_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D4_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D4_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D4_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="2" $FormData{'R1_slave'} onclick='disabledforslave();'>
	  </td>
	  <td width="94%" colspan="2" align="left"><b>$_R{'SLAVEZONE'}</b>
          </td>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'MASTERSERVERNAME'}:</b>
	  </TD>
	  <TD WIDTH="47%"><input type="text" name="slavenetname" size="23" value="$FormData{'slavenetname'}">
          </td>
        </tr>        
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <TR>
          <TD WIDTH="100%" colspan="3" align="right">&nbsp;<input type="submit" class="clsButton" value=" $_R{'ADD'} " name="submit">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>
HEREFWD


########################################

my $PAGEREV = <<HEREREV;
<SCRIPT>
function disabledforslave()
{
	document.addsub.adminemail.disabled=true;	
	document.addsub.refresh.disabled=true;	
	document.addsub.retry.disabled=true;	
	document.addsub.expire.disabled=true;	
	document.addsub.minttl.disabled=true;	
	document.addsub.D1.disabled=true;	
	document.addsub.D2.disabled=true;	
	document.addsub.D3.disabled=true;	
	document.addsub.D4.disabled=true;	
	
}
function enableformaster()
{
	document.addsub.adminemail.disabled=false;	
	document.addsub.refresh.disabled=false;	
	document.addsub.retry.disabled=false;	
	document.addsub.expire.disabled=false;	
	document.addsub.minttl.disabled=false;	
	document.addsub.D1.disabled=false;	
	document.addsub.D2.disabled=false;	
	document.addsub.D3.disabled=false;	
	document.addsub.D4.disabled=false;	
	
}
</SCRIPT>

<FORM METHOD="POST" NAME="addsub" ACTION="server_dns_subzone.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="addsubzone" value="on">
<INPUT TYPE="hidden" NAME="zname" value="SUB_THISPARENTZONE">
<INPUT TYPE="hidden" NAME="direction" value="reverse">
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <tr>
          <TD WIDTH="53%" NOWRAP colspan="2"><b>$_R{'DNSSUBNAME'}:</b>
          </TD>
          <TD NOWRAP WIDTH="47%"><INPUT TYPE="text" NAME="subzonename" value="$FormData{'subzonename'}" SIZE=10 MAXLENGTH=45> <B>.SUB_THISPARENTDISP </B>
          </TD>
        </tr>

        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="1" $FormData{'R1_master'} onclick='enableformaster();'>
          </td>
          <td width="94%" colspan="2" align="left"><b>$_R{'MASTERZONE'}</b>
          </td>
        </tr>
	        
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'ADMINEMAIL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="adminemail" value="$FormData{'adminemail'}" SIZE=23 MAXLENGTH=45>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'REFRESHAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="refresh" SIZE=5 MAXLENGTH=15 value="$FormData{'refresh'}">
            <select size="1" name="D1">
              <option $FormData{'D1_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D1_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D1_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D1_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D1_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'RETRYAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="retry"  SIZE=5 MAXLENGTH=15 value="$FormData{'retry'}">
            <select size="1" name="D2">
              <option $FormData{'D2_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D2_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D2_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D2_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D2_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </TR>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'EXPIREAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="expire"  SIZE=5 MAXLENGTH=15 value="$FormData{'expire'}">
            <select size="1" name="D3">
              <option $FormData{'D3_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D3_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D3_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D3_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D3_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'MINIMUMTTL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="minttl"  SIZE=5 MAXLENGTH=15 value="$FormData{'minttl'}">
            <select size="1" name="D4">
              <option $FormData{'D4_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D4_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D4_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D4_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D4_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="2" $FormData{'R1_slave'} onclick='disabledforslave();'>
	  </td>
	  <td width="94%" colspan="2" align="left"><b>$_R{'SLAVEZONE'}</b>
          </td>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%" NOWRAP>&nbsp;<b>$_R{'MASTERSERVERNAME'}:</b>
	  </TD>
	  <TD WIDTH="47%"><input type="text" name="slavenetname" size="23" value="$FormData{'slavenetname'}">
          </td>
        </tr>        
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <TR>
          <TD WIDTH="100%" colspan="3" align="right">&nbsp;<input type="submit" class="clsButton" value=" $_R{'ADD'} " name="submit">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>
HEREREV



###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html,$parentzonename) = @_;

	my $dispname;

	# Set the server status.
	#$html =~ s/SERVERSTATUS/$ServiceStatus/s;

	$html =~ s/SUB_THISPARENTZONE/$parentzonename/s;

	$dispname = $parentzonename;
	
	if ($parentzonename =~ /(\.)$/ )
	{
		# If the name ends with a period, strip it off.
		$dispname = $`;
	}
	
	$html =~ s/SUB_THISPARENTDISP/$dispname/s;

	# Add formatted version of parent zone name (w/o trailing dots) into page for view.

	return $html;
}

###############################################################################
# InvalidIP
###############################################################################

sub InvalidIP
{
	my ($ipaddress,$useless) = @_;
	
	my ($ipA,$ipB,$ipC,$ipD);
	my ($t1,$t2);
	my $tempIP = $ipaddress;

	my $chkval = ( $tempIP =~ /(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})/ );

	if (!($chkval) )
	{
		# If the string does not match the approved format
		return 1;  # false
	}
	elsif ( ($` ne '') || ($' ne '') || ($& eq '') )
	{
		return 1;
		# Matched, but there's extra data before or after the string.
	}
	else
	{
		($ipA,$ipB,$ipC,$ipD) = ( $tempIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
		
		if ( ($ipA < 0) || ($ipA > 255) )
		{
			return 1;
		}
		elsif ( ($ipB < 0) || ($ipB > 255) )
		{
			return 1;
		}
		elsif ( ($ipC < 0) || ($ipC > 255) )
		{
			return 1;
		}
		elsif ( ($ipD < 0) || ($ipD > 255) )
		{
			return 1;
		}
		else
		{
			return 0;
		}

	}

}

###############################################################################
# InvalidTime
###############################################################################

sub InvalidTime
{
	my ($tval1,$useless) = @_;

	# Begin checking for the time values.
	# Valid include:
	# 1, 1S1, 1M1, 1H1, 1D1, 1W1, 1S1M1H1D1W1

	if ($tval1 =~ /^(((\d+)[SsMmHhDdWw])*)(\d+)$/ )
	{
		# Match value?
		return 0;
	}
	else
	{
		return 1;
	}

}

###############################################################################
# The main body of the script.
###############################################################################

my $html;
my $fn_out1;
my $mymaster = $cgi->param('R1');
my $mymasterlen = length($cgi->param('slavenetname'));
my ($ia,$ib,$ic,$ic1,$ic2,$id,$ie,$ie2);
my $filename;


my $parentzone;
my $fullparentzone;
my $z_assembled;
my $z_display;
my $parentfile;
my ($i,$j);
my @z_config_info;
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);

$parentzone = $cgi->param('zname');
$fullparentzone = $parentzone . ".";


# Detect if we are a reverse zone, by checking the parent's zone name.

my $iszonereverse;
if ( ($cgi->param('zname') =~ /.in-addr.arpa$/) || ($cgi->param('direction') eq 'reverse') )
{
	$iszonereverse = 1;
}
else
{
	$iszonereverse = 0;
}



# If applying a change...
if ( ($cgi->param('submit')) && ($cgi->param('addsubzone')) )
{
	if ($cgi->param('subzonename') eq '')
	{
		$Result = 870006;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		
		$UseFocus = 1;
		$FocusForm = 'addsub';
		$FocusField = 'subzonename';
		goto DISPLAYMAINFORM;
	}

	my $subzonename = lc($cgi->param('subzonename'));
	my $slavenetname = lc($cgi->param('slavenetname'));
	
	$z_assembled = $subzonename . "." . $fullparentzone;
	$z_display = $subzonename . "." . $parentzone;

	# Check to see is Master or Slave is checked and check to see if the Slaves Primary Master is set
	if ( ( ($mymaster eq '2') && ($mymasterlen) ) || ($mymaster eq '1') )
	{
		# Pass the form values into DNSConfig->addzone to create the zone information in the named.conf
		if ($mymaster eq '2')
		{
			$ia = DNSConfig->zone_add($z_display,\$fn_out1, $mymaster, $slavenetname );

			if ($ia eq 0)
			{
				# Now add the link from the parent zone.

				# Get parent zone file information.

				$id = DNSConfig->zone_getinfo(\@z_config_info,$parentzone);

				if ($id ne 0)
				{
					# Unable to get info for parent zone file.
					$Result = 870007;
					$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
					goto DISPLAYMAINFORM;
				}

				if ( ($z_config_info[0] ne '') && ($z_config_info[1] ne '') )
				{
					# If we find it, that is.
					$parentfile = $z_config_info[1];

					# Add NS SUB record to parent zone.

					if ($iszonereverse eq 1)
					{
						$ie = DNSZone->zone_addrecord($parentzone,$parentfile,"NS",$subzonename,'localhost.');
					}
					else
					{
						$ie = DNSZone->zone_addrecord($parentzone,$parentfile,"NS",$subzonename,'dns.' . $fullparentzone);
					}
					$ie2 = DNSZone->zone_increment($parentzone,$parentfile);

					if ( ($ie ne 0) || ($ie2 ne 0) )
					{
						# Unable to get info for parent zone file.
						$Result = 870008;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						goto DISPLAYMAINFORM;
					}

				}

			}
		}
		else
		{
			# Look for empty parameters, and hurt them.
			
			if ( ($iszonereverse eq 0) && ($cgi->param('subzoneipaddr') eq '') )
			{
				$Result = 870005;
				$FocusField = 'subzoneipaddr';
			}
			
			elsif ($cgi->param('adminemail') eq '')
			{
				$Result = 870005;
				$FocusField = 'adminemail';
			}
			
			elsif ($cgi->param('refresh') eq '')
			{
				$Result = 870005;
				$FocusField = 'refresh';
			}
			elsif (InvalidTime($cgi->param('refresh')))
			{
				$Result = 870019;
				$FocusField = 'refresh';
			}
			
			elsif ($cgi->param('retry') eq '')
			{
				$Result = 870005;
				$FocusField = 'retry';
			}
			elsif (InvalidTime($cgi->param('retry')))
			{
				$Result = 870019;
				$FocusField = 'retry';
			}
			
			elsif ($cgi->param('expire') eq '')
			{
				$Result = 870005;
				$FocusField = 'expire';
			}
			elsif (InvalidTime($cgi->param('expire')))
			{
				$Result = 870019;
				$FocusField = 'expire';
			}
			
			elsif ($cgi->param('minttl') eq '')
			{
				$Result = 870005;
				$FocusField = 'minttl';
			}
			elsif (InvalidTime($cgi->param('minttl')))
			{
				$Result = 870019;
				$FocusField = 'minttl';
			}
			
			if ( ($Result eq 870005) || ($Result eq 870019) )
			{
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				$UseFocus = 1;
				$FocusForm = 'addsub';
				goto DISPLAYMAINFORM;
			}

			if ( ($iszonereverse eq 0) && (InvalidIP($cgi->param('subzoneipaddr'))) )
			{
				$Result = 870009;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addsub';
				$FocusField = 'subzoneipaddr';
				goto DISPLAYMAINFORM;
			}

			if ($cgi->param('adminemail') !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
			{
				# Invalid e-mail address!

				$Result = 870010;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addsub';
				$FocusField = 'adminemail';
				goto DISPLAYMAINFORM;
			}

			$ia = DNSConfig->zone_add($z_display,\$fn_out1, $mymaster);
			if ($ia eq 0)
			{
				
				if ($iszonereverse eq 1)
				{
					$ib = DNSZone->zone_construct_rev($z_assembled,$fn_out1,'localhost.');
				}
				else
				{
					$ib = DNSZone->zone_construct_fwd($z_assembled,$fn_out1,$cgi->param('subzoneipaddr'));
				}


				if($ib eq 0)
				{
					$ic1 = DNSZone->zone_getinfo($z_assembled,$fn_out1,\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

					if ($ic1 ne 0)
					{
						# Unable to get info for newly created zone file.
						$Result = 870003;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						goto DISPLAYMAINFORM;
					}

					$pz_soa[2] = $cgi->param('adminemail');
					$pz_soa[4] = $cgi->param('refresh') . $cgi->param('D1');
					$pz_soa[5] = $cgi->param('retry') . $cgi->param('D2');
					$pz_soa[6] = $cgi->param('expire') . $cgi->param('D3');
					$pz_soa[7] = $cgi->param('minttl') . $cgi->param('D4');

					$ic2 = DNSZone->zone_changeheader($z_assembled,$fn_out1,\@pz_soa);

					if ($ic2 ne 0)
					{
						# Unable to set header info for newly created zone file.
						$Result = 870004;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						goto DISPLAYMAINFORM;
					}

					# Get parent zone file information.
			
					$id = DNSConfig->zone_getinfo(\@z_config_info,$parentzone);
			
					if ($id ne 0)
					{
						# Unable to get info for parent zone file.
						$Result = 870007;
						$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
						goto DISPLAYMAINFORM;
					}

					if ( ($z_config_info[0] ne '') && ($z_config_info[1] ne '') )
					{
						# If we find it, that is.
						$parentfile = $z_config_info[1];
			
						# Add NS SUB record to parent zone.
						
						if ($iszonereverse eq 1)
						{
							$ie = DNSZone->zone_addrecord($parentzone,$parentfile,"NS",$subzonename,'localhost.');
						}
						else
						{
							$ie = DNSZone->zone_addrecord($parentzone,$parentfile,"NS",$subzonename,'dns.' . $fullparentzone);
						}
						$ie2 = DNSZone->zone_increment($parentzone,$parentfile);

						if ( ($ie ne 0) || ($ie2 ne 0) )
						{
							# Unable to get info for parent zone file.
							$Result = 870008;
							$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
							goto DISPLAYMAINFORM;
						}

					}

				}

			}

		}
		
		if ($ia != 0 || $ib != 0)
		{
			# Unable to add or construct subzone.
			$Result = 870002;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		else
		{
			print $cgi->redirect("server_dns_edit.cgi?zname=$parentzone&newsubzonemessage=1&newsubzonename=" . $z_display );
			exit(0);
		}	

	}

	else
	{
		$Result = 870001;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';	
		
		$UseFocus = 1;
		$FocusForm = 'addsub';
		$FocusField = 'slavenetname';
	}	

}

#####################################

DISPLAYMAINFORM:

if ($iszonereverse eq 1)
{
	$html = SetPage($PAGEREV,$parentzone);
}
else
{
	$html = SetPage($PAGEFWD,$parentzone);
}

my %var;

$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8700;
$var{'title'}			= $_R{'IDT:8700'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'} =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
				{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$parentzone"},
			];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}

DisplayMain->printDocument(\%var);

exit(0);

