#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_fan.cgi                                                    #
#                                                                             #
# Description: CGI needed to display fan information.                         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# GetCurrentRPM
###############################################################################

sub GetCurrentRPM
{
	my $CommandKey	= shift;
	my $CurrentRPM	= 0;

	# Validate the input parameter(s)...
	if (defined($CommandKey))
	{
		# Get the requested data.
		$CurrentRPM = Safe::backtick("/usr/ins/intel/bin/asm/bakAsm $CommandKey");
	}

	return $CurrentRPM;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_fan.lrc', 'server_hwinfo.lrc', 'server_alerts.lrc' ]);
	my $Configuration		= new ConfigurationData;
	my %_R					= ( 'IDT:300'				=> '',
								'FAN_THRESHOLDS'		=> '',
								'CRITICAL_LOWER'		=> '',
								'WARNING_LOWER'			=> '',
								'CURRENT'				=> '',
								'RPM'					=> '',
								'WARNING'				=> '',
								'CRITICAL'				=> '',
								'IDT:500'				=> '',
								'CONFIGURE_ALERTS'		=> '' );
	my @FanData			= ( { 'c-label' => 'FAN0', 'label' => 'FAN_1', 'command-key' => 'get_fan_0' },
							{ 'c-label' => 'FAN1', 'label' => 'FAN_2', 'command-key' => 'get_fan_1' },
							{ 'c-label' => 'FAN2', 'label' => 'FAN_3', 'command-key' => 'get_fan_2' },
							{ 'c-label' => 'FAN3', 'label' => 'FAN_4', 'command-key' => 'get_fan_3' },
							{ 'c-label' => 'FAN4', 'label' => 'FAN_5', 'command-key' => 'get_fan_4' } );
	my $Feedback;
	my $Result;
	my $html;
	my $F;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Try to open the configuration file...
	if (0 == $Configuration->Attach('/etc/asm/asm.conf'))
	{
		# Run through each of the fan(s)...
		foreach $F (@FanData)
		{
			# Get the label.
			$F->{'label'} = $StringResource->_S($F->{'label'});

			# Get the upper and lower thresholds.
			$F->{'warning-lower'}	= $Configuration->GetValueKeyed("$F->{'c-label'}_THRESHOLD");
			$F->{'critical-lower'}	= $Configuration->GetValueKeyed("$F->{'c-label'}_CRITICAL_THRESHOLD");

			# Get the current fan RPM.
			$F->{'current'} = GetCurrentRPM($F->{'command-key'});

			# Get the indicator positions.
			$F->{'i-position-left'}		= int((($F->{'current'} / 10000) * 100));
			$F->{'i-position-right'}	= (100 - int((($F->{'current'} / 9000) * 100)));

			# Get the current status...
			if ($F->{'critical-lower'} > $F->{'current'})
			{
				# Critical.
				$F->{'status'}	= " <img src=/images/critical.gif> $_R{'CRITICAL'}";
				$F->{'bgcolor'} = '#ff0000'
			}
			elsif ($F->{'warning-lower'} > $F->{'current'})
			{
				# Warning.
				$F->{'status'}	= " <img src=/images/warning.gif> $_R{'WARNING'}";
				$F->{'bgcolor'} = '#ffff00';
			}
			else 
			{
				# Normal.
				$F->{'status'}	= '';
				$F->{'bgcolor'} = '#008000';
			}
		}
	}
	else
	{
		# Run through each of the fan(s)...
		foreach $F (@FanData)
		{
			# Safe defaults.
			$F->{'label'}				= $StringResource->_S($F->{'label'});
			$F->{'warning-lower'}		= 0;
			$F->{'critical-lower'}		= 0;
			$F->{'current'}				= 0;
			$F->{'i-position-left'}		= 0; 
			$F->{'i-position-right'}	= 100;
			$F->{'status'} 				= '';
		}
	}

	# The top of the page.
	$html = <<HERE;
<table border=0 width=375>
<tr>
	<td>
		<table border=0 cellPadding=0 cellSpacing=1 width=100%>
        <tr>
			<td colSpan=3 width=100%><b>$_R{'FAN_THRESHOLDS'}</b></td>
		</tr>
HERE

	# Run through each of the fan(s)...
	foreach $F (@FanData)
	{
		# Add on the information the current fan.
		$html .= <<HERE;
		<tr>
			<td colspan=3 width=100%>&nbsp;</td>
		</tr>
		<tr>
			<td colspan=3 width=100%><b>$F->{'label'}</b>$F->{'status'}</td>
		</tr>
		<tr>
			<td width=135 align=center rowspan=3>
				<table border=1 cellpadding=0 cellspacing=0 width=100>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=$F->{'i-position-left'}% bgcolor=$F->{'bgcolor'}>&nbsp;</td>
							<td width=$F->{'i-position-right'}%></td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
			<td width=110>$_R{'CURRENT'}&nbsp;<br></td>
			<td width=133><b>$F->{'current'} $_R{'RPM'}</b></td>
		</tr>
		<tr>
			<td width=110>$_R{'WARNING_LOWER'}</td>
			<td width=133>$F->{'warning-lower'} $_R{'RPM'}</td>
		</tr>
		<tr>
			<td width=110>$_R{'CRITICAL_LOWER'}</td>
			<td width=133>$F->{'critical-lower'} $_R{'RPM'}</td>
		</tr>
HERE
	}

	# Add on the tail of the page.
	$html .= <<HERE;
		<tr>
			<td colSpan=3 width=100%>&nbsp;</td>
		</tr>
		</table>
		<br>
		<a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>
	</td>
</tr>
</table>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:500'}, 'location' => 'server_hwinfo.cgi' } ],
								 'root-link' => 'server_status', 'class' => 'server_fan', 'id' => 300, 'title' => $_R{'IDT:300'} });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
