#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_ftp.cgi                                                    #
#                                                                             #
# Description: CGI needed to display and modify FTP settings.                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use VirtualServ;
use LanguageResource;
use DisplayMain;
use MonSettings;
use Daemon;
use CGI;

###############################################################################
# GetFTPStatus
###############################################################################

sub GetFTPStatus
{
	my $Return	= 0;
	my $Inetd	= new Daemon('inetd', '');
	
	# Get the status of inetd daemon...
	if ($Inetd->GetStatus())
	{
		# Try to open the configuration file...
		if (open(INETDCONF, '< /etc/inetd.conf'))
		{
			# Loop through the file...
			while(<INETDCONF>) 
			{
				# If a line matching ftp appears...
				if (/^ftp/) 
				{
					# FTP is configured to run.
					$Return = 1;
					last;
				}
			}

			# Close the file.	
			close(INETDCONF);
		}
	}

	return $Return;
}

###############################################################################
# ControlFTP
###############################################################################

sub ControlFTP
{
	my $Control = shift;
	my $Return	= 0;
	
	# Try to open the configuration file...
	if (open(INETDCONF, '< /etc/inetd.conf'))
	{
		my @ConfigurationData = <INETDCONF>;

		# Close the file.
		close(INETDCONF);

		# Try to open the configuration file...
		if (open(INETDCONF, '> /etc/inetd.conf'))
		{
			my $INETDaemon = new Daemon('inet');
			my $Line;

			# Loop through the file...
			foreach $Line (@ConfigurationData) 
			{
				# Based on the control...
				if ((0 == $Control) &&
					($Line =~ /^ftp/))
				{ 
					# Add on the comment marker.
					print INETDCONF "#$Line";
				}
				elsif ((1 == $Control) &&
					   ($Line =~ /(\#+)(ftp.*)$/))
				{ 
					# Take off the comment marker.
					print INETDCONF "$2\n"; 
				}
				else 
				{ 
					# Just an ordinary line.
					print INETDCONF $Line; 
				}
			}
					
			# Close the file.	
			close(INETDCONF);

			# Restart the daemon.
			$INETDaemon->Control(2);
		}
	}

	return $Return;
}

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my $MaxLoginFailureAttempts	= $cgi->param('max_login_failure_attempts');
		my $MaxLoginUsers			= $cgi->param('max_login_users');
		my $MaxFileTransferSize		= $cgi->param('max_file_transfer_size');
		my $MaxFileTransferIdle		= $cgi->param('max_file_transfer_idle');

		if ($MaxLoginFailureAttempts !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 180000;
		}
		elsif ($MaxLoginUsers !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 180001;
		}
		elsif (($MaxFileTransferSize !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 180002;
		}
		elsif ($MaxFileTransferIdle !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 180003;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_ftp.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:1800'						=> '',
								'SERVICE'						=> '',
								'STATUS'						=> '',
								'CONTROLS'						=> '',
								'FTP'							=> '',
								'START'							=> '',
								'STOP'							=> '',
								'CONFIRM_STOP'					=> '',
								'RUNNING'						=> '',
								'STOPPED'						=> '',
                                'ALLOW_ANONYMOUS_LOGIN'			=> '',
								'BANNER_MESSAGE_USERS'			=> '',
								'FAILED_LOGIN_ATTEMPTS'			=> '',
								'MAX_LOGGED_IN_USERS'			=> '',
								'MAX_SESSION_DATA_TRANSFER'		=> '',
								'IDLE_TIME_BEFORE_DISCONNECT'	=> '',
								'MB'							=> '',
								'SEC'							=> '',
								'APPLY'							=> '' );
	my $ServiceStatus;
	my $EnableAnonymousFTP;
	my @BannerSystemUsers;
	my $MaxLoginFailureAttempts;
	my $MaxLoginUsers;
	my $MaxFileTransferSize;
	my $MaxFileTransferIdle;
	my $Dummy;
	my $Feedback;
	my $Result;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# If toggling the daemon state...
	if ((defined($cgi->param('control_start'))) ||
		(defined($cgi->param('control_stop'))))
	{
		my $Monitor = new MonSettings();
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('ftp', (($cgi->param('control_start')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}
	
	SWITCH:
	{
		# Control the services.
		if (defined($cgi->param('control_start')))	{ ControlFTP(1); last SWITCH; }
		if (defined($cgi->param('control_stop')))	{ ControlFTP(0); last SWITCH; }
	}

	# Get the status information.
	$ServiceStatus	= (GetFTPStatus()) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";
	
	# If applying data...
	if (defined($cgi->param('apply')))
	{
		# Validate the data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Set the applied data.
			VirtualServ::setftp($cgi->param('max_login_failure_attempts'),
								$cgi->param('max_file_transfer_idle'),
								$cgi->param('max_file_transfer_size'),
								$cgi->param('max_login_users'),
								'',
								$cgi->param('banner_system_users'),
								$cgi->param('enable_anonymous_ftp'));
		}
		else
		{
			$EnableAnonymousFTP			= $cgi->param('enable_anonymous_ftp') || '';
			@BannerSystemUsers			= $cgi->param('banner_system_users');
			$MaxLoginFailureAttempts	= $cgi->param('max_login_failure_attempts');
			$MaxLoginUsers				= $cgi->param('max_login_users');
			$MaxFileTransferSize		= $cgi->param('max_file_transfer_size');
			$MaxFileTransferIdle		= $cgi->param('max_file_transfer_idle');
			
			# Get the feedback message.
			$Feedback = $StringResource->_S($Result);
		}
	}
	
	# If there aren't any negative results...
	unless ($Result)
	{
		# Get the configuration data.
		($MaxLoginFailureAttempts, $MaxFileTransferIdle, $MaxFileTransferSize, $MaxLoginUsers) = VirtualServ::ftp_conf();
		($Dummy, $EnableAnonymousFTP) = VirtualServ::checkAnon(); 
		@BannerSystemUsers = VirtualServ::ftp_msg();

		# Adjust the file size.
		$MaxFileTransferSize /= 1048576;
	}
	
	# Adjust the checkbox value.
	$EnableAnonymousFTP = 'checked' if ($EnableAnonymousFTP eq 'yes');
	
	my $html = <<HERE;
<form method=post action=server_ftp.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<br>
			<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
			<tr> 
				<td width=100%>
					<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
					<tr>
						<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
						<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
						<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'FTP'}</b></td>
						<td width=15%><font color=#008000>$ServiceStatus</font></td>
						<td width=40% align=left>
							<input type=submit class=clsButton value='$_R{'START'}' name=control_start>
							<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
						</td>
					</tr>
					</table>
				</td>
			</tr>
			</table>
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=95% valign=top><input type=checkbox name=enable_anonymous_ftp value=yes $EnableAnonymousFTP>&nbsp;<b>$_R{'ALLOW_ANONYMOUS_LOGIN'}</b></td>
				<td width=10%></td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% colspan=2><b>$_R{'BANNER_MESSAGE_USERS'}</b></td>
			</tr>
			<tr>
				<td width=100% colspan=2><textarea name=banner_system_users rows=4 cols=41 wrap=physical>@BannerSystemUsers</textarea></td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td nowrap width=95%><b>$_R{'FAILED_LOGIN_ATTEMPTS'}</b></td>
				<td width=10%><input type=text name=max_login_failure_attempts size=3 maxlength=3 value='$MaxLoginFailureAttempts'></td>
			</tr>
			<tr>
				<td nowrap width=95%><b>$_R{'MAX_LOGGED_IN_USERS'}</b>&nbsp;</td>
				<td width=10%><input type=text name=max_login_users size=3 maxlength=4 value='$MaxLoginUsers'></td>
			</tr>
			<tr>
				<td nowrap width=64%><b>$_R{'MAX_SESSION_DATA_TRANSFER'}</b></td>
				<td width=36%><input type=text name=max_file_transfer_size size=3 maxlength=4 value='$MaxFileTransferSize'>$_R{'MB'}</td>
			</tr>
			<tr>
				<td nowrap width=64%><b>$_R{'IDLE_TIME_BEFORE_DISCONNECT'}</b></td>
				<td width=36%><input type=text name=max_file_transfer_idle size=4 maxlength=5 value='$MaxFileTransferIdle'>$_R{'SEC'}</td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% align=right colspan=2>
					<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>
				</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<br>
	<a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>
</form>
HERE
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_ftp', 'id' => 1800, 'title' => $_R{'IDT:1800'},
								 'root-link' => 'server_ftp', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
