#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_mail.cgi                                                   #
#                                                                             #
# Description: CGI needed to display and modify mail settings.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Unix::AliasFile;
use LanguageResource;
use DisplayMain;
use MonSettings;
use Daemon;
use Safe;
use CGI;

###############################################################################
# GetMailConfiguration
###############################################################################

sub GetMailConfiguration 
{
	my $var_ref = $_[0];
	my ($smart_relay_host, $unresolvable_domains, @array);

	open (SENDMAIL_MC, "/etc/mail/sendmail.mc");
	while (<SENDMAIL_MC>) 
	{
		if (/`SMART_HOST', `(\S+)(?=')/ ) 
		{ 
			$smart_relay_host= $1; 
		}
		elsif (/^FEATURE.*`accept_unresolvable_domains'/ ) 
		{ 
			$unresolvable_domains = ''; 
		}
		elsif (/^dnl.*FEATURE.*`accept_unresolvable_domains'/ ) 
		{ 
			$unresolvable_domains = 'checked'; 
		}
		elsif (/^FEATURE.*`dnsbl'/ ) 
		{ 
			$var_ref->{rbl} = 'checked'; 
		}
		elsif (/^dnl.*FEATURE.*`dnsbl'/ ) 
		{ 
			$var_ref->{rbl} = ''; 
		}
		elsif (/^define.*`LOCAL_MAILER_MAX', `(\S+)(?=')/ ) 
		{ 
			$var_ref->{max_incoming_message_size} = $1/1048576; 
		}
		elsif (/^define.*`SMTP_MAILER_MAX', `(\S+)(?=')/ ) 
		{ 
			$var_ref->{max_outgoing_message_size} = $1/1048576; 
		}
		
		push(@array, $_);
	}
	close(SENDMAIL_MC);

	return ($smart_relay_host, $unresolvable_domains, \@array);
}

###############################################################################
# SetMailConfiguration
###############################################################################

sub SetMailConfiguration 
{
	my $vars_ref = $_[0];
	my $array_ref = $vars_ref->{arrayref};

	foreach (@$array_ref)  
	{
		if (/`SMART_HOST'/) 
		{ 
			$_ = "define(`SMART_HOST\', `$vars_ref->{smart_relay_host}\')\n" 
		}
        elsif (/`accept_unresolvable_domains'/) 
		{
			unless ($vars_ref->{unresolvable_domains}) 
			{
				$_ = "FEATURE(`accept_unresolvable_domains\')\n";
			} 
			else 
			{
				$_ = "dnl FEATURE(`accept_unresolvable_domains\')\n";
			}
        }
        elsif (/`dnsbl'/ ) 
		{
			if ($vars_ref->{q}->param('rbl')) 
			{
				$_ = "FEATURE(`dnsbl\')\n";
			}
			else 
			{
				$_ = "dnl FEATURE(`dnsbl\')\n";
			}
        }
        elsif (/`LOCAL_MAILER_MAX'/ ) 
		{
			$_ = "define(`LOCAL_MAILER_MAX\', `" . int($vars_ref->{max_incoming_message_size} * 1048576) . "\')\n"
        }
        elsif (/`SMTP_MAILER_MAX'/ ) 
		{
			$_ = "define(`SMTP_MAILER_MAX\', `" . int($vars_ref->{max_outgoing_message_size} * 1048576) . "\')\n"
        }
	}
 
	open (SENDMAIL_MC_NEW, ">/etc/mail/sendmail.mc.tmp");
	print SENDMAIL_MC_NEW @$array_ref;
	close(SENDMAIL_MC_NEW);
	
	rename("/etc/mail/sendmail.mc", "/etc/mail/sendmail.mc.orig");	# Move sendmail.mc to sendmail.mc.orig
	rename("/etc/mail/sendmail.mc.tmp", "/etc/mail/sendmail.mc");	# Move sendmail.mc.tmp to sendmail.mc 
} 

###############################################################################
# GetAccessConfiguration
###############################################################################
 
sub GetAccessConfiguration 
{
	my $vars_ref = $_[0];
	my ($rejects, $access_db);
	
	open (access_db, "/etc/mail/access");

	while (<access_db>) 
	{
		unless ((/^#/) or
				(/^\s+$/)) 
		{
			if (/REJECT/) 
			{ 
				$_ =~ s/\s*REJECT\s*/\n/;
				$rejects .= $_;
			}
			else 
			{
				$access_db .= $_;
			}
		}
	}

	return ($rejects, $access_db);
	close(access_db);
}         

###############################################################################
# SetAccessConfiguration
###############################################################################

sub SetAccessConfiguration 
{
	my $vars_ref = shift;
	my $reject_list = shift;
	my $access_db = shift;

	# Replace the newlines and semicolons with a space.
	$reject_list =~ s/[\n;,]/ /g;
	
	# Add a space at the end of the string
	$reject_list = "$reject_list ";	
	
	# Shrink each occurance of multiple spaces into 1
	$reject_list =~ s/\s+/ /g;	
	
	# Add the word REJECT and a new line char after every word.
	$reject_list =~ s/(\S+ )/$1\t\tREJECT\n/g;
	
	# If reject list is empty then clean it up
	$reject_list = "" unless $reject_list =~ /\w/g;
	
	# Remove blank lines from $access_db
	$access_db =~ s/^\n//mg;	
	
	open (access_db, ">/etc/mail/access");
	flock(access_db, 2);
	print access_db 
"# Check the /usr/lib/sendmail-cf/README file for a description
# of the format of this file. (search for access_db in that file)
# The /usr/lib/sendmail-cf/README is part of the sendmail-doc
# package.
#
$reject_list#
#
$access_db";
	
	close(access_db);
	
	# Run makemap to create a hash table from the plain text access file.
	Safe::system("makemap hash /etc/mail/access.db < /etc/mail/access");
} 

###############################################################################
# GetHostNameConfiguration
###############################################################################

sub GetHostNameConfiguration
{
    my $vars_ref = $_[0];
	
	open(local_host_names, "/etc/mail/local-host-names");

	my $host_aliases;
	while (<local_host_names>) 
	{
		unless ((/^#/) or (/^\s*\n/)) 
		{
			$host_aliases .= "$_; ";
		}
	}

	close(local_host_names);

	$host_aliases =~ s/\n//g;
	$host_aliases = substr($host_aliases, 0, -2) if ($host_aliases =~ /; $/);

	return $host_aliases;
}

###############################################################################
# SetHostNameConfiguration
###############################################################################

sub SetHostNameConfiguration 
{
	my $vars_ref = $_[0];
	my $host_aliases = $_[1];

	# Replace the newlines and semicolons with a space.
	$host_aliases =~ s/[\n;,]/ /g; 
	
	# Add a space at the end of the string
	$host_aliases = "$host_aliases ";
	
	# Shrink each occurance of multiple spaces into 1
	$host_aliases =~ s/\s+/ /g;
	
	# Add a new line char after every word.
	$host_aliases =~ s/(\S+) /$1\n/g;

	open (local_host_names, ">/etc/mail/local-host-names");
	flock(local_host_names, 2);
	print local_host_names 
"# local-host-names - include all aliases for your machine here.
# Entries from obsoleted /etc/sendmail.cw
#
";

	print local_host_names $host_aliases;
	close(local_host_names);
}

###############################################################################
# GetRelayConfiguration
###############################################################################

sub GetRelayConfiguration
{
	my $vars_ref = $_[0];
	
	open (relay_domains, "/etc/mail/relay-domains");

	my $relay_domains;
	while (<relay_domains>) 
	{
		unless ((/^#/) or (/^\s*\n/)) 
		{
			$relay_domains .= "$_; ";
		}
	}
	close(relay_domains);

	$relay_domains =~ s/\n//g;
	$relay_domains = substr($relay_domains, 0, -2) if ($relay_domains =~ /; $/);

	return $relay_domains;
}                             

###############################################################################
# SetRelayConfiguration
###############################################################################

sub SetRelayConfiguration
{
	my $vars_ref = $_[0];
	my $relay_domains = $_[1];

	# Replace the newlines and semicolons with a space.
	$relay_domains =~ s/[\n;,]/ /g; 
	
	# Add a space at the end of the string
	$relay_domains = "$relay_domains ";
	
	# Shrink each occurance of multiple spaces into 1
	$relay_domains =~ s/\s+/ /g;
	
	# Add a new line char after every word.
	$relay_domains =~ s/(\S+) /$1\n/g;

	open (relay_domains, ">/etc/mail/relay-domains");
	flock(relay_domains, 2);
	print relay_domains $relay_domains;
	close(relay_domains);
}

###############################################################################
# SetPostMasterAddress
###############################################################################

sub SetPostMasterAddress
{
	my $PostMasterAddress = shift;

	# Open the alias file in read/write mode.
	my $Alias = new Unix::AliasFile("/etc/aliases");
	
	# Set postmaster alias.
	$Alias->alias("postmaster", $PostMasterAddress);
	
	# Save changes.
	$Alias->commit();
	
	# Generate the /etc/aliases.db file from /etc/aliases
	Safe::system("/usr/bin/newaliases");
}

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my %FormData;
		my @HostAliases;
		my @RelayDomains;
		my @RejectList;
		my $Item;

		# Cache the form data.
		$FormData{'post_master_address'}		= $cgi->param('post_master_address');
		$FormData{'smart_relay_host'}			= $cgi->param('smart_relay_host');
		$FormData{'host_aliases'}				= $cgi->param('host_aliases');
		$FormData{'relay_domains'}				= $cgi->param('relay_domains');
		$FormData{'reject_list'}				= $cgi->param('reject_list');
		$FormData{'max_incoming_message_size'}	= $cgi->param('max_incoming_message_size');
		$FormData{'max_outgoing_message_size'}	= $cgi->param('max_outgoing_message_size');
		
		# Get rid of control characters.
		$FormData{'host_aliases'}	=~ s/\n/;/sg;
		$FormData{'host_aliases'}	=~ s/\,/;/g;
		$FormData{'host_aliases'}	=~ s/\cM//g;
		$FormData{'relay_domains'}	=~ s/\n/;/sg;
		$FormData{'relay_domains'}	=~ s/\,/;/g;
		$FormData{'relay_domains'}	=~ s/\cM//g;
		$FormData{'reject_list'}	=~ s/\n/;/sg;
		$FormData{'reject_list'}	=~ s/\,/;/g;
		$FormData{'reject_list'}	=~ s/\cM//g;

		# Break out the names.
		@HostAliases	= split(/;/, $FormData{'host_aliases'});
		@RelayDomains	= split(/;/, $FormData{'relay_domains'});
		@RejectList		= split(/;/, $FormData{'reject_list'});
        
		if (($FormData{'post_master_address'}) &&
			(!(($FormData{'post_master_address'} =~ /^(\w|-|_)+$/) ||
			   ($FormData{'post_master_address'} =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/))))
		{
			# Set the return code.
			$Return = 190000;
		}
		elsif (($FormData{'max_incoming_message_size'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 190001;
		}
		elsif (($FormData{'max_outgoing_message_size'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 190002;
		}
		elsif ($FormData{'smart_relay_host'} !~ /^(\w|-|\.|_)*$/)
		{
			# Set the return code.
			$Return = 190003;
		}

		# If things are going well..
		if (0 == $Return)
		{
			# Run through all of the host aliases...
			foreach $Item (@HostAliases)
			{
				# Trim the white space.
				$Item =~ s/^\s*(.*?)\s*$/$1/;

				# If the item is not a host/domain.
				if ($Item !~ /^(\w|-|\.|_)*$/)
				{
					# Set the error code.
					$Return = 190004;
					last;
				}
			}
		}

		# If things are going well..
		if (0 == $Return)
		{
			# Run through all of the host aliases...
			foreach $Item (@RelayDomains)
			{
				# Trim the white space.
				$Item =~ s/^\s*(.*?)\s*$/$1/;
				
				# If the item is not a host/domain.
				if ($Item !~ /^(\w|-|\.|_)*$/)
				{
					# Set the error code.
					$Return = 190005;
					last;
				}
			}
		}
		
		# If things are going well..
		if (0 == $Return)
		{
			# Run through all of the host aliases...
			foreach $Item (@RejectList)
			{
				# Trim the white space.
				$Item =~ s/^\s*(.*?)\s*$/$1/;
				
				# If the item is not a host/domain.
				if (($Item) &&
					(!(($Item =~ /^(\w|-|\.|_)*$/) ||               
					   ($Item =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/))))
				{
					# Set the error code.
					$Return = 190006;
					last;
				}
			}
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_mail.lrc', 'server_alerts.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:1900'						=> '',
								'SERVICE'						=> '',
								'STATUS'						=> '',
								'CONTROLS'						=> '',
								'MAIL'							=> '',
								'START'							=> '',
								'STOP'							=> '',
								'CONFIRM_STOP'					=> '',
								'RESTART'						=> '',
								'RUNNING'						=> '',
								'STOPPED'						=> '',
								'POSTMASTER_E-MAIL_ADDRESS'		=> '',
								'SMART_RELAY_HOSTNAME'			=> '',
								'REJECT_UNRESOLVED_HOST_E-MAIL'	=> '',
								'ENABLE_RBL'					=> '',
								'E-MAIL_ALIASES'				=> '',
								'E-MAIL_RELAY'					=> '',
								'E-MAIL_REJECT'					=> '',
								'MAX_MESSAGE_SIZE_INCOMING'		=> '',
								'MAX_MESSAGE_SIZE_OUTGOING'		=> '',
								'MB'							=> '',
								'MAIL_SERVICE_LOG'				=> '',
								'CONFIGURE_ALERTS'				=> '',
								'APPLY'							=> '' );
	my $MailDaemon			= new Daemon('sendmail', ': accepting connections');
	my %FormData;
	my $ServiceStatus;
	my $Feedback;
	my $Result;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the form data.
	$FormData{'post_master_address'}		= $cgi->param('post_master_address');
	$FormData{'smart_relay_host'}			= $cgi->param('smart_relay_host');
	$FormData{'unresolvable_domains'}		= $cgi->param('unresolvable_domains');
	$FormData{'rbl'}						= $cgi->param('rbl');
	$FormData{'host_aliases'}				= $cgi->param('host_aliases');
	$FormData{'relay_domains'}				= $cgi->param('relay_domains');
	$FormData{'reject_list'}				= $cgi->param('reject_list');
	$FormData{'max_incoming_message_size'}	= $cgi->param('max_incoming_message_size');
	$FormData{'max_outgoing_message_size'}	= $cgi->param('max_outgoing_message_size');
	
	# Get rid of control characters.
	$FormData{'host_aliases'}	=~ s/\n/;/sg;
	$FormData{'host_aliases'}	=~ s/\,/;/g;
	$FormData{'host_aliases'}	=~ s/\cM//g;
	$FormData{'relay_domains'}	=~ s/\n/;/sg;
	$FormData{'relay_domains'}	=~ s/\,/;/g;
	$FormData{'relay_domains'}	=~ s/\cM//g;
	$FormData{'reject_list'}	=~ s/\n/;/sg;
	$FormData{'reject_list'}	=~ s/\,/;/g;
	$FormData{'reject_list'}	=~ s/\cM//g;
	
	# If toggling the daemon state...
	if ((defined($cgi->param('control_start'))) ||
		(defined($cgi->param('control_stop'))))
	{
		my $Monitor = new MonSettings();
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('smtp', (($cgi->param('control_start')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}
	
	SWITCH:
	{
		# Control the services.
		if (defined($cgi->param('control_start')))		{ $MailDaemon->Control(1); sleep(1); last SWITCH; }
		if (defined($cgi->param('control_stop')))		{ $MailDaemon->Control(0); sleep(1); last SWITCH; }
		if (defined($cgi->param('control_restart')))	{ $MailDaemon->Control(2); sleep(1); last SWITCH; }
	}

	# If applying data...
	if (defined($cgi->param('apply')))
	{
		# Validate the data.
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			my %Variables;
			
			$| = 1;
			
			$Variables{'q'} = $cgi;
			($Variables{'smart_relay_host'}, $Variables{'unresolvable_domains'}, $Variables{'arrayref'}) = GetMailConfiguration(\%Variables);
			$Variables{'smart_relay_host'}			= $FormData{'smart_relay_host'};
			$Variables{'max_incoming_message_size'}	= int($FormData{'max_incoming_message_size'});
			$Variables{'max_outgoing_message_size'}	= int($FormData{'max_outgoing_message_size'});
			$Variables{'host_aliases'} 				= $FormData{'host_aliases'};
			$Variables{'relay_domains'}				= $FormData{'relay_domains'};
			$Variables{'rejects'}					= $FormData{'rejects'};
			$Variables{'unresolvable_domains'}		= ($FormData{'unresolvable_domains'}) ? 'checked' : '';
			$Variables{'rbl'}						= ($FormData{'rbl'}) ? 'checked' : '';
				
			# Write the /etc/mail/sendmail.mc file (Works)
			SetMailConfiguration(\%Variables);						
	
			($Variables{'rejects_tmp'}, $Variables{'access_db'}) = GetAccessConfiguration(\%Variables);
	
			# Write the /etc/mail/access file.
			SetAccessConfiguration(\%Variables, $FormData{'reject_list'}, $Variables{'access_db'});
	
			# Write the /etc/mail/local-host-names file.
			SetHostNameConfiguration(\%Variables, $FormData{'host_aliases'});
	
			# Write the /etc/mail/relay-domains file.
			SetRelayConfiguration(\%Variables, $FormData{'relay_domains'});
	
			#Set the postmaster alias.
			SetPostMasterAddress($FormData{'post_master_address'});

			# Get the current status.
			$Variables{'sendmail_status'} = (Safe::system("/etc/rc.d/init.d/sendmail status > /dev/null 2>&1")) ? 0 : 1;
	
			# Update sendmail if necessary...
			if ($Variables{'sendmail_status'}) 
			{
				Safe::system("/usr/bin/m4 /etc/mail/sendmail.mc > /etc/sendmail.cf; /etc/rc.d/init.d/sendmail restart > /dev/null 2>&1");
				
				# Give the service enough time to start.
				sleep(1);
			} 
			else
			{ 
				Safe::system("/usr/bin/m4 /etc/mail/sendmail.mc > /etc/sendmail.cf;");
			}
		}
		else
		{
			# Get the feedback message.
			$Feedback = $StringResource->_S($Result);
		}
	}
	else
	{
		# Get the data.
		($FormData{'smart_relay_host'}, $FormData{'unresolvable_domains'}, $FormData{'arrayref'}) = GetMailConfiguration(\%FormData);
		$FormData{'host_aliases'}	= GetHostNameConfiguration(\%FormData);
		$FormData{'relay_domains'}	= GetRelayConfiguration(\%FormData);
		($FormData{'reject_list'}, $FormData{'access_db'}) = GetAccessConfiguration(\%FormData);

		# Determine postmaster alias from /etc/aliases
		my $al = new Unix::AliasFile ("/etc/aliases", mode=>"r");
		my $Postmasters = [$al->alias('postmaster')];
		$FormData{'post_master_address'} = join(',', @{ $Postmasters });
	}
	
	# Get the status information.
	$ServiceStatus	= ($MailDaemon->GetStatus()) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";

	$html = <<HERE;
<form method=post action=server_mail.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
			<tr> 
				<td width=100%> 
					<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
					<tr>
						<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
                        <td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
						<td width=40% align=left bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'MAIL'}</b></td>
						<td width=15%>$ServiceStatus</td>
						<td width=40% align=left nowrap>
							<input type=submit class=clsButton value='$_R{'START'}' name=control_start>
							<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
							<input type=submit class=clsButton value='$_R{'RESTART'}' name=control_restart>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			</table>
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=4>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% colspan=4>
					<font color=black><b>$_R{'POSTMASTER_E-MAIL_ADDRESS'}</b></font>&nbsp;&nbsp;
					<input type=text name=post_master_address size=21 maxlength=35 value='$FormData{'post_master_address'}'>
				</td>
			</tr>
			<tr>
				<td width=100% colspan=4>
					<font color=black><b>$_R{'SMART_RELAY_HOSTNAME'}</b></font>&nbsp;&nbsp;
					<input type=text name=smart_relay_host size=26 maxlength=40 value='$FormData{'smart_relay_host'}'>
				</td>
			</tr>
			<tr>
				<td width=100% valign=top colspan=4>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% valign=top colspan=4>
					<input type=checkbox name=unresolvable_domains value=checked $FormData{'unresolvable_domains'}>&nbsp;<font color=black><b>$_R{'REJECT_UNRESOLVED_HOST_E-MAIL'}</b></font>
				</td>
			</tr>
			<tr>
				<td width=100% valign=top colspan=4>
					<input type=checkbox name=rbl value=checked $FormData{'rbl'}>&nbsp;<font color=black><b>$_R{'ENABLE_RBL'}</b></font>
				</td>
			</tr>
			<tr>
				<td width=100% colspan=4>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% colspan=4><font color=black><b>$_R{'E-MAIL_ALIASES'}</b></font></td>
			</tr>
			<tr>
				<td width=100% colspan=4><textarea name=host_aliases rows=2 cols=40 wrap=physical>$FormData{'host_aliases'}</textarea></td>
			</tr>
			<tr>
				<td width=100% colspan=4><font color=black><b>$_R{'E-MAIL_RELAY'}</b></font></td>
			</tr>
			<tr>
				<td width=100% colspan=4><textarea name=relay_domains rows=2 cols=40 wrap=physical>$FormData{'relay_domains'}</textarea></td>
			</tr>
			<tr>
				<td width=100% colspan=4><font color=black><b>$_R{'E-MAIL_REJECT'}</b></font></td>
			</tr>
			<tr>
				<td width=100% colspan=4><textarea name=reject_list rows=2 cols=40 wrap=physical>$FormData{'reject_list'}</textarea></td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=74%><b><font color=black>$_R{'MAX_MESSAGE_SIZE_INCOMING'}</font></b></td>
				<td width=19% align=right nowrap colspan=2><input type=text name=max_incoming_message_size size=3 maxlength=3 value='$FormData{'max_incoming_message_size'}'>$_R{'MB'}</td>
				<td width=7%></td>
			</tr>
			<tr>
				<td width=74%><b><font color=black>$_R{'MAX_MESSAGE_SIZE_OUTGOING'}</font></b></td>
				<td width=19% align=right nowrap colspan=2><input type=text name=max_outgoing_message_size size=3 maxlength=3 value='$FormData{'max_outgoing_message_size'}'>$_R{'MB'}</td>
				<td width=7%></td>
			</tr>
			<tr>
				<td width=100% colspan=4>&nbsp;</td>
			</tr>
			<tr>
				<td width=77% colspan=2 align=left></td>
				<td width=23% colspan=2 align=right><input type=submit class=clsButton value='$_R{'APPLY'}' name=apply></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<br>
	<a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>&nbsp;&nbsp;&nbsp;<a href=server_system_logs.cgi?logtype=mail&backto=mail>$_R{'MAIL_SERVICE_LOG'}</a>
</form>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_mail', 'id' => 1900, 'title' => $_R{'IDT:1900'},
								 'root-link' => 'server_mail', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
