#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_power.cgi                                                  #
#                                                                             #
# Description: CGI needed to control the power state of the system.           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;
use DisplayMain;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_power.lrc', 'common.lrc' ]);
	my $Configuration		= new ConfigurationData;
	my %_R					= ( 'IDT:1300'					=> '',
								'WARNING'			=> '',
								'REBOOT_SHUTDOWN_WARNING'	=> '',
								'REBOOT_CONFIRM'	=> '', 
								'SHUTDOWN_CONFIRM'	=> '');
	my $Action				= (defined($cgi->param('command')) ||
							   defined($cgi->param('command_reboot')) ||
							   defined($cgi->param('command_shutdown'))) ? 1 : 0;
	my $Feedback;
	my $Result;
	my $html				= '';
	my $ConfirmMessage		= '';

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# If there is an action...
	if (($Action) &&
		(!defined($cgi->param('action_decline'))))
	{
		my $Command				= (defined($cgi->param('command'))) ? $cgi->param('command') : ((defined($cgi->param('command_reboot'))) ? 'command_reboot' : 'command_shutdown');
		my $Confirmed			= (defined($cgi->param('action_confirm'))) ? 1 : 0;
		my $DisplayConfirmation	= 1;
		my %_RE					= ( 'YES'	=> '',
									'NO'	=> '' );
		
		# Get the string resources.
		$StringResource->GetStrings(\%_RE);
		
		# If the command is accepted...
		if (($Confirmed) &&
			(defined($cgi->param('command'))))
		{
			my $InitLevel = ('command_reboot' eq $Command) ? 6 : 0;
			
			# Try to shut down the system...
			unless ($Result = Safe::backtick("/sbin/init $InitLevel"))
			{
				my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_power.lrc' ]);

				# Log the event.
				$Log->Append('security', 'information', ('command_reboot' eq $Command) ? 130002 : 130003, [ $cgi->remote_user() ]);

				# Force a logout.
				print $cgi->redirect( -url => '/logout.cgi?message=down' );
				$DisplayConfirmation = 0;
			}
			else
			{
				# Get the appropriate error message.
				$Result		= ('command_reboot' eq $Command) ? 130000 : 130001;
				$Feedback	= $StringResource->_S($Result);
			}
		}
		
							   
		# If the confirmation message should be displayed...
		if ($DisplayConfirmation)
		{
			# Set the confirmation message.
		    $ConfirmMessage = ($Command eq 'command_reboot') ? 'REBOOT_CONFIRM' : 'SHUTDOWN_CONFIRM';
			
			$html = <<HERE;
	<form method=post action=server_power.cgi enctype="application/x-www-form-urlencoded">
		<input type=hidden name=command value=$Command>
		<table border=0 width=375 bgcolor=#eeeeee>
		<tr>
			<td>
				<table border=0 width=100%>
				<tr>
					<td width=17% valign=top><font color="red"><b>$_R{'WARNING'}</b></font></td>
					<td width=83%>$_R{$ConfirmMessage}</td>
				</tr>
				<tr>
					<td colspan=2 width=100%>&nbsp;</td>
				</tr>
				<tr>
					<td align=left width=10><input type=submit name=action_confirm value='$_RE{'YES'}' class=clsButton></td>
					<td align=right width=83%><input type=submit name=action_decline value='$_RE{'NO'}' class=clsButton></td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</form>
HERE
		}
	}
	else
	{
		my %_RE	= ( 'REBOOT'	=> '',
					'SHUTDOWN'	=> '' );

		# Get the string resources.
		$StringResource->GetStrings(\%_RE);
		
		$html = <<HERE;
<form method=post action=server_power.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=17% valign=top><font color="red"><b>$_R{'WARNING'}</b></font></td>
				<td width=83%>$_R{'REBOOT_SHUTDOWN_WARNING'}</td>
			</tr>
			<tr>
				<td colspan=2 width=100%>&nbsp;</td>
			</tr>
			<tr>
				<td align=left width=17%><input type=submit name=command_reboot value='$_RE{'REBOOT'}' class=clsButton></td>
				<td align=right width=83%><input type=submit name=command_shutdown value='$_RE{'SHUTDOWN'}' class=clsButton></td>
	    	</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	}

	# If there is a page to display...
	if ($html)
	{
		# Display the page.
		DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
									 'class' => 'server_power', 'id' => 1300, 'title' => $_R{'IDT:1300'},
									 'root-link' => 'server_power', 'result' => $Result, 'feedback' => $Feedback });
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
