#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: server_route.cgi                                                  #
#                                                                             #
# Description: CGI needed to display and modify network route configurations. #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;
use DisplayMain;
use RouteConfig;
use CGI;

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($Use, $FormDataRef)	= @_;
	my $Return = 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}
	
	# Set the key.
	$Key = ('modify' eq $Use) ? 'n_' : '';

	# Validate the data...
	if (($FormDataRef->{$Key . 'destination'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) || (255 < $1) || (255 < $2) || (255 < $3) || (255 < $4))
	{
		# Set the return code.
		$Return = 460006;
	}
	elsif (($FormDataRef->{$Key . 'gateway'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) || (255 < $1) || (255 < $2) || (255 < $3) || (255 < $4))
	{
		# Set the return code.
		$Return = 460007;
	}
	elsif (($FormDataRef->{$Key . 'netmask'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) || (255 < $1) || (255 < $2) || (255 < $3) || (255 < $4))
	{
		# Set the return code.
		$Return = 460008;
	}
	else
	{
		# Purify the values.
		$FormDataRef->{$Key . 'destination'} = sprintf("%d.%d.%d.%d", split(/\./, $FormDataRef->{$Key . 'destination'}));
		$FormDataRef->{$Key . 'gateway'}	 = sprintf("%d.%d.%d.%d", split(/\./, $FormDataRef->{$Key . 'gateway'}));
		$FormDataRef->{$Key . 'netmask'}	 = sprintf("%d.%d.%d.%d", split(/\./, $FormDataRef->{$Key . 'netmask'}));
	}
	
	return $Return;
}

###############################################################################
# AddRoute
###############################################################################

sub AddRoute
{
	my ($User, $FormDataRef)	= @_;
	my $Return = 0;
	
	# Add the route.
	if (0 == RouteConfig->configRoute($FormDataRef->{'destination'}, $FormDataRef->{'gateway'}, $FormDataRef->{'netmask'}, $FormDataRef->{'nic'}, ''))
	{
		my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_network.lrc' ]);
		
		# Log the event.
		$Log->Append('security', 'information', 460004, [ $User, $FormDataRef->{'destination'}, $FormDataRef->{'gateway'}, $FormDataRef->{'netmask'}, $FormDataRef->{'nic'} ]);
	}
	else
	{
		$Return = 460002;
	}

	return $Return;
}

###############################################################################
# DeleteRoute
###############################################################################

sub DeleteRoute
{
	my ($User, $FormDataRef)	= @_;
	my $Return = 0;
	
	# Delete the route.
	if (0 == RouteConfig->deleteRoute($FormDataRef->{'destination'}, $FormDataRef->{'gateway'}, $FormDataRef->{'netmask'}, $FormDataRef->{'nic'}, ''))
	{
		my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_network.lrc' ]);
		
		# Log the event.
		$Log->Append('security', 'information', 460005, [ $User, $FormDataRef->{'destination'}, $FormDataRef->{'gateway'}, $FormDataRef->{'netmask'}, $FormDataRef->{'nic'} ]);
	}
	else
	{
		$Return = 460000;
	}

	return $Return;
}

###############################################################################
# ModifyRoute
###############################################################################

sub ModifyRoute
{
	my ($User, $FormDataRef)	= @_;
	my $Return = 0;
	
	# If the configuration is different...
	unless (($FormDataRef->{'n_nic'}         eq $FormDataRef->{'o_nic'}) &&
			($FormDataRef->{'n_gateway'}     eq $FormDataRef->{'o_gateway'}) && 
			($FormDataRef->{'n_netmask'}     eq $FormDataRef->{'o_netmask'}) &&
			($FormDataRef->{'n_destination'} eq $FormDataRef->{'o_destination'}))
	{
		# Try to delete the old route and create the new one...
		if ((0 == RouteConfig->deleteRoute($FormDataRef->{'o_destination'}, $FormDataRef->{'o_gateway'}, $FormDataRef->{'o_netmask'}, $FormDataRef->{'o_nic'}, '')) &&
			(0 == RouteConfig->configRoute($FormDataRef->{'n_destination'}, $FormDataRef->{'n_gateway'}, $FormDataRef->{'n_netmask'}, $FormDataRef->{'n_nic'}, 'yes')))
		{
			my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_network.lrc' ]);

			# Log the event.
			$Log->Append('security', 'information', 460003, [ $User, 
															  $FormDataRef->{'o_destination'}, $FormDataRef->{'o_gateway'}, $FormDataRef->{'o_netmask'}, $FormDataRef->{'o_nic'},
															  $FormDataRef->{'n_destination'}, $FormDataRef->{'n_gateway'}, $FormDataRef->{'n_netmask'}, $FormDataRef->{'n_nic'} ]);
		}
		else
		{
			$Return = 460001;
		}
	}
	
	return $Return;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef) = @_;
	my $Return;

	# Check for a button press...
	if (defined($cgi->param('route_add')))
	{
		# Redirect to the route add.
		$Return = 'route_add';
	}
	else
	{
		my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_network.lrc', 'common.lrc' ]);
		my %_R					= ( 'IDT:4500'				=> '',
									'IDT:4600'				=> '',
									'DEVICE'				=> '',
									'ROUTES'				=> '',
									'DESTINATION'			=> '',
									'GATEWAY'				=> '',
									'NETMASK'				=> '',
									'ADAPTER'				=> '',
									'ADD'					=> '',
									'DELETE'				=> '',
									'CONFIRM_ROUTE_DELETE'	=> '',
									'NO_ROUTES_SELECTED'	=> '',
									'APPLY'					=> '' );
		my %FormData;
		my @RouteList			= RouteConfig->listRoutes();
		my $RouteHTML;
		my $Route;
		my $Feedback;
		my $Result;
		my $html;
	
		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# If taking an action...
		if (defined($cgi->param('route_delete')))
		{
			# Get the form data...
			($FormData{'destination'}, $FormData{'gateway'}, $FormData{'netmask'}, $FormData{'nic'}) = split(/\,/, $cgi->param('route_selection'));
			
			# Try to delete the specified route.
			$Result = DeleteRoute($cgi->remote_user(), \%FormData);
			
			# Get the new list.
			@RouteList = RouteConfig->listRoutes();

            # If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		elsif (defined($cgi->param('apply')))
		{
			# If there is a selection...
			if ($cgi->param('route_selection'))
			{
				# Get the form data...
				($FormData{'o_destination'}, $FormData{'o_gateway'}, $FormData{'o_netmask'}, $FormData{'o_nic'}, $FormData{'o_name'}) = split(/\,/, $cgi->param('route_selection'));

				# Get the new data.
				$FormData{'n_destination'}	= $cgi->param($FormData{'o_name'} . '_dest');
				$FormData{'n_gateway'}		= $cgi->param($FormData{'o_name'} . '_gw');
				$FormData{'n_netmask'}		= $cgi->param($FormData{'o_name'} . '_nmask');
				$FormData{'n_nic'}			= $FormData{'o_nic'};

				# Validate the form data.
				if (0 == ($Result = ValidateFormData('modify', \%FormData)))
				{
					# Modify the route.
					$Result = ModifyRoute($cgi->remote_user(), \%FormData);
				}
			}
			else
			{
				$Result = 460009;
			}
			
			# Get the new list.
			@RouteList = RouteConfig->listRoutes();

			# If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		
		# If there are any routes...
		if (scalar(@RouteList))
		{
			my $Adapter	= $StringResource->_S('ADAPTER');
	
			# Run through the route list...
			foreach $Route (@RouteList)
			{
				# Exclude the default gateway...
				if ('0.0.0.0' ne $Route->{'destination'})
				{
					my $Name		= "$Route->{'nic'}:$Route->{'destination'}";
					my $RouteData	= "$Route->{'destination'},$Route->{'gateway'},$Route->{'netmask'},$Route->{'nic'},$Name";
					my $Device		= $Adapter . (('eth0' eq $Route->{'nic'}) ? '1' : '2');
					my $Destination	= $Name . '_dest';
					my $Gateway		= $Name . '_gw';
					my $Netmask		= $Name . '_nmask';

					$RouteHTML .= <<HERE;		
		<tr bgcolor=white>
			<td width=120><input type=radio name=route_selection value=$RouteData>&nbsp;$Device</td>
			<td><input type=text size=14 maxlength=15 name=$Destination value='$Route->{'destination'}'></td>
			<td><input type=text size=14 maxlength=15 name=$Gateway value='$Route->{'gateway'}'></td>
			<td><input type=text size=14 maxlength=15 name=$Netmask value='$Route->{'netmask'}'></td>
		</tr>
HERE
				}
			}
		}
		
		# If there are no routes...
		unless ($RouteHTML)
		{
			$RouteHTML .= '<tr align=left valign=center><td colspan=4><b>' . $StringResource->_S('NO_ROUTES_AVAILABLE') . '</b></td></tr>'
		}
	
		$html = <<HERE;
<script>
function confirmDeleteRoute()
{
	var bReturn = false;
	
	for(i = 1; i < document.server_route.route_selection.length; i++)
	{
		if (document.server_route.route_selection[i].checked)
		{
			return confirm('$_R{'CONFIRM_ROUTE_DELETE'}');
		}
	}
	
	if (i == document.server_route.route_selection.length)
	{
		alert('$_R{'NO_ROUTES_SELECTED'}');
	}
	
	return bReturn;
}
</script>
<form method=post name=server_route action=server_route.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 bgcolor=#eeeeee width=375>
	<tr>
		<td>
			<table border=0>
			<tr align=left valign=center>
				<td><b>$_R{'ROUTES'}</b></td>
			</tr>
			<tr>
				<td align=left nowrap colspan=2>
					<input type=submit name=route_add value='$_R{'ADD'}' class=clsButton>
					<input type=submit name=route_delete value='$_R{'DELETE'}' class=clsButton onclick="return confirmDeleteRoute();">
				</td>
			</tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>&nbsp;</td>
	</tr>
	<tr>
		<td>
			<table bordercolor=#eeeeee border=1 cellpadding=2 bgcolor=#ffffff cellspacing=0>
			<tr bgcolor=#808080>
				<td height=35><font color=white><b>$_R{'DEVICE'}</b></font></td>
				<td height=35><font color=white><b>$_R{'DESTINATION'}</b></font></td>
				<td height=35><font color=white><b>$_R{'GATEWAY'}</b></font></td>
				<td height=35><font color=white><b>$_R{'NETMASK'}</b></font></td>
			</tr>
$RouteHTML
			</table>
		</td>
	</tr> 
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td align=right><input type=submit name=apply value='$_R{'APPLY'}' class=clsButton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<input type=hidden name=type value=default>
</form>
HERE

		# Overriding result...
		$Result = $DisplayDataRef->{'result'} unless(defined($Result));

		# Fill out the display information.
		$DisplayDataRef->{'q'}				= $cgi;
		$DisplayDataRef->{'html'}			= $html;
		$DisplayDataRef->{'class'}			= 'server_network';
		$DisplayDataRef->{'id'}				= 4600;
		$DisplayDataRef->{'title'}			= $_R{'IDT:4600'};
		$DisplayDataRef->{'root-link'}		= 'server_network';
		$DisplayDataRef->{'result'}			= $Result;
		$DisplayDataRef->{'feedback'}		= $Feedback;
		$DisplayDataRef->{'cookiecrumbs'}	= [ { 'name' => $_R{'IDT:4500'}, 'location' => 'server_network.cgi' } ];

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# RenderRouteAdd
###############################################################################

sub RenderRouteAdd
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_network.lrc', 'common.lrc' ]);
	my %FormData			= (	'nic'			=> 'eth0',
								'destination'  	=> '',
								'gateway'   	=> '',
								'netmask'		=> '' );
	my $Key;
	my $Feedback;
	my $Result;
	my $html;
	my $Return;

	# If adding a route...
	if (defined($cgi->param('add')))
	{
		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
		}

		# Validate the form data.
		if (0 == ($Result = ValidateFormData('add', \%FormData)))
		{
			# Try to add the route...
			if (0 == ($Result = AddRoute($cgi->remote_user(), \%FormData)))
			{
				# Set the result.
				$DisplayDataRef->{'result'}	= $Result;
				
				# Move on to the main page.
				$Return = 'default';
			}
		}

		# If there was an error...
		if ($Result)
		{
			# Get the error string.
			$Feedback = $StringResource->_S($Result);
		}
	}

    # If a destination is not set...
	unless($Return)
	{
		my %_R				= ( 'IDT:4500'		=> '',
								'IDT:4600'		=> '',
								'IDT:4700'		=> '',
								'INTERFACE'		=> '',
								'DESTINATION'	=> '',
								'GATEWAY'		=> '',
								'NETMASK'		=> '',
								'ADD'			=> '');
		my $Adapter			= $StringResource->_S('ADAPTER');
		my @Adapter			= ( $Adapter . '1', $Adapter . '2' );
		my $AdapterHTML		= '';

		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# Prime the adapter selection.
		$AdapterHTML .= "\t\t\t\t\t<option " . (('eth0' eq $FormData{'nic'}) ? 'selected ' : '') . "value=eth0>$Adapter[0]\n";
		$AdapterHTML .= "\t\t\t\t\t<option " . (('eth1' eq $FormData{'nic'}) ? 'selected ' : '') . "value=eth1>$Adapter[1]\n";

		$html = <<HERE;
<form method=post action=server_route.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=route_add>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>&nbsp;
			<table bordercolor=#eeeeee border=1 cellpadding=2 bgcolor=#ffffff cellspacing=0>
			<tr bgcolor=#808080>
				<td height=35><font color=white><b>$_R{'INTERFACE'}</b></font></td>
				<td height=35><font color=white><b>$_R{'DESTINATION'}</b></font></td>
				<td height=35><font color=white><b>$_R{'GATEWAY'}</b></font></td>
				<td height=35><font color=white><b>$_R{'NETMASK'}</b></font></td>
			</tr>
			<tr bgcolor=white>
				<td>
					<select name=nic>
$AdapterHTML
					</select>
				</td>
				<td width=11%><input type=test size=14 maxlength=15 name=destination value='$FormData{destination}'></td>
				<td width=22%><input type=test size=14 maxlength=15 name=gateway value='$FormData{gateway}'></td>
				<td width=22%><input type=test size=14 maxlength=15 name=netmask value='$FormData{netmask}'></td>
			</tr>
			</table>
		</td>
	</tr> 
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td align=right><input type=submit name=add value='$_R{'ADD'}' class=clsButton>&nbsp;<input type=reset class=clsButton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
		
		# Fill out the display information.
		$DisplayDataRef->{'q'}				= $cgi;
		$DisplayDataRef->{'html'}			= $html;
		$DisplayDataRef->{'class'}			= 'server_network';
		$DisplayDataRef->{'id'}				= 4700;
		$DisplayDataRef->{'title'}			= $_R{'IDT:4700'};
		$DisplayDataRef->{'root-link'}		= 'server_network';
		$DisplayDataRef->{'result'}			= $Result;
		$DisplayDataRef->{'feedback'}		= $Feedback;
		$DisplayDataRef->{'cookiecrumbs'}	= [ { 'name' => $_R{'IDT:4500'}, 'location' => 'server_network.cgi' }, { 'name' => $_R{'IDT:4600'}, 'location' => 'server_route.cgi' } ];

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'   eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);				last SWITCH; }
		if ('route_add' eq $FormType) { $FormType = RenderRouteAdd($cgi, \%DisplayData, $Redirect);	last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	else
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
