#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_system_logs.cgi                                            #
#                                                                             #
# Description: CGI needed to display logged system information.               #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;
use ConfigHandler;
use LanguageTime;
use DisplayMain;
use LogParser;
use Safe;
use CGI;

###############################################################################
# GetSELContents
###############################################################################

sub GetSELContents
{
	my ($Line, $Type, $Length, $Date, $Time, $Details);
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_system_logs.lrc' ]);
	my @SELContents;
	my @SELRawLines;
	my $Description;
	my $Month;
	my $Day;
	my $Year;
	my $i = 0;

	# Get the SEL event log contents.
	@SELRawLines = split(/^/m, Safe::backtick('/usr/ins/intel/bin/asm/bakAsm get_eventlog'));
	
	# Loop though the output...
	foreach $Line (@SELRawLines)
	{
		# Parse the line.
		($Type, $Length, $Date, $Time, $Details) = split ' ', $Line;

		# Get the actual description.
		$Description = GetSELEntryDescription($Type, $Details);

		# Check to see if the description should be skipped.
		next if (($Description eq "ignore") or ($StringResource->_S("ASM_SEL_DESCRIPTION_$Type") eq ''));

		# Get the month and day.
		($Month, $Day, $Year) = split(/\//, $Date);

		# Add the entry to the array.
		$SELContents[$i] = $StringResource->_S("MONTH_$Month") . " " . $Day . " " . $Time . "~"  . $StringResource->_S("ASM_SEL_DESCRIPTION_$Type") . (($Description ne '') ? " ($Description)" : " ");
		
		# Move to the next element.
		++$i;
	}

	return @SELContents;
}

###############################################################################
# GetSELEntryDescription
###############################################################################

sub GetSELEntryDescription
{
	my ($AlertType, $Details)	= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_system_logs.lrc' ]);
	my $Description				= '';
	my $ThresholdType;
	my $ThresholdID;

	# If this is a POST software...
	if (8 == $AlertType)
	{
		my $dword = hex(substr($Details, 0, 8));
		my $secondDword = 0;
		my $i;

		for ($i = 0; $i < 32; $i++)
		{
			if ($dword & (1 << $i))
			{
				if (24 == $i)
				{
					# Second dword of options to check.
					$secondDword = 1;
				}
				else
				{
					$Description .= (($Description ne '') ? ", " : '') . $StringResource->_S("ASM_SEL_POST_ERRORS_DWORD1_$i");
			 	}
			}
		}
	  	
		if ($secondDword)
		{
			$dword = hex(substr($Details, 8));

			for ($i = 0; $i < 32; $i++)
			{
				if ($dword & (1 << $i))
				{
					$Description .= (($Description ne '') ? ", " : '') . $StringResource->_S("ASM_SEL_POST_ERRORS_DWORD2_$i");
			 	}
			}
		}
	}

	# If a threashold has passed...
	if (10 == $AlertType)
	{
		if ('ffff' eq substr($Details, 0, 4))
		{
			# A non-event (CPU/Mem utilization) that we ignore.
			return "ignore";
		}
		
		$ThresholdType	= substr($Details, 0, 2);
		$ThresholdID	= substr($Details, 6, 2);
		$Description	.= $StringResource->_S("ASM_SEL_THRESHOLD_ID_$ThresholdID") . " " . $StringResource->_S("ASM_SEL_THRESHOLD_TYPE_$ThresholdType");
	}

	# If this is disk mirroring...
	if ('ae' eq $AlertType)
	{
		$Description .= $StringResource->_S('ASM_SEL_RAID_0');
	}

	# If a predictive failure threashold has passed...
	if (82 == $AlertType)
	{
		if ('ffff' eq substr($Details, 0, 4))
		{
			# A non-event (CPU/Mem utilization) that we ignore..
			return "ignore";
		}
		
		$ThresholdType	= substr($Details, 0, 2);
		$ThresholdID	= substr($Details, 6, 2);
		$Description	.= $StringResource->_S("ASM_SEL_PFTHRESHOLD_ID_$ThresholdID") . " " . $StringResource->_S("ASM_SEL_THRESHOLD_TYPE_$ThresholdType");
	}

	return $Description;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi					= shift;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_system_logs.lrc' ]);
	my $TimeResource		= new LanguageTime($PreferredLanguage, '/usr/ins/intel/gui/locale');
	my %_R					= ( 'IDT:1200'			=> '',
								'LAST_BOOT'			=> '',
								'SERVICE_LOGS'		=> '',
								'SERVER_LOGS'		=> '',
								'WEB'				=> '',
								'MAIL'				=> '',
								'SECURITY'			=> '',
								'SYSTEM'			=> '',
								'SOFTWARE_ALERTS'	=> '',
								'HARDWARE_ALERTS'	=> '',
								'REPORTED'			=> '',
								'DESCRIPTION'		=> '' );
	my %DisplayParameters;
	my @LogData;
	my $LogType				= $cgi->param('logtype');
	my $EntryDisplayLimit	= ConfigHandler::getConfig(ConfigHandler::getPath('CONSTANTS'), 'MAX_LOGS_TO_DISPLAY');
    my $LastBootTime;
	my $html1				= '';
	my $html2				= '';
	my $html3				= '';
	my $html4				= '';
	my $html5				= '';
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe default.
	$LogType = 'system' unless(defined($LogType));

	# Get the last boot time.
	$LastBootTime = Safe::backtick('/usr/bin/procinfo');
	$LastBootTime = substr($LastBootTime, index($LastBootTime, 'Bootup:') + 8, 27); # TODO - deal with the internationalization

	# Set up the top of the page...
	if (('web'  eq $LogType) ||
		('mail' eq $LogType))
	{
		$html1 = <<HERE;
		<table border=1 cellpadding=0 cellSpacing=0 width=375>
		<tr>
HERE
	}
	else
	{
		$html1 = <<HERE;
		<b>$_R{'LAST_BOOT'}</b>&nbsp;$LastBootTime<br><br>
		<b>$_R{'SERVICE_LOGS'}</b><br>&nbsp;
		<b><a href=server_system_logs.cgi?logtype=web>$_R{'WEB'}</a></b><br>&nbsp;
		<b><a href=server_system_logs.cgi?logtype=mail>$_R{'MAIL'}</a></b><br><br>
		<b>$_R{'SERVER_LOGS'}</b>
		<table border=1 cellpadding=3 cellspacing=0 bordercolor=#eeeeee width=375>
		<tr>
HERE
	}

	# If necessary, set up the title bar...
	if ('security' eq $LogType)
	{
$html2 = <<HERE;
			<td nowrap width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=system><b><font color=#ffffff>$_R{'SYSTEM'}</font></b></a></td>
			<td nowrap width=90 bgcolor=#ffffff><font color=#000000>$_R{'SECURITY'}</font></td>
			<td width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=error><b><font color=#ffffff>$_R{'SOFTWARE_ALERTS'}</font></b></a></td>
			<td width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=hardware><b><font color=#ffffff>$_R{'HARDWARE_ALERTS'}</font></b></a></td>
HERE
	}
	elsif ('system' eq $LogType)
	{
$html2 = <<HERE;
			<td nowrap width=90 bgcolor=#ffffff><font color=#000000>$_R{'SYSTEM'}</font></td>
			<td nowrap width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=security><b><font color=#ffffff>$_R{'SECURITY'}</font></b></a></td>
			<td width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=error><b><font color=#ffffff>$_R{'SOFTWARE_ALERTS'}</font></b></a></td>
			<td width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=hardware><b><font color=#ffffff>$_R{'HARDWARE_ALERTS'}</font></b></a></td>
HERE
	}
	elsif('error' eq $LogType)
	{
$html2 = <<HERE;
			<td nowrap width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=system><b><font color=#ffffff>$_R{'SYSTEM'}</font></b></a></td>
			<td nowrap width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=security><b><font color=#ffffff>$_R{'SECURITY'}</font></b></a></td>
			<td width=90 bgcolor=#FFFFFF><font color=#000000>$_R{'SOFTWARE_ALERTS'}</font></td>
			<td width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=hardware><b><font color=#ffffff>$_R{'HARDWARE_ALERTS'}</font></b></a></td>
HERE
	}
	elsif('hardware' eq $LogType)
	{
$html2 = <<HERE;
			<td nowrap width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=system><b><font color=#ffffff>$_R{'SYSTEM'}</font></b></a></td>
			<td nowrap width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=security><b><font color=#ffffff>$_R{'SECURITY'}</font></b></a></td>
			<td width=90 bgcolor=#808080><a href=server_system_logs.cgi?logtype=error><b><font color=#ffffff>$_R{'SOFTWARE_ALERTS'}</font></b></a></td>
			<td width=90 bgcolor=#FFFFFF><font color=#000000>$_R{'HARDWARE_ALERTS'}</font></td>
HERE
	}
	elsif(('web'  eq $LogType) || 
		  ('mail' eq $LogType))
	{
		$html2 = '';
	}

	# The middle of the page.
	$html3 = <<HERE;
		</tr>
		</table>
		<table border=1 cellpadding=4 cellspacing=0 bordercolor=#ffffff width=100%>
		<tr>
			<td width=16 bgcolor=#808080>&nbsp;</td>
			<td nowrap width=105 bgcolor=#808080><b><font color=#ffffff>$_R{'REPORTED'}</font></b></td>
			<td width=254 bgcolor=#808080><b><font color="#ffffff">$_R{'DESCRIPTION'}</font></b></td>
		</tr>
HERE

	# Get the log data requested...
	if (('web'    eq $LogType) || 
		('mail'   eq $LogType) || 
		('system' eq $LogType))
	{ 
		# Get the log data from the service logs...
		@LogData = LogParser::parseServiceLogs($LogType, 'en');  # TODO - deal with the internationalization
		
		# Resort the list.
		@LogData = reverse(@LogData);
	}
	elsif ('hardware' eq $LogType)
	{ 
		# Get the log data from the SEL (System Event Log).
		@LogData = GetSELContents(); 
		# Resort the list.
		@LogData = reverse(@LogData);
	}
	else
	{ 
		my $Log			= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log');
		my $Enumerator;
		
		# Set the preferred language.
		$Log->SetPreferredLanguage($PreferredLanguage);

		# Try and get the enumerator.
		if ($Enumerator = $Log->enumeratorElements(400))
		{
			my $Element;
			my $Message;
			my $i		= 0;
			
			# Run through the elements...
			while ($Enumerator->hasMoreElements())
			{
				# If there is an element and it's type matches...
				if (($Element = $Enumerator->nextElement()) &&
					($LogType eq $Element->GetCategory()) &&
					($Message = $Element->GetMessage()))
				{
					# If the log type is 'error'...
					if ('error' eq $LogType)
					{
						# Merge the data into the common stream.
						$LogData[$i] = join('~', $Element->GetType(), $TimeResource->FormatTime($StringResource->_S('TIME_FORMAT'), localtime($Element->GetTimeStamp())), $Message);
					}
					else
					{
						# Merge the data into the common stream.
						$LogData[$i] = join('~', $TimeResource->FormatTime($StringResource->_S('TIME_FORMAT'), localtime($Element->GetTimeStamp())), $Message);
					}
					
					# Move on.
					++$i;
				}
			}
		}
	}

	# If there is no data...
	unless (@LogData)
	{
		# Create a filler entry.
		$LogData[0] = join('~', $TimeResource->FormatTime($StringResource->_S('TIME_FORMAT'), localtime()), $StringResource->_S('NO_ENTRIES_FOUND'));
	}
	
	# If there is log data...
	if (@LogData)
	{
		my $Date;
		my $Message;
		my $Severity;
		my $bgcolor		= '#eeeeee';
		
		# Realign the entry display limit.
		$EntryDisplayLimit = (@LogData > $EntryDisplayLimit) ? $EntryDisplayLimit : @LogData;

		# Run though the data...
		for (my $i = 0; $i < $EntryDisplayLimit; ++$i)
		{
			# If the log does not have health association...
			unless ('error' eq $LogType)
			{
				# Get the date and message.
				($Date, $Message) = split(/~/, $LogData[$i], 2);

				# No severity.
				$Severity = '/images/blank.gif';
			}
			else
			{
				# Get the date and message.
				($Severity, $Date, $Message) = split(/~/, $LogData[$i], 3);

				# If there was no message...
				if (!$Message)
				{
					# Re-split the data.
					($Date, $Message) = split(/~/, $LogData[$i], 2);
				}

				# Get the severity image.
				$Severity = ('critical' eq lc($Severity)) ? '/images/critical.gif' : (('warning' eq lc($Severity)) ? '/images/warning.gif' : '/images/blank.gif');
			}

			# Build the table entry.
			$html4 .= <<HERE;
		<tr>
			<td width=16 bgcolor=$bgcolor><img align=center border=0 height=16 src=$Severity></td>
			<td nowrap width=105 bgcolor=$bgcolor>$Date</td>
			<td width=254 bgcolor=$bgcolor>$Message</td>
		</tr>
HERE

			# Toggle the background color.
			$bgcolor = ($bgcolor eq '#eeeeee') ? 'white' : '#eeeeee';
		}
	}

	# Finish off the table.
	$html5 = '</table>';

	# Set up the baseline display parameters.
	$DisplayParameters{'q'}			= $cgi;
	$DisplayParameters{'html'}		= $html1 . $html2 . $html3 . $html4 . $html5;
	$DisplayParameters{'bgcolor'}	= 'white';
	$DisplayParameters{'class'}		= 'server_system_logs';
	$DisplayParameters{'id'}		= 1200;
	
	# If there are any sublogs to display...
	if ('mail' eq $LogType)
	{
		# Set up the title and help.
		$DisplayParameters{'title'}			= $StringResource->_S('IDT:1200:mail');
		$DisplayParameters{'helpIndicator'} = $LogType;

		# Set up the cookie crumbs...
		if ('mail' eq $cgi->param('backto'))
		{
			$DisplayParameters{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('MAIL'), 'location' => 'server_mail.cgi' } ];
			$DisplayParameters{'root-link'}		= 'server_mail';
		}
		else
		{
			$DisplayParameters{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('IDT:1200'), 'location' => 'server_system_logs.cgi' } ];
			$DisplayParameters{'root-link'}		= 'server_system_logs';
		}
	}
	elsif ('web' eq $LogType)
	{
		# Set up the title and help.
		$DisplayParameters{'title'}			= $StringResource->_S('IDT:1200:web');
		$DisplayParameters{'helpIndicator'} = $LogType;
		
		# Set up the cookie crumbs...
		if ('web' eq $cgi->param('backto'))
		{
			$DisplayParameters{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('WEB'), 'location' => 'server_http.cgi' } ];
			$DisplayParameters{'root-link'}		= 'server_http';
		}
		else
		{
			$DisplayParameters{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('IDT:1200'), 'location' => 'server_system_logs.cgi' } ];
			$DisplayParameters{'root-link'}		= 'server_system_logs';
		}
	}
	else
	{
		# Just set up the title.
		$DisplayParameters{'title'}		= $StringResource->_S('IDT:1200');
		$DisplayParameters{'root-link'} = 'server_system_logs';
	}

	# Display the page.
    DisplayMain->printDocument(\%DisplayParameters);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
