#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_telnet.cgi                                                 #
#                                                                             #
# Description: CGI needed to display and modify the telnet and OpenSSH        #
#              settings.                                                      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Daemon;
use CGI;

###############################################################################
# GetTelnetStatus
###############################################################################

sub GetTelnetStatus
{
	my $Return = 0;
	my $Inetd	= new Daemon('inetd', '');
	
	# Get the status of inetd daemon...
	if ($Inetd->GetStatus())
	{
		# Try to open the configuration file...
		if (open(INETDCONF, '< /etc/inetd.conf'))
		{
			# Loop through the file...
			while(<INETDCONF>) 
			{
				# If a line matching telnet appears...
				if (/^telnet/) 
				{
					# Telnet is configured to run.
					$Return = 1;
					last;
				}
			}
					
			# Close the file.	
			close(INETDCONF);
		}
	}

	return $Return;
}

###############################################################################
# ControlTelnet
###############################################################################

sub ControlTelnet
{
	my $Control = shift;
	my $Return	= 0;
	
	# Try to open the configuration file...
	if (open(INETDCONF, '< /etc/inetd.conf'))
	{
		my @ConfigurationData = <INETDCONF>;

		# Close the file.
		close(INETDCONF);

		# Try to open the configuration file...
		if (open(INETDCONF, '> /etc/inetd.conf'))
		{
			my $INETDaemon = new Daemon('inet');
			my $Line;

			# Loop through the file...
			foreach $Line (@ConfigurationData) 
			{
				# Based on the control...
				if ((0 == $Control) &&
					($Line =~ /^telnet/))
				{ 
					# Add on the comment marker.
					print INETDCONF "#$Line";
				}
				elsif ((1 == $Control) &&
					   ($Line =~ /(\#+)(telnet.*)$/))
				{ 
					# Take off the comment marker.
					print INETDCONF "$2\n"; 
				}
				else 
				{ 
					# Just an ordinary line.
					print INETDCONF $Line; 
				}
			}
					
			# Close the file.	
			close(INETDCONF);

			# Restart the daemon.
			$INETDaemon->Control(2);
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_telnet.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:1700'				=> '',
								'TELNET'				=> '',
								'OPENSSH'				=> '',
								'SERVICE'				=> '',
								'STATUS'				=> '',
								'CONTROLS'				=> '',
								'STOP'					=> '',
								'START'					=> '',
								'CONFIRM_STOP_TELNET'	=> '',
								'CONFIRM_STOP_OPENSSH'	=> '',
								'RUNNING'				=> '',
								'STOPPED'				=> '' );
	my $SSHDaemon			= new Daemon('sshd');
	my %ServiceStatus;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	SWITCH:
	{
		# Control the services.
		if (defined($cgi->param('control_start_telnet')))	{ ControlTelnet(1); last SWITCH; }
		if (defined($cgi->param('control_stop_telnet')))	{ ControlTelnet(0); last SWITCH; }
		if (defined($cgi->param('control_start_openssh')))	{ $SSHDaemon->Control(1); last SWITCH; }
		if (defined($cgi->param('control_stop_openssh')))	{ $SSHDaemon->Control(0); last SWITCH; }
	}

	# Get the status information.
	$ServiceStatus{'telnet'}	= (GetTelnetStatus()) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";
	$ServiceStatus{'openssh'}	= ($SSHDaemon->GetStatus()) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";
	
    my $html = <<HERE;
<form action=server_telnet.cgi method=post enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
			<tr> 
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td> 
				<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td> 
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td> 
			</tr> 
			<tr> 
				<td width=40%><b>$_R{'TELNET'}</b></td>
				<td width=15%><font color=#008000>$ServiceStatus{'telnet'}</font></td> 
				<td width=40% align=left nowrap>
					<input type=submit class=clsButton value='$_R{'START'}' name=control_start_telnet> 
					<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop_telnet onclick="javascript: return confirm('$_R{'CONFIRM_STOP_TELNET'}');">
				</td> 
			</tr> 
			</table> 
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			</table>
			<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
			<tr> 
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td> 
				<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td> 
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td> 
			</tr> 
			<tr> 
				<td width=40%><b>$_R{'OPENSSH'}</b></td> 
				<td width=15%><font color=#008000>$ServiceStatus{'openssh'}</font></td> 
				<td width=40% align=left nowrap>
					<input type=submit class=clsButton value='$_R{'START'}' name=control_start_openssh> 
					<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop_openssh onclick="javascript: return confirm('$_R{'CONFIRM_STOP_OPENSSH'}');">
				</td>
			</tr>
			</table> 
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			</table>
		</td> 
	</tr> 
	</table> 
</form>
HERE
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_telnet', 'id' => 1700, 'title' => $_R{'IDT:1700'},
								 'root-link' => 'server_telnet' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
