#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: server_title.cgi                                                  #
#                                                                             #
# Description: Contains CGI script needed to render the server administrator  #
#              title frame.                                                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
    my $html;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $ImageResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/images');
	my $ImgPathTopbarLeft	= $ImageResource->ResolveResourceFile('topbar_left-server.jpg');
	my $ImgPathOnlineGuide	= $ImageResource->ResolveResourceFile('onlineguide-server.jpg');
	my $ImgPathLogOut		= $ImageResource->ResolveResourceFile('logout-server.jpg');
	my $Domain				= Safe::backtick('/bin/hostname --fqdn');
	my $User				= $cgi->remote_user();
	my $BodyTag;
	my %OffsetDomain;
	my %OffsetUser;
	my $GOLabel				= '';

	# Safe defaults.
	$ImgPathTopbarLeft	= '' unless(defined($ImgPathTopbarLeft));
	$ImgPathOnlineGuide	= '' unless(defined($ImgPathOnlineGuide));
	$ImgPathLogOut		= '' unless(defined($ImgPathLogOut));

	# Get just the hidden part of the file name.
	$ImgPathTopbarLeft	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
	$ImgPathOnlineGuide	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
	$ImgPathLogOut		=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;

	# Based on the user agent...
	if (-1 < (index($cgi->user_agent(), 'MSIE')))
	{ 
		# Use a MSIE compatible parameters.
		$BodyTag 				= "<body bgcolor=#ffffff background=/images/topbar_bkgd_mid.jpg leftmargin=0 topmargin=0>"; 
		$OffsetDomain{'top'}	= '25px';
		$OffsetDomain{'left'}	= '69px';
		$OffsetUser{'top'}		= '39px';
		$OffsetUser{'left'}		= '70px';
	}
	else
	{ 
		# Use a Netscape compatible parameters.
		$BodyTag				= "<body bgcolor=#ffffff onResize=location.reload() background=/images/topbar_bkgd_mid_netscape.jpg leftmargin=0 topmargin=0>";
		$OffsetDomain{'top'}	= '30px';
		$OffsetDomain{'left'}	= '70px';
		$OffsetUser{'top'}		= '45px';
		$OffsetUser{'left'}		= '70px';
	}
	
	# Check for the group operations label...
	if (Safe::backtick('/sbin/chkconfig --list wcdd') =~ /3:on/)
	{  
		# Get the image path.
		my $ImgPathGO	= $ImageResource->ResolveResourceFile('groupoperations-server.jpg');
		$ImgPathGO		=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
		
		# Turn on the label.
		$GOLabel = "<div id=layer1 style='left: 505px; position: absolute; top: 2px'><a href=/cgi-bin/ema/go_index.cgi target=_blank><img border=0 src=$ImgPathGO width=110 height=17></a></div>";
	}

	$html = <<HERE;
<!doctype html public -//ietf//dtd html//en>
<html>
	<head>
		<link rel=stylesheet href=/style/style.css type=text/css>
	</head>
$BodyTag         
	<table cellspacing=0 border=0 cellpadding=0 width=101%>
	<tr>
		<td width=399>
			<img src=$ImgPathTopbarLeft border=0 width=399 height=101>
		</td>
		<td width=100%>&nbsp;</td>
		<td width=365>
			<img src=/images/topbar_bkgd_right.jpg border=0 width=365 height=101>
		</td>
	</tr>
	</table>        
	<div id=sliver style='LEFT: -1px; POSITION: absolute; TOP: 0px'>
		<img border=0 src=/images/topbar_bkgd_left_sliver.jpg width=2 height=101>
	</div>
	<div id=connectedto style='LEFT: $OffsetDomain{'left'}; POSITION: absolute; TOP: $OffsetDomain{'top'}'>
		<p class=clsTitleFrame>
		<font size=3><b>$Domain</b></font>
	</div>
	<div id=loggedin style='LEFT: $OffsetUser{'left'}; POSITION: absolute; TOP: $OffsetUser{'top'}'>
		<p class=clsTitleFrame>
		<font size=1><b>$User</b></font> 
	</div>
	$GOLabel
	<div id=layer2 style='left: 620px; position: absolute; top: 2px'>
		<a href=/logout.cgi?message=logout target=_top><img border=0 src=$ImgPathLogOut width=57 height=15></a>
	</div>
	<div id=layer3 style='left: 690px; position: absolute; top: 2px'>
		<a href=/guide/server/start.htm target=_blank><img src=$ImgPathOnlineGuide border=0 width=74 height=15></a>
	</div>
</body>
</html>
HERE

	print($cgi->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT'));
	print($html);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
