#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: user_status.cgi                                                   #
#                                                                             #
# Description: CGI needed to display and modify user configurations.          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use Virtusertable;
use VirtualFtp;
use DisplayMain;
use DomainMail;
use Authorize;
use Quotaall;
use Users;
use CGI;

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}
	
	# Validate the data...
	if (($FormDataRef->{'name_full'} ne '') &&
		($FormDataRef->{'name_full'} !~ /^\S+(\s*\S+)*$/))
	{
		# Set the return code.
		$Return = 910000;
	}
	elsif (($FormDataRef->{'disk_quota'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
	       (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
	{
		# Set the return code.
		$Return = 910004;
	}
	else
	{
		my @MailAliasListList;
		my $MailAliasList		= $FormDataRef->{'e_mail_alias'};
		my $BadAliasCount;

		# Clean up the list.
		$MailAliasList =~ s/\n/ /sg;
		$MailAliasList =~ s/\,/ /g;
		$MailAliasList =~ s/\;/ /g;
		$MailAliasList =~ s/\cM//g;

		# Get the individual addresses.
		@MailAliasListList = split(/\s+/, $MailAliasList);

		# Run though the mail list...
		for (@MailAliasListList)
		{
			# Look for bad addresses...
			if ($_ =~ /^(\w|-|_)+$/)
			{
				next;
			}
			else
			{
				# Increment the bad count.
				++$BadAliasCount;
			}
		}

		# If there are any bad addresses...
		if ($BadAliasCount)
		{
			# Set the return code.
			$Return = (1 < $BadAliasCount) ? 910006 : 910005;
		}
	}

	return $Return;
}

###############################################################################
# GetHomeUsage
###############################################################################

sub GetHomeUsage
{
	my $DFData = Safe::backtick('/bin/df -k /home');
    
	# Get the data.
	if ($DFData =~ /\/dev\/\S+\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/g)
	{
		return $1;
	}

	return 1;
}

###############################################################################
# ModifyUser
###############################################################################

sub ModifyUser
{
	my ($FormDataRef)	= @_;
	my $IsPrimaryDA		= Users::primaryDA($FormDataRef->{'name_login'});
	my $Return			= 0;

	# Make sure the primary domain administrator is not demoted...
	if ((1 == $IsPrimaryDA) &&
		('none' eq $FormDataRef->{'rights'}))
	{
		# Set the return code.
		$Return = 910014;
	}
	else
	{
		my ($UserID, $UserGID, $Home)	= (getpwnam($FormDataRef->{'name_login'}))[2,3,7];
		my @UserList					= Users::listUsrs();
		my $VirtualDomain				= Users::getUserDomain($FormDataRef->{'name_login'});
		my $Domain						= '';
		my $User;
		my $Shell;
		my @Quota;
		my $QuotaLimit;
		my $QuotaTotal;
		my $SpaceAvailable;
		my $SurrogateGroup;
		
		# Run through the user list...
		foreach $User (@UserList) 
		{
			# If the user matches the login name...
			if ($User->{'user'} eq $FormDataRef->{'name_login'})
			{
				# Cache the user's domain.
				$Domain = $User->{'domain'};
				last;
			}
		}
		
		# Remove "chroot" stuff from 
		# the home directory of the user.
		$Home =~ s/(\.\/)//;
		$Home =~ s/\/$//;

		# If the rights are specified...
		if (defined($FormDataRef->{'rights'}))
		{
			# If the rights are adminsitrator...
			if ('domain' eq $FormDataRef->{'rights'}) 
			{
				# Domain administrator.
				$SurrogateGroup				= $FormDataRef->{'domain'};
				$FormDataRef->{'domain'}	= 10001;
			}
			elsif ('server' eq $FormDataRef->{'rights'}) 
			{
				# Server administrator.
				$SurrogateGroup				= $FormDataRef->{'domain'};
				$FormDataRef->{'domain'}	= 10000;
			}
		}
	
		# Set up the Telnet access...
		$Shell = ($FormDataRef->{'telnet_access'}) ? '/bin/bash' : '/bin/false';

		# Get the quota data.
		@Quota			= Quotaall::repquota({ 'domain' => $VirtualDomain });
		$QuotaLimit		= $Quota[2] / 1024; # Sometimes the quota stuff returns 0.
		$QuotaTotal		= Quotaall::quotaCheck($Domain) / 1024;

		# If the quota limit for the domain is zero.
		$QuotaLimit = GetHomeUsage() if(0 == $QuotaLimit);

		# Get the space available.
		$SpaceAvailable	= $QuotaLimit - $QuotaTotal;

		# Check for space and size limits on the quota...
		if ($SpaceAvailable <= $FormDataRef->{'disk_quota'}) 
		{
			# Set the return code.
			$Return = 910009;
		}
		elsif (1000 < $FormDataRef->{'disk_quota'})
		{
			# Set the return code.
			$Return = 910010;
		}
		elsif (0 >= $FormDataRef->{'disk_quota'})
		{
			# Set the return code.
			$Return = 910016;
		}
		else
		{
			# If the quota changed...
			if (($FormDataRef->{'disk_quota'} != $FormDataRef->{'old_disk_quota'}) &&
                (0 != $QuotaLimit))
			{
				my $QuotaRemaining = $SpaceAvailable;
				
				# If there is any quota remaining...
				$QuotaRemaining += $FormDataRef->{'old_disk_quota'} if($QuotaRemaining >= 0); 
				
				# Check for the combined quota of the users for the domain.
				if (($FormDataRef->{'disk_quota'} >= $QuotaRemaining) && 
					($FormDataRef->{'old_disk_quota'} < $FormDataRef->{'disk_quota'})) 
				{
					# Set the return code.
					$Return = 910017;
				}
				elsif (($FormDataRef->{'disk_quota'} > $FormDataRef->{'old_disk_quota'}) &&
                       ($FormDataRef->{'disk_quota'} >= $QuotaRemaining))
				{
					# Set the return code.
					$Return = 910018;
				}
			}

			# If things are still going good...
			unless ($Return)
			{
				my @AliasList	= split(/\s+/, $FormDataRef->{'e_mail_alias'});
				my $UserExists	= 0;
				my $Alias;      

				# Run through the alias list...
				foreach $Alias (@AliasList)
				{
					# Run through the user list...
					foreach $User (@UserList)
					{
						# If there is a match...
						if ($User eq $Alias)
						{
							$UserExists = 1;
							last;
						}
					}

					# Bail out...
					last if($UserExists);
				}

				# If there is not a duplicate aliased user...
				unless ($UserExists)
				{
					# If there is an e-mail alias...
					if ('' ne $FormDataRef->{'e_mail_alias'})
					{
						# Update the alias.
						if (0 != DomainMail::modAlias($FormDataRef->{'name_login'}, $VirtualDomain, $FormDataRef->{'e_mail_alias'}))
						{
							# Set the return code.
							$Return = 910012;
						}
					}
					elsif ('' eq $FormDataRef->{'e_mail_alias'})
					{
						# Clear the alias.
						DomainMail::modAlias($FormDataRef->{'name_login'});
					}
					
					# If things are still going good...
					unless ($Return)
					{
						# Set the quota.
						Quotaall::setquota($UserID, $FormDataRef->{'disk_quota'});

						# Modify the user information.
						Users::usrMod($FormDataRef->{'name_login'}, $FormDataRef->{'name_login'}, $FormDataRef->{'name_full'}, $Home, $FormDataRef->{'domain'}, $Shell, $SurrogateGroup);

						# If FTP access is needed...
						if ($FormDataRef->{'ftp_access'})
						{
							# Enable FTP.
							DomainMail::enftp($FormDataRef->{'name_login'});
						}
						else
						{
							# Disable FTP.
							DomainMail::noftp($FormDataRef->{'name_login'});
						}

						# If mail access is needed...
						if ($FormDataRef->{'mail_access'})
						{
							# Enable FTP.
							DomainMail::enmail($FormDataRef->{'name_login'}); 
						}
                        else
						{
							# Disable FTP.
							DomainMail::nomail($FormDataRef->{'name_login'});
						}
					}
				}
				else
				{
					# Set the return code.
					$Return = 910011;
				}
			}
		}
	}

	return $Return;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'user_status.lrc', 'common.lrc' ]);
	my %FormData			= (	'domain'				=> '',
								'name_full'	   			=> '',
								'name_login'   			=> '',
								'e_mail_alias'			=> '',
								'telnet_access'			=> '',
								'ftp_access'			=> '',
								'mail_access'			=> '',
								'rights'				=> 'none',
								'disk_quota'			=> 0 );
	my @UserList			= Users::listUsrs();
	my $User				= $cgi->param('user');
	my $RemoteUser			= $cgi->remote_user();
	my $RemoteUserGID		= (getpwnam($RemoteUser))[3];
	my $AdministratorRights	= ((10000 == $RemoteUserGID) || (10001 == $RemoteUserGID)) ? 1 : 0;
	my $NoOverrideAttempt;
	my $UserRef;
	my $Index;
	my $Item;
	my $Level;
	my $Key;
	my $Feedback;
	my $Result;
	my $html;

	# If the user is logged in directly...
	$User = ((10000 != $RemoteUserGID) && (10001 != $RemoteUserGID)) ? $RemoteUser : '';

	# If there still is no user...
	if ('' eq $User)
	{
		# Try to get the target user.
		($User, $Level) = Authorize::getTargetUser();
			
		# Bail out if user is unauthorized...
		exit(0) if($Level == $Authorize::PRIV_NONE);
	}
	
	# Run through the user list...
	foreach $Item (@UserList) 
	{
		# If the user name matches...
		if ($Item->{'user'} eq $User) 
		{
			# Get the index.
			$Index = $Item->{'num'};
			last;
		}
	}

	# Get the user information.
	$UserRef = $UserList[$Index];
	
	# Bail out if user is unauthorized...
	exit(0) if((0 == $UserRef->{'gid'}) || ('root' eq $UserRef->{'user'}));
	
	# Check for a domain administrator trying to modify a server administrator...
	$NoOverrideAttempt = ((10001 == $RemoteUserGID) && (10000 == $UserRef->{'gid'})) ? 0 : 1;
	
	# If the remote user has administrator rights...
	if (($NoOverrideAttempt) &&
		($AdministratorRights))
	{
		my %_R					= ( 'NAME_FULL'				=> '',
									'NAME_LOGIN'			=> '',
									'E_MAIL_ALIASES_OPT'	=> '',
									'TELNET_ACCESS'			=> '',
									'FTP_ACCESS'			=> '',
									'MAIL_ACCESS'			=> '',
									'ADMINISTRATOR_RIGHTS'	=> '',
									'SERVER_ADMINISTRATOR'	=> '',
									'DOMAIN_ADMINISTRATOR'	=> '',
									'NONE'					=> '',
									'USER_DISK_QUOTA'		=> '',
									'MB'					=> '',
									'APPLY'					=> '' );
		
		# Re-use for HTML.
		$AdministratorRights = '';

		# Get the string resources.
		$StringResource->GetStrings(\%_R);

		# If modifying a user...
		if (defined($cgi->param('apply')))
		{
			# Get the form data from the form...
			foreach $Key (keys(%FormData))
			{
				# Get the data...
				$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
			}
	
			# Validate the form data.
			if (0 == ($Result = ValidateFormData(\%FormData)))
			{
				# Clean up the list.
				$FormData{'e_mail_alias'} =~ s/\n/ /sg;
				$FormData{'e_mail_alias'} =~ s/\,/ /g;
				$FormData{'e_mail_alias'} =~ s/\;/ /g;
				$FormData{'e_mail_alias'} =~ s/\cM//g;
				
				# Try to modify the user...
				if (0 == ($Result = ModifyUser(\%FormData)))
				{
					# Get the updated list.
					@UserList = Users::listUsrs();

					# Run through the user list...
					foreach $Item (@UserList) 
					{
						# If the user name matches...
						if ($Item->{'user'} eq $User) 
						{
							# Get the index.
							$Index = $Item->{'num'};
							last;
						}
					}

					# Get the user information.
					$UserRef = $UserList[$Index];
				}
			}
	
			# If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		else
		{
			# Get the form data.
			$FormData{'domain'}			= $UserRef->{'domain'};
			$FormData{'rights'}			= ((10000 == $UserRef->{'gid'}) ? 'server' : ((10001 == $UserRef->{'gid'}) ? 'domain' : 'none'));
			$FormData{'name_full'}		= $UserRef->{'real'};
			$FormData{'name_login'}		= $UserRef->{'user'};
			$FormData{'e_mail_alias'}	= DomainMail::userAlias($UserRef->{'user'}, $UserRef->{'domain_name'});
			$FormData{'telnet_access'}	= ($UserRef->{'shell'} eq '/bin/false') ? 0 : 1;
			$FormData{'ftp_access'}		= (DomainMail::ftpaccess($UserRef->{'user'})) ? 0 : 1;
			$FormData{'mail_access'}	= (DomainMail::mailaccess($UserRef->{'user'})) ? 0 : 1;
			$FormData{'disk_quota'}		= (Quotaall::repquota({ 'user' => $UserRef->{'user'}}))[1]/1024;
		}

		# Check for the special case of the server 
		# administrator viewing themselves...
		if (($RemoteUser eq $UserRef->{'user'}) && 
			(10000 == $RemoteUserGID) &&
			(10000 == $UserRef->{'gid'})) 
		{
			# The server administrator cannot modify their 
			# rights (to prevent accidently losing the last server administrator).
			$AdministratorRights = '<input type=hidden name=rights value=server>';
		}
		else 
		{
			my $ServerAdministrator = '';
	
			# Set up the radio state...
			$FormData{'rights_server'}	= ('server' eq $FormData{'rights'}) ? 'checked' : '';
			$FormData{'rights_domain'}	= ('domain' eq $FormData{'rights'}) ? 'checked' : '';
			$FormData{'rights_none'}	= ('none'   eq $FormData{'rights'}) ? 'checked' : '';
		
			# If the remote user is a server administrator...
			if (10000 == $RemoteUserGID)
			{
				# If the user is a server administrator...
				if (10000 == $UserRef->{'gid'})
				{	
					# Set the checkbox.
					$FormData{'rights_server'}	= 'checked';
					$FormData{'rights_domain'}	= '';
					$FormData{'rights_none'}	= '';
				}
				
				# If we are serveradmin but currently selected user is not serveradmin
				$ServerAdministrator = <<HERE;
				<tr align=left valign=center>
					<td width=25>&nbsp;<input type=radio name=rights value=server $FormData{'rights_server'}></td>
					<td nowrap width=315><b>$_R{'SERVER_ADMINISTRATOR'}</b></td>
				</tr>
HERE
			}

			$AdministratorRights = <<HERE;
		<tr align=left valign=center>
			<td nowrap><b>$_R{'ADMINISTRATOR_RIGHTS'}</b></td>
		</tr>
		<tr align=left valign=center>
			<td colspan=2>
				<table cellspacing=0 border=0 cellpadding=0 width=100%>
$ServerAdministrator
				<tr align=left valign=center>
					<td width=25>&nbsp;<input type=radio name=rights value=domain $FormData{'rights_domain'}></td>
					<td nowrap width=315><b>$_R{'DOMAIN_ADMINISTRATOR'}</b></td>
				</tr>
				<tr align=left valign=center>
					<td width=25>&nbsp;<input type=radio name=rights value=none $FormData{'rights_none'}></td>
					<td nowrap width=315><b>$_R{'NONE'}</b></td>
				</tr>
				</table>
			</td>
		</tr>
HERE
		}

		# Re-align the checkboxes...
		$FormData{'telnet_access'}	= ($FormData{'telnet_access'}) ? 'checked' : '';
		$FormData{'ftp_access'}		= ($FormData{'ftp_access'}) ? 'checked' : '';
		$FormData{'mail_access'}	= ($FormData{'mail_access'}) ? 'checked' : '';
		
		# Escape the users full name so it displays properly (Wer'e escaping the ' symbol)
		$FormData{'name_full'} 		=~ s/'/&#039;/g;
		
		# Pad out the mail alias...
		$FormData{'e_mail_alias'} = join(',', (split(/\s+/, $FormData{'e_mail_alias'})));
		$FormData{'e_mail_alias'} = join(', ', (split(/,/, $FormData{'e_mail_alias'})));
	
		$html = <<HERE;
<form method=post action=user_status.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 width=100%>
			<input type=hidden name=domain value=$FormData{'domain'}>
			<input type=hidden name=name_login value=$FormData{'name_login'}>
			<input type=hidden name=old_disk_quota value=$FormData{'old_disk_quota'}>
			<tr>
				<td><b>$_R{'NAME_FULL'}</b></td>
				<td><input type=text size=20 maxlength=32 name=name_full value='$FormData{'name_full'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'NAME_LOGIN'}</b></td>
				<td>$FormData{'name_login'}</td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td valign=top><b>$_R{'E_MAIL_ALIASES_OPT'}</b></td>
				<td valign=top><textarea name=e_mail_alias rows=2 cols=20>$FormData{'e_mail_alias'}</textarea></td>
			</tr>			
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=telnet_access $FormData{'telnet_access'}>&nbsp;<b>$_R{'TELNET_ACCESS'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=ftp_access $FormData{'ftp_access'}>&nbsp;<b>$_R{'FTP_ACCESS'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=mail_access $FormData{'mail_access'}>&nbsp;<b>$_R{'MAIL_ACCESS'}</b></td>
			</tr>
$AdministratorRights
			<tr>
				<td nowrap><b>$_R{'USER_DISK_QUOTA'}</b></td>
				<td><input type=text size=4 maxlength=10 name=disk_quota value='$FormData{'disk_quota'}'>$_R{'MB'}</td>
			</tr>
			<tr>
				<td colspan=2 align=right><input type=submit name=apply value='$_R{'APPLY'}' class=clsButton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	}
	else
	{
		my %_R					= ( 'NAME_FULL'				=> '',
									'NAME_LOGIN'			=> '',
									'E_MAIL_ALIASES'		=> '',
									'TELNET_ACCESS_LABEL'	=> '',
									'FTP_ACCESS_LABEL'		=> '',
									'MAIL_ACCESS_LABEL'		=> '',
									'USER_DISK_QUOTA'		=> '',
									'MB'					=> '',
									'ENABLED'				=> '',
									'DISABLED'				=> '' );
		
		# Get the string resources.
		$StringResource->GetStrings(\%_R);

		# Get the form data.
		$FormData{'e_mail_alias'}	= DomainMail::userAlias($UserRef->{'user'}, $UserRef->{'domain_name'});
		$FormData{'telnet_access'}	= ($UserRef->{'shell'} eq '/bin/false') ? 0 : 1;
		$FormData{'ftp_access'}		= (DomainMail::ftpaccess($UserRef->{'user'})) ? 0 : 1;
		$FormData{'mail_access'}	= (DomainMail::mailaccess($UserRef->{'user'})) ? 0 : 1;
		$FormData{'disk_quota'}		= (Quotaall::repquota({ 'user' => $UserRef->{'user'}}))[1]/1024;

		# Get the display status.
		$FormData{'telnet_access'}	= ($FormData{'telnet_access'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		$FormData{'ftp_access'}		= ($FormData{'ftp_access'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		$FormData{'mail_access'}	= ($FormData{'mail_access'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		
		# Bring up the overrid note if needed...
		unless ($NoOverrideAttempt)
		{
			# Override note is needed.
			$NoOverrideAttempt = '<tr><td colspan=2><b>' . $StringResource->_S('NOTE') . '&nbsp;</b>' . $StringResource->_S('DOMAIN_OVERRIDE_NOTE') . '</td></tr><tr><td colspan=2>&nbsp;</td></tr>';
		}
		else
		{
			# No note needed.
			$NoOverrideAttempt = '';
		}

		$html = <<HERE;
<table border=0 width=375 bgcolor=white>
$NoOverrideAttempt
<tr>
	<td width=50%><b>$_R{'NAME_FULL'}</b></td>
	<td>$UserRef->{'real'}</td>
</tr>
<tr>
	<td><b>$_R{'NAME_LOGIN'}</b></td>
	<td>$UserRef->{'user'}</td>
</tr>
<tr>
	<td colspan=2>&nbsp</td>
</tr>
<tr>
	<td valign=top><b>$_R{'E_MAIL_ALIASES'}</b></td>
	<td valign=top>$FormData{'e_mail_alias'}</td>
</tr>			
<tr>
	<td colspan=2>&nbsp</td>
</tr>
<tr>
	<td nowrap><b>$_R{'TELNET_ACCESS_LABEL'}</b></td>
	<td>$FormData{'telnet_access'}</td>
</tr>
<tr>
	<td nowrap><b>$_R{'FTP_ACCESS_LABEL'}</b></td>
	<td>$FormData{'ftp_access'}</td>
</tr>
<tr>
	<td nowrap><b>$_R{'MAIL_ACCESS_LABEL'}</b></td>
	<td>$FormData{'mail_access'}</td>
</tr>
<tr>
	<td colspan=2>&nbsp</td>
</tr>
<tr>
	<td nowrap><b>$_R{'USER_DISK_QUOTA'}</b></td>
	<td>$FormData{'disk_quota'}&nbsp;$_R{'MB'}</td>
</tr>
</table>
HERE
	}

	# Fill out the display information.
	$DisplayDataRef->{'q'}				= $cgi;
	$DisplayDataRef->{'html'}			= $html;
	$DisplayDataRef->{'class'}			= 'user_status';
	$DisplayDataRef->{'id'}				= 9100;
	$DisplayDataRef->{'title'}			= $StringResource->_S('IDT:9100');
	$DisplayDataRef->{'root-link'}		= 'user_status';
	$DisplayDataRef->{'helpIndicator'}	= ($AdministratorRights) ? 'domain' : 'user';
	$DisplayDataRef->{'result'}			= $Result;
	$DisplayDataRef->{'feedback'}		= $Feedback;

	return 0;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'  eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);	last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	else
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
