#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: login_navbar.cgi                                                  #
#                                                                             #
# Description: Contains CGI script needed to render the main login screen.    #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)					= @_;
    my $html;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'login_navbar.lrc' ]);
	my $ImageResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/images');
	my $FileResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/help');
	my %_R						= ( 'USER_NAME'			=> '',
									'HINT_USER_NAME'	=> '',
									'PASSWORD'			=> '',
									'HINT_PASSWORD'		=> '',
									'LOGIN'				=> '',
									'LOGIN_TROUBLE'		=> '',
									'COPYRIGHT'			=> '' );
	my $ImgPathLoginSelected	= $ImageResource->ResolveResourceFile('login_login_selected.jpg');
    my $ImgPathSettingUp		= $ImageResource->ResolveResourceFile('login_setting_up.jpg');
	my $ImgPathAboutWeb			= $ImageResource->ResolveResourceFile('login_about_web.jpg');
    my $FilePathSettingUp		= $FileResource->ResolveResourceFile('setting_up.htm');
	my $FilePathAboutWeb		= $FileResource->ResolveResourceFile('about_web.htm');
	my $FirstbootCookie			= $cgi->cookie('fb2login');
	my $Welcome;
	my $Message					= $cgi->param('message');

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe defaults.
	$ImgPathLoginSelected	= '' unless(defined($ImgPathLoginSelected));
	$ImgPathSettingUp		= '' unless(defined($ImgPathSettingUp));
	$ImgPathAboutWeb		= '' unless(defined($ImgPathAboutWeb));
	$FilePathSettingUp		= '' unless(defined($FilePathSettingUp));
	$FilePathAboutWeb		= '' unless(defined($FilePathAboutWeb));

	# Get just the hidden part of the file name.
	$ImgPathLoginSelected	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
	$ImgPathSettingUp		=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
	$ImgPathAboutWeb		=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
	$FilePathSettingUp		=~ s/^\/usr\/ins\/intel\/gui(\/locale\/help\/.*)$/$1/;
	$FilePathAboutWeb		=~ s/^\/usr\/ins\/intel\/gui(\/locale\/help\/.*)$/$1/;

	# Get just the letter code.
	$PreferredLanguage		= $ImgPathLoginSelected;
	$PreferredLanguage		=~ s/^.*\/(\w{2}).*$/$1/;
	$PreferredLanguage		= '/locale/images/' . $PreferredLanguage . 'login_';

    # Check for the first boot cookie...
	$Welcome = ('Y' eq $FirstbootCookie) ? $StringResource->_S('FINISHED_SETUP_WIZARD') : $StringResource->_S('WELCOME');
	
	SWITCH:
	{
		if ('logout' eq $Message)      { $Message = $StringResource->_S('LOGOUT_SUCCESSFUL'); last SWITCH; }
		if (('sysdown' eq $Message) ||
			('down'    eq $Message))   { $Message = '<font color=red>'. $StringResource->_S('PLEASE_WAIT') . '</font>'; last SWITCH; }
		if ('login_error' eq $Message) { $Message = '<font color=red>'. $StringResource->_S('LOGIN_FAILED') . '</font>'; last SWITCH; }
		if ('new_login' eq $Message)   { $Message = $StringResource->_S('LOGIN_TO_SERVER'); last SWITCH; }
		if ('biosupdate' eq $Message) { $Message = '<font color=red>'.$StringResource->_S('PLEASE_WAIT_BIOSUP') . '</font>'; last SWITCH; }
		$Message = $StringResource->_S('LOGIN_TO_SERVER');
	}

	$html = <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<meta name=copyright content="Copyright (c) 2000 Intel Corporation, et al.">
		<link rel=stylesheet href=/style/style.css type=text/css>
		<base target=_parent>
	</head>
<body bgcolor=#ffffff onresize=location.reload() onload=setFocus()>
	<script>
		function setFocus()
		{
			document.login.credential_0.focus();
		}
		
		var selected_item = 'login';
	
		function changeImages(itemname)
		{
			if (itemname != selected_item)
			{
				var picvalue = eval("document.images."+ itemname +".src");
				
				if(picvalue.indexOf('hover') != -1)
				{
					eval("document.images."+ itemname +".src = '$PreferredLanguage"+itemname+".jpg';");
				}
				else
				{
					eval("document.images."+ itemname +".src = '$PreferredLanguage"+itemname+"_hover.jpg';");
				}
			}
		}
	</script>
	<table cellspacing="0" cellpadding="0" width="840">
	<tr>
		<td width=126 valign=top>
			<p><a target=_self href=/login_navbar.cgi><img src=$ImgPathLoginSelected border=0 width=148 height=27></a></p>
			<p><a target=_self href=$FilePathSettingUp onmouseover="changeImages('setting_up');" onmouseout="changeImages('setting_up');"><img id=setting_up name=setting_up src=$ImgPathSettingUp border=0 width=148 height=27></a></p>
			<p><a target=_self href=$FilePathAboutWeb onmouseover="changeImages('about_web');" onmouseout="changeImages('about_web');"><img id=about_web name=about_web src=$ImgPathAboutWeb border=0 width=148 height=27></a></p>
		</td>
		<td width=642 valign=top>
			<table cellspacing=0 cellpadding=0>
			<tr>
				<td width=* valign=top>
					<p class=clsTitlePage>
					<b>$Welcome</b><br>
					<h5>$Message&nbsp;</h5>
					<form name=login method=post action=/LOGIN>
						<input type=hidden name=destination value=/login/index.cgi>
						<table cellspacing=0 cellpadding=0 style="font-size: 0.8em;">
						<tr style="padding-bottom: 4;">
							<td style="padding-right:16;"><label class=InputLabel for=username title='$_R{'HINT_USER_NAME'}' accesskey=u>$_R{'USER_NAME'}</label></td>
							<td><input type=text id=username name=credential_0 style="font-size: 1em; width: 175; height: 23" title='$_R{'HINT_USER_NAME'}' size=15 tabindex=1></td>
						</tr>
						<tr>
							<td><label class=InputLabel for=password title='$_R{'HINT_PASSWORD'}' accesskey=p>$_R{'PASSWORD'}</label></td>
							<td><input type=password id=password name=credential_1 style="font-size: 1em; width: 175; height: 23" title='$_R{'HINT_PASSWORD'}' size=15 tabindex=2></td>
							<td style="padding-left: 10; padding-right: 10;"><input type=submit class=clsButton tabindex=3 value=' $_R{'LOGIN'} '></td>
						</tr>
						<tr>
							<td colspan=2 style="height: 16;"></td>
						</tr>
						</table>
					</form>
					<div id=idCopyrightText class=text style="margin-top: 50;">
						<div style="margin-bottom: 12; width: 90%; overflow: visible;">$_R{'LOGIN_TROUBLE'}</div>
						<div style="width: 550; overflow: visible;">$_R{'COPYRIGHT'}</div>
					</div>
				</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</body>
</html>
HERE

	# Check for the first boot cookie...
	if ('Y' eq $FirstbootCookie) 
	{
		# Set the expiration and kill the cookie.
		print $cgi->header(-cookie=>$cgi->cookie( -name=>'fb2login', -value=>'N', -path=>'/'), -expires=>'Mon, 21-May-1971 00:00:00 GMT');
	}
	else
	{
		# Set the expiration.
		print($cgi->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT'));
	}
	
	print($html);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
