package AuthCustom;

###############################################################################
#                                                                             #
# Filename: AuthCustom.pm                                                     #
#                                                                             #
# Description: Contains script needed to provide custom authorization for     #
#              Apache.                                                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib "/usr/ins/intel/gui/perllib";

use strict;
use Apache;
use Apache::Constants qw(:common);
use vars qw(@ISA);
use AuthCookie;
use Password;

@ISA = qw(AuthCookie);

use vars qw($PRIV_NONE $PRIV_USER $PRIV_DOMAIN $PRIV_SERVER);
$PRIV_NONE   = 0;
$PRIV_USER   = 1;
$PRIV_DOMAIN = 2;
$PRIV_SERVER = 3;

sub authen_cred ($$\@) {
    my $self = shift;
    my $r = shift;
    my @creds = @_;

    join(":", @creds);
}

sub authen_ses_key ($$$) {
    my $self = shift;
    my $r = shift;
    my($user, $password) = split(/:/, shift, 2);

    # Get the authentication requirements (from Apache) in an array ref
    my $reqs_arr = $r->requires or return; 

    my ($requirement, $args);
    foreach my $req (@$reqs_arr) {
      ($requirement, $args) = split /\s+/, $req->{requirement}, 2;
      $args = '' unless defined $args;
      last if ($requirement eq 'valid-user') or ($requirement eq 'domain-admin') or ($requirement eq 'server-admin');
    }                                     

    if ($requirement eq 'valid-user') {
	$requirement = $PRIV_USER;
    } 
    elsif ($requirement eq 'domain-admin') {
	$requirement = $PRIV_DOMAIN;
    } 
    elsif ($requirement eq 'server-admin') {
	$requirement = $PRIV_SERVER;
    } 

    my ($priv_level, @pwent);

    # If requirement is domain-admin or server-admin then check privilege level
    if (($requirement eq $PRIV_DOMAIN) or ($requirement eq $PRIV_SERVER)) {
      @pwent = getpwnam($user);

      if ($pwent[3] == 10000) {
        # User is a Server Admin (gid is 10000)
        $priv_level = $PRIV_SERVER;
      } 
      elsif ($pwent[3] == 10001) {
        # User is a Domain Admin...
        $priv_level = $PRIV_DOMAIN;
      } 

    }
    # Otherwise assign a default privilege level
    else {
	$priv_level = $PRIV_USER;
    }

    if ($priv_level >= $requirement) {
      my $pwverify = Password->checkPassword($user, $password);
      if ($pwverify eq "VALID") { 
	 return $user;
      } else {
	 return;
      }
    }
    else {
	 return;
    }
}

1;
