package Backup;

###############################################################################
#                                                                             #
# Filename: Backup.pm                                                         #
#                                                                             #
# Description: Contains script needed to facilitate domain and system         #
#              backup/restore.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use POSIX qw(strftime tmpnam);
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use IO::File;
use IO::Select;
use LanguageResource;
use LanguageLogCGI;
use Safe;
$|=1;

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw(initialize_backup_sequence restore_from_web calculate_disk_usage);
$VERSION = '0.04';

# Globals
my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'Backup.lrc' ]);

#####################################################################
# initialize_backup_sequence() 					    # 
#								    #
#								    #
#####################################################################
sub initialize_backup_sequence {

#  my ($self, $var_ref) = @_;
  my $self = shift;
  my $var_ref = shift;

  my $l10nNoBackupData = $StringResource->_S('NO_BACKUP_DATA');

  my $i = 0;   # Define counter.
  my (@files_to_backup, @targets_to_backup);
#  $var_ref->{tar_cmd} = "/bin/tar";  # Use czOp for output to stdout
  my %backup_targets = (
			users_mail		=> "server_mail",
			system_config		=> "/etc/",
			user_home_dirs		=> "/home/domain",
			home_paritition		=> "/home",
			intershop4		=> "/home/isroot4",
			domain_email		=> "domain_mail",
			domain_user_home_dirs	=> "users",
			domain_web_site		=> "web",
  );

  if ($var_ref->{q}->param('system_to_backup') eq 'entire_system') {
	@files_to_backup = qw(/);
  }
  elsif ($var_ref->{q}->param('system_to_backup') eq 'entire_domain') {
	@files_to_backup = "/home/domain/$var_ref->{domain}/";
  }
  else {
	@targets_to_backup = $var_ref->{q}->param('files_to_backup');
 
	# Assign backup target paths into an array
	foreach (@targets_to_backup) {
		if ($_ eq "users_mail") {
			$var_ref->{mail_to_backup} = $_;
		}
		elsif ($_ eq "domain_email") {
                        $var_ref->{mail_to_backup} = $_;
                }
		else {
                $files_to_backup[$i] = $backup_targets{$_};
                $i++;
		}
  	}
  }

  $var_ref->{files_to_backup} =  [@files_to_backup];

  # Check to see what button was pressed and then take appropriate action
  if ( ($var_ref->{q}->param('start_backup')) and 
	((@{$var_ref->{files_to_backup}}) or ($var_ref->{mail_to_backup})) ) {
     perform_backup($var_ref);
  }
  else {
	  $var_ref->{feedback} = "$l10nNoBackupData";
	  $var_ref->{caller}->make_body($var_ref);
  }
 
} # End of initialize_backup_sequence()

#####################################################################
# perform_backup()                                                  #
#####################################################################
sub perform_backup {
 
 my ($var_ref) = @_;
 $|=1;
 my ($backup_cmd, $data, $return_val);

 my $l10nNoBackupData = $StringResource->_S('NO_BACKUP_DATA');
 my $l10nBackupCompleted = $StringResource->_S('BACKUP_COMPLETED');

 # Set files_to_backup to current directory if ...
 $var_ref->{files_to_backup} =  ['.'] if ($var_ref->{q}->param('system_to_backup') eq 'entire_domain');	

  if (($var_ref->{mail_to_backup} eq "users_mail") 
	and !($var_ref->{caller} eq "BackupDomain") ) {

       	my $path_arr_ref = get_user_home_dirs("system");
       	foreach (@$path_arr_ref) {
	  next unless (-e "$_/mbox"); 
	  push(@{$var_ref->{files_to_backup}}, "$_/mbox");
       	}
     }
  elsif ($var_ref->{mail_to_backup} eq "domain_email") {

       	my $path_arr_ref = get_user_home_dirs("domain", $var_ref->{domain});
       	foreach (@$path_arr_ref) {
	  next unless (-e "$_/mbox"); 
	  push(@{$var_ref->{files_to_backup}}, "$_/mbox");
       	}
  }

 if ($var_ref->{backup_medium} eq "web") {
         if ((!@{$var_ref->{files_to_backup}}) and (!$var_ref->{mail_to_backup})) {
	     $var_ref->{feedback} = $l10nNoBackupData;
	     $var_ref->{caller}->make_body($var_ref);
          }
         else {


             print $var_ref->{q}->header(-type=>'application/octet-stream',
	     'Content-Disposition'=>"inline; filename=$var_ref->{backup_filename}");

	     # The following backup mechanism needs to be enhanced so that Apache doesn't end up
	     # caching all the tar output and increasing in size.

	     if ($var_ref->{caller} eq "BackupDomain") {
		$backup_cmd = "(cd /home/domain/$var_ref->{domain}; /bin/tar cOpP @{$var_ref->{files_to_backup}} --exclude /proc --exclude /dev | /bin/gzip)";
	     }
	     else
	     {
		$backup_cmd = "/bin/tar cOpP @{$var_ref->{files_to_backup}} --exclude /proc --exclude /dev --exclude /home/backup | /bin/gzip";
	     }

	     my $FH = Safe::pipe_read("$backup_cmd");
		while (sysread $FH, $data, 1024) {
		# syswrite does not return the number of bytes read.  This is different from the documented
		# behaviour.  In this case it returns "1" instead of "1024" -Cheema
  		$return_val = syswrite STDOUT, $data;
		last if ($return_val <= 0);
	     }
	     Safe::pipe_close($FH);
          }
 }
 elsif ($var_ref->{backup_medium} eq "server_hd") {
         if ((!@{$var_ref->{files_to_backup}}) and (!$var_ref->{mail_to_backup})) {
	     $var_ref->{feedback} = $l10nNoBackupData;
	     $var_ref->{caller}->make_body($var_ref);
         }
         else {
	    if (confirm_avaialable_disk_space($var_ref)) {
	        unless (-d "/home/backup") {		# If the backup dir doesn't exist, create it in
		   mkdir "/home/backup", 0700;		# in a secure fashion.
	        }

	        umask 077;				# Change umask so that backup files created are not
	     						# world readable.

		# Delete any file that exists with the same name that we are using to create a new archive.
		unlink "/home/backup/$var_ref->{backup_filename}";
             	$var_ref->{PID} = Safe::spawn_daemon("/bin/tar czpPf \"/home/backup/$var_ref->{backup_filename}\"" .
			" @{$var_ref->{files_to_backup}} --exclude /proc --exclude /dev --exclude /home/backup");

		$var_ref->{backup_filename} = CGI::escape($var_ref->{backup_filename});
		# Redirect user to a new page with status of the backup process.
	        print $var_ref->{q}->redirect(-uri=>"/cgi-bin/server/server_backup.cgi?type=status&archive=$var_ref->{backup_filename}&pid=$var_ref->{PID}",-nph=>'1');

	    } else {
		# Need to make this l10n Compliant
		$var_ref->{feedback} = $StringResource->_S('NOT_ENOUGH_SPACE_TO_BACKUP_SELECTED_DATA');
	        $var_ref->{caller}->make_body($var_ref);
	    }
        };
 
 }
 else {
	# Need to make this l10n Compliant
	$var_ref->{feedback} = $StringResource->_S('DID_NOT_SELECT_BACKUP_MEDIUM');
	$var_ref->{caller}->make_body($var_ref);
 }

 
}  #End of perform_backup()

#
# confirm_avaialable_disk_space();
#

sub confirm_avaialable_disk_space {

my $var_ref = shift;
my ($total_disk_space_needed, $total_disk_space_available, $size);
my @files_to_backup = $var_ref->{q}->param('files_to_backup');

  if ($var_ref->{q}->param('system_to_backup') eq 'entire_system') {
	$total_disk_space_needed = $var_ref->{q}->param('entire_system_size');

		if ($total_disk_space_needed =~ /k/) {
			chop($total_disk_space_needed);
			$total_disk_space_needed = $total_disk_space_needed / 1024;
		} elsif ($total_disk_space_needed =~ /G/) {
			chop($total_disk_space_needed);
			$total_disk_space_needed = $total_disk_space_needed  * 1024;
		} elsif ($total_disk_space_needed =~ /M/) {
			chop($total_disk_space_needed);
		}

	
	# $total_disk_space_needed =~ s/\D//g;
  } elsif (@files_to_backup) {
	foreach (@files_to_backup) {
		# $_ .= "_size":
		# $total_disk_space_needed += $var_ref->{q}->param("$_" . "_size");
		$size = $var_ref->{q}->param("$_" . "_size");
		
		if ($size =~ /k/) {
			chop($size);
			$total_disk_space_needed += $size / 1024;
		} elsif ($size =~ /G/) {
			chop($size);
			$total_disk_space_needed += $size * 1024;
		} elsif ($size =~ /M/) {
			chop($size);
			$total_disk_space_needed += $size;
		}
	}
  }

  my $df_info;
  my $FH = Safe::pipe_read("/bin/df --block-size=1048576");
  sysread $FH, $df_info, 1024;
  Safe::pipe_close $FH;

  while ($df_info =~ /\/dev\/\S+\s*\S+\s*\S+\s*(\S+)\s*\S+\s*(\S+)/g) {
        if ($2 eq "/home") {
           $total_disk_space_available = $1;
        }
  }       

  if ($total_disk_space_needed < $total_disk_space_available) {
	return 1;
  } else {
	return;
  }

}


#
# calculate_disk_usage()                                            #
#

sub calculate_disk_usage {
 my ($self, $var_ref) = @_;

if ($var_ref->{caller} eq "BackupDomain") {

  my ($FH, $domain_usage, $domain_web_usage, $domain_mail_usage, $filesize, $domain_users_usage);

  $FH = Safe::pipe_read("/usr/bin/du -sh /home/domain/$var_ref->{domain}/.");
  sysread $FH, $domain_usage, 1024;
  $var_ref->{domain_usage} = $1 if ($domain_usage =~ /^(\S+)/);
  Safe::pipe_close $FH;

  $FH = Safe::pipe_read("/usr/bin/du -sh /home/domain/$var_ref->{domain}/web");
  sysread $FH, $domain_web_usage, 1024;
  $var_ref->{domain_web_usage} = $1 if ($domain_web_usage =~ /^(\S+)/);
  Safe::pipe_close $FH;

  my $path_arr_ref;

  eval { $path_arr_ref = get_user_home_dirs("domain", $var_ref->{domain}) };

  if ($@) {
	return $@;
  }

  foreach (@$path_arr_ref) {
       ($filesize)= (stat("/home/domain/$var_ref->{domain}/$_/mbox"))[7];
	$var_ref->{domain_mail_usage} += $filesize;
  }
  $var_ref->{domain_mail_usage} = Backup::ConvertSize($var_ref->{domain_mail_usage});

  $FH = Safe::pipe_read("/usr/bin/du -sh /home/domain/$var_ref->{domain}/users");
  sysread $FH, $domain_users_usage, 1024;
  $var_ref->{domain_users_usage} = $1 if ($domain_users_usage =~ /^(\S+)/);
  Safe::pipe_close $FH;
  
  return; 
}

  my ($disk_free, $etc_usage, $is4_usage, $disk_usage, $FH);

  $FH = Safe::pipe_read("/bin/df -h --block-size=1");  # get info in bytes
  sysread $FH, $disk_free, 1024;
  Safe::pipe_close $FH;
  

	while ($disk_free =~ /\/dev\/\S+\s*\S+\s*(\S+)\s*\S+\s*\S+\s*(\S+)/g)
	{
        if ($2 eq "/home")
		{
			$var_ref->{home_partition_usage} = $1;

			if (opendir(DIR, "/home"))
			{
				my @files = readdir(DIR);
				closedir DIR; 

				# For each file get size and time information
				foreach (@files)
				{
					next if ( ($_ eq ".") or ($_ eq "..") );
					next if (/^domain/);

					my $file_usage;
					$FH = Safe::pipe_read("/usr/bin/du -sb /home/$_");
					sysread $FH, $file_usage, 1024;
					Safe::pipe_close $FH;

					$file_usage = $1 if $file_usage =~ /^\s*(\S+)/;

					$var_ref->{home_partition_usage} = $var_ref->{home_partition_usage} - $file_usage;
				}   

				# Change unit of size on home_partition_usage to a human readable format
				$var_ref->{home_partition_usage} = ConvertSize($var_ref->{home_partition_usage});
			}
			else
			{
				my $Log		= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'Backup.lrc' ]);
				my $Return	= 1400000;

				# Log the event.
				$Log->Append('error', 'critical', $Return);

				return $Return;
			}
		}
        elsif ($2 eq "/")
		{
			$var_ref->{root_partition_usage} = $1;
        }
	}
 
  $FH = Safe::pipe_read("/usr/bin/du -sh /etc");
  sysread $FH, $etc_usage, 1024;
  Safe::pipe_close $FH;

  $var_ref->{etc_dir_usage} = ConvertSize($1) if ($etc_usage =~ /^(\S+)/);

  # Calculate Intershop4 disk space usage.

  $FH = Safe::pipe_read("/usr/bin/du -sh /home/isroot4");
  sysread $FH, $is4_usage, 1024;
  Safe::pipe_close $FH;

  $var_ref->{intershop4_usage} = ConvertSize($1) if ($is4_usage =~ /^(\S+)/);

  # Calculate entire system mail usage.

  # Rewind the file pointer to the beginning of the /etc/passwd file
  setpwent;

  # Read from password database, one entry at a time.
  my ($name,$uid,$dir,$size,$TotalMailSize);
  while (($name,$uid,$dir) = (getpwent())[0,2,7]) {

    # Determine if the user is system user or a domain user
    # and add to total size accordingly.
    if (($uid == 0) or ($uid > 499)) {
      $size = (stat("$dir/mbox"))[7];
      $TotalMailSize += $size;
    }

  }
  endpwent;

  $TotalMailSize = ConvertSize($TotalMailSize);
  $var_ref->{mail_dir_usage} = $TotalMailSize;

  # Get df to return disk usage in MegaBytes
  
  $FH = Safe::pipe_read("/bin/df --block-size=1048576");
  sysread $FH, $disk_usage, 1024;
  Safe::pipe_close $FH;

  # Go through all mounts and add drive space usage.
  while ($disk_usage =~ /\/dev\/\S+\s*\S+\s*(\S+)\s*\S+\s*\S+\s*(\S+)/g) {
        $var_ref->{total_drive_usage} += $1;
  }

  # If the total is < 1024 then tag an M (for MegaBytes) else divide by 1024 and
  # tag a G (for GigaBytes)
  if ($var_ref->{total_drive_usage} < 1024) {
    $var_ref->{total_drive_usage} = "$var_ref->{total_drive_usage}M";
  }
  else {
    $var_ref->{total_drive_usage} = sprintf("%.2f", $var_ref->{total_drive_usage}/1024) . "G";
	# sprintf("%.0f", $free_memory/1024)
  }
  
 $var_ref->{total_drive_usage} = ConvertSize($var_ref->{total_drive_usage});

}  #End of calculate_disk_usage


######################################
# restore_from_web()    This function simply tries to restore from an archive
#                       uploaded from the web browser
######################################
sub restore_from_web {
  my ($self, $var_ref) = @_;

  # Get the name of the file being uploaded
  my $filename = $var_ref->{q}->param('upload');
  my ($tmp_filename, $fh, $restore_cmd, $data, $run_lilo, $back_link);

  my $l10nNoFileToUpload = $StringResource->_S('NO_FILE_TO_UPLOAD');
  my $l10nStartingFileRecovery = $StringResource->_S('STARTING_FILE_RECOVERY');
  my $l10nFilesToBeRestored = $StringResource->_S('FILES_TO_BE_RESTORED');
  my $l10nFilesRestored = $StringResource->_S('FILES_RESTORED');
  my $l10nBackToBackRes = $StringResource->_S('BACK_TO_BACKUP_RESTORE');

  unless (-d "/home/backup") {            # If the backup dir doesn't exist, create it in
          mkdir "/home/backup", 0700;     # in a secure fashion.
  } 

  # Get a temporary filename that does not clash with an existing file.
  # And then get a file handle to this temporary file name
  do {  $tmp_filename = tmpnam();
	$tmp_filename =~ s/\/tmp\//\/home\/backup\//g;
     }
  until $fh = IO::File->new($tmp_filename, O_RDWR|O_CREAT|O_EXCL);

  # If the user did not send a file name at all do this:
  if (!$filename) { 
        $var_ref->{feedback} = $l10nNoFileToUpload;
        $var_ref->{caller}->make_body($var_ref);
  }
  else {

        # Determine the system command we want to run to restore data from
        # $tmp_filename file.
        if ($var_ref->{caller} eq "BackupDomain") {
            $restore_cmd = "(cd /home/domain/$var_ref->{domain}; /bin/tar xvpPzf \"$tmp_filename\")";
            $back_link = "domain_backup.cgi";
        }
        else
        {   
            $restore_cmd = "(cd /; /bin/tar xvpPzf \"$tmp_filename\")";
            $back_link = "server_backup.cgi";
        } 

        print $var_ref->{q}->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT'), 
	      $var_ref->{q}->start_html(-title=>$StringResource->_S('RESTORE_FILE_LIST'),
				        -BGCOLOR=>'white'), 
		  "<b><font face=arial size=1><u><a href=$back_link>&lt;&nbsp;$l10nBackToBackRes</a></u>&nbsp;</font></b>",
   	      "<p>$l10nStartingFileRecovery: $filename ...
	      <p>$l10nFilesToBeRestored:<pre><hr noshade>"; 
 
	 my ($bytesread, $buffer);

	 # Read from the temporary file created by CGI.pm and write to the
	 # filehandle created above.
	my $counter = 0;
         while ($bytesread=read($filename,$buffer,1024)) {
            print $fh $buffer;
	    $counter++;
         }

	 # Done writing.  Close filehandle.
         close($fh);

	 # If the filename passed did not have any data then print an error.
	 unless ( ($counter > 0) or ($buffer) ) {
		print $StringResource->_S('BACKUP_FILE_NOT_FOUND');
		return;
	 }

	 # Now do stuff wth $tmp_filename and later delete it.

	    # Read any output from the system command.  In this case we should should only get a list
	    # files being untarr'ed.
  	    my $FH = Safe::pipe_read("$restore_cmd");
	    my $sel = new IO::Select( $FH );
  	    while (sysread $FH, $data, 80) {
		# If the restore included kernel image and lilo related files then run lilo
		if ( ($data =~ /^\/boot/) or ($data =~ /\n\/boot/) ) {
			$run_lilo = 1;
		}

	    	# Write the output sent by the restore command to the browser.
     		syswrite STDOUT, $data;

                # Print dots while waiting on Filehandle to get some data that we can read.
                while (!$sel->can_read(2)) {
                        print STDOUT ".\n";
                        last if $sel->has_error(1);
                }         
  	    }
  	    Safe::pipe_close($FH);

	    if ($run_lilo) {
		run_lilo($var_ref);
	    }

	 # Delete the $tmp_filename file
         unlink $tmp_filename;

	 # Print the footers
         print	"</pre>
	 $l10nFilesRestored", $var_ref->{q}->end_html();
      }
 
}  # End of restore_from_web()


######################################
# restore_from_server_hd()    This function simply tries to restore from an archive
#                       uploaded from the web browser
######################################
sub restore_from_server_hd {
  my ($self, $var_ref) = @_; 
  my ($filename, $restore_cmd, $data, $run_lilo, $back_link);

  my $l10nStartingFileRecovery = $StringResource->_S('STARTING_FILE_RECOVERY');
  my $l10nFilesToBeRestored = $StringResource->_S('FILES_TO_BE_RESTORED');
  my $l10nFilesRestored = $StringResource->_S('FILES_RESTORED');
  my $l10nFileNotFound = $StringResource->_S('NO_FILE');
  my $l10nBackToBackRes=$StringResource->_S('BACK_TO_BACKUP_RESTORE');
  my $l10nRestoreFileList=$StringResource->_S('RESTORE_FILE_LIST');

  if ($var_ref->{caller} eq "BackupDomain") {
    $filename = "/home/domain/$var_ref->{domain}/backup/" . $var_ref->{q}->param('restore_file');
    $restore_cmd = "(cd /home/domain/$var_ref->{domain}; /bin/tar xvpPzf \"$filename\")";
    $back_link = "domain_backup.cgi";
  }
  else { 
    $filename = "/home/backup/" . $var_ref->{q}->param('restore_file');
    $restore_cmd = "(cd /; /bin/tar xvpPzf \"$filename\")";
    $back_link = "server_backup.cgi";
  }

  print $var_ref->{q}->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT'), 
	$var_ref->{q}->start_html(-title=>$l10nRestoreFileList,
  				   -BGCOLOR=>'white'),
        "<b><font face=arial size=1><u><a href=$back_link>&lt;&nbsp;$l10nBackToBackRes</a></u>&nbsp;</font></b><p>";

  if (-e "$filename") {
    print   "$l10nStartingFileRecovery: $filename ...
                <p>$l10nFilesToBeRestored:<pre><hr noshade>";

	    # Execute the restore command
	    $|=1;	# Disable print buffering
	    my $FH = Safe::pipe_read("$restore_cmd");
	    my $sel = new IO::Select( $FH );
            while (sysread $FH, $data, 80) {

                if ( ($data =~ /^\/boot/) or ($data =~ /\n\/boot/) ) {
                        $run_lilo = 1;
                }

                # Write the output sent by the restore command to the browser.
                syswrite STDOUT, $data;

		# Print dots while waiting on Filehandle to get some data that we can read.
		while (!$sel->can_read(2)) {
			print STDOUT ".\n";
			last if $sel->has_error(1);
		}
            }
            Safe::pipe_close($FH);

            if ($run_lilo) {
                run_lilo($var_ref);
            }            


    print  "</pre>
    $l10nFilesRestored";
  }
  else {
    print "<p> $l10nFileNotFound$filename";
  }

  print $var_ref->{q}->end_html();

} # End of restore_from_server_hd() 


#
# ConvertSize takes a number and converts it into another unit.
#	If no desired unit is specified then it tries to convert it into a human
# 	readable format.
#	
#	Usage: ConvertSize("size", "desired_denomination")
#	Usage: ConvertSize("2048k", "M")
#
#	Valid denominations are b, k, M and G for bytes, kilobytes, Megabytes and Gigabytes
#

sub ConvertSize {
 my $size = shift;
 my $desired_denomination = shift;

 # Convert everything to bytes first
 if ($size =~ /k/i) {
   $size = $size * 1024;
 }
 elsif ($size =~ /M/i) {
   $size = $size * 1024 * 1024;
 }
 elsif ($size =~ /G/i) {
   $size = $size * 1024 * 1024 * 1024;
 }

 # Check for desired denomination, if it exists, convert to it
 if ($desired_denomination) {
   if ($desired_denomination =~ /b/i) {
     	$size = sprintf("%.0f", $size) . " " . $StringResource->_S('BYTES');
   }
   elsif ($desired_denomination =~ /k/i) {
     	$size = sprintf("%.2f", $size / 1024) . " " . $StringResource->_S('KB'); 
   }
   elsif ($desired_denomination =~ /M/i) {
     	$size = sprintf("%.2f", $size / 1024 / 1024) . " " . $StringResource->_S('MB');
   }
   elsif ($desired_denomination =~ /G/i) {
     	$size = $size = sprintf("%.2f", $size / 1024 / 1024 / 1024) . " " . $StringResource->_S('GB');
   }
 
 } # End of desired_denomination check
 else {
   if ($size < 1024) { 
	$size = sprintf("%.0f", $size) . " " . $StringResource->_S('BYTES');	
   }
   elsif ($size < 1048576) { 
	$size = sprintf("%.2f", $size / 1024) . " " . $StringResource->_S('KB');	
   }
   elsif ($size < 1073741824) { 
	$size = sprintf("%.2f", $size / 1024 / 1024) . " " . $StringResource->_S('MB');	
   }
   else {
	$size = sprintf("%.2f", $size / 1024 / 1024 / 1024) . " " . $StringResource->_S('GB');	
   }

 }

 return $size;

}

#
# run_lilo runs lilo to install the boot loader. 
#

sub run_lilo {

    my $var_ref = shift;
    my ($mount_point, $hda3_mounted);
 
    # Determine the mount point for /emboot partition
    if (open (MTAB, "/etc/mtab")) {
        while (<MTAB>) {
 
                /\/dev\/(\S+)\s*\/(\S+)/g;
 
                if (($1) and ($2)) {
                        if ($2 eq "boot") {
                          # print "1: $1 \t2: $2 \n";
                          $mount_point = $1;
                          $mount_point =~ s/\d//;
                          $mount_point .= "3";
                        }
                        if ($2 eq "emboot") {
                          $hda3_mounted = 1;
                        }
                }
        }
        if ($hda3_mounted) {
          Safe::system("/sbin/lilo");
        }
        else {
          Safe::system("/bin/mount /dev/$mount_point /emboot; /sbin/lilo; /bin/umount /emboot");
        }
    }
    else {
    	print_error($var_ref, $StringResource->_S('FILE_OPEN_ERROR') . ": /etc/mtab");
    }
}

#
# sub get_user_home_dirs() expects the name of target to get user paths for.  The targets could be
# "system" or "domain".  It returns a reference to an array with user dirs.
#

sub get_user_home_dirs {

	my $target = shift;
	my $domain = shift if ($target eq "domain");

	my $counter = 0;
	my (@path, $uid, $dir);
	
	if ($target eq "system") {

		setpwent;
		while (($uid,$dir) = (getpwent())[2,7]) {
			if (($uid == 0) or ($uid > 499)) {
			  $path[$counter] = "$dir";
			  $counter++;
			}
		}
		endpwent;
		return \@path;
	}
	elsif ($target eq "domain")
	{
		if (opendir(DIR, "/home/domain/$domain/users"))
		{
			my @files = readdir(DIR);
			closedir DIR;
			
			# For each file get size and time information
			foreach (@files) {
			next if ( ($_ eq ".") or ($_ eq "..") );
			$path[$counter] = "users/$_";
			$counter++;
			}   
			return \@path;
		}
		else
		{
			my $Log		= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'Backup.lrc' ]);
			my $Return	= 1400001;

			# Log the event.
			$Log->Append('error', 'warning', $Return, [ "/home/domain/$domain/users" ]);
			
			die("Unable to open the domain directory /home/domain/$domain/users.");
		}
	}
}


#
# sub print_error() prints out whatever error the calling function sends it.
#

sub print_error {
  my $var_ref = shift;
  my $error_msg = shift;
  print $var_ref->{q}->header,
	$var_ref->{q}->start_html(-title=>$StringResource->_S('ERROR'),
                                  -BGCOLOR=>'white'),
	$var_ref->{q}->h1($StringResource->_S('ERROR_OCCURED')),
        "<br>$error_msg", $var_ref->{q}->end_html;
} 



1;
__END__
