package ConfigHandler;

###############################################################################
#                                                                             #
# Filename: ConfigHandler.pm                                                  #
#                                                                             #
# Description: This module provides the ability to read a configuration file  #
#              containing a list of name=value pairs, and return the value    # 
#              corresponding to a given name.                                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;

#####################################################################
#Function: getConfig
#Arguments: listed in order -- 
#       1. Name of the config file (including the path)
#       2. The key for which a config value is to be retrieved
#Return: The config value corresponding to the key
#Description: This function looks for the key in the config file,
#       and returns the config value corresponding to that key.
#####################################################################

sub getConfig
{
        my $configFilePath = shift(@_);
        my $key = shift(@_);
	my $locale = "en";

	if(!$configFilePath)
        {
		return ""; 
	}

        if(!(-e $configFilePath) || (-z $configFilePath))
	{
		return "";
	}

	if(!$key || ($key eq ""))
	{ 
		return ""; 
	}

        my $line = "";
        my @splitLine = ("", "");

        # open the appropriate log file
        open (CONFIGFILE, "$configFilePath") or print STDERR "Can't open config file: $!";

        # search the file, one line at a time, for a string matching the key
        while($line = <CONFIGFILE>)
        {
			# search the line for a matching key
			if($line =~ /\b\A$key=/)
			{
				my $i;
				my $Cache = '';

				# split the matched line at the '=' character
				@splitLine = split /=/, $line;

				# Reassemble the line if there are '=' signs
				for $i (1 .. $#splitLine)
				{
					# For extra stuff...
					if (1 != $i)
					{
						# Add the '=' back in.
						$Cache .= '=';
					}

					# Append the data.
					$Cache .= $splitLine[$i];
				}

				# Rebuild the line.
				$splitLine[1] = $Cache;

				# remove the trailing '\n'
				chomp($splitLine[1]);
			}
        }

        # close the file handler
        close(CONFIGFILE);

        return $splitLine[1];
}

#########################################################################
#Function: getPath
#Arguments: listed in order -- 
#       1. The key for which a config value is to be retrieved
#Return: The config value corresponding to the key in the 'paths' file
#Description: This function looks for the key in the 'paths' config file,
#       and returns the path corresponding to that key.
#########################################################################

sub getPath
{
	my $pathsFile = "/usr/ins/intel/gui/config/paths";
	my $key = shift(@_);
	my $locale = "en";

	if(!$key || ($key eq ""))
	{ 
		return ""; 
	}

	return getConfig($pathsFile, $key);
}
1;
