package DisplayMain;

###############################################################################
#                                                                             #
# Filename: DisplayMain.pm                                                    #
#                                                                             #
# Description: CGI needed to display the core Web UI.                         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use QuickHelp;
use LanguageResource;

###############################################################################
# printDocument
###############################################################################

sub printDocument 
{ 
	my ($this, $PageData)	= @_;
	my $HeaderRef			= { };
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'DisplayMain.lrc' ]);
	my $RootLink			= $PageData->{'root-link'};
	my $FormFocus;
	my $Indentation			= $PageData->{'margin-left'};
	my $Feedback			= '';
	my $CookieCrumbs		= '';
	my $Title				= '';
	my $QuickHelpContent;
	my $Line;

	# Build the root link if needed...
	if (defined($RootLink))
	{
		$RootLink = <<HERE;
	<script>
		selected_item = "$RootLink";
		if(parent.navbar.selected_item)
			parent.navbar.selectitem("$RootLink");
	</script>
HERE
	}
	else
	{
		# Safe default.
		$RootLink = '';
	}
	
	# If there is a focus field on the page...
	if (defined($PageData->{'focus-field'}))
	{
		my $Form		= $PageData->{'focus-form'};
		my $Selected	= '';

		# Safe default.
		$Form		= 'forms[0]' unless(defined($Form));
		$Selected	= "document.$Form.$PageData->{'focus-field'}.select();" if((defined($PageData->{'result'})) && (0 != $PageData->{'result'}));

		$FormFocus = <<HERE;
	<script>
		this.focus();
		document.$Form.$PageData->{'focus-field'}.focus();
		$Selected
	</script>
HERE
	}
	else
	{
		# Safe default.
		$FormFocus = '';
	}

	# Set up the indentation.
	if (defined($Indentation))
	{
		my $Spacing = $Indentation * 10;

		$Indentation = <<HERE;
		<td align=left width=$Spacing nowrap>&nbsp;</td>
HERE
	}
	else
	{
		# Safe default.
		$Indentation = '';
	}
	
	# Safe default.
	$PageData->{'bgcolor'}	= 'white' unless(defined($PageData->{'bgcolor'}));
	$PageData->{'refresh'}	= ''  unless(defined($PageData->{'refresh'}));
	$PageData->{'q'}		= new CGI unless(defined($PageData->{'q'}));

	# If there are any results to display...
	if (defined($PageData->{'result'}))
	{
		my $Heading;
		my $FontColor;
		
		# If there is any status information...
		if ((defined($Feedback = $PageData->{'feedback'})) &&
			(length($Feedback)))
		{
			# Append a line break.
			$Feedback = "<br>$Feedback";
		}
		else
		{
			# Safe default.
			$Feedback = '';
		}

		# If things are good...
		if (0 == $PageData->{'result'})
		{
			# Set the sucessful heading and font color.
			$Heading	= $StringResource->_S('CHANGES_APPLIED');
			$FontColor	= 'green';
		}
		else
		{
			# Set the unsucessfull heading and font color.
			$Heading	= $StringResource->_S('OPERATION_FAILED');
			$FontColor	= 'red';
		}

		# Render the status string.
		$Feedback = "\t\t\t<font color=$FontColor><b>$Heading $Feedback</b></font>\n";
	}
  	
	# If there are crumbs...
	if (defined($PageData->{'cookiecrumbs'}))
	{
		my $CookieCrumb;
		
		# Run through any cookie crumbs...
		for $CookieCrumb (@{ $PageData->{'cookiecrumbs'} })
		{
			# Add on a space if necessary...
			$CookieCrumbs .= '&nbsp;' if($CookieCrumbs ne '');

			# Add in the cookie crumb.
			$CookieCrumbs .= "<b><font face=arial size=1><u><a href=$CookieCrumb->{location}>&lt;&nbsp;$CookieCrumb->{name}</a></u>&nbsp;</font></b>";
		}

		# Add on the tab indent if necessary.
        $CookieCrumbs = "\t\t\t$CookieCrumbs";
			
		# Add on the line feed.
		$CookieCrumbs .= "\n";
	}

	# If the page title is available...
	if (defined($PageData->{'title-html'}))
	{
		# Set up the page title.
		$Title = "\t\t\t$PageData->{'title-html'}\n";
	}
	elsif (defined($PageData->{'title'}))
	{
		# Set up the page title.
		$Title = "\t\t\t<p class=clsTitlePage>$PageData->{'title'}</p>\n";
	}

	# Get the quick help content.
	$QuickHelpContent = $this->GetQuickHelpContent($PageData);

	# Build the header information.
	$HeaderRef->{-cookies}	= $PageData->{'cookies'} if($PageData->{'cookies'});
	$HeaderRef->{-refresh}	= $PageData->{'refresh'} if($PageData->{'refresh'});
	$HeaderRef->{-refresh}	= "$PageData->{refresh_rate}; URL=$PageData->{refresh_url}?refresh_rate=$PageData->{refresh_rate}" if($PageData->{'refresh_rate'});
	$HeaderRef->{-expires}	= 'Mon, 21-May-1971 00:00:00 GMT' unless($PageData->{'cache'});
	
	# Print the header...
	print $PageData->{'q'}->header($HeaderRef);
	
	# Print the body.
	print <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<link rel=stylesheet href=/style/style.css>
	</head>
	<body bgcolor=white marginheight=2 marginwidth=2 leftmargin=0 topmargin=2>
$RootLink
	<table align=left cellspacing=0 cellpadding=0 border=0 bgcolor=$PageData->{'bgcolor'} width=100%>
	<tr align=left valign=top>
$Indentation
		<td align=left>
			<table align=left cellspacing=0 cellpadding=0 border=0 width=375>
$CookieCrumbs
			<tr>
				<td>
$Feedback
				</td>
			</tr>
			</table>
		</td>
	</tr>
	<tr align=left valign=top>
$Indentation
		<td align=left>
			<table align=left cellspacing=0 cellpadding=0 border=0 width=100%>
			<tr>
				<td>
$Title			
				</td>
			</tr>
			<tr align=left valign=top>
				<td valign=top>
$PageData->{'html'}
				</td>
$QuickHelpContent
			</tr>
			</table>
		</td>
	</tr>
	</table>
$FormFocus
	</body>
</html>
HERE
}

###############################################################################
# GetQuickHelpContent
###############################################################################

sub GetQuickHelpContent
{
	my ($this, $PageData)	= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $QuickHelp			= new QuickHelp($PreferredLanguage, $PageData->{'class'});
	my $RemoteUser			= $PageData->{'q'}->remote_user();
	my $AuthorizationGID;
	my $AccessLevel;
	my $QuickHelpTitle;
	my $QuickHelpContent;
	my $ID;
	
	# Set the authorization GID.
	$AuthorizationGID = (defined($RemoteUser)) ? (getpwnam($RemoteUser))[3] : -1;

	# Determine the user access level...
	SWITCH:
	{
		if (10000 == $AuthorizationGID) { $AccessLevel = 1; last SWITCH; } # Server administrator.
		if (10001 == $AuthorizationGID) { $AccessLevel = 2; last SWITCH; } # Domain administrator.

		# Safe default.
		$AccessLevel=3; # Regular user.
	}

    # Build the quick help ID.
	$ID = $PageData->{'id'};
	$ID .= ":$PageData->{'helpIndicator'}" if(defined($PageData->{'helpIndicator'}));
	$ID .= ":$AccessLevel";

	# Set the quick help ID.
	$QuickHelp->SetID($ID);
	
	# Try to get the title and content.
	unless ((defined($QuickHelpTitle	= $QuickHelp->GetTitle())) &&
			(defined($QuickHelpContent	= $QuickHelp->GetContent())) &&
			(length($QuickHelpTitle)) && (length($QuickHelpContent)))
	{
		# Build the basic ID.
		$ID = $PageData->{'id'};
		$ID .= ":$PageData->{'helpIndicator'}" if(defined($PageData->{'helpIndicator'}));
		
		# Set the ID.
		$QuickHelp->SetID($ID);

		# Try again to get the title and content.
		$QuickHelpTitle		= $QuickHelp->GetTitle();
		$QuickHelpContent	= $QuickHelp->GetContent()
	}
	
	# Get the quick help information...
	if ((defined($QuickHelpTitle)) && (defined($QuickHelpContent)) &&
		(length($QuickHelpTitle)) && (length($QuickHelpContent)))
	{
		# Safe default.
		$PageData->{'spillover'} = '' unless(defined($PageData->{'spillover'}));

		# If there is spillover...
		if ($PageData->{'spillover'})
		{
			# Place it in a cell.
			$PageData->{'spillover'}  = '<tr><td>' . $PageData->{'spillover'} . ' </td></tr>';
		}

		# Set up the quick help content.
		$QuickHelpContent = <<HERE;
				<td valign=top>&nbsp;&nbsp;</td> 
				<td valign=top width=1>
					<table width=183 align=right border=0 cellspacing=0 cellpadding=0>
					<tr>
						<td>
							<table align=left border=1 cellspacing=0 cellpadding=3 bordercolor=#808080>
							<tr bgcolor=gray>
								<td bgcolor=#808080><b><font color=#ffffff>$QuickHelpTitle</font></b></td>
							</tr>
							<tr bgcolor=#ffffcc>
								<td valign=top align=left>
									$QuickHelpContent
									<p><img width=183 src='/images/blank.gif' height=1></p>
								</td>
							</tr>
							</table>
						</td>
					</tr>
					$PageData->{'spillover'}
					</table>
				</td> 
HERE
	}
	else
	{
		# Safe default.
		$QuickHelpContent = '';
	}

	return $QuickHelpContent;
}

1;

__END__
