package DistributionDirective;

###############################################################################
#                                                                             #
# Filename: DistributionDirective.pm                                          #
#                                                                             #
# Description: Contains script needed to create a distribution directive.     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;

###############################################################################
# DistributionDirective::new
###############################################################################

sub new
{
	my ($class, $ID, $Class, $UserName, $UserPassword, $TargetURI, $TargetPort, $TargetNodes) = @_;

	# Initialize the object.
	my $this = 
	{
		"id"			=> $ID,
		"class"			=> $Class,
		"user-name"		=> $UserName,
		"user-password"	=> $UserPassword,
		"target-URI"	=> $TargetURI,
		"target-port"	=> $TargetPort,
		"target-nodes"	=> $TargetNodes,
	};

	# Align the object with this package.
    bless($this, $class);

	# Return the object.
	return $this;
}

###############################################################################
# DistributionDirective::GetDocument
###############################################################################

sub GetDocument
{
	my $this = shift;

	# Return the document as it currently exists.
	return 	"<directive>" . "\n" .
				"<id>" .			$this->{'id'} . 			"</id>" .				"\n" .
				"<class>" .			$this->{'class'} . 			"</class>" .			"\n" .
				"<user-name>" .		$this->{'user-name'} . 		"</user-name>" .		"\n" .
				"<user-password>" .	$this->{'user-password'} .	"</user-password>" .	"\n" .
				"<target-URI>" .	$this->{'target-URI'} .		"</target-URI>" .		"\n" .
				"<target-port>" .	$this->{'target-port'} .	"</target-port>" .		"\n" .
				"<target-nodes>" .	$this->{'target-nodes'} .	"</target-nodes>" .		"\n" .
			"</directive>" . "\n";
}

###############################################################################
# DistributionDirective::GetID
###############################################################################

sub GetID
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'id'};
}

###############################################################################
# DistributionDirective::SetID
###############################################################################

sub SetID
{
	my ($this, $ID) = @_;
	
	# Set the data.
	$this->{'id'} = $ID;

	# Return the new value.
	return $this->{'id'};
}

###############################################################################
# DistributionDirective::GetClass
###############################################################################

sub GetClass
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'class'};
}

###############################################################################
# DistributionDirective::SetClass
###############################################################################

sub SetClass
{
	my ($this, $Class) = @_;
	
	# Set the data.
	$this->{'class'} = $Class;

	# Return the new value.
	return $this->{'class'};
}

###############################################################################
# DistributionDirective::GetUserName
###############################################################################

sub GetUserName
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'user-name'};
}

###############################################################################
# DistributionDirective::SetUserName
###############################################################################

sub SetUserName
{
	my ($this, $UserName) = @_;
	
	# Set the data.
	$this->{'user-name'} = $UserName;

	# Return the new value.
	return $this->{'user-name'};
}

###############################################################################
# DistributionDirective::GetUserPassword
###############################################################################

sub GetUserPassword
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'user-password'};
}

###############################################################################
# DistributionDirective::SetUserPassword
###############################################################################

sub SetUserPassword
{
	my ($this, $UserPassword) = @_;
	
	# Set the data.
	$this->{'user-password'} = $UserPassword;

	# Return the new value.
	return $this->{'user-password'};
}

###############################################################################
# DistributionDirective::GetTargetURI
###############################################################################

sub GetTargetURI
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'target-URI'};
}

###############################################################################
# DistributionDirective::SetTargetURI
###############################################################################

sub SetTargetURI
{
	my ($this, $TargetURI) = @_;
	
	# Set the data.
	$this->{'target-URI'} = $TargetURI;

	# Return the new value.
	return $this->{'target-URI'};
}

###############################################################################
# DistributionDirective::GetTargetPort
###############################################################################

sub GetTargetPort
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'target-port'};
}

###############################################################################
# DistributionDirective::SetTargetPort
###############################################################################

sub SetTargetPort
{
	my ($this, $TargetPort) = @_;
	
	# Set the data.
	$this->{'target-port'} = $TargetPort;

	# Return the new value.
	return $this->{'target-port'};
}

###############################################################################
# DistributionDirective::GetTargetNodes
###############################################################################

sub GetTargetNodes
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'target-nodes'};
}

###############################################################################
# DistributionDirective::SetTargetNodes
###############################################################################

sub SetTargetNodes
{
	my ($this, $TargetNodes) = @_;
	
	# Set the data.
	$this->{'target-nodes'} = $TargetNodes;

	# Return the new value.
	return $this->{'target-nodes'};
}

1;

__END__
