package IteratorHash;

###############################################################################
#                                                                             #
# Filename: IteratorHash.pm                                                   #
#                                                                             #
# Description: Iteration class used to enumerate hash elements.               #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;

###############################################################################
# IteratorHash::new
###############################################################################

sub new
{
	my ($class, $HashRef) = @_;

	# Initialize the object.
	my $this = 
	{
		'location'	=> -1,
		'hash-ref'	=> $HashRef,
		'hash-keys'	=> [ keys(%{ $HashRef }) ],
	};

	# Align the object with this package.
    bless($this, $class);

	# Return the object.
	return $this;
}

###############################################################################
# IteratorHash::hasNext
###############################################################################

sub hasNext
{
	my $this = shift;

	# Return the current element count.
	return (defined($this->{'hash-ref'})) ? defined($this->{'hash-keys'}->[$this->{'location'} + 1]) : 0;
}

###############################################################################
# IteratorHash::next
###############################################################################

sub next
{
	my $this = shift;
	
	# Return the next element.
	return (defined($this->{'hash-ref'})) ? $this->{'hash-ref'}->{$this->{'hash-keys'}->[++$this->{'location'}]} : undef;
}

###############################################################################
# IteratorHash::remove
###############################################################################

sub remove
{
	my $this = shift;
	
	# If the array reference is available...
	if (defined($this->{'hash-ref'}))
	{
		# Make sure that 'next' has been called...
		if (-1 != $this->{'location'})
		{
			# Delete the hash entry.
			delete($this->{'hash-ref'}->{$this->{'hash-keys'}->[$this->{'location'}]});

			# Remove the specified key.
			splice(@{ $this->{'hash-keys'} }, $this->{'location'}, 1);

			# Back up the current location by one.
			$this->{'location'} = (1 == $this->{'location'}) ? 0 : ($this->{'location'} - 1);
		}
	}
}

1;

__END__

