package LanguageTime;

###############################################################################
#                                                                             #
# Filename: LanguageTime.pm                                                   #
#                                                                             #
# Description: Contains script needed to manage time resource data.           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use POSIX qw(strftime);

###############################################################################
# LanguageTime::new
###############################################################################

sub new
{
	my ($class, $PreferredLanguage, $RootDirectory) = @_;
	
	# Initialize the object.
	my $this =
	{
		'string-resource'	=> new LanguageResource($PreferredLanguage, $RootDirectory, [ 'LanguageTime.lrc' ]),
	};
	
	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# LanguageTime::FormatTime
###############################################################################

sub FormatTime
{
	my ($this, $Format, @Data)	= @_;
	my $Time;

	# Validate the input parameter(s)...
	if ((defined($Format)) &&
		(defined(@Data)))
	{
		my $IntFormat;

		# Look for the abbreviated week day...
		if (($Format =~ /%a/) &&
			(($` !~ /%+$/) || (0 == (length($&) % 2))))
		{
			# Add onto the itermediate format line.
			$IntFormat .= "a:%w,";
		}
		
		# Look for the abbreviated week day...
		if (($Format =~ /%A/) &&
			(($` !~ /%+$/) || (0 == (length($&) % 2))))
		{
			# Add onto the itermediate format line.
			$IntFormat .= "A:%w,";
		}
	
		# Look for the abbreviated week day...
		if (($Format =~ /%b/) &&
			(($` !~ /%+$/) || (0 == (length($&) % 2))))
		{
			# Add onto the itermediate format line.
			$IntFormat .= "b:%m,";
		}

		# Look for the abbreviated week day...
		if (($Format =~ /%B/) &&
			(($` !~ /%+$/) || (0 == (length($&) % 2))))
		{
			# Add onto the itermediate format line.
			$IntFormat .= "B:%m,";
		}

		# Look for the abbreviated week day...
		if (($Format =~ /%p/) &&
			(($` !~ /%+$/) || (0 == (length($&) % 2))))
		{
			# Add onto the itermediate format line.
			$IntFormat .= "p:%k,";
		}

		# Look for the abbreviated week day...
		if (($Format =~ /%P/) &&
			(($` !~ /%+$/) || (0 == (length($&) % 2))))
		{
			# Add onto the itermediate format line.
			$IntFormat .= "P:%k,";
		}
		
		# If there is an intermediate format...
		if (defined($IntFormat))
		{
			# Get the necessary map information.
			my @MapPoints = split(/,/, strftime($IntFormat, @Data));
			my %Map;
			my $Parsed;
			my $Token;
            my $Point;
			my $Data;
			my $Key;

			# Run through each of the conversion points...
			foreach $Point (@MapPoints)
			{
				($Token, $Data) = split(/:/, $Point);

				SWITCH :
				{
					if ('a' eq $Token) { $Map{'%a'} = $this->{'string-resource'}->_S("DAY_ABRV:$Data"); last SWITCH; }
					if ('A' eq $Token) { $Map{'%A'} = $this->{'string-resource'}->_S("DAY_LONG:$Data"); last SWITCH; }
					if ('b' eq $Token) { $Map{'%b'} = $this->{'string-resource'}->_S("MONTH_ABRV:$Data"); last SWITCH; }
					if ('B' eq $Token) { $Map{'%B'} = $this->{'string-resource'}->_S("MONTH_LONG:$Data"); last SWITCH; }
					if ('p' eq $Token) { $Data = (12 > $Data) ? 'MS_AM_UC' : 'MS_PM_UC'; $Map{'%p'} = $this->{'string-resource'}->_S($Data); last SWITCH; }
					if ('P' eq $Token) { $Data = (12 > $Data) ? 'MS_AM_LC' : 'MS_PM_LC'; $Map{'%P'} = $this->{'string-resource'}->_S($Data); last SWITCH; }
				}
			}

			# Run through the map and replace all of the localized strings...
			foreach $Key (keys(%Map))
			{
				# Run through the format string...
				while ($Format =~ s/.//)
				{
					# Cache the character.
					$Parsed .= $&;

					# Look for a token...
					if ('%' eq $&)
					{
						# Pull off another character.
						$Format =~ s/.//;

						# Cache the character.
						$Parsed .= $&;

						# If the character is not an escape character...
						if ('%' ne $&)
						{
							# Replace the key with the mapped value.
							$Parsed =~ s/$Key$/$Map{$Key}/;
						}
					}
				}

				# Reset the variable...
				$Format = $Parsed;
				
				# Reset the cache.
				$Parsed = '';
			}
		}

		# Build the time...
		$Time = strftime($Format, @Data); 
	}

	return $Time;
}

###############################################################################
# LanguageTime::GetCurrentTime
###############################################################################

sub GetCurrentTime
{
	# Format the current time...
	return FormatTime(@_, localtime());
}

1;

__END__
