package Mail;

###############################################################################
#                                                                             #
# Filename: Mail.pm                                                           #
#                                                                             #
# Description: Provides functions to manipulate Sendmail and its              #
#              configuration files.                                           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use MLDBM 'DB_File';
use LanguageLogCGI;
use VirtualDomain;
use Safe;

require Exporter;
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
@EXPORT_OK = qw(set_mail_domain set_mail_domain_aliases get_mail_domain_aliases get_site_from_domain 
		disable_mail enable_mail delete_mail_domain mail_check restart_sendmail);

$|=1;

#
# set_mail_domain
# 	[ Usage: set_mail_domain(domain1, www.domain.com) ]
#
sub set_mail_domain {

  my (%vars);
  $vars{site}   = shift;
  my $domain_name = shift;

  $vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
  open_config_file(\%vars);

  # Get values from the database into a temporary variable
  my $hash_ref = $vars{db}->{$vars{site}}; 

  $vars{old_domain_name} = $hash_ref->{domain_name} if ($hash_ref->{domain_name});

  # Update the entry/create an entry
  $hash_ref->{domain_name} = $domain_name;
  $vars{db}->{$vars{site}} = $hash_ref;

  write_local_host_names_file(\%vars);
  write_relay_domains_file(\%vars);
  close_config_file(\%vars);

}

#
# delete_mail_domain
# 	[ Usage: delete_mail_domain(domain1, www.domain.com) ]
#
sub delete_mail_domain {

  my (%vars);
  $vars{site}   = shift;
  my $domain_name = shift;

  $vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
  open_config_file(\%vars);

  # Get values from the database into a temporary variable
  my $hash_ref = $vars{db}->{$vars{site}};

  $vars{old_domain_name} = $hash_ref->{domain_name} if ($hash_ref->{domain_name});
  $vars{old_domain_aliases} = $hash_ref->{domain_aliases} if ($hash_ref->{domain_aliases});  

  # Update the entry/create an entry
  $vars{db}->{$vars{site}} = "";

  write_local_host_names_file(\%vars);
  write_relay_domains_file(\%vars);
  close_config_file(\%vars);

}


#
# set_mail_domain_aliases
#	[ Usage: set_mail_domain_aliases("domain1", "www.domain1.com", "www.domain2.com", ...) ]
#
sub set_mail_domain_aliases {

  my (%vars);
  $vars{site} = shift;
  my @domain_aliases = @_;

  $vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
  open_config_file(\%vars);

  # Get values from the database into a temporary variable
  my $hash_ref = $vars{db}->{$vars{site}};

  $vars{old_domain_aliases} = $hash_ref->{domain_aliases} if ($hash_ref->{domain_aliases});  

  $hash_ref->{domain_aliases} = [@domain_aliases];
  $vars{db}->{$vars{site}} = $hash_ref;

  write_local_host_names_file(\%vars);
  write_relay_domains_file(\%vars);
  close_config_file(\%vars);

}

#
# get_mail_domain_aliases
#	[ Usage: get_mail_domain_aliases("domain1") ]
#
sub get_mail_domain_aliases {

  my @domain_aliases;
  my (%vars);
  $vars{site} = shift;

  $vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
  open_config_file(\%vars);

  # Get values from the database into a temporary variable
  @domain_aliases = @{$vars{db}->{$vars{site}}->{domain_aliases}};

  close_config_file(\%vars);  

  return @domain_aliases;

}

#
# read_from_local_host_names_file()
#	[ Usage: read_from_local_host_names_file(%vars) ]
#

sub read_from_local_host_names_file
{
	my $HashRef = shift;

	if (open(LOCALHOSTNAMES, '/etc/mail/local-host-names'))
	{
		while (<LOCALHOSTNAMES>)
		{
			unless ((/^#/) or (/^\s*\n/))
			{
				# Remove newlines and spaces
				s/[\s+\n]//g;
				$HashRef->{'local_host_names'}->{$_} = " ";
			}
		}
		
		# Close the file.
		close(LOCALHOSTNAMES);
	}
	else
	{
		# Log the error.
		LogMessage(1500001, 'error', 'critical', [ '/etc/mail/local-host-names' ], $HashRef->{site});
	}
}


#
# read_from_relay_domains_file()
#	[ Usage: read_from_relay_domains_file(%vars) ]
#

sub read_from_relay_domains_file
{
	my $HashRef = shift;
	
	if (open(RELAYDOMAINS, '/etc/mail/relay-domains'))
	{
		while (<RELAYDOMAINS>)
		{
			unless ((/^#/) or (/^\s*\n/))
			{
				# Remove newlines and spaces
				s/[\s+\n]//g;
				$HashRef->{'relay_domains'}->{$_} = " ";
			}
		}
		
		# Close the file.
		close(RELAYDOMAINS);
	}
	else
	{
		# Log the error.
		LogMessage(1500001, 'error', 'critical', [ '/etc/mail/relay-domains' ], $HashRef->{site});
	}
}

###############################################################################
# Mail::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef, $Domain) = @_;
	my $Location	= ($Domain) ? "/home/domain/$Domain/logs/error.log" : '/usr/ins/intel/gui/logs/server.log';
	my $Log			= new LanguageLogCGI($Location, '/usr/ins/intel/gui/locale', [ 'Mail.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}


#
# write_local_host_names_file
#	[ Usage: write_local_host_names_file(\%vars) ]
#

sub write_local_host_names_file
{
	my $HashRef = shift;
	my (@domain_aliases, $domain_name);

	read_from_local_host_names_file($HashRef);
	
	# Delete from hash the old domain name 
	if ($HashRef->{old_domain_name})
	{
		delete $HashRef->{local_host_names}->{$HashRef->{old_domain_name}};
	}
	
	if ($HashRef->{old_domain_aliases})
	{
		foreach (@{$HashRef->{old_domain_aliases}})
		{
			delete $HashRef->{local_host_names}{$_};
		}
	}

	if (open(LOCALHOSTNAMES, '> /etc/mail/local-host-names'))
	{
		if (flock(LOCALHOSTNAMES, 2))
		{
			print LOCALHOSTNAMES <<HERE;
# local-host-names - include all aliases for your machine here.
# Entries from obsoleted /etc/sendmail.cw
#
HERE

			# Check to see if mail was disabled for this domain
			unless ($HashRef->{db}->{$HashRef->{site}}->{mail_disabled_by})
			{
				# Print the new domain name if it exists.
				$HashRef->{local_host_names}->{$HashRef->{db}->{$HashRef->{site}}->{domain_name}} = " " if (defined($HashRef->{db}->{$HashRef->{site}}->{domain_name}));

				# Print the new domain aliases if they exist.
				if ($HashRef->{db}->{$HashRef->{site}}->{domain_aliases})
				{ 
					foreach (@{$HashRef->{db}->{$HashRef->{site}}->{domain_aliases}})
					{
						$HashRef->{local_host_names}->{$_} = " ";
					}
				}
			} # End of mail_disabled check

			# Go through the hash with values from the config file and print them back to the config file
			foreach (keys %{$HashRef->{local_host_names}})
			{
				print LOCALHOSTNAMES "$_\n";
			}

			# Close the file.
			close(LOCALHOSTNAMES);
		}
		else
		{
			# Log the error.
			LogMessage(1500003, 'error', 'critical', [ '/etc/mail/local-host-names' ], $HashRef->{site});
		}
	}
	else
	{
		# Log the error.
		LogMessage(1500002, 'error', 'critical', [ '/etc/mail/local-host-names' ], $HashRef->{site});
	}
}

#
# write_relay_domains_file
#	[ Usage: write_relay_domains_file(\%vars) ]
#

sub write_relay_domains_file
{
	my $HashRef = shift;
	my (@domain_aliases, $domain_name);
	
	read_from_relay_domains_file($HashRef);
	
	# Delete from hash the old domain name 
	if ($HashRef->{old_domain_name})
	{
		delete $HashRef->{relay_domains}->{$HashRef->{old_domain_name}};
	}
	
	if ($HashRef->{old_domain_aliases})
	{
		foreach (@{$HashRef->{old_domain_aliases}})
		{
			delete $HashRef->{relay_domains}{$_};
		}
	}
	
	if (open(RELAYDOMAINS, '> /etc/mail/relay-domains'))
	{
		if (flock(RELAYDOMAINS, 2))
		{
			# Check to see if mail was disabled for this domain
			unless ($HashRef->{db}->{$HashRef->{site}}->{mail_disabled_by})
			{
				# Print the new domain name if it exists.
				$HashRef->{relay_domains}->{$HashRef->{db}->{$HashRef->{site}}->{domain_name}} = " " if (defined($HashRef->{db}->{$HashRef->{site}}->{domain_name}));

				# Print the new domain aliases if they exist.
				if ($HashRef->{db}->{$HashRef->{site}}->{domain_aliases})
				{ 
					foreach (@{$HashRef->{db}->{$HashRef->{site}}->{domain_aliases}})
					{
						$HashRef->{relay_domains}->{$_} = " ";
					}
				}
			} # End of mail_disabled check

			# Go through the hash with values from the config file and print them back to the config file
			foreach (keys %{$HashRef->{relay_domains}})
			{
				print RELAYDOMAINS "$_\n";
			}

			# Close the file.
			close(RELAYDOMAINS);
		}
		else
		{
			# Log the error.
			LogMessage(1500003, 'error', 'critical', [ '/etc/mail/relay-domains' ], $HashRef->{site});
		}
	}
	else
	{
		# Log the error.
		LogMessage(1500002, 'error', 'critical', [ '/etc/mail/relay-domains' ], $HashRef->{site});
	}
}

#
# open_config_file
#
sub open_config_file {

	my $HashRef = shift;
	my (%db);
	
	unless (tie(%db, 'MLDBM', $HashRef->{config_file}))
	{
		# Log the error.
		LogMessage(1500000, 'error', 'critical', [ $HashRef->{config_file} ], $HashRef->{site});
	}

	$HashRef->{db} = \%db;
}

#
# close_config_file
#
sub close_config_file {

  my $HashRef = shift;

  # Close database file 
  untie %{$HashRef->{db}};  

}

#
# get_site_from_domain   converts www.domain.com to domain1
# 	[ Usage: get_site_from_domain("www.domain.com") ]
#
sub get_site_from_domain {

  my $domain_name = shift;
  my @domains = VirtualDomain::list_vws();  
  my $site;

  foreach (@domains) {

    if ($domain_name eq $_->{vdn}) { 
      $site = $_->{dom_grp};
      return $site;
      last;
    }
  }

}

#
# disable_mail 
#	[ Usage: disable_mail($variables{site}, $Level) ]
#

sub disable_mail
{
	my %vars;
	$vars{site} = shift;
	my $Level	= shift;
	my $Return	= 0;
	
	$vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
	open_config_file(\%vars);
	
	# Get values from the database into a temporary variable
	my $hash_ref = $vars{db}->{$vars{site}};
	
	unless ($hash_ref->{mail_disabled_by})
	{
		$vars{old_domain_name}		= $hash_ref->{domain_name} if ($hash_ref->{domain_name});
		$vars{old_domain_aliases}	= $hash_ref->{domain_aliases} if ($hash_ref->{domain_aliases});
		
		# Update the entry/create an entry
		$hash_ref->{mail_disabled_by} = $Level;
		$vars{db}->{$vars{site}} = $hash_ref;
		
		write_local_host_names_file(\%vars);
		write_relay_domains_file(\%vars);
		close_config_file(\%vars);
		
		restart_sendmail();
		
		# Log the message.
		LogMessage(1500005, 'security', 'information', [ $hash_ref->{domain_name} ], $vars{site});
	}
	
	return $Return;
}

#
# enable_mail 
#	[ Usage: enable_mail($variables{site}, $Level) ]
#

sub enable_mail
{
	my %vars;
	$vars{site} = shift;
	my $Level	= shift;
	my $Return	= 0;
	
	$vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
	open_config_file(\%vars);
	
	# Get values from the database into a temporary variable
	my $hash_ref = $vars{db}->{$vars{site}};
	
	if ($Level >= $hash_ref->{mail_disabled_by})
	{
		# Update the entry/create an entry
		$hash_ref->{mail_disabled_by}	= '';
		$hash_ref->{mail_enabled_by}	= $Level;
		$vars{db}->{$vars{site}}		= $hash_ref;
		
		write_local_host_names_file(\%vars);
		write_relay_domains_file(\%vars);
		close_config_file(\%vars);
		
		restart_sendmail();
		
		# Log the message.
		LogMessage(1500004, 'security', 'information', [ $hash_ref->{domain_name} ], $vars{site});
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

#
# mail_check
#       [ Usage: mail_check("domain1") ]
#	Returns "checked" if mail is disabled for domain or a " " if it is not disabled.
#
sub mail_check {

  my $site   = shift;
  my (%vars);

  $vars{config_file} = "/usr/ins/intel/gui/config/sendmail.db";
  open_config_file(\%vars);

  if ($vars{db}->{$site}->{mail_disabled_by}) {
    return "checked";
  }
  else {
    return " ";
  }

}

#
# restart_sendmail
#       [ Usage: restart_sendmail() ]
#
sub restart_sendmail {

Safe::system("/etc/rc.d/init.d/sendmail", "restart");

}

1;

__END__
