package MonPeriod;

###############################################################################
#                                                                             #
# Filename: MonPeriod.pm                                                      #
#                                                                             #
# Description: Container class for a monitor alert directive.                 #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
# Aggregates: MonAlert, MonAlertTolerance                                     #
#                                                                             #
# References: IteratorArray                                                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonAlert;
use MonAlertTolerance;
use IteratorArray;

###############################################################################
# MonPeriod::new
###############################################################################

sub new
{
	my ($class, $Period, $Label) = @_;

	# Initialize the object.
	my $this = 
	{
		'period'			=> $Period,
		'label'				=> $Label,
		'alert-interval'	=> undef,
		'alert-tolerance'	=> undef,
		'alert-paired'		=> 0,
		'alert-limit'		=> undef,
		'alert-downtime'	=> undef,
		'alert-list'		=> [ ],
	};

	# Align the object with this package.
    bless($this, $class);

	# Return the object.
	return $this;
}

###############################################################################
# MonPeriod::getPeriod
###############################################################################

sub getPeriod
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'period'};
}

###############################################################################
# MonPeriod::setPeriod
###############################################################################

sub setPeriod
{
	my ($this, $Period) = @_;
	
	# Set the data.
	$this->{'period'} = $Period;

	return 0;
}

###############################################################################
# MonPeriod::getLabel
###############################################################################

sub getLabel
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'label'};
}

###############################################################################
# MonPeriod::setLabel
###############################################################################

sub setLabel
{
	my ($this, $Label) = @_;
	
	# Set the data.
	$this->{'label'} = $Label;

	return 0;
}

###############################################################################
# MonPeriod::getAlertInterval
###############################################################################

sub getAlertInterval
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'alert-interval'};
}

###############################################################################
# MonPeriod::setAlertInterval
###############################################################################

sub setAlertInterval
{
	my ($this, $AlertInterval) = @_;
	
	# Set the data.
	$this->{'alert-interval'} = $AlertInterval;

	return 0;
}

###############################################################################
# MonPeriod::getAlertTolerance
###############################################################################

sub getAlertTolerance
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'alert-tolerance'};
}

###############################################################################
# MonPeriod::setAlertTolerance
###############################################################################

sub setAlertTolerance
{
	my ($this, $AlertTolerance) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($AlertTolerance->isa('MonAlertTolerance'))
	{
		# Set the data.
		$this->{'alert-tolerance'} = $AlertTolerance;
	}
	else
	{
		$Return = 20003001;
	}

	return $Return;
}

###############################################################################
# MonPeriod::areAlertsPaired
###############################################################################

sub areAlertsPaired
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'alert-paired'};
}

###############################################################################
# MonPeriod::setAlertsPaired
###############################################################################

sub setAlertsPaired
{
	my ($this, $AlertsPaired) = @_;
	
	# Set the data.
	$this->{'alert-paired'} = $AlertsPaired;

	return 0;
}

###############################################################################
# MonPeriod::getAlertLimit
###############################################################################

sub getAlertLimit
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'alert-limit'};
}

###############################################################################
# MonPeriod::setAlertLimit
###############################################################################

sub setAlertLimit
{
	my ($this, $AlertLimit) = @_;
	
	# Set the data.
	$this->{'alert-limit'} = $AlertLimit;

	return 0;
}

###############################################################################
# MonPeriod::getServiceDowntimeAlertLimit
###############################################################################

sub getServiceDowntimeAlertLimit
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'alert-downtime'};
}

###############################################################################
# MonPeriod::setServiceDowntimeAlertLimit
###############################################################################

sub setServiceDowntimeAlertLimit
{
	my ($this, $ServiceDowntimeAlertLimit) = @_;
	
	# Set the data.
	$this->{'alert-downtime'} = $ServiceDowntimeAlertLimit;

	return 0;
}

###############################################################################
# MonPeriod::addAlert
###############################################################################

sub addAlert
{
	my ($this, $Alert) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Alert->isa('MonAlert'))
	{
		# Cache the object reference.
		push(@{ $this->{'alert-list'} }, $Alert);
	}
	else
	{
		$Return = 20003002;
	}

	return $Return;
}

###############################################################################
# MonPeriod::iteratorAlerts
###############################################################################

sub iteratorAlerts
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'alert-list'})) &&
		(scalar(@{ $this->{'alert-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'alert-list'});
	}

	return $Iterator;
}

1;

__END__
