package MonWatch;

###############################################################################
#                                                                             #
# Filename: MonWatch.pm                                                       #
#                                                                             #
# Description: Container class for a monitor watch directive.                 #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
# Aggregates: MonService, MonHostGroup                                        #
#                                                                             #
# References: IteratorArray                                                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonService;
use MonHostGroup;
use IteratorArray;

###############################################################################
# MonWatch::new
###############################################################################

sub new
{
	my ($class, $HostGroup) = @_;

	# Initialize the object.
	my $this = 
	{
		'host-group'	=> $HostGroup,
		'service-list'	=> [ ],
	};

	# Align the object with this package.
    bless($this, $class);

	# Return the object.
	return $this;
}

###############################################################################
# MonService::getHostGroup
###############################################################################

sub getHostGroup
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'host-group'};
}

###############################################################################
# MonService::setHostGroup
###############################################################################

sub setHostGroup
{
	my ($this, $HostGroup) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($HostGroup->isa('MonHostGroup'))
	{
		# Cache the object reference.
		push(@{ $this->{'host-group'} }, $HostGroup);
	}
	else
	{
		$Return = 20002001;
	}

	return $Return;
}

###############################################################################
# MonPeriod::addService
###############################################################################

sub addService
{
	my ($this, $Service) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Service->isa('MonService'))
	{
		# Cache the object reference.
		push(@{ $this->{'service-list'} }, $Service);
	}
	else
	{
		$Return = 20002002;
	}

	return $Return;
}

###############################################################################
# MonPeriod::iteratorServices
###############################################################################

sub iteratorServices
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'service-list'})) &&
		(scalar(@{ $this->{'service-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'service-list'});
	}

	return $Iterator;
}

1;

__END__
