package ProductInformation;

###############################################################################
#                                                                             #
# Filename: ProductInformation.pm                                             #
#                                                                             #
# Description: Contains script needed to access product information.          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;

###############################################################################
# ProductInformation::new
###############################################################################

sub new
{
	my ($class) = @_;
	
	# Initialize the object.
	my $this =
	{
		'data-resource'	=> new ConfigurationData(),
    };
	
	# Align the object with this package.
	bless($this, $class);
	
	# Try to attach to the file.
	$this->{'data-resource'}->Attach('/usr/ins/product.info');

	# Return the object.
	return $this;
}

###############################################################################
# ProductInformation::getManufacturer
###############################################################################

sub getManufacturer
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('MANUFACTURER');
}

###############################################################################
# ProductInformation::getManufacturerID
###############################################################################

sub getManufacturerID
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('MANUFACTURER_ID');
}

###############################################################################
# ProductInformation::getDescription
###############################################################################

sub getDescription
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('DESCRIPTION');
}

###############################################################################
# ProductInformation::getClassification
###############################################################################

sub getClassification
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('CLASSIFICATION');
}

###############################################################################
# ProductInformation::getVersion
###############################################################################

sub getVersion
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('VERSION');
}

###############################################################################
# ProductInformation::getFamily
###############################################################################

sub getFamily
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('FAMILY');
}

###############################################################################
# ProductInformation::getModel
###############################################################################

sub getModel
{
	my $this = shift;
	my $Model;

	# Look for a storefront configuration...
	if (2 == $this->getClassification())
	{
		# The IS4Control should be installed...
		eval
		{
			require IS4Control;

			# Get the model number.
			$Model = '13' . sprintf('%02d', IS4Control->totalLicenses());
		};
	}
	else
	{
		# Get the model.
		$Model = $this->{'data-resource'}->GetValueKeyed('MODEL');
	}

	return $Model;
}

###############################################################################
# ProductInformation::getSeries
###############################################################################

sub getSeries
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('SERIES');
}

###############################################################################
# ProductInformation::getCopyright
###############################################################################

sub getCopyright
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('COPYRIGHT');
}

###############################################################################
# ProductInformation::getTrademark
###############################################################################

sub getTrademark
{
	my $this = shift;

	# Return the data.
	return $this->{'data-resource'}->GetValueKeyed('TRADEMARK');
}

1;

__END__

