package SoftwareSupport;

###############################################################################
#                                                                             #
# Filename: SoftwareSupport.pm                                                #
#                                                                             #
# Description: Contains script needed to enable group operations.             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Safe;
use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;

###############################################################################
# new
###############################################################################

sub new
{
	my ($this, $ProductID, $ProductVersion, $DirCache, @Manifest) = @_;

	# Create a blob of data.
	$this = 
	{
		"product-id"		=> $ProductID,
		"product-version"	=> $ProductVersion,
		"dir-cache"			=> $DirCache,
		"manifest"			=> [@Manifest],
	};

	# Align the object with this package.
    bless($this, 'SoftwareSupport');

	# Return the object.
	return $this;
}

###############################################################################
# GetOfficialProductManifest
###############################################################################

sub GetOfficialProductManifest
{
	my ($this, $Manifest) = @_;
	my $Status;
	my $URLConfigFile     = '/usr/ins/intel/gui/config/swupdates.conf';
	
	# Validate the input parameter(s)...
	if ($Manifest)
	{
		my $UserAgent	 = new LWP::UserAgent;
		my $Response; 
		my $UpdateSite;
                
		# Get the update site.
		$UpdateSite = Safe::backtick("/bin/cat $URLConfigFile");
		chomp($UpdateSite);
                
		# Fill out the entire URL.
		$UpdateSite	= $UpdateSite . "/" . $this->{'product-id'} . '_manifest.htm';
		$UpdateSite = lc($UpdateSite);

		# Clean out the provided manifest.
		undef(@$Manifest);

		# Get the response.
		$Response = $UserAgent->request(HTTP::Request->new(GET => $UpdateSite));

		# If the response is good...
		if (($Status = $Response->is_success) &&
			($Status = $this->ProcessManifest($Response->content, $Manifest)))
		{
			# Set the memeber data.
			$this->{'manifest'} = [@$Manifest];
		}
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# GetCachedProductManifest
###############################################################################

sub GetCachedProductManifest
{
	my ($this, $Manifest, $Adopt) = @_;
	my $Status;

	# Validate the input parameter(s)...
	if (($Manifest) &&
		($this->{'dir-cache'}))
	{    
		# Clean out the provided manifest.
		undef(@$Manifest);

		# Try to open the cached manifest...
		if ($Status = open(MANIFEST, "$this->{'dir-cache'}manifest"))
		{
			my $ManifestRaw;
			
			# Read in the manifest.
			if (($Status = (262144 > read(MANIFEST, $ManifestRaw, 262144))) &&
				($Status = $this->ProcessManifest($ManifestRaw, $Manifest)) &&
				($Adopt))
			{
				# Set the memeber data.
				$this->{'manifest'} = [@$Manifest];                 
			}
			
			# Close the file.
			close(MANIFEST);
		}
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# GetCachedPackageListSupported
###############################################################################

sub GetCachedPackageListSupported
{
	my ($this, $PackageList) = @_;
	my $Status;
	
	# Validate the input parameter(s)...
	if (($PackageList) &&
		($this->{'dir-cache'}))
	{
		# Clean out the provided package list.
		undef(@$PackageList);

		# Everything is good.
		$Status = 1;

		# Try to open the cache directory.
		if (opendir(CACHEDIR, $this->{'dir-cache'}))
		{
			# Get all of the files.
			my @FileList	= readdir(CACHEDIR);
			my $FileRef;
			
			# Close the directory handle.
			closedir(CACHEDIR);
	
			# Run through the file list.
			foreach $FileRef (@FileList)
			{
				# If the file is a .rpm file.
				if ($FileRef =~ /^.*?(\.i386)?\.rpm/)
				{
					my $Manifest	= $this->{'manifest'}; 
					my $PackageData;
					my $i;
                       
					# For each element...
					for $i (0 .. $#$Manifest)
					{                    
						# Look for a matching file name in the manifest.
						if ($Manifest->[$i]{'file-name'} eq $FileRef)
						{
							# Create a temporary hash.
							$PackageData = {};

							# Get the field data.
							$PackageData->{'version'}		= $Manifest->[$i]{'version'};
							$PackageData->{'file-name'}		= $Manifest->[$i]{'file-name'};
							$PackageData->{'display-name'}	= $Manifest->[$i]{'display-name'};
							$PackageData->{'model'}			= $Manifest->[$i]{'model'};
							
							# Add the element to the list.
							push(@$PackageList, $PackageData);

							last;
						}
					}
				}                                
			}
		}                
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# GetCachedPackageListUnsupported
###############################################################################

sub GetCachedPackageListUnsupported
{
	my ($this, $PackageList) = @_;
	my $Status;
	
	# Validate the input parameter(s)...
	if (($PackageList) &&
		($this->{'dir-cache'}))
	{
		# Clean out the provided package list.
		undef(@$PackageList);

		# Everything is good.
		$Status = 1;

		# Try to open the cache directory.
		if (opendir(CACHEDIR, $this->{'dir-cache'}))
		{
			# Get all of the files.
			my @FileList	= readdir(CACHEDIR);
			my $FileRef;
			my $RPMName;
			
			# Close the directory handle.
			closedir(CACHEDIR);
	
			# Run through the file list.
			foreach $FileRef (@FileList)
			{
				# If the file is a .rpm file.
				if ($FileRef =~ /^.*?(\.i386)?\.rpm/)
				{
					my $Manifest		= $this->{'manifest'}; 
					my $PackageData;
					my $PackageFound	= 0;
					my $i;

					# For each element...
					for $i (0 .. $#$Manifest)
					{
						# Look for a matching file name in the manifest.
						if ($PackageFound = ($Manifest->[$i]{'file-name'} eq $FileRef))
						{
							last;
						}
                    }

					# If the package was not found in the manifest...
					if (!$PackageFound)
					{
						# Create a temporary hash.
						$PackageData = {};

						# Prep the substrings.
						$_ = $FileRef;
						$PackageData->{'version'}		= $_;
						$PackageData->{'file-name'}		= $FileRef;
						$PackageData->{'display-name'}	= $_;
						$PackageData->{'model'}			= [ '?' ];

						# Get the field data.
						$RPMName						= $this->{'dir-cache'} . '/' . $FileRef;                                                 
						$PackageData->{'version'}		= Safe::backtick("/bin/rpm -qp --queryformat '%{VERSION}-%{RELEASE}' $RPMName 2>/dev/null");
						$PackageData->{'display-name'}	= Safe::backtick("/bin/rpm -qp --queryformat '%{NAME}' $RPMName 2>/dev/null");						                                                

						# Add the element to the list.
						push(@$PackageList, $PackageData);
					}
				}
			}
		}
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# DeleteProductManifestCached
###############################################################################

sub DeleteCachedProductManifest
{
	my ($this) = @_;
	my $Status;

	# If the object is initialized...
	if ($this->{'dir-cache'})
	{
		# Delete the current manifest.
		$Status = unlink($this->{'dir-cache'} . 'manifest');
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# CacheProductManifest
###############################################################################

sub CacheProductManifest
{
	my ($this, $Preserve) = @_;
	my $Status;
	
	# If the object is initialized...
	if ($this->{'dir-cache'})
	{
		my @ManifestOld;

		# Clean out the manifest.
		undef(@ManifestOld);

		# Make the cache directory.
		mkdir($this->{'dir-cache'}, 511);

		# Try to get the old manifest.
		if ($Preserve)
		{
			# Get the old manifest.
			$this->GetCachedProductManifest(\@ManifestOld, 0);
		}
		
		# Delete the current manifest.
		DeleteCachedProductManifest;

		# If the file handle can be opened...
		if ($Status = open(MANIFEST, ">$this->{'dir-cache'}manifest"))
		{
			my $Manifest = $this->{'manifest'}; 
			my ($i, $j);

			# Set the beginning of the manifest.
			print MANIFEST "<PackageList ProductVersion=\"$this->{'product-version'}\">\n";
			
			# If the old manifest is available...
			if (defined(@ManifestOld))
			{
				# For each element in the new manifest...
				for $i (0 .. $#ManifestOld)
				{
					my $PackageInfoFound = 0;
					
					# For each element in the old manifest...
					for $j (0 .. $#$Manifest)
					{
						my $ManifestNewRef = $Manifest->[$j]{'model'};
						my $ManifestOldRef = $ManifestOld[$i]{'model'};
						my $o;
						my $p;

						# Run through the outer list...
						for $o (0 .. $#$ManifestNewRef)
						{
							# Run through the inner list...
							for $p (0 .. $#$ManifestOldRef)
							{
								last if($PackageInfoFound = ($o eq $p));
							}

							# Try to bail out early.
							last unless($PackageInfoFound);
						}

						# Make sure there is a difference...
						if ($PackageInfoFound = (($PackageInfoFound) &&
												 (scalar(@{$Manifest->[$j]{'model'}}) == scalar(@{$ManifestOld[$i]{'model'}})) &&
												 ($Manifest->[$j]{'display-name'}     eq $ManifestOld[$i]{'display-name'}) &&
												 ($Manifest->[$j]{'version'}          eq $ManifestOld[$i]{'version'})      &&
												 ($Manifest->[$j]{'location'}         eq $ManifestOld[$i]{'location'})     &&
												 ($Manifest->[$j]{'file-name'}        eq $ManifestOld[$i]{'file-name'})))
						{
							last;
						}
					}

					# If the package was not found...
					if (!$PackageInfoFound)
					{
						# Re-join the model numbers.
						my $Model = join(',', @{ $ManifestOld[$i]{'model'} });
						
						# Set the current package information.
						print MANIFEST "<Package Model=\"$Model\" DisplayName=\"$ManifestOld[$i]{'display-name'}\" Version=\"$ManifestOld[$i]{'version'}\" Location=\"$ManifestOld[$i]{'location'}\" FileName=\"$ManifestOld[$i]{'file-name'}\"/>\n";
					}
				}
			}
				
			# For each element...
			for $i (0 .. $#$Manifest)
			{
				# Re-join the model numbers.
				my $Model = join(',', @{ $Manifest->[$i]{'model'} });

				# Set the current package information.
				print MANIFEST "<Package Model=\"$Model\" DisplayName=\"$Manifest->[$i]{'display-name'}\" Version=\"$Manifest->[$i]{'version'}\" Location=\"$Manifest->[$i]{'location'}\" FileName=\"$Manifest->[$i]{'file-name'}\"/>\n";
			}

			# Set the ending of the manifest.
			print MANIFEST "</PackageList>\n";

			# Close the file.
			close(MANIFEST);
		}
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# ProcessManifest
###############################################################################

sub ProcessManifest
{
	my ($this, $ManifestRaw, $Manifest)	= @_;
	my $Status;

	# Validate the input parameter(s)...
	if ($ManifestRaw)
	{
		my $PackageListBegin	= '<PackageList ProductVersion="' . $this->{'product-version'} . '">';
		my $PackageListEnd		= '</PackageList>';
		my @ManifestList;
		my $Line;
		my $i;
		
		# Prep the manifest.
		$ManifestRaw	=~ s/\n//sg;
		$ManifestRaw	=~ s/\cM//g;
		
		# Try to match the correct package information...
		if ($ManifestRaw =~ /^.*$PackageListBegin(.*?)$PackageListEnd.*$/)
		{
			# Parse down the raw manifest...
			$ManifestRaw	=~ s/^.*$PackageListBegin(.*?)$PackageListEnd.*$/$1/;
			@ManifestList	=  split(/<Package(.*?)\/>/, $ManifestRaw);

			# Run through the list.
			for $i (0 .. $#ManifestList)
			{
				# If the element has something in it...
				if ((defined($ManifestList[$i])) &&
					(1 < length($ManifestList[$i])))
				{
					# Create a temporary hash.
					$Line = {};

					# Prep the substrings.
					$Line->{'version'} = $Line->{'location'} = $Line->{'file-name'} = $Line->{'display-name'} = $Line->{'model'} = $ManifestList[$i];

					# Validate the data...
					if (($Line->{'version'}      =~ /.*Version="(.*?)".*/) &&
						($Line->{'location'}     =~ /.*Location="(.*?)".*/) &&
						($Line->{'file-name'}    =~ /.*FileName="(.*?)".*/) &&
						($Line->{'display-name'} =~ /.*DisplayName="(.*?)".*/) &&
						($Line->{'model'}        =~ /.*Model="(.*?)".*/))
					{
						# Get the data.
						$Line->{'version'}		=~ s/.*Version="(.*?)".*/$1/;
						$Line->{'location'}		=~ s/.*Location="(.*?)".*/$1/;
						$Line->{'file-name'}	=~ s/.*FileName="(.*?)".*/$1/;
						$Line->{'display-name'}	=~ s/.*DisplayName="(.*?)".*/$1/;
						$Line->{'model'}		=~ s/.*Model="(.*?)".*/$1/;
						$Line->{'model'}		= [ split(/,/, $Line->{'model'}) ];
						
						# Add the element to the list.
						push(@$Manifest, $Line);
					}
				}
			}

			$Status = 1;
		}
		else
		{
			$Status = 0;
		}
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# GetProductUpdate
###############################################################################

sub GetProductUpdate
{
	my ($this, $URL, $Target) = @_;

	# Validate the input parameter(s)...
	if (($URL) &&
		($Target) &&
		($this->{'dir-cache'}))
	{
		# Get the response.
		my $UserAgent	= LWP::UserAgent->new;
		my $Response	= $UserAgent->request(HTTP::Request->new(GET => $URL), $this->{'dir-cache'} . $Target);
		
		# If the response is good...
		return ($Response->is_success);
	}

	return 0;
}

###############################################################################
# GetProductID
###############################################################################

sub GetProductID
{
	my $this = shift;

	# Return the requested data.
	return $this->{'product-id'};
}

###############################################################################
# SetProductID
###############################################################################

sub SetProductID
{
	my ($this, $ProductID) = @_;

	# Set the data.
	$this->{'product-id'} = $ProductID;

	# Return the new value.
	return $this->{'product-id'};
}

###############################################################################
# GetProductVersion
###############################################################################

sub GetProductVersion
{
	my $this = shift;

	# Return the requested data.
	return $this->{'product-version'};
}

###############################################################################
# SetProductVersion
###############################################################################

sub SetProductVersion
{
	my ($this, $ProductVersion) = @_;

	# Set the data.
	$this->{'product-version'} = $ProductVersion;

	# Return the new value.
	return $this->{'product-version'};
}

###############################################################################
# GetDirCache
###############################################################################

sub GetDirCache
{
	my $this = shift;

	# Return the requested data.
	return $this->{'dir-cache'};
}

###############################################################################
# SetDirCache
###############################################################################

sub SetDirCache
{
	my ($this, $DirCache) = @_;

	# Set the data.
	$this->{'dir-cache'} = $DirCache;

	# Return the new value.
	return $this->{'dir-cache'};
}

1;

__END__
