#!/usr/bin/perl 
package Users;
$VERSION="0.54";

###############################################################################
#                                                                             #
# Filename: Users.pm                                                          #
#                                                                             #
# Description: This module provides configuration support for managing system #
#              users.                                                         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 
use strict;
use CGI;
use IO::File;
use VirtualDomain;
use LanguageResource;
use LanguageLogCGI;
use Quotaall;
use DomainMail;
use Password;
use ConfigHandler;
use Safe;
require 'syscall.ph';

# Globals
my $PreferredLanguage	= (defined($ENV{'HTTP_ACCEPT_LANGUAGE'})) ? (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0] : 'en-US';
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'Users.lrc' ]);

###############################################################################
# Users::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef) = @_;
	my $Log = new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'Users.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}

######################################################################
# global settings
######################################################################
# path to the /etc/passwd
use vars qw($passwd $shadow);
$passwd = "/etc/passwd";
$shadow = "/etc/shadow";

# a flock() is on this file while virtusertable operations
# are in progress
use vars qw($lockfile_passwd $lockfile_shadow);
$lockfile_passwd = "/var/lock/usrpasswd.lock";
$lockfile_shadow = "/var/lock/usrshadow.lock";

######################################################################
# global variables
######################################################################

# the file handle for the lock file
use vars qw($fh_passwd $fh_shadow);
$fh_shadow = undef;
$fh_passwd = undef;

######################################################################


#############
sub usrAdd {
        my $result = "User";
	my ($real,$hom,$grp,$skel,$shel,$sgrp,$login,$passw) = @_;
	$skel = '/usr/ins/intel/gui/skel/user'; 
	my $domainadmin = 10001;
	my $serveradmin = 10000;	
	my @mod_arg;
	my $ex_user = getpwnam($login);
        return ( $StringResource->_S('USER_EXISTS_TRY_AGAIN', [ $login ] )) if ($ex_user);
	my @user_args = ("-c","$real","-s","$shel","-d","$hom","-m","-k","$skel","-g","$grp","$login");
	if ($sgrp ne "") { push @user_args,"-G", $sgrp; }
	my $user_spec = "/usr/sbin/useradd";
	my $user_mod = "/usr/sbin/usermod";
        
	# Lock here 
	my $return_lock = Users::lock();
        return $return_lock if ($return_lock != 1); 
	$result = Safe::system($user_spec , @user_args);
        if ($result != 0) {
                        $result = Safe::backtick("$user_spec @user_args 2>&1");
                        ($result) = (split /:/,$result)[1];
                       	# unlock before returning
			Users::unlock();
			return( $StringResource->_S('USER_ADD_FAILED', [ $login ] ). ": $result");
        }

	if ($grp == $domainadmin ) { 
		$hom =~ s/\/(\w+)\/(\w+)\/(\w+\d+)\/(\S+)/\/$1\/$2\/$3\/\.\/$4/;
                my $home_for_chroot = "/$1/$2/$3/";
                &create_chroot_environment("$home_for_chroot");
	        @mod_arg = ("-d","$hom","$login");
                Safe::system($user_mod,@mod_arg);
	}
	elsif (($grp != $serveradmin) && ($grp != $domainadmin)) {
                &create_chroot_environment("$hom");
		$hom = "$hom/./";
		@mod_arg = ("-d","$hom","$login");
		Safe::system($user_mod,@mod_arg);
	 }
	
	# Change passwd here 
	my $return_pass = &usrPasswd($login,$passw);
	unless ( $return_pass =~ /YES/ ) {
                $return_pass = $StringResource->_S('PASSWORD_SETTING_FAILED', [ $login ]);
                LogMessage('PASSWORD_SETTING_FAILED', 'security', 'information', [ $login ]);
        	Users::unlock();
	        return $return_pass;
        }
	# unlock here also 
	Users::unlock();			
	chmod 0711, $hom;
	mbox($login,$hom);
	$result = "User";
	return $result;
}

sub createAdmin {
	my ($login,$real,$passw,$previlage,$hom,$sgrp) = @_;
	my $skel = '/usr/ins/intel/gui/skel/user'; 
	my $user_mod = "/usr/sbin/usermod";
	my ($result,$grp,$ex_user,@mod_arg);
	$ex_user = getpwnam($login);
        return ("exist") if ($ex_user);
	if ($hom eq '') {	
		unless ( -d "/home/domain/domain/users") {
			mkdir("/home/domain/domain/users", 0775);	
		}	
		 $hom = "/home/domain/domain/users/$login"; 
	}
	if ($previlage eq 'serveradmin') {$grp = '10000'; }
	if ($previlage eq 'domainadmin') { $grp = '10001'; }
	my $shel = '/bin/bash';	
	# Lock here
        my $return_lock = Users::lock();
        return $return_lock if ($return_lock != 1);
	my @user_args = ("-c","$real","-s","$shel","-d","$hom","-m","-k","$skel","-g","$grp");
	if ($sgrp ne "") { push @user_args,"-G", $sgrp; }
	push(@user_args,$login); 
	my $user_spec = "/usr/sbin/useradd";
        
	$result = Safe::system($user_spec , @user_args);
	if ($result != 0) {
                        $result = Safe::backtick("$user_spec @user_args 2>&1");
                        ($result) = (split /:/,$result)[1];
                       	Users::unlock();
			 return($StringResource->_S('USER_ADD_FAILED', [ $login ] ). ": $result");
        }


	if ($previlage eq 'domainadmin') {
                $hom =~ s/\/(\w+)\/(\w+)\/(\w+\d+)\/(\S+)/\/$1\/$2\/$3\/\.\/$4/;
                my $home_for_chroot = "/$1/$2/$3/";
                @mod_arg = ("-d","$hom","$login");
                &create_chroot_environment("$home_for_chroot");
                Safe::system($user_mod,@mod_arg);
        }

	################################################################################
	# Set Password
	################################################################################
	$result = Password->changePassword($login, $passw);
	if (0 == $result) { $result = 'YES'; }
	else 			  { return ($StringResource->_S('PASSWORD_SETTING_FAILED', [ $login ] )); }
	Users::unlock();
	mbox($login,$hom);
	chmod 0711, $hom;
	$ex_user = getpwnam($login);
	# Set quota for PDA from here ... 20 MB.
        Quotaall::setquota($ex_user,"20");
	# Default of 20 MB for everyone.
	return ($result);
}
############
sub usrMod {
	my ($newlog,$oldname,$real,$hom,$grp,$shel,$sgrp) = @_;
        my $result;
	my $gid = (getpwnam($oldname))[3];
	my $domainadmin = 10001;
	my $serveradmin = 10000;
	$shel = "/bin/bash" if (!$shel);
	my $user_spec = "/usr/sbin/usermod";
	
	if ($grp == $domainadmin ) {
                $hom =~ s/\/(\w+)\/(\w+)\/(\w+\d+)\/(\S+)/\/$1\/$2\/$3\/\.\/$4/;
                my $home_for_chroot = "/$1/$2/$3/";
                &create_chroot_environment("$home_for_chroot");
        }
        elsif (($grp != $serveradmin) && ($grp != $domainadmin)) {
                &create_chroot_environment("$hom");
                $hom = "$hom/./";
        }
	my @user_args = ("-c","$real","-s","$shel","-d","$hom","-m","-l","$newlog");
	if ($sgrp ne "") { push(@user_args,"-G",$sgrp); }
	if ($grp ne "") { push(@user_args,"-g",$grp); }
	push(@user_args, $oldname);
        # Lock here
        my $return_lock = Users::lock();
        return $return_lock if ($return_lock != 1);

	$result = Safe::system($user_spec , @user_args);
	if ($result != 0) {
                        $result = Safe::backtick("$user_spec @user_args 2>&1");
                        ($result) = (split /:/,$result)[1];
                        Users::unlock();
			return( $StringResource->_S('USER_MOD_FAILED', [ $newlog ] ). ": $result");
        }
	Users::unlock();
        return 1;
}
##########
sub usrDel
{
        my ($user) = @_;
        my $result;
	# Lock here
        my $return_lock = Users::lock();
        return $return_lock if ($return_lock != 1);
	my $uid = getpwnam($user);
	my $quota = 0;
	my ($home) = (getpwnam($user))[7];
	my $spool_mail = "/var/spool/mail\/$user";
	# if ( -e "$spool_mail") {
	#	unlink($spool_mail);
	# } 

	my @user_arg = ("$user");
	if ($home) { @user_arg = ("-r","$user"); }
        my $userdel = "/usr/sbin/userdel";
	DomainMail::modAlias($user,'','');
	
	# delete users from "ftpusers" if listed
	my $ret1 = DomainMail::ftpaccess($user);
	DomainMail::enftp($user) if ($ret1);
	my $ret2 = DomainMail::mailaccess($user);
	DomainMail::enmail($user) if ($ret2);

	# Set quota to null 
	Quotaall::setquota($uid,$quota);
        $result = Safe::system($userdel, @user_arg) if($uid);
	if ($result != 0) {
		if ($uid) {	 
			$result = Safe::backtick("$userdel @user_arg 2>&1"); 
			($result) = (split /:/,$result)[1];
			$result .= "<BR>";			
			Users::unlock();
			return( $StringResource->_S('USER_DELETE_FAILED', [ $user ] ) . ": $result");
		}
	}
	Users::unlock();
	return 1;
}

#########
sub usrPasswd
{
	my ($usr, $npw) = @_;
	my $result = Password->changePassword($usr, $npw);
	if (0 == $result) { $result = 'YES'; }
		return ($result);
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
sub systemRun
{    
        my ($runspec) = @_;
        my $pipespec = "$runspec |";
        my $fho = new IO::File $pipespec;
        unless ( defined $fho )
        {
                # $logo->append_something_to_a_log;
                exit(1);
        }
        my $result = <$fho>;
        $fho->close;
        return $result;

}

################################################################
sub listUsrs {
       my $line = 0;
	my (@paswd,@val,$dom,$domain_name)= '';
	my @domains = VirtualDomain::list_vws();
	my $sadmin = 10000;
	my $dadmin = 10001;
	my $server_admin = $StringResource->_S('SERVER_ADMIN');
	my $domain_admin = $StringResource->_S('DOMAIN_ADMIN');
	my ($useralias,$right);
	my $type;

	# preload a hashtable with the domains.
	# the key is the domain group (site name) and the value is the vdn (name of domain).
	my %domains_hash;
	foreach my $item (@domains) {
		$domains_hash{$item->{dom_grp}} = $item->{vdn};
	}

        open(PSD, '/etc/passwd');
        while(<PSD>) {
                s/\r|\n//g;
                if (/\S/ && !/^[#\+\-]/) {
                        @paswd = split(/:/, $_, -1);
#                        if ($paswd[2] > '100') {
				($dom) = (split /\//, $paswd[5])[3];

				# get the name of the domain from the previously loaded hash table
				if ( ($dom ne '') && (exists $domains_hash{$dom}) && ($domains_hash{$dom} ne '') ) {
					$domain_name = $domains_hash{$dom};
				} else {
					$domain_name = 'None';
				}

				$useralias = DomainMail::userAlias($paswd[0],$domain_name);
				$right = 'User';
				$right = $server_admin if ($paswd[3] == $sadmin);
				$right = $domain_admin if ($paswd[3] == $dadmin);
				$type  = 0;
				$type  = $sadmin if ($paswd[3] == $sadmin);
				$type  = $dadmin if ($paswd[3] == $dadmin);
				$paswd[4] = "&nbsp" if ($paswd[4] eq '');	
				$useralias = "&nbsp" if ($useralias eq '');	
				push(@val, { 'user' => $paswd[0],   'pass' => $paswd[1],
                                    'uid' => $paswd[2],    'gid' => $paswd[3],
                                    'real' => $paswd[4],   'home' => $paswd[5],
				    'domain'=> $dom,       'shell' => $paswd[6],  
				    'line' => $line,	   'domain_name' => $domain_name,	
				    'right' => $right,	   'type' => $type, 'mailalias' => $useralias,  	
                                    'num' => scalar(@val) });
#			$idx{$paswd[0]} = $val[$#val];
#                	}	     
		   }
		$domain_name = $useralias = '';
                $line++;
                }
        close(PSD);
return @val;
}
########################################################
sub listGrps
{
my (@val, $line, %idx, @grp, $i, $j);
$line = 0;
open(GRP, '/etc/group');
while(<GRP>) {
        s/\r|\n//g;
        if (/\S/ && !/^[#\+\-]/) {
                @grp = split(/:/, $_, -1);
                push(@val, { 'group' => $grp[0],  'pass' => $grp[1],
                            'gid' => $grp[2],    'mem' => $grp[3],
                            'line' => $line,    'num' => scalar(@grp) });
#  	       $idx{$grp[0]} = $grp[$#rv];
                }
        $line++;
        }
close(GRP);

return @val;

}

##############################
sub maxGid
{
	my (@group, $item,@gids, $gid,);	
	@group = &listGrps;
	@gids = sort { "$a->{'gid'}" <=> "$b->{'gid'}" } @group;
		foreach $item ( @gids) {
			$gid = $item->{'gid'};
		}
	return $gid;

}


#######################################
sub domainGrp 

{
	my ($newgid, $lastgrp,@group,@gids,@dgrp,$item,$grpname, $newgrp,$grpnum)='';
	my $domain_gid = '11001';
        @group = listGrps;
        @gids = sort { "$a->{'gid'}" <=> "$b->{'gid'}" } @group;
	foreach $item (@gids) {
		if ($item->{'group'} =~ /domain(\d+)/) {
			$grpname = $item->{'group'};
			push(@dgrp,$grpname);
		}
	}
	my $totaldomain = scalar(@dgrp);
	@dgrp = sort {$a <=> $b} @dgrp;
	$lastgrp = pop(@dgrp);	
	if ($lastgrp =~ /domain(\d+)/) { 
		$grpnum = $1;
		$newgrp = 'domain' . ($grpnum + 1);
		$newgid = (getgrnam($lastgrp) + 1); 
	}		
	else {
		$newgrp = 'domain' . '1';
		$newgid = $domain_gid;
	}		
	return ($newgrp, $newgid , $lastgrp);

}

##################################################
sub grpAdd

{
	my ($nwgrp, $nwgid) = @_;
	my (@add_args,$add_grp, $grpadd, $result, $lgrp);
	if ($nwgrp eq '' && $nwgid eq '') { 
		($nwgrp, $nwgid, $lgrp) = domainGrp;
	}
	@add_args = ("-g", "$nwgid","$nwgrp");
	$grpadd = "/usr/sbin/groupadd";
        Safe::system($grpadd, @add_args) == 0 or  return($StringResource->_S('DOMAIN_GROUP_CREATE_FAILED', [ $nwgrp ] ) . ": $!");
}


####################################################
sub grpDel 

{
	my ($grpnam, $some) = @_;

        my $del_args = ("$grpnam");
        my $groupdel = "/usr/sbin/groupdel";
       
        Safe::system($groupdel, $del_args) == 0 or return($StringResource->_S('DOMAIN_DELETE_FAILED', [ $grpnam ] ) . ": $!");
}


######################################################

sub mbox 
{
	my ($user,$home) = @_;
	my $touch = "/bin/touch";
	my $gid = (getpwnam('mail'))[3];
	my $uid = (getpwnam($user))[2];
	my $spool_mail = "/var/spool/mail";
	Safe::system($touch,"$home/mbox");
	chown $uid,$gid,"$home/mbox";	
	chmod(0660,"$home/mbox");
	# symlink("$home/mbox","$spool_mail/$user");
	# chown $uid,$gid,"$spool_mail/$user";	
}
sub getUserDomain {

        my ($user,$some) = @_;
        my ($usr,$domain,$i,$item,$vdname);
        my $none = 'None';
        my @alluser = &listUsrs();
        foreach $item (@alluser) {
                if ($item->{user} eq $user) {
                	$usr = $item->{user}; 
                	$domain = $item->{domain};
        		$vdname = $item->{domain_name};
			last;
		}
	}

        return $none if (!($domain =~ /domain/));
        return $vdname;

}

#
# &create_chroot_environment
#
# Given a valid directory, create an FTP chroot() environment
# underneath it, by creating the proper directories, special files,
# and hard linking the static binaries.
#
# NOTE: Because hard links are used, the specified directory MUST
# reside on the same partition as the central binary directory.
# That is, the /home partition.
#

sub create_chroot_environment {
	my ($directory) = @_;

	my $ftp_anonymous_root = ConfigHandler::getPath("FTP_ANONYMOUS_ROOT");
	my $bin_directory = $ftp_anonymous_root . "/bin";
	my $lib_directory = $ftp_anonymous_root . "/lib";
	my $etc_directory = $ftp_anonymous_root . "/etc";

	#
	# scan {bin,lib,etc} directories to get a list of files
	#

	# /bin
	opendir DIR, $bin_directory;
	my @bin_files = grep { (! /^\./) && -f "$bin_directory/$_" } readdir(DIR);
	closedir DIR;

	# /lib
	opendir DIR, $lib_directory;
	my @lib_files = grep { (! /^\./) && -f "$lib_directory/$_" } readdir(DIR);
	closedir DIR;

	# /etc
	opendir DIR, $etc_directory;
	my @etc_files = grep { (! /^\./) && -f "$etc_directory/$_" } readdir(DIR);
	closedir DIR;

	#
	# create directories
	#

	if (! -d $directory) {
		return ($StringResource->_S('NO_DIRECTORY') . ": $!");
	}

	# /bin
	if (! -d "$directory/bin") {
		mkdir("$directory/bin",0777)
			|| return ($StringResource->_S('FTP_DIR_CREATE_ERROR') . ": $!");
	}

	# /lib
	if (! -d "$directory/lib") {
		mkdir("$directory/lib",0777)
			|| return ($StringResource->_S('FTP_DIR_CREATE_ERROR') . ": $!");
	}

	# /etc
	if (! -d "$directory/etc") {
		mkdir("$directory/etc",0777)
			|| return ($StringResource->_S('FTP_DIR_CREATE_ERROR') . ": $!");
	}

	#
	# make the hard links
	#

	# /bin
	foreach my $i (@bin_files) {
		if (! -f "$directory/bin/$i") {
			link("$bin_directory/$i","$directory/bin/$i")
				|| return ($StringResource->_S('CANNOT_LINK') . ": $!");
		}
	}

	# /lib
	foreach my $i (@lib_files) {
		if (! -f "$directory/lib/$i") {
			link("$lib_directory/$i","$directory/lib/$i")
				|| return ($StringResource->_S('CANNOT_LINK') . ": $!");
		}
	}

	# /etc
	foreach my $i (@etc_files) {
		if (! -f "$directory/etc/$i") {
			link("$etc_directory/$i","$directory/etc/$i")
				|| return ($StringResource->_S('CANNOT_LINK') . ": $!");
		}
	}

	#
	# misc operations
	#

	# assure proper modes
	chmod 0711, "$directory/bin", "$directory/lib";
	chmod 0755, "$directory/etc";

	# assure proper ownership
	chown 0, 0, "$directory/bin", "$directory/etc", "$directory/lib";
1;
}

#######################################

sub primaryDA {
	
	my ($user,$some) = @_;
	return "0" if ($user eq '');
	my ($home_for_domain,$home_uid);
	my ($uid,$home) = (getpwnam($user))[2,7];	
	$home =~ /\/(\w+)\/(\w+)\/(\w+\d+)\/(\S+)/;
        $home_for_domain = "/$1/$2/$3/";
	$home_uid = (stat($home_for_domain))[4];	
	if ($uid == $home_uid) { return "1"; }
	return "0";

}

##########################################
# locking mechanism for /etc/passwd and /etc/shadow;

# Open a lock file for append (creating it if needed) and
# try to get a flock() on it.


sub lock {
    $fh_passwd = new IO::File;
    $fh_shadow = new IO::File;
    $fh_shadow->open(">> $lockfile_shadow") || return  ( $StringResource->_S('PASSWD_FILE_OPEN_ERROR'));
    flock($fh_shadow,2) || return ( $StringResource->_S('PASSWD_FILE_LOCK_ERROR'));
    $fh_passwd->open(">> $lockfile_passwd") || return  ( $StringResource->_S('PASSWD_FILE_OPEN_ERROR'));
    flock($fh_passwd,2) || return ( $StringResource->_S('PASSWD_FILE_LOCK_ERROR'));
    return "1";	

}

# &unlock

# Close the file handle to the lock file, thereby
# releasing the lock.  We DO NOT try to unlink the file,
# since that would raise concurrency issues.

sub unlock {
    $fh_shadow->close;
    $fh_passwd->close;
    undef $fh_shadow;
    undef $fh_passwd;

}

1;

