package VirtualDomain;

###############################################################################
#                                                                             #
# Filename: VirtualDomain.pm                                                  #
#                                                                             #
# Description: This module provides configuration support for managing        #
#              virtual domains.                                               #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use vars qw(@ISA @EXPORT @EXPORT_OK);
use strict;
use LanguageResource;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 
use File::Copy;
use Usable;
use VirtualServ;
use NetConfig;
use RouteConfig;
use Users;
use VirtualFtp;
use SetNic;
use Safe;

# Globals
my $PreferredLanguage	= (defined($ENV{'HTTP_ACCEPT_LANGUAGE'})) ? (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0] : 'en-US';
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'VirtualDomain.lrc' ]);

############################################################################
# SName: list_vws() - Read the http.conf file and generate a hash 
#	        array of virtual Server configuration
# Arguments: None
# Return Value:Array of Virtual web server configuration

sub list_vws {
my (%mm,@rv,@lines,$l,$g1,$g2,$g3,$g4);
my ($adm,$mail);
my $APACHE_CONF = "/etc/httpd/conf/httpd.conf";
if (!(-e "$APACHE_CONF")) { return; }
open(APCON,"$APACHE_CONF") || return ($StringResource->_S('APACHE_CONF_FILE_MISSING', [ $APACHE_CONF ] ));
	while(<APCON>){
		s/\r|\n//g;
		if (/^<VirtualHost .*>$/ ... /^<\/VirtualHost>$/) {
		push(@lines, $_);	
		}
    }
close(APCON);
# Read each line of virtual configuration and generate a hash array 
foreach $l (@lines) {
	if ($l =~ /^<VirtualHost ([\d\.]+):(\d+)/) { $mm{'vdip'} = $1; $mm{'port'} = $2; }
	if ($l =~ /ServerName (\S+)/) { $mm{'vdn'} = $1; }	
	if ($l =~ /ServerAdmin (\S+)/) { 
			($adm,$mail) = split (/\@/, $1);
			$mm{'vdadm'} = $adm; 
		}
	if ($l =~ /ServerAlias (\S+)/) { $mm{'vdalias'} = $1; }	
	if ($l =~ /DocumentRoot (\S+)/) { 
		$mm{'docroot'} = $1; 
		($g1, $g2, $g3, $g4) = split /\//, $1;					
		$mm{'dom_grp'} = $g4;
		$mm{'dom_hom'} = "\/$g2\/$g3";
		}	
	if ($l =~ /ErrorLog (\S+)/) { $mm{'errlog'} = $1; }	
	if ($l =~ /TransferLog (\S+)$/) { $mm{'acclog'} = $1; }	
	if ($l =~ /ScriptAlias\s+\/cgi-bin\/\s+(\S+)$/) { $mm{'vdcgi'} = 'yes'; }
	if ($l =~ /server-parsed (\S+)$/) { $mm{'vdssi'} = 'yes'; }	
	if ($l =~ /RewriteCond (\S+)$/) {
		$l =~ s/(\D)//g;
		$mm{'vdport'} = $l;
		}	
	if ($l =~ /FrontpageEnable$/) { $mm{'vdfpx'} = 'yes'; }
	if ($l =~ /SSLEnable$/) { $mm{'vdssl'} = 'yes'; }
	if ($l =~ /^<\/VirtualHost>$/) { 
		$mm{'idx'} = scalar(@rv); 	
		$mm{'edit'} = ($mm{'vdn'});
		push @rv, { %mm };
	$mm{vdadm} = $mm{port} = $mm{vdn} = $mm{vdip} = $mm{vdssl} = $mm{vdcgi} = $mm{vdssi} = $mm{vdalias} = $mm{vdfpx} = $adm = $mail = $g1 = $g2 = $g3 = $g4 = $l = '';
	}
}

return (@rv);
}

################################################################################
# This adds the Virtual Server Configuration to the Apache
# SName: vd_conf( IP, ServerName, ServerAlias, Admin,Port, SSI, CGI)
# Default Virtual Server Configuration added to the apache configuration file
# This is for Virtual Web domain support with cgi-wrapper for every domain ###
# Arguments are $_[0] to $_[7] in the following order 
# ( IP, hostName,admin,Serveralias,Docroot,port,SSI,CGI,SSL )	
sub vd_conf 
{
	my ($ip,$vdname,$admin,$alias,$grp,$ssi,$cgi,$ssl,$sfpx) = @_;
	my ($home,$web,$err_log,$acc_log,$http,$port);
	$home = "/home/domain/$grp";
	$web = 'web';
	$err_log = 'logs/error_log';
	$acc_log = 'logs/access_log';
	
	if ($ssl eq 'Yes')
	{
		 $port = '443';
		 $http = 'https';
	}
	else
	{ 
		$port = '80';
		$http = 'http';
	}

	# Support for Name and IP based Virtual hosting
	

	my $my='$';
	# print CONF "<VirtualHost $ip>";
	print CONF "<VirtualHost $ip:$port>\n";
	print CONF "ServerName $vdname\n";
	print CONF "ServerAdmin $admin\@$_[1]\n";
	print CONF "ServerAlias $alias\n";
	print CONF "DocumentRoot $home/$web\n";
	print CONF "ErrorLog \"|/usr/ins/intel/bin/webspy $home\/$err_log\"\n";
	print CONF "TransferLog \"|/usr/ins/intel/bin/webspy $home\/$acc_log\"\n";
	print CONF "RewriteEngine on\n";
	print CONF "RewriteCond %{HTTP_HOST}	!^$ip(:$port)?$my\n";
	print CONF "RewriteCond %{HTTP_HOST}	!^$vdname(:$port)?$my\n";
	print CONF "RewriteRule ^/(.*)		$http://$vdname/\$1 [L,R]\n";
	print CONF "RewriteOptions inherit\n";
	print CONF "AliasMatch ^/~([^/]+)(/(.*))? $home/users/\$1/web/\$3\n";
	print CONF "AliasMatch ^/users/([^/]+)(/(.*))? $home/users/\$1/web/\$3\n";

	if ($cgi eq 'Yes')
	{
		print CONF "ScriptAlias /cgi-bin/ $home/web/cgi-bin/\n";
	}

	if ($sfpx eq 'Yes')
	{
		print CONF "FrontpageEnable\n";
	}

	if ( ($ssi eq 'Yes') || ($sfpx eq 'Yes') )
	{
		print CONF "<Directory /home/domain/$grp/web/>\n";

		if ($ssi eq 'Yes')
		{
			print CONF "    Addhandler server-parsed .shtml\n";
			print CONF "    Addhandler server-parsed .html\n";
			print CONF "    AddType text/html .shtml\n";
			print CONF "    Options +Includes\n";
		}

       	        if ($sfpx eq 'Yes')
       	        {
			print CONF "    AllowOverride AuthConfig Limit Indexes Options\n";
       	        }

		print CONF "</Directory>\n";
	}

	if ($ssl eq 'Yes')
	{ 
		if ( ( -e "$home/cert/certificate") && (-e "$home/cert/key") )
		{
			print CONF "SSLEnable\n";
			print CONF "SSLengine on\n";
			print CONF "SSLCertificateFile $home/cert/certificate\n";
			print CONF "SSLCertificateKeyFile $home/cert/key\n";
		}
	}

	print CONF "</VirtualHost>\n";

}



################################################################################
#SName: create_vd() -
#Argument: IP, Servername, Admin, Alias, SSI, CGI, FXP
#

sub create_vd {

	my ($ip,$vdname,$admin,$alias,$grp,$ssi,$cgi,$ssl,$sfpx) = @_;
	my ($vd_config,$config,$namehost,$flag);
	my $APACHE_CONF = "/etc/httpd/conf/httpd.conf";
        
	if ( -e $APACHE_CONF )
	{
		copy($APACHE_CONF, $APACHE_CONF.".old") or return($StringResource->_S('APACHE_CONF_FILE_BACKUP_FAIL', [ $APACHE_CONF, " $!" ] ));
	}
	open (FHH,$APACHE_CONF) or &error($StringResource->_S('FILE_OPEN_ERROR', [ ": $!" ] ));
	open (CONF, ">$APACHE_CONF."."old") or &error($StringResource->_S('FILE_OPEN_ERROR', [ ": $!" ] ));
	flock(CONF, 2) or &error($StringResource->_S('FILE_LOCK_ERROR', [ ": $! "] ));
	while(<FHH>){
        if (/^<VirtualHost $ip(.*)>$/ ... /^<\/VirtualHost>$/) {
                        $vd_config  .= $_;
                        return($StringResource->_S('HOSTNAME_ALREADY_IN_USE', [ ": $vdname" ] )) if (/^ServerName\s+$vdname$/);
                }
	elsif(/^NameVirtualHost\s+$ip/) {
		$flag = 'on';
		$namehost = "NameVirtualHost $ip";
	}
		
        else {
                        $config .= $_;
                }
        }
                print CONF "$config";
                my($var1,$var2,$var3) = ip_domain($ip);
                if($var1 eq $ip) {
                        if($flag eq 'on') {
				print CONF "\nNameVirtualHost $ip\n";
			}
			else {	
				print CONF "\nNameVirtualHost $ip\n";
                        }
			vd_conf($ip,$vdname,$admin,$alias,$grp,$ssi,$cgi,$ssl,$sfpx);
                        print CONF "$vd_config";
                }
        	else {
                        print CONF "$vd_config";
                        vd_conf($ip,$vdname,$admin,$alias,$grp,$ssi,$cgi,$ssl,$sfpx);
                }

	close(CONF);
	close(FHH);
	copy($APACHE_CONF.".old", $APACHE_CONF) or return($StringResource->_S('APACHE_CONF_FILE_SAVE_ERROR', [ " $!" ] ));
	unlink($APACHE_CONF.".old");

}

##################################################################################                        
#SName: del_vws()- delete the Virtual Server Configuration from httpd.conf
#Argumnet: IP, Server Name
#
#This can also be done by using the "idx" from the hash table instead of passing the ip 
#and server name to this function
sub del_vws
{
	my ( $vdname,$nwgrp,$admin,$ip ) =  @_;
        
	my ($conf,$item,@abc,$config,$val,$namehost);	
	my $home = "/home/domain/$nwgrp/web";
	my $APACHE_CONF = "/etc/httpd/conf/httpd.conf";
	if ( -e $APACHE_CONF )
	{
		copy($APACHE_CONF, $APACHE_CONF.".old") or return ($StringResource->_S('APACHE_CONF_FILE_BACKUP_FAIL', [ $APACHE_CONF, " $!" ]));
	}
	open (CONF, $APACHE_CONF."\.old") or return ($StringResource->_S('FILE_OPEN_ERROR', [ ": $!" ]));
	open (FHH, ">$APACHE_CONF") or return($StringResource->_S('FILE_OPEN_ERROR', [ ": $!" ] ));
	flock(FHH, 2) or return ($StringResource->_S('FILE_LOCK_ERROR', [ ": $!" ] ));

        while(<CONF>){
                if (/^<VirtualHost $ip(.*)>$/ ... /^<\/VirtualHost>$/) {
                        $config .= $_;
                        if (/^<\/VirtualHost>$/) {
                                $val = scalar(@abc);
                                push @abc,$val,$config;
                                $config = '';
                        }
                }
                elsif (/^NameVirtualHost\s+$ip$/) {
                        $namehost = $_;
                }
                else {  
                        $conf .= $_;
                }
        }
        print FHH "$conf";
        print FHH "$namehost\n" if ($val);

        foreach $item (@abc){
                if (($item =~ /ServerName\s+$vdname/) && ($item =~ /DocumentRoot\s+$home/)) {
                }
                else {  
                        print FHH "$item"  if ($item =~ /S+/);
                }
        }
	close(FHH);
	close(CONF);
	unlink($APACHE_CONF.".old");

}

######################################################################################
sub delVip 
{
	my ($vdname,$nwgrp,$admin,$ip) = @_;
	## Check for IP Address if it is being used by some other virtual server. 
	### Call this also from out side of this subroutine
	my ($item,$vip);
	
	my @num_v_nics = NetConfig->listNetInterfaces();
	my ($result1,$result2,$result3) = ip_domain($ip);
	
	if ($result1 eq  '')
	{
        	foreach  $item (@num_v_nics)
		{
                	 $vip = $item->{'virtual'} if ($item->{'ipaddress'} eq $ip);
                }
		if (!$vip)
		{
########		return($StringResource->_S('IPADDR_IN_USE'));
		}
		else
		{	
			NetConfig->deleteNetInterface('',$ip);
			RouteConfig->deleteRoute($ip, '0.0.0.0', '', "eth0")
		}
	}
	
	my $value = cleanup($vdname,$nwgrp,$admin,$ip);
        
	if ($value ne '')
	{
                return($StringResource->_S('DOMAIN_DELETE_FAILED', [ " $value : $!" ])); 
        }
	
	return '';
}

#############
# Check for the IP and Server Name if the IP is already being used by some other virtual server. 
#
sub ip_domain 
{
        my ($ipadd,$domain) = @_;
        my (@domains, $item, $result1, $result2,$result3);
        @domains = list_vws();
        if ($domain ne '') {
                foreach $item (@domains) {
                        if (($ipadd eq $item->{'vdip'}) and ($domain ne $item->{vdn})) {
                                $result1 = $item->{'vdip'};
                                $result2 = $item->{'vdn'};
                                $result3 = $item->{'dom_grp'};
                        	last;
			}
                }


        }
        else {  
                foreach $item (@domains) {
                        if ($ipadd eq $item->{'vdip'}) {
                                $result1 = $item->{'vdip'};
                                $result2 = $item->{'vdn'};
                                $result3 = $item->{'dom_grp'};
                        	last;
			}
                }
        }
return ($result1,$result2,$result3);
}
#############################################################################


###################################################################################
######################################################################################################
# Steps to follow > Create Group  > Create User > Create Home dir > Create Home directory >
# > Create VIP if req. > Create FTP conf > create http.conf > Create Mail >
# Front Page Support to work with next .....
# create a new group for the domain and then use this name to create the home directory for the domain. 
# So here it goes.....
######################################################################################################

sub create_dom_home 	
{

	my ($vdname,$nwgrp,$admin,$passd,$real,$ssl,$priv,$admin_ssl) = @_;
	my ($domain_hom,$logs,$web,$er_log,$ac_log,$ip,$ne,$c,$user_home,$home_path,$done);
	if ($priv eq '') {$priv = 'domainadmin'; }
	$domain_hom = "/home/domain";
	$user_home ='users';
	$home_path = "$domain_hom/$nwgrp/$user_home/$admin";
	$web = "web";
	$logs = 'logs';
	$er_log = 'error_log';
	$ac_log = 'access_log';
	$done = 'yes';
	my $cert = 'cert';	
	my $ssl_cert = '/etc/httpd/ssl';

	# Alert : This may be used for checking if some dork has created a file with this name 
	# and rename it as *.old file

	unless (-d "$domain_hom/$nwgrp") {
		mkdir("$domain_hom/$nwgrp", 0775) or return($StringResource->_S('DOMAIN_HOME_CREATE_ERROR', [ ": $!" ] ));
		}
	# Create simlink for the domain directory to make it clear for the domains...
	unless(-e "$domain_hom/$vdname") {
		symlink("$domain_hom/$nwgrp","$domain_hom/$vdname");
	}
	# Create Home dir for users............
	unless (-d "$domain_hom/$nwgrp/$user_home") { 
		mkdir("$domain_hom/$nwgrp/$user_home", 0775) or return($StringResource->_S('USER_HOME_CREATE_ERROR', [ $home_path, " $!" ] ));
   		}

	########## Copy SSL certificate in home dir....


	if ($ssl eq 'Yes') {
		mkdir("$domain_hom/$nwgrp/$cert", 0660) or return("Error creating $nwgrp/$cert  dir:  $!");
		Safe::system("/bin/cp","$ssl_cert/certificate","$domain_hom/$nwgrp/$cert/");	
		Safe::system("/bin/cp","$ssl_cert/request","$domain_hom/$nwgrp/$cert/");	
		Safe::system("/bin/cp","$ssl_cert/key","$domain_hom/$nwgrp/$cert/");	

		unlink("$ssl_cert/key");
		unlink("$ssl_cert/request");
		unlink("$ssl_cert/certificate");
		}	


	########################
	# Create users also.

	if ( -d "$domain_hom/$nwgrp/$user_home" ) {
	my $var1 = Users::createAdmin($admin,$real,$passd,$priv,$home_path,$nwgrp) or return ($StringResource->_S('USER_CREATE_ERROR', [ $admin, " $!" ] ));
		return $var1 unless ($var1 =~ /YES/);
	}
	else { return ("User home directory does not exists"); }

	unless (-d "$domain_hom/$nwgrp/$web") { 
		mkdir("$domain_hom/$nwgrp/$web", 0775) or return($StringResource->_S('DOMAIN_WEB_DIR_CREATE_ERROR', [ ": $!" ] ));
		Safe::system("/bin/cp -fR /usr/ins/intel/gui/skel/domain/* $domain_hom/$nwgrp/$web");
		}
	Safe::system("/bin/chown","-R","$admin:$nwgrp","$domain_hom/$nwgrp/$web");
	# Change index.html page to reflect the right URL of domain ....
	# 	
		
	Usable::changeIndex($vdname,"$domain_hom/$nwgrp/$web",$nwgrp,'',$admin_ssl);
	chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$domain_hom/$nwgrp/$web");
	chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$domain_hom/$nwgrp");
	chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$domain_hom/$vdname");
	chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$domain_hom/$nwgrp/$user_home");
	chmod(00755,"$domain_hom/$nwgrp/$web");
   		
	# Create Web Log directory for the virtual server
	unless (-d "$domain_hom/$nwgrp/$logs") { 
		mkdir("$domain_hom/$nwgrp/$logs", 0775) or return($StringResource->_S('LOG_DIR_CREATE_ERROR', [ ": $!" ] ));
		chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$domain_hom/$nwgrp/$logs");
  		
		}
	
		return $done;
}


###########################################################################################################
#   This would add the virtual IP for the Virtual Web server ..
#   If the IP address already exists then do next step ... don't die....

sub create_vip
{
	my ($ipadd,$netmask,$nwgrp, $admin,$vdname,$sftp) = @_;
	my ($domain_hom,$ip,$done);
	        
	$done = 'yes';
	
	my $nic1 = SetNic::generateIPAlias('eth0');
	$nic1 = "eth0:".$nic1;
	
	if ($sftp eq 'Yes')
	{
		setFtp($ipadd,$nwgrp,$admin,$vdname);
	}
	        
	NetConfig->changeNic($nic1,$ipadd,$netmask);
	RouteConfig->createRoute($ipadd, '', '', "eth0", 0);
	        
	return $done;
}

#############################################################################################
# Create Virtual FTP server Configuration ............
# Add in ftpserver , create config tree, create ftp home, create ftpaccess ............

sub setFtp 

{

	my ($ipadd,$nwgrp,$admin,$vdname) = @_;
        my ($domain_hom,$ip,$ne,$c,$home_path,$done,$ftpserver,$conf,$ins_gui);
	my (@domains,$result1,$result2,$text);
        $domain_hom = '/home/domain';
        $ftpserver = 'ftpservers'; 
        $ins_gui = '/usr/ins/intel/wu-ftpd';
        $conf = 'conf';
        $done = 'yes';
	$text = $StringResource->_S('VIRTUAL_FTP_SERVICE_NOT_AVAILABLE');
	@domains = list_vws();
        foreach my $item (@domains) {
                 if ($ipadd eq $item->{'vdip'}) {
                        $result1 = $item->{'vdip'};
                        $result2 = $item->{'vdn'};
                }
	return $text if(($result2 ne $vdname) and ($result1 eq $ipadd)); 
	}	
	
	# Create Virtual FTP server Configuration ............
        # Add in ftpserver , create config tree, create ftp home, create ftpaccess ............

	my $ret_val = VirtualFtp::getFtpStatus($ipadd,$nwgrp);	
	unless ($ret_val) { 

		VirtualFtp::ftpserverAdd($ipadd,$nwgrp);

	        unless (-d "$ins_gui/$conf/$nwgrp") {
	
                	mkdir("$ins_gui/$conf/$nwgrp", 0775) or &error($StringResource->_S('FTP_CONFIG_CREATE_ERROR', [ ": $!" ] )); 
               		chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$ins_gui/$conf/$nwgrp");
                }
                # Config files ofr Virtual FTP but require unique IP else it will be shared.
                # Requite to change the 1st two lines in ftpaccess

                Safe::system("/bin/cp","$ins_gui/$conf/ftpconversions","$ins_gui/$conf/$nwgrp/ftpconversionss");
                Safe::system("/bin/cp","$ins_gui/$conf/ftpusers","$ins_gui/$conf/$nwgrp/ftpusers");

        	unless (-d "$domain_hom/$nwgrp/ftp") {
                	mkdir("$domain_hom/$nwgrp/ftp", 0775) or &error($StringResource->_S('FTP_DIR_CREATE_ERROR', [ ": $!" ] ));
                	chown((getpwnam($admin))[2],(getgrnam($nwgrp))[2], "$domain_hom/$nwgrp/ftp");
                
		}
			VirtualServ::set_virtual_ftp($domain_hom,$nwgrp,$vdname,$admin);
             # Files required to Anonymous ftp
             
	     Users::create_chroot_environment("$domain_hom/$nwgrp/ftp/");
	}	
	return $done;	
}


#############################################################################################
sub delFtp
{
	my ($ip,$nwgrp) = @_;
	my (@line,$command,$args);
        my $ftp_home = "/usr/ins/intel/wu-ftpd/conf";
	
	my $ret_val1 = VirtualFtp::getFtpStatus($ip,$nwgrp);
	
	if ($ret_val1) {
		VirtualFtp::ftpserverDel($ip,$nwgrp);
	}	
	# Delete logs, users, ftp , web 
	# Need to use some other way to remove this ...
	$command = 'rm';
	$args = '-rf';
	if ($nwgrp =~ /domain(\d+)/) {
		if (-d "$ftp_home/$nwgrp") {
                	system($command,$args,"$ftp_home/$nwgrp");
        	}      
	}

}

#################################################################################################
# Clean group if exists, Clean Users if.., clean home dir for domain and clean everything. 
#
sub cleanup
{
	my ($vdname,$nwgrp,$admin,$ip) = @_;
	my ($command, $args,$domain_hom, $uid, $home, $gid,@line);
	$domain_hom = "/home/domain";
	my $symlink = "/home/domain/$vdname";
        my $user_home ='users';
       	my $web = "web";
        my $logs = 'logs';
        my $done = '';
	($uid, $home) = (getpwnam($admin))[2,7];
	$gid = getgrnam($nwgrp);
	Users::usrDel($admin) if ($uid);
	Users::grpDel($nwgrp) if ($gid);
        unlink("$symlink") if (-l "$symlink");	
	
	delFtp($ip, $nwgrp);
	$command = 'rm';
	$args = '-rf';
	if ($nwgrp =~ /domain(\d+)/) {
		if (-d "$domain_hom/$nwgrp") {
			system($command, $args, "$domain_hom/$nwgrp");
		}	
	}
	return $done;
}
##################################################################################
sub mod_home 
{
	my ($vdname,$nwgrp,$admin,$ip) = @_;
	my ($command, $args,$domain_hom, $uid, $home, $gid);
        $domain_hom = "/home/domain/$nwgrp";
	my $symlink = "/home/domain/$vdname";
        my $user_home ='users';
       	my $web = "web";
        my $logs = 'logs';
        my $done = 'yes';
	($uid, $home) = (getpwnam($admin))[2,7];
	$gid = getgrnam($nwgrp);
#	Modify the user 
#	Users::usrDel($admin) if ($uid);

	if (-l "$symlink") {			
		unlink("$symlink"); 
		unless(-e "$domain_hom/$vdname") {
			symlink("$domain_hom/$nwgrp","$domain_hom/$vdname");
		}

	}
	return $done;
}

######################################################################
# &activate($domainname, $port, $username, $password, $group);

# Run the appropriate fpsrvadm.exe command to activate frontpage
# on a virtual web site specified by the arguments:

# $domainname - the domain name of the virtual web site. (e.g. "test1.com")
# $port       - the numeric port that the web sites runs on.  (e.g. 80)
# $username   - the username of the domain owner
# $password   - the password of the domain owner
# $group      - the unix group of the domain owner

# The fpsrvadm.exe program will use the specified $username and
# $password to create an entry in a web authentication file.  The
# user will use this username and password to connect with the
# frontpage client.


sub activateFpx {
    my ($domainname,$port,$username,$password,$group) = @_;

        my $path_to_fpsrvadm =
                "/usr/local/frontpage/currentversion/bin/fpsrvadm.exe";
        my $path_to_httpd_conf =
        "/etc/httpd/conf/httpd.conf";
	my $path_to_frontpage_configuration =
		"/usr/local/frontpage";

    # set the proper umask
    umask(0022);
    # were all the needed parameters specified?
    if ( (! $domainname) ||
         (! $port)       ||
         (! $username)   ||
         (! $password)   ||
         (! $group) ) {
        return 0;
    }

# build the command
    my @cmd = ($path_to_fpsrvadm,
               '-operation', 'install',
               '-t', 'apache-fp',
               '-m', $domainname,
               '-p', ($domainname . ':' . $port),
               '-u', $username,
               '-pw', $password,
               '-s', $path_to_httpd_conf,
               '-xu', $username,
               '-xg', $group);

    # debug...
    #print join(" ",@cmd),"\n";

    # run the command.
    my $ret_value = &configFpx(@cmd);

    # there is a bug in fpsrvadm.exe that causes the frontpage
    # configuration file to be created with port 80 in its filename,
    # even if another port was specified.  work around the bug by
    # copying the file.  (We can't just rename the file, since there
    # may be a FP-enabled web site running on port 80 that just got
    # its .cnf file wiped out by fpsrvadm.exe...)
    if ($port != 80) {
        if ( -f "$path_to_frontpage_configuration/$domainname:80.cnf" ) {
            Safe::system('/bin/cp', '-p',
                   "$path_to_frontpage_configuration/$domainname:80.cnf",
                   "$path_to_frontpage_configuration/$domainname:$port.cnf");
	}
    }

    # Also make a copy for port 443, in case the user wants to enable
    # SSL on this domain at some point in the future.
    if ($port != 443) {
        if ( -f "$path_to_frontpage_configuration/$domainname:80.cnf" ) {
            Safe::system('/bin/cp', '-p',
                   "$path_to_frontpage_configuration/$domainname:80.cnf",
                   "$path_to_frontpage_configuration/$domainname:443.cnf");
        }
    }

    return $ret_value;
}


# &deactivate($domainname, $port);

# Deactivate frontpage on the specified virtual web site.


sub deactivateFpx {
    my ($domainname, $port) = @_;

        my $path_to_fpsrvadm =
        "/usr/local/frontpage/currentversion/bin/fpsrvadm.exe";
        my $path_to_httpd_conf =
        "/etc/httpd/conf/httpd.conf";

    # set the proper umask
    umask(0022);

    # were all the needed parameters specified?
    if ( (! $domainname) || (! $port) ) {
        return 0;
    }

    # build the command
    my @cmd = ($path_to_fpsrvadm,
               '-operation', 'uninstall',
               '-t', 'apache-fp',
               '-p', ($domainname . ':' . $port),
               '-s', $path_to_httpd_conf);

    # debug..
    #print join(" ",@cmd),"\n";

    # run the command.
    return &configFpx(@cmd);
}



# &run_command(@command)

# forks, reopens standard out and error to /dev/null,
# and runs the command specified by the array.


sub configFpx {
    my @cmd = @_;
    my $completed = 0;

    # fork!
    my $pid = open(CHILD,"-|");

    if ($pid) {           #### parent process

        while (<CHILD>) {
            # scan the output of fpsrvadm for the
            # word "completed", which indicates success
            if ( /completed/i ) {
                $completed = 1;
            }
        }
        close(CHILD);

    } else {              #### child process
        # exec the command
        exec @cmd;
#        exit;
    }

    if ($completed) {
        return 1;
    } else {
        return 0;
    }

}

#
# resetPasswordFpx
#
# Use the "security" operation of fpsrvadm to reset the FrontPage
# password to a known value.
#
# If $old_username and $new_username are identical, this subroutine
# only changes the password.  If they are different, a new FrontPage
# user is created with the new username, and then the old username
# is removed.
#

sub resetPasswordFpx {
    my ($domainname,$port,$old_username,$new_username,$password,$group) = @_;

        my $path_to_fpsrvadm =
                "/usr/local/frontpage/currentversion/bin/fpsrvadm.exe";
        my $path_to_httpd_conf =
        "/etc/httpd/conf/httpd.conf";
	my $path_to_frontpage_configuration =
		"/usr/local/frontpage";

    # set the proper umask
    umask(0022);
    # were all the needed parameters specified?
    if ( (! $domainname) ||
         (! $port)       ||
         (! $old_username)   ||
         (! $new_username)   ||
         (! $password)   ||
         (! $group) ) {
        return 0;
    }

    # build the command
    my @cmd = ($path_to_fpsrvadm,
               '-operation', 'security',
               '-w', '/',              ## security always wants the root web
               '-a', 'administrators', ## we always update the FP administrator
               '-t', 'apache-fp',
               '-m', $domainname,
               '-p', $port,            ## "simple" port number here, as opposed to activateFpx
               '-u', $new_username,
               '-pw', $password,
               '-s', $path_to_httpd_conf,
               '-xu', $new_username,
               '-xg', $group);

    # run the command.
    my $ret_value = &configFpx(@cmd);

    # if the username is changing, attempt to delete the old FrontPage user.
    if ($old_username ne $new_username) {
        # build the command
        @cmd = ($path_to_fpsrvadm,
                   '-operation', 'security',
                   '-w', '/',              ## security always wants the root web
                   '-a', 'remove',         ## an AccessType of "remove" deletes the user.
                   '-t', 'apache-fp',
                   '-m', $domainname,
                   '-p', $port,            ## "simple" port number here, as opposed to activateFpx
                   '-u', $old_username,    ## remove the OLD username
                   '-pw', $password,
                   '-s', $path_to_httpd_conf,
                   '-xu', $new_username,
                   '-xg', $group);

        # run the command.
        &configFpx(@cmd);
    }

    return $ret_value;
}

1;

